@@ -5,7 +5,7 @@
 // Author:      David Elliott <dfe@cox.net>
 // Modified by:
 // Created:     2007/05/02
-// RCS-ID:      $Id: trackingrectmanager.h 51595 2008-02-08 09:12:04Z DE $
+// RCS-ID:      $Id: trackingrectmanager.h 58757 2009-02-08 11:45:59Z VZ $
 // Copyright:   (c) 2007 Software 2000 Ltd.
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -18,7 +18,7 @@
 
 class wxCocoaTrackingRectManager
 {
-    DECLARE_NO_COPY_CLASS(wxCocoaTrackingRectManager)
+    wxDECLARE_NO_COPY_CLASS(wxCocoaTrackingRectManager);
 public:
     wxCocoaTrackingRectManager(wxWindow *window);
     void ClearTrackingRect();
