@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: docview.h 53546 2008-05-10 21:02:36Z VZ $
+// RCS-ID:      $Id: docview.h 64569 2010-06-12 18:21:42Z VZ $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -19,6 +19,7 @@
 #include "wx/list.h"
 #include "wx/string.h"
 #include "wx/frame.h"
+#include "wx/filehistory.h"
 
 #if wxUSE_PRINTING_ARCHITECTURE
     #include "wx/print.h"
@@ -31,39 +32,37 @@ class WXDLLIMPEXP_FWD_CORE wxDocTemplate;
 class WXDLLIMPEXP_FWD_CORE wxDocManager;
 class WXDLLIMPEXP_FWD_CORE wxPrintInfo;
 class WXDLLIMPEXP_FWD_CORE wxCommandProcessor;
-class WXDLLIMPEXP_FWD_CORE wxFileHistory;
 class WXDLLIMPEXP_FWD_BASE wxConfigBase;
 
+class wxDocChildFrameAnyBase;
+
 #if wxUSE_STD_IOSTREAM
   #include "wx/iosfwrap.h"
 #else
   #include "wx/stream.h"
 #endif
 
-// Document manager flags
+// Flags for wxDocManager (can be combined).
 enum
 {
-    wxDOC_SDI = 1,
-    wxDOC_MDI,
-    wxDOC_NEW,
-    wxDOC_SILENT,
-    wxDEFAULT_DOCMAN_FLAGS = wxDOC_SDI
+    wxDOC_NEW    = 1,
+    wxDOC_SILENT = 2
 };
 
 // Document template flags
 enum
 {
     wxTEMPLATE_VISIBLE = 1,
-    wxTEMPLATE_INVISIBLE,
+    wxTEMPLATE_INVISIBLE = 2,
     wxDEFAULT_TEMPLATE_FLAGS = wxTEMPLATE_VISIBLE
 };
 
 #define wxMAX_FILE_HISTORY 9
 
-class WXDLLEXPORT wxDocument : public wxEvtHandler
+class WXDLLIMPEXP_CORE wxDocument : public wxEvtHandler
 {
 public:
-    wxDocument(wxDocument *parent = (wxDocument *) NULL);
+    wxDocument(wxDocument *parent = NULL);
     virtual ~wxDocument();
 
     // accessors
@@ -76,9 +75,16 @@ public:
     void SetDocumentName(const wxString& name) { m_documentTypeName = name; }
     wxString GetDocumentName() const { return m_documentTypeName; }
 
+    // access the flag indicating whether this document had been already saved,
+    // SetDocumentSaved() is only used internally, don't call it
     bool GetDocumentSaved() const { return m_savedYet; }
     void SetDocumentSaved(bool saved = true) { m_savedYet = saved; }
 
+    // return true if the document hasn't been modified since the last time it
+    // was saved (implying that it returns false if it was never saved, even if
+    // the document is not modified)
+    bool AlreadySaved() const { return !IsModified() && GetDocumentSaved(); }
+
     virtual bool Close();
     virtual bool Save();
     virtual bool SaveAs();
@@ -103,6 +109,10 @@ public:
     // modified to false)
     virtual bool OnSaveModified();
 
+    // if you override, remember to call the default
+    // implementation (wxDocument::OnChangeFilename)
+    virtual void OnChangeFilename(bool notifyViews);
+
     // Called by framework if created automatically by the default document
     // manager: gives document a chance to initialise and (usually) create a
     // view
@@ -110,18 +120,22 @@ public:
 
     // By default, creates a base wxCommandProcessor.
     virtual wxCommandProcessor *OnCreateCommandProcessor();
-    virtual wxCommandProcessor *GetCommandProcessor() const { return m_commandProcessor; }
-    virtual void SetCommandProcessor(wxCommandProcessor *proc) { m_commandProcessor = proc; }
+    virtual wxCommandProcessor *GetCommandProcessor() const
+        { return m_commandProcessor; }
+    virtual void SetCommandProcessor(wxCommandProcessor *proc)
+        { m_commandProcessor = proc; }
 
     // Called after a view is added or removed. The default implementation
     // deletes the document if this is there are no more views.
     virtual void OnChangedViewList();
 
+    // Called from OnCloseDocument(), does nothing by default but may be
+    // overridden. Return value is ignored.
     virtual bool DeleteContents();
 
     virtual bool Draw(wxDC&);
     virtual bool IsModified() const { return m_documentModified; }
-    virtual void Modify(bool mod) { m_documentModified = mod; }
+    virtual void Modify(bool mod);
 
     virtual bool AddView(wxView *view);
     virtual bool RemoveView(wxView *view);
@@ -129,7 +143,7 @@ public:
     const wxList& GetViews() const { return m_documentViews; }
     wxView *GetFirstView() const;
 
-    virtual void UpdateAllViews(wxView *sender = (wxView *) NULL, wxObject *hint = (wxObject *) NULL);
+    virtual void UpdateAllViews(wxView *sender = NULL, wxObject *hint = NULL);
     virtual void NotifyClosing();
 
     // Remove all views (because we're closing the document)
@@ -137,25 +151,22 @@ public:
 
     // Other stuff
     virtual wxDocManager *GetDocumentManager() const;
-    virtual wxDocTemplate *GetDocumentTemplate() const { return m_documentTemplate; }
-    virtual void SetDocumentTemplate(wxDocTemplate *temp) { m_documentTemplate = temp; }
+    virtual wxDocTemplate *GetDocumentTemplate() const
+        { return m_documentTemplate; }
+    virtual void SetDocumentTemplate(wxDocTemplate *temp)
+        { m_documentTemplate = temp; }
 
-    // Get title, or filename if no title, else [unnamed]
-    //
-    // NB: this method will be deprecated in wxWidgets 3.0, you still need to
-    //     override it if you need to modify the existing behaviour in this
-    //     version but use GetUserReadableName() below if you just need to call
-    //     it
-    virtual bool GetPrintableName(wxString& buf) const;
+    // Get the document name to be shown to the user: the title if there is
+    // any, otherwise the filename if the document was saved and, finally,
+    // "unnamed" otherwise
+    virtual wxString GetUserReadableName() const;
 
-#if wxABI_VERSION >= 20805
-    wxString GetUserReadableName() const
-    {
-        wxString s;
-        GetPrintableName(s);
-        return s;
-    }
-#endif // wxABI 2.8.5+
+#if WXWIN_COMPATIBILITY_2_8
+    // use GetUserReadableName() instead
+    wxDEPRECATED_BUT_USED_INTERNALLY(
+        virtual bool GetPrintableName(wxString& buf) const
+    );
+#endif // WXWIN_COMPATIBILITY_2_8
 
     // Returns a window that can be used as a parent for document-related
     // dialogs. Override if necessary.
@@ -178,15 +189,17 @@ protected:
     virtual bool DoSaveDocument(const wxString& file);
     virtual bool DoOpenDocument(const wxString& file);
 
+    // the default implementation of GetUserReadableName()
+    wxString DoGetUserReadableName() const;
+
 private:
     DECLARE_ABSTRACT_CLASS(wxDocument)
-    DECLARE_NO_COPY_CLASS(wxDocument)
+    wxDECLARE_NO_COPY_CLASS(wxDocument);
 };
 
-class WXDLLEXPORT wxView: public wxEvtHandler
+class WXDLLIMPEXP_CORE wxView: public wxEvtHandler
 {
 public:
-    //  wxView(wxDocument *doc = (wxDocument *) NULL);
     wxView();
     virtual ~wxView();
 
@@ -199,16 +212,19 @@ public:
     wxWindow *GetFrame() const { return m_viewFrame ; }
     void SetFrame(wxWindow *frame) { m_viewFrame = frame; }
 
-    virtual void OnActivateView(bool activate, wxView *activeView, wxView *deactiveView);
+    virtual void OnActivateView(bool activate,
+                                wxView *activeView,
+                                wxView *deactiveView);
     virtual void OnDraw(wxDC *dc) = 0;
     virtual void OnPrint(wxDC *dc, wxObject *info);
-    virtual void OnUpdate(wxView *sender, wxObject *hint = (wxObject *) NULL);
+    virtual void OnUpdate(wxView *sender, wxObject *hint = NULL);
     virtual void OnClosingDocument() {}
     virtual void OnChangeFilename();
 
     // Called by framework if created automatically by the default document
     // manager class: gives view a chance to initialise
-    virtual bool OnCreate(wxDocument *WXUNUSED(doc), long WXUNUSED(flags)) { return true; }
+    virtual bool OnCreate(wxDocument *WXUNUSED(doc), long WXUNUSED(flags))
+        { return true; }
 
     // Checks if the view is the last one for the document; if so, asks user
     // to confirm save data (if modified). If ok, deletes itself and returns
@@ -218,9 +234,6 @@ public:
     // Override to do cleanup/veto close
     virtual bool OnClose(bool deleteWindow);
 
-    // Extend event processing to search the document's event table
-    virtual bool ProcessEvent(wxEvent& event);
-
     // A view's window can call this to notify the view it is (in)active.
     // The function then notifies the document manager.
     virtual void Activate(bool activate);
@@ -232,18 +245,30 @@ public:
     virtual wxPrintout *OnCreatePrintout();
 #endif
 
+    // implementation only
+    // -------------------
+
+    // set the associated frame, it is used to reset its view when we're
+    // destroyed
+    void SetDocChildFrame(wxDocChildFrameAnyBase *docChildFrame);
+
 protected:
+    // hook the document into event handlers chain here
+    virtual bool TryBefore(wxEvent& event);
+
     wxDocument*       m_viewDocument;
     wxString          m_viewTypeName;
     wxWindow*         m_viewFrame;
 
+    wxDocChildFrameAnyBase *m_docChildFrame;
+
 private:
     DECLARE_ABSTRACT_CLASS(wxView)
-    DECLARE_NO_COPY_CLASS(wxView)
+    wxDECLARE_NO_COPY_CLASS(wxView);
 };
 
 // Represents user interface (and other) properties of documents and views
-class WXDLLEXPORT wxDocTemplate: public wxObject
+class WXDLLIMPEXP_CORE wxDocTemplate: public wxObject
 {
 
 friend class WXDLLIMPEXP_FWD_CORE wxDocManager;
@@ -258,8 +283,8 @@ public:
                   const wxString& ext,
                   const wxString& docTypeName,
                   const wxString& viewTypeName,
-                  wxClassInfo *docClassInfo = (wxClassInfo *) NULL,
-                  wxClassInfo *viewClassInfo = (wxClassInfo *)NULL,
+                  wxClassInfo *docClassInfo = NULL,
+                  wxClassInfo *viewClassInfo = NULL,
                   long flags = wxDEFAULT_TEMPLATE_FLAGS);
 
     virtual ~wxDocTemplate();
@@ -272,13 +297,16 @@ public:
 
     // Helper method for CreateDocument; also allows you to do your own document
     // creation
-    virtual bool InitDocument(wxDocument* doc, const wxString& path, long flags = 0);
+    virtual bool InitDocument(wxDocument* doc,
+                              const wxString& path,
+                              long flags = 0);
 
     wxString GetDefaultExtension() const { return m_defaultExt; }
     wxString GetDescription() const { return m_description; }
     wxString GetDirectory() const { return m_directory; }
     wxDocManager *GetDocumentManager() const { return m_documentManager; }
-    void SetDocumentManager(wxDocManager *manager) { m_documentManager = manager; }
+    void SetDocumentManager(wxDocManager *manager)
+        { m_documentManager = manager; }
     wxString GetFileFilter() const { return m_fileFilter; }
     long GetFlags() const { return m_flags; }
     virtual wxString GetViewName() const { return m_viewTypeName; }
@@ -290,7 +318,7 @@ public:
     void SetDefaultExtension(const wxString& ext) { m_defaultExt = ext; }
     void SetFlags(long flags) { m_flags = flags; }
 
-    bool IsVisible() const { return ((m_flags & wxTEMPLATE_VISIBLE) == wxTEMPLATE_VISIBLE); }
+    bool IsVisible() const { return (m_flags & wxTEMPLATE_VISIBLE) != 0; }
 
     wxClassInfo* GetDocClassInfo() const { return m_docClassInfo; }
     wxClassInfo* GetViewClassInfo() const { return m_viewClassInfo; }
@@ -311,23 +339,26 @@ protected:
     wxClassInfo*      m_docClassInfo;
     wxClassInfo*      m_viewClassInfo;
 
-    // Called by CreateDocument and CreateView to create the actual document/view object.
-    // By default uses the ClassInfo provided to the constructor. Override these functions
-    // to provide a different method of creation.
+    // Called by CreateDocument and CreateView to create the actual
+    // document/view object.
+    //
+    // By default uses the ClassInfo provided to the constructor. Override
+    // these functions to provide a different method of creation.
     virtual wxDocument *DoCreateDocument();
     virtual wxView *DoCreateView();
 
 private:
     DECLARE_CLASS(wxDocTemplate)
-    DECLARE_NO_COPY_CLASS(wxDocTemplate)
+    wxDECLARE_NO_COPY_CLASS(wxDocTemplate);
 };
 
 // One object of this class may be created in an application, to manage all
 // the templates and documents.
-class WXDLLEXPORT wxDocManager: public wxEvtHandler
+class WXDLLIMPEXP_CORE wxDocManager: public wxEvtHandler
 {
 public:
-    wxDocManager(long flags = wxDEFAULT_DOCMAN_FLAGS, bool initialize = true);
+    // NB: flags are unused, don't pass wxDOC_XXX to this ctor
+    wxDocManager(long flags = 0, bool initialize = true);
     virtual ~wxDocManager();
 
     virtual bool Initialize();
@@ -340,32 +371,34 @@ public:
     void OnFileRevert(wxCommandEvent& event);
     void OnFileSave(wxCommandEvent& event);
     void OnFileSaveAs(wxCommandEvent& event);
+    void OnMRUFile(wxCommandEvent& event);
+#if wxUSE_PRINTING_ARCHITECTURE
     void OnPrint(wxCommandEvent& event);
     void OnPreview(wxCommandEvent& event);
+    void OnPageSetup(wxCommandEvent& event);
+#endif // wxUSE_PRINTING_ARCHITECTURE
     void OnUndo(wxCommandEvent& event);
     void OnRedo(wxCommandEvent& event);
 
     // Handlers for UI update commands
     void OnUpdateFileOpen(wxUpdateUIEvent& event);
-    void OnUpdateFileClose(wxUpdateUIEvent& event);
+    void OnUpdateDisableIfNoDoc(wxUpdateUIEvent& event);
     void OnUpdateFileRevert(wxUpdateUIEvent& event);
     void OnUpdateFileNew(wxUpdateUIEvent& event);
     void OnUpdateFileSave(wxUpdateUIEvent& event);
-    void OnUpdateFileSaveAs(wxUpdateUIEvent& event);
     void OnUpdateUndo(wxUpdateUIEvent& event);
     void OnUpdateRedo(wxUpdateUIEvent& event);
 
-    void OnUpdatePrint(wxUpdateUIEvent& event);
-    void OnUpdatePreview(wxUpdateUIEvent& event);
-
-    // Extend event processing to search the view's event table
-    virtual bool ProcessEvent(wxEvent& event);
-
     // called when file format detection didn't work, can be overridden to do
     // something in this case
     virtual void OnOpenFileFailure() { }
 
     virtual wxDocument *CreateDocument(const wxString& path, long flags = 0);
+
+    // wrapper around CreateDocument() with a more clear name
+    wxDocument *CreateNewDocument()
+        { return CreateDocument(wxString(), wxDOC_NEW); }
+
     virtual wxView *CreateView(wxDocument *doc, long flags = 0);
     virtual void DeleteTemplate(wxDocTemplate *temp, long flags = 0);
     virtual bool FlushDoc(wxDocument *doc);
@@ -402,27 +435,14 @@ public:
     // Views or windows should inform the document manager
     // when a view is going in or out of focus
     virtual void ActivateView(wxView *view, bool activate = true);
-    virtual wxView *GetCurrentView() const;
+    virtual wxView *GetCurrentView() const { return m_currentView; }
 
     wxList& GetDocuments() { return m_docs; }
     wxList& GetTemplates() { return m_templates; }
 
-    // Make a default document name
-    //
-    // NB: this method is renamed to MakeNewDocumentName() in wx 3.0, you still
-    //     need to override it if your code needs to customize the default name
-    //     generation but if you just use it from your code, prefer the version
-    //     below which is forward-compatible with wx 3.0
-    virtual bool MakeDefaultName(wxString& buf);
-
-#if wxABI_VERSION >= 20808
-    wxString MakeNewDocumentName() const
-    {
-        wxString s;
-        wx_const_cast(wxDocManager *, this)->MakeDefaultName(s);
-        return s;
-    }
-#endif // wx ABI >= 2.8.8
+    // Return the default name for a new document (by default returns strings
+    // in the form "unnamed <counter>" but can be overridden)
+    virtual wxString MakeNewDocumentName();
 
     // Make a frame title (override this to do something different)
     virtual wxString MakeFrameTitle(wxDocument* doc);
@@ -438,26 +458,56 @@ public:
     virtual void FileHistoryUseMenu(wxMenu *menu);
     virtual void FileHistoryRemoveMenu(wxMenu *menu);
 #if wxUSE_CONFIG
-    virtual void FileHistoryLoad(wxConfigBase& config);
+    virtual void FileHistoryLoad(const wxConfigBase& config);
     virtual void FileHistorySave(wxConfigBase& config);
 #endif // wxUSE_CONFIG
 
     virtual void FileHistoryAddFilesToMenu();
     virtual void FileHistoryAddFilesToMenu(wxMenu* menu);
 
-    wxString GetLastDirectory() const { return m_lastDirectory; }
+    wxString GetLastDirectory() const;
     void SetLastDirectory(const wxString& dir) { m_lastDirectory = dir; }
 
     // Get the current document manager
     static wxDocManager* GetDocumentManager() { return sm_docManager; }
 
+#if WXWIN_COMPATIBILITY_2_8
+    // deprecated, override GetDefaultName() instead
+    wxDEPRECATED_BUT_USED_INTERNALLY(
+        virtual bool MakeDefaultName(wxString& buf)
+    );
+#endif
+
 #if WXWIN_COMPATIBILITY_2_6
     // deprecated, use GetHistoryFilesCount() instead
     wxDEPRECATED( size_t GetNoHistoryFiles() const );
 #endif // WXWIN_COMPATIBILITY_2_6
 
+
 protected:
-    long              m_flags;
+    // Open the MRU file with the given index in our associated file history.
+    void DoOpenMRUFile(unsigned n);
+#if wxUSE_PRINTING_ARCHITECTURE
+    virtual wxPreviewFrame* CreatePreviewFrame(wxPrintPreviewBase* preview,
+                                               wxWindow *parent,
+                                               const wxString& title);
+#endif // wxUSE_PRINTING_ARCHITECTURE
+
+    // hook the currently active view into event handlers chain here
+    virtual bool TryBefore(wxEvent& event);
+
+    // return the command processor for the current document, if any
+    wxCommandProcessor *GetCurrentCommandProcessor() const;
+
+    // this method tries to find an active view harder than GetCurrentView():
+    // if the latter is NULL, it also checks if we don't have just a single
+    // view and returns it then
+    wxView *GetActiveView() const;
+
+    // activate the first view of the given document if any
+    void ActivateDocument(wxDocument *doc);
+
+
     int               m_defaultDocumentNameCounter;
     int               m_maxDocsOpen;
     wxList            m_docs;
@@ -467,9 +517,13 @@ protected:
     wxString          m_lastDirectory;
     static wxDocManager* sm_docManager;
 
+#if wxUSE_PRINTING_ARCHITECTURE
+    wxPageSetupDialogData m_pageSetupDialogData;
+#endif // wxUSE_PRINTING_ARCHITECTURE
+
     DECLARE_EVENT_TABLE()
     DECLARE_DYNAMIC_CLASS(wxDocManager)
-    DECLARE_NO_COPY_CLASS(wxDocManager)
+    wxDECLARE_NO_COPY_CLASS(wxDocManager);
 };
 
 #if WXWIN_COMPATIBILITY_2_6
@@ -480,61 +534,282 @@ inline size_t wxDocManager::GetNoHistoryFiles() const
 #endif // WXWIN_COMPATIBILITY_2_6
 
 // ----------------------------------------------------------------------------
-// A default child frame
+// Base class for child frames -- this is what wxView renders itself into
+//
+// Notice that this is a mix-in class so it doesn't derive from wxWindow, only
+// wxDocChildFrameAny does
 // ----------------------------------------------------------------------------
 
-class WXDLLEXPORT wxDocChildFrame : public wxFrame
+class WXDLLIMPEXP_CORE wxDocChildFrameAnyBase
 {
 public:
-    wxDocChildFrame(wxDocument *doc,
-                    wxView *view,
-                    wxFrame *frame,
-                    wxWindowID id,
-                    const wxString& title,
-                    const wxPoint& pos = wxDefaultPosition,
-                    const wxSize& size = wxDefaultSize,
-                    long type = wxDEFAULT_FRAME_STYLE,
-                    const wxString& name = wxT("frame"));
-    virtual ~wxDocChildFrame(){}
+    // default ctor, use Create() after it
+    wxDocChildFrameAnyBase()
+    {
+        m_childDocument = NULL;
+        m_childView = NULL;
+        m_win = NULL;
+    }
 
-    // Extend event processing to search the view's event table
-    virtual bool ProcessEvent(wxEvent& event);
+    // full ctor equivalent to using the default one and Create(0
+    wxDocChildFrameAnyBase(wxDocument *doc, wxView *view, wxWindow *win)
+    {
+        Create(doc, view, win);
+    }
 
-    void OnActivate(wxActivateEvent& event);
-    void OnCloseWindow(wxCloseEvent& event);
+    // method which must be called for an object created using the default ctor
+    //
+    // note that it returns bool just for consistency with Create() methods in
+    // other classes, we never return false from here
+    bool Create(wxDocument *doc, wxView *view, wxWindow *win)
+    {
+        m_childDocument = doc;
+        m_childView = view;
+        m_win = win;
+
+        if ( view )
+            view->SetDocChildFrame(this);
+
+        return true;
+    }
+
+    // dtor doesn't need to be virtual, an object should never be destroyed via
+    // a pointer to this class
+    ~wxDocChildFrameAnyBase()
+    {
+        // prevent the view from deleting us if we're being deleted directly
+        // (and not via Close() + Destroy())
+        if ( m_childView )
+            m_childView->SetDocChildFrame(NULL);
+    }
 
     wxDocument *GetDocument() const { return m_childDocument; }
     wxView *GetView() const { return m_childView; }
     void SetDocument(wxDocument *doc) { m_childDocument = doc; }
     void SetView(wxView *view) { m_childView = view; }
-    bool Destroy() { m_childView = (wxView *)NULL; return wxFrame::Destroy(); }
+
+    wxWindow *GetWindow() const { return m_win; }
 
 protected:
+    // we're not a wxEvtHandler but we provide this wxEvtHandler-like function
+    // which is called from TryBefore() of the derived classes to give our view
+    // a chance to process the message before the frame event handlers are used
+    bool TryProcessEvent(wxEvent& event)
+    {
+        return m_childView && m_childView->ProcessEventLocally(event);
+    }
+
+    // called from EVT_CLOSE handler in the frame: check if we can close and do
+    // cleanup if so; veto the event otherwise
+    bool CloseView(wxCloseEvent& event);
+
+
     wxDocument*       m_childDocument;
     wxView*           m_childView;
 
-private:
-    DECLARE_CLASS(wxDocChildFrame)
-    DECLARE_EVENT_TABLE()
-    DECLARE_NO_COPY_CLASS(wxDocChildFrame)
+    // the associated window: having it here is not terribly elegant but it
+    // allows us to avoid having any virtual functions in this class
+    wxWindow* m_win;
+
+
+    wxDECLARE_NO_COPY_CLASS(wxDocChildFrameAnyBase);
 };
 
 // ----------------------------------------------------------------------------
-// A default parent frame
+// Template implementing child frame concept using the given wxFrame-like class
+//
+// This is used to define wxDocChildFrame and wxDocMDIChildFrame: ChildFrame is
+// a wxFrame or wxMDIChildFrame (although in theory it could be any wxWindow-
+// derived class as long as it provided a ctor with the same signature as
+// wxFrame and OnActivate() method) and ParentFrame is either wxFrame or
+// wxMDIParentFrame.
 // ----------------------------------------------------------------------------
 
-class WXDLLEXPORT wxDocParentFrame : public wxFrame
+template <class ChildFrame, class ParentFrame>
+class WXDLLIMPEXP_CORE wxDocChildFrameAny : public ChildFrame,
+                                            public wxDocChildFrameAnyBase
 {
 public:
-    wxDocParentFrame();
-    wxDocParentFrame(wxDocManager *manager,
-                     wxFrame *frame,
-                     wxWindowID id,
-                     const wxString& title,
-                     const wxPoint& pos = wxDefaultPosition,
-                     const wxSize& size = wxDefaultSize,
-                     long style = wxDEFAULT_FRAME_STYLE,
-                     const wxString& name = wxFrameNameStr);
+    typedef ChildFrame BaseClass;
+
+    // default ctor, use Create after it
+    wxDocChildFrameAny() { }
+
+    // ctor for a frame showing the given view of the specified document
+    wxDocChildFrameAny(wxDocument *doc,
+                       wxView *view,
+                       ParentFrame *parent,
+                       wxWindowID id,
+                       const wxString& title,
+                       const wxPoint& pos = wxDefaultPosition,
+                       const wxSize& size = wxDefaultSize,
+                       long style = wxDEFAULT_FRAME_STYLE,
+                       const wxString& name = wxFrameNameStr)
+    {
+        Create(doc, view, parent, id, title, pos, size, style, name);
+    }
+
+    bool Create(wxDocument *doc,
+                wxView *view,
+                ParentFrame *parent,
+                wxWindowID id,
+                const wxString& title,
+                const wxPoint& pos = wxDefaultPosition,
+                const wxSize& size = wxDefaultSize,
+                long style = wxDEFAULT_FRAME_STYLE,
+                const wxString& name = wxFrameNameStr)
+    {
+        if ( !wxDocChildFrameAnyBase::Create(doc, view, this) )
+            return false;
+
+        if ( !BaseClass::Create(parent, id, title, pos, size, style, name) )
+            return false;
+
+        this->Connect(wxEVT_ACTIVATE,
+                      wxActivateEventHandler(wxDocChildFrameAny::OnActivate));
+        this->Connect(wxEVT_CLOSE_WINDOW,
+                      wxCloseEventHandler(wxDocChildFrameAny::OnCloseWindow));
+
+        return true;
+    }
+
+    virtual bool Destroy()
+    {
+        // FIXME: why exactly do we do this? to avoid activation events during
+        //        destructions maybe?
+        m_childView = NULL;
+        return BaseClass::Destroy();
+    }
+
+protected:
+    // hook the child view into event handlers chain here
+    virtual bool TryBefore(wxEvent& event)
+    {
+        return TryProcessEvent(event) || BaseClass::TryBefore(event);
+    }
+
+private:
+    void OnActivate(wxActivateEvent& event)
+    {
+        BaseClass::OnActivate(event);
+
+        if ( m_childView )
+            m_childView->Activate(event.GetActive());
+    }
+
+    void OnCloseWindow(wxCloseEvent& event)
+    {
+        if ( CloseView(event) )
+            Destroy();
+        //else: vetoed
+    }
+
+    wxDECLARE_NO_COPY_TEMPLATE_CLASS_2(wxDocChildFrameAny,
+                                        ChildFrame, ParentFrame);
+};
+
+// ----------------------------------------------------------------------------
+// A default child frame: we need to define it as a class just for wxRTTI,
+// otherwise we could simply typedef it
+// ----------------------------------------------------------------------------
+
+#ifdef __VISUALC6__
+    // "non dll-interface class 'wxDocChildFrameAny<>' used as base interface
+    // for dll-interface class 'wxDocChildFrame'" -- this is bogus as the
+    // template will be DLL-exported but only once it is used as base class
+    // here!
+    #pragma warning (push)
+    #pragma warning (disable:4275)
+#endif
+
+typedef wxDocChildFrameAny<wxFrame, wxFrame> wxDocChildFrameBase;
+
+class WXDLLIMPEXP_CORE wxDocChildFrame : public wxDocChildFrameBase
+{
+public:
+    wxDocChildFrame()
+    {
+    }
+
+    wxDocChildFrame(wxDocument *doc,
+                    wxView *view,
+                    wxFrame *parent,
+                    wxWindowID id,
+                    const wxString& title,
+                    const wxPoint& pos = wxDefaultPosition,
+                    const wxSize& size = wxDefaultSize,
+                    long style = wxDEFAULT_FRAME_STYLE,
+                    const wxString& name = wxFrameNameStr)
+        : wxDocChildFrameBase(doc, view,
+                              parent, id, title, pos, size, style, name)
+    {
+    }
+
+    bool Create(wxDocument *doc,
+                wxView *view,
+                wxFrame *parent,
+                wxWindowID id,
+                const wxString& title,
+                const wxPoint& pos = wxDefaultPosition,
+                const wxSize& size = wxDefaultSize,
+                long style = wxDEFAULT_FRAME_STYLE,
+                const wxString& name = wxFrameNameStr)
+    {
+        return wxDocChildFrameBase::Create
+               (
+                    doc, view,
+                    parent, id, title, pos, size, style, name
+               );
+    }
+
+private:
+    DECLARE_CLASS(wxDocChildFrame)
+    wxDECLARE_NO_COPY_CLASS(wxDocChildFrame);
+};
+
+// ----------------------------------------------------------------------------
+// wxDocParentFrame and related classes.
+//
+// As with wxDocChildFrame we define a template base class used by both normal
+// and MDI versions
+// ----------------------------------------------------------------------------
+
+// Base class containing type-independent code of wxDocParentFrameAny
+//
+// Similarly to wxDocChildFrameAnyBase, this class is a mix-in and doesn't
+// derive from wxWindow.
+class WXDLLIMPEXP_CORE wxDocParentFrameAnyBase
+{
+public:
+    wxDocParentFrameAnyBase() { m_docManager = NULL; }
+
+    wxDocManager *GetDocumentManager() const { return m_docManager; }
+
+protected:
+    wxDocManager *m_docManager;
+
+    wxDECLARE_NO_COPY_CLASS(wxDocParentFrameAnyBase);
+};
+
+// This is similar to wxDocChildFrameAny and is used to provide common
+// implementation for both wxDocParentFrame and wxDocMDIParentFrame
+template <class BaseFrame>
+class WXDLLIMPEXP_CORE wxDocParentFrameAny : public BaseFrame,
+                                             public wxDocParentFrameAnyBase
+{
+public:
+    wxDocParentFrameAny() { }
+    wxDocParentFrameAny(wxDocManager *manager,
+                        wxFrame *frame,
+                        wxWindowID id,
+                        const wxString& title,
+                        const wxPoint& pos = wxDefaultPosition,
+                        const wxSize& size = wxDefaultSize,
+                        long style = wxDEFAULT_FRAME_STYLE,
+                        const wxString& name = wxFrameNameStr)
+    {
+        Create(manager, frame, id, title, pos, size, style, name);
+    }
 
     bool Create(wxDocManager *manager,
                 wxFrame *frame,
@@ -543,40 +818,115 @@ public:
                 const wxPoint& pos = wxDefaultPosition,
                 const wxSize& size = wxDefaultSize,
                 long style = wxDEFAULT_FRAME_STYLE,
-                const wxString& name = wxFrameNameStr);
+                const wxString& name = wxFrameNameStr)
+    {
+        m_docManager = manager;
 
-    // Extend event processing to search the document manager's event table
-    virtual bool ProcessEvent(wxEvent& event);
+        if ( !BaseFrame::Create(frame, id, title, pos, size, style, name) )
+            return false;
 
-    wxDocManager *GetDocumentManager() const { return m_docManager; }
+        this->Connect(wxID_EXIT, wxEVT_COMMAND_MENU_SELECTED,
+                      wxCommandEventHandler(wxDocParentFrameAny::OnExit));
+        this->Connect(wxEVT_CLOSE_WINDOW,
+                      wxCloseEventHandler(wxDocParentFrameAny::OnCloseWindow));
 
-    void OnExit(wxCommandEvent& event);
-    void OnMRUFile(wxCommandEvent& event);
-    void OnCloseWindow(wxCloseEvent& event);
+        return true;
+    }
 
 protected:
-    wxDocManager *m_docManager;
+    // hook the document manager into event handling chain here
+    virtual bool TryBefore(wxEvent& event)
+    {
+        if ( m_docManager && m_docManager->ProcessEventLocally(event) )
+            return true;
+
+        return BaseFrame::TryBefore(event);
+    }
 
 private:
-    typedef wxFrame base_type;
-    DECLARE_CLASS(wxDocParentFrame)
-    DECLARE_EVENT_TABLE()
-    DECLARE_NO_COPY_CLASS(wxDocParentFrame)
+    void OnExit(wxCommandEvent& WXUNUSED(event))
+    {
+        this->Close();
+    }
+
+    void OnCloseWindow(wxCloseEvent& event)
+    {
+        if ( m_docManager && !m_docManager->Clear(!event.CanVeto()) )
+        {
+            // The user decided not to close finally, abort.
+            event.Veto();
+        }
+        else
+        {
+            // Just skip the event, base class handler will destroy the window.
+            event.Skip();
+        }
+    }
+
+
+    wxDECLARE_NO_COPY_CLASS(wxDocParentFrameAny);
 };
 
+typedef wxDocParentFrameAny<wxFrame> wxDocParentFrameBase;
+
+class WXDLLIMPEXP_CORE wxDocParentFrame : public wxDocParentFrameBase
+{
+public:
+    wxDocParentFrame() : wxDocParentFrameBase() { }
+
+    wxDocParentFrame(wxDocManager *manager,
+                     wxFrame *parent,
+                     wxWindowID id,
+                     const wxString& title,
+                     const wxPoint& pos = wxDefaultPosition,
+                     const wxSize& size = wxDefaultSize,
+                     long style = wxDEFAULT_FRAME_STYLE,
+                     const wxString& name = wxFrameNameStr)
+        : wxDocParentFrameBase(manager,
+                               parent, id, title, pos, size, style, name)
+    {
+    }
+
+    bool Create(wxDocManager *manager,
+                wxFrame *parent,
+                wxWindowID id,
+                const wxString& title,
+                const wxPoint& pos = wxDefaultPosition,
+                const wxSize& size = wxDefaultSize,
+                long style = wxDEFAULT_FRAME_STYLE,
+                const wxString& name = wxFrameNameStr)
+    {
+        return wxDocParentFrameBase::Create(manager,
+                                            parent, id, title,
+                                            pos, size, style, name);
+    }
+
+private:
+    DECLARE_CLASS(wxDocParentFrame)
+    wxDECLARE_NO_COPY_CLASS(wxDocParentFrame);
+};
+
+#ifdef __VISUALC6__
+    // reenable warning 4275
+    #pragma warning (pop)
+#endif
+
 // ----------------------------------------------------------------------------
 // Provide simple default printing facilities
 // ----------------------------------------------------------------------------
 
 #if wxUSE_PRINTING_ARCHITECTURE
-class WXDLLEXPORT wxDocPrintout : public wxPrintout
+class WXDLLIMPEXP_CORE wxDocPrintout : public wxPrintout
 {
 public:
-    wxDocPrintout(wxView *view = (wxView *) NULL, const wxString& title = wxT("Printout"));
-    bool OnPrintPage(int page);
-    bool HasPage(int page);
-    bool OnBeginDocument(int startPage, int endPage);
-    void GetPageInfo(int *minPage, int *maxPage, int *selPageFrom, int *selPageTo);
+    wxDocPrintout(wxView *view = NULL, const wxString& title = _("Printout"));
+
+    // implement wxPrintout methods
+    virtual bool OnPrintPage(int page);
+    virtual bool HasPage(int page);
+    virtual bool OnBeginDocument(int startPage, int endPage);
+    virtual void GetPageInfo(int *minPage, int *maxPage,
+                             int *selPageFrom, int *selPageTo);
 
     virtual wxView *GetView() { return m_printoutView; }
 
@@ -585,91 +935,36 @@ protected:
 
 private:
     DECLARE_DYNAMIC_CLASS(wxDocPrintout)
-    DECLARE_NO_COPY_CLASS(wxDocPrintout)
+    wxDECLARE_NO_COPY_CLASS(wxDocPrintout);
 };
 #endif // wxUSE_PRINTING_ARCHITECTURE
 
-// ----------------------------------------------------------------------------
-// File history management
-// ----------------------------------------------------------------------------
-
-class WXDLLEXPORT wxFileHistory : public wxObject
-{
-public:
-    wxFileHistory(size_t maxFiles = 9, wxWindowID idBase = wxID_FILE1);
-    virtual ~wxFileHistory();
-
-    // Operations
-    virtual void AddFileToHistory(const wxString& file);
-    virtual void RemoveFileFromHistory(size_t i);
-    virtual int GetMaxFiles() const { return (int)m_fileMaxFiles; }
-    virtual void UseMenu(wxMenu *menu);
-
-    // Remove menu from the list (MDI child may be closing)
-    virtual void RemoveMenu(wxMenu *menu);
-
-#if wxUSE_CONFIG
-    virtual void Load(wxConfigBase& config);
-    virtual void Save(wxConfigBase& config);
-#endif // wxUSE_CONFIG
-
-    virtual void AddFilesToMenu();
-    virtual void AddFilesToMenu(wxMenu* menu); // Single menu
-
-    // Accessors
-    virtual wxString GetHistoryFile(size_t i) const;
-    virtual size_t GetCount() const { return m_fileHistoryN; }
-
-    const wxList& GetMenus() const { return m_fileMenus; }
-
-#if wxABI_VERSION >= 20802
-    // Set/get base id
-    void SetBaseId(wxWindowID baseId) { m_idBase = baseId; }
-    wxWindowID GetBaseId() const { return m_idBase; }
-#endif // wxABI 2.8.2+
-
-#if WXWIN_COMPATIBILITY_2_6
-    // deprecated, use GetCount() instead
-    wxDEPRECATED( size_t GetNoHistoryFiles() const );
-#endif // WXWIN_COMPATIBILITY_2_6
-
-protected:
-    // Last n files
-    wxChar**          m_fileHistory;
-    // Number of files saved
-    size_t            m_fileHistoryN;
-    // Menus to maintain (may need several for an MDI app)
-    wxList            m_fileMenus;
-    // Max files to maintain
-    size_t            m_fileMaxFiles;
-
-private:
-    // The ID of the first history menu item (Doesn't have to be wxID_FILE1)
-    wxWindowID m_idBase;
-
-    DECLARE_DYNAMIC_CLASS(wxFileHistory)
-    DECLARE_NO_COPY_CLASS(wxFileHistory)
-};
-
-#if WXWIN_COMPATIBILITY_2_6
-inline size_t wxFileHistory::GetNoHistoryFiles() const
-{
-    return m_fileHistoryN;
-}
-#endif // WXWIN_COMPATIBILITY_2_6
-
+// For compatibility with existing file formats:
+// converts from/to a stream to/from a temporary file.
 #if wxUSE_STD_IOSTREAM
-// For compatibility with existing file formats:
-// converts from/to a stream to/from a temporary file.
-bool WXDLLEXPORT wxTransferFileToStream(const wxString& filename, wxSTD ostream& stream);
-bool WXDLLEXPORT wxTransferStreamToFile(wxSTD istream& stream, const wxString& filename);
+bool WXDLLIMPEXP_CORE
+wxTransferFileToStream(const wxString& filename, wxSTD ostream& stream);
+bool WXDLLIMPEXP_CORE
+wxTransferStreamToFile(wxSTD istream& stream, const wxString& filename);
 #else
-// For compatibility with existing file formats:
-// converts from/to a stream to/from a temporary file.
-bool WXDLLEXPORT wxTransferFileToStream(const wxString& filename, wxOutputStream& stream);
-bool WXDLLEXPORT wxTransferStreamToFile(wxInputStream& stream, const wxString& filename);
+bool WXDLLIMPEXP_CORE
+wxTransferFileToStream(const wxString& filename, wxOutputStream& stream);
+bool WXDLLIMPEXP_CORE
+wxTransferStreamToFile(wxInputStream& stream, const wxString& filename);
 #endif // wxUSE_STD_IOSTREAM
 
+
+// these flags are not used anywhere by wxWidgets and kept only for an unlikely
+// case of existing user code using them for its own purposes
+#ifdef WXWIN_COMPATIBILITY_2_8
+enum
+{
+    wxDOC_SDI = 1,
+    wxDOC_MDI,
+    wxDEFAULT_DOCMAN_FLAGS = wxDOC_SDI
+};
+#endif // WXWIN_COMPATIBILITY_2_8
+
 #endif // wxUSE_DOC_VIEW_ARCHITECTURE
 
 #endif // _WX_DOCH__
