@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: gdicmn.h 44095 2007-01-05 13:36:49Z VZ $
+// RCS-ID:      $Id: gdicmn.h 64883 2010-07-11 10:44:03Z VZ $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -27,16 +27,18 @@
 // forward declarations
 // ---------------------------------------------------------------------------
 
-class WXDLLIMPEXP_CORE wxBitmap;
-class WXDLLIMPEXP_CORE wxBrush;
-class WXDLLIMPEXP_CORE wxColour;
-class WXDLLIMPEXP_CORE wxCursor;
-class WXDLLIMPEXP_CORE wxFont;
-class WXDLLIMPEXP_CORE wxIcon;
-class WXDLLIMPEXP_CORE wxPalette;
-class WXDLLIMPEXP_CORE wxPen;
-class WXDLLIMPEXP_CORE wxRegion;
-class WXDLLIMPEXP_BASE wxString;
+class WXDLLIMPEXP_FWD_CORE wxBitmap;
+class WXDLLIMPEXP_FWD_CORE wxBrush;
+class WXDLLIMPEXP_FWD_CORE wxColour;
+class WXDLLIMPEXP_FWD_CORE wxCursor;
+class WXDLLIMPEXP_FWD_CORE wxFont;
+class WXDLLIMPEXP_FWD_CORE wxIcon;
+class WXDLLIMPEXP_FWD_CORE wxPalette;
+class WXDLLIMPEXP_FWD_CORE wxPen;
+class WXDLLIMPEXP_FWD_CORE wxRegion;
+class WXDLLIMPEXP_FWD_BASE wxString;
+class WXDLLIMPEXP_FWD_CORE wxIconBundle;
+class WXDLLIMPEXP_FWD_CORE wxPoint;
 
 // ---------------------------------------------------------------------------
 // constants
@@ -78,9 +80,18 @@ enum wxBitmapType
     wxBITMAP_TYPE_TGA,
     wxBITMAP_TYPE_MACCURSOR,
     wxBITMAP_TYPE_MACCURSOR_RESOURCE,
+
+    wxBITMAP_TYPE_MAX,
     wxBITMAP_TYPE_ANY = 50
 };
 
+//  Polygon filling mode
+enum wxPolygonFillMode
+{
+    wxODDEVEN_RULE = 1,
+    wxWINDING_RULE
+};
+
 // Standard cursors
 enum wxStockCursor
 {
@@ -124,14 +135,23 @@ enum wxStockCursor
     wxCURSOR_BASED_ARROW_UP,
     wxCURSOR_BASED_ARROW_DOWN,
 #endif // X11
-
     wxCURSOR_ARROWWAIT,
+#ifdef __WXMAC__
+    wxCURSOR_OPEN_HAND,
+    wxCURSOR_CLOSED_HAND,
+#endif
 
     wxCURSOR_MAX
 };
 
 #ifndef __WXGTK__
-    #define wxCURSOR_DEFAULT wxCURSOR_ARROW
+    #define wxCURSOR_DEFAULT        wxCURSOR_ARROW
+#endif
+
+#ifndef __WXMAC__
+    // TODO CS supply openhand and closedhand cursors
+    #define wxCURSOR_OPEN_HAND      wxCURSOR_HAND
+    #define wxCURSOR_CLOSED_HAND    wxCURSOR_HAND
 #endif
 
 // ---------------------------------------------------------------------------
@@ -140,12 +160,12 @@ enum wxStockCursor
 
 /* Useful macro for creating icons portably, for example:
 
-    wxIcon *icon = new wxICON(mondrian);
+    wxIcon *icon = new wxICON(sample);
 
   expands into:
 
-    wxIcon *icon = new wxIcon("mondrian");      // On wxMSW
-    wxIcon *icon = new wxIcon(mondrian_xpm);    // On wxGTK
+    wxIcon *icon = new wxIcon("sample");      // On wxMSW
+    wxIcon *icon = new wxIcon(sample_xpm);    // On wxGTK
  */
 
 #ifdef __WXMSW__
@@ -156,16 +176,16 @@ enum wxStockCursor
     #define wxICON(X) wxIcon(wxT(#X))
 #elif defined(__WXMGL__)
     // Initialize from an included XPM
-    #define wxICON(X) wxIcon( (const char**) X##_xpm )
+    #define wxICON(X) wxIcon( X##_xpm )
 #elif defined(__WXDFB__)
     // Initialize from an included XPM
-    #define wxICON(X) wxIcon( (const char**) X##_xpm )
+    #define wxICON(X) wxIcon( X##_xpm )
 #elif defined(__WXGTK__)
     // Initialize from an included XPM
-    #define wxICON(X) wxIcon( (const char**) X##_xpm )
+    #define wxICON(X) wxIcon( X##_xpm )
 #elif defined(__WXMAC__)
     // Initialize from an included XPM
-    #define wxICON(X) wxIcon( (const char**) X##_xpm )
+    #define wxICON(X) wxIcon( X##_xpm )
 #elif defined(__WXMOTIF__)
     // Initialize from an included XPM
     #define wxICON(X) wxIcon( X##_xpm )
@@ -191,7 +211,7 @@ enum wxStockCursor
       defined(__WXDFB__)   || \
       defined(__WXCOCOA__)
     // Initialize from an included XPM
-    #define wxBITMAP(name) wxBitmap( (const char**) name##_xpm )
+    #define wxBITMAP(name) wxBitmap(name##_xpm)
 #else // other platforms
     #define wxBITMAP(name) wxBitmap(name##_xpm, wxBITMAP_TYPE_XPM)
 #endif // platform
@@ -204,7 +224,7 @@ enum wxStockCursor
 // wxSize
 // ---------------------------------------------------------------------------
 
-class WXDLLEXPORT wxSize
+class WXDLLIMPEXP_CORE wxSize
 {
 public:
     // members are public for compatibility, don't use them directly.
@@ -216,18 +236,18 @@ public:
 
     // no copy ctor or assignment operator - the defaults are ok
 
-    bool operator==(const wxSize& sz) const { return x == sz.x && y == sz.y; }
-    bool operator!=(const wxSize& sz) const { return x != sz.x || y != sz.y; }
-
-    wxSize operator+(const wxSize& sz) const { return wxSize(x + sz.x, y + sz.y); }
-    wxSize operator-(const wxSize& sz) const { return wxSize(x - sz.x, y - sz.y); }
-    wxSize operator/(int i) const { return wxSize(x / i, y / i); }
-    wxSize operator*(int i) const { return wxSize(x * i, y * i); }
-
     wxSize& operator+=(const wxSize& sz) { x += sz.x; y += sz.y; return *this; }
     wxSize& operator-=(const wxSize& sz) { x -= sz.x; y -= sz.y; return *this; }
-    wxSize& operator/=(const int i) { x /= i; y /= i; return *this; }
-    wxSize& operator*=(const int i) { x *= i; y *= i; return *this; }
+    wxSize& operator/=(int i) { x /= i; y /= i; return *this; }
+    wxSize& operator*=(int i) { x *= i; y *= i; return *this; }
+    wxSize& operator/=(unsigned int i) { x /= i; y /= i; return *this; }
+    wxSize& operator*=(unsigned int i) { x *= i; y *= i; return *this; }
+    wxSize& operator/=(long i) { x /= i; y /= i; return *this; }
+    wxSize& operator*=(long i) { x *= i; y *= i; return *this; }
+    wxSize& operator/=(unsigned long i) { x /= i; y /= i; return *this; }
+    wxSize& operator*=(unsigned long i) { x *= i; y *= i; return *this; }
+    wxSize& operator/=(double i) { x = int(x/i); y = int(y/i); return *this; }
+    wxSize& operator*=(double i) { x = int(x*i); y = int(y*i); return *this; }
 
     void IncTo(const wxSize& sz)
         { if ( sz.x > x ) x = sz.x; if ( sz.y > y ) y = sz.y; }
@@ -235,10 +255,12 @@ public:
         { if ( sz.x < x ) x = sz.x; if ( sz.y < y ) y = sz.y; }
 
     void IncBy(int dx, int dy) { x += dx; y += dy; }
+    void IncBy(const wxPoint& pt);
     void IncBy(const wxSize& sz) { IncBy(sz.x, sz.y); }
     void IncBy(int d) { IncBy(d, d); }
 
     void DecBy(int dx, int dy) { IncBy(-dx, -dy); }
+    void DecBy(const wxPoint& pt);
     void DecBy(const wxSize& sz) { DecBy(sz.x, sz.y); }
     void DecBy(int d) { DecBy(d, d); }
 
@@ -271,11 +293,103 @@ public:
     int GetY() const { return y; }
 };
 
+inline bool operator==(const wxSize& s1, const wxSize& s2)
+{
+    return s1.x == s2.x && s1.y == s2.y;
+}
+
+inline bool operator!=(const wxSize& s1, const wxSize& s2)
+{
+    return s1.x != s2.x || s1.y != s2.y;
+}
+
+inline wxSize operator+(const wxSize& s1, const wxSize& s2)
+{
+    return wxSize(s1.x + s2.x, s1.y + s2.y);
+}
+
+inline wxSize operator-(const wxSize& s1, const wxSize& s2)
+{
+    return wxSize(s1.x - s2.x, s1.y - s2.y);
+}
+
+inline wxSize operator/(const wxSize& s, int i)
+{
+    return wxSize(s.x / i, s.y / i);
+}
+
+inline wxSize operator*(const wxSize& s, int i)
+{
+    return wxSize(s.x * i, s.y * i);
+}
+
+inline wxSize operator*(int i, const wxSize& s)
+{
+    return wxSize(s.x * i, s.y * i);
+}
+
+inline wxSize operator/(const wxSize& s, unsigned int i)
+{
+    return wxSize(s.x / i, s.y / i);
+}
+
+inline wxSize operator*(const wxSize& s, unsigned int i)
+{
+    return wxSize(s.x * i, s.y * i);
+}
+
+inline wxSize operator*(unsigned int i, const wxSize& s)
+{
+    return wxSize(s.x * i, s.y * i);
+}
+
+inline wxSize operator/(const wxSize& s, long i)
+{
+    return wxSize(s.x / i, s.y / i);
+}
+
+inline wxSize operator*(const wxSize& s, long i)
+{
+    return wxSize(s.x * i, s.y * i);
+}
+
+inline wxSize operator*(long i, const wxSize& s)
+{
+    return wxSize(s.x * i, s.y * i);
+}
+
+inline wxSize operator/(const wxSize& s, unsigned long i)
+{
+    return wxSize(s.x / i, s.y / i);
+}
+
+inline wxSize operator*(const wxSize& s, unsigned long i)
+{
+    return wxSize(s.x * i, s.y * i);
+}
+
+inline wxSize operator*(unsigned long i, const wxSize& s)
+{
+    return wxSize(s.x * i, s.y * i);
+}
+
+inline wxSize operator*(const wxSize& s, double i)
+{
+    return wxSize(int(s.x * i), int(s.y * i));
+}
+
+inline wxSize operator*(double i, const wxSize& s)
+{
+    return wxSize(int(s.x * i), int(s.y * i));
+}
+
+
+
 // ---------------------------------------------------------------------------
 // Point classes: with real or integer coordinates
 // ---------------------------------------------------------------------------
 
-class WXDLLEXPORT wxRealPoint
+class WXDLLIMPEXP_CORE wxRealPoint
 {
 public:
     double x;
@@ -283,53 +397,261 @@ public:
 
     wxRealPoint() : x(0.0), y(0.0) { }
     wxRealPoint(double xx, double yy) : x(xx), y(yy) { }
+    wxRealPoint(const wxPoint& pt);
 
-    wxRealPoint operator+(const wxRealPoint& pt) const { return wxRealPoint(x + pt.x, y + pt.y); }
-    wxRealPoint operator-(const wxRealPoint& pt) const { return wxRealPoint(x - pt.x, y - pt.y); }
+    // no copy ctor or assignment operator - the defaults are ok
 
-    bool operator==(const wxRealPoint& pt) const
-    {
-        return wxIsSameDouble(x, pt.x) && wxIsSameDouble(y, pt.y);
-    }
-    bool operator!=(const wxRealPoint& pt) const { return !(*this == pt); }
+    //assignment operators
+    wxRealPoint& operator+=(const wxRealPoint& p) { x += p.x; y += p.y; return *this; }
+    wxRealPoint& operator-=(const wxRealPoint& p) { x -= p.x; y -= p.y; return *this; }
+
+    wxRealPoint& operator+=(const wxSize& s) { x += s.GetWidth(); y += s.GetHeight(); return *this; }
+    wxRealPoint& operator-=(const wxSize& s) { x -= s.GetWidth(); y -= s.GetHeight(); return *this; }
 };
 
 
-class WXDLLEXPORT wxPoint
+inline bool operator==(const wxRealPoint& p1, const wxRealPoint& p2)
+{
+    return wxIsSameDouble(p1.x, p2.x) && wxIsSameDouble(p1.y, p2.y);
+}
+
+inline bool operator!=(const wxRealPoint& p1, const wxRealPoint& p2)
+{
+    return !(p1 == p2);
+}
+
+inline wxRealPoint operator+(const wxRealPoint& p1, const wxRealPoint& p2)
+{
+    return wxRealPoint(p1.x + p2.x, p1.y + p2.y);
+}
+
+
+inline wxRealPoint operator-(const wxRealPoint& p1, const wxRealPoint& p2)
+{
+    return wxRealPoint(p1.x - p2.x, p1.y - p2.y);
+}
+
+
+inline wxRealPoint operator/(const wxRealPoint& s, int i)
+{
+    return wxRealPoint(s.x / i, s.y / i);
+}
+
+inline wxRealPoint operator*(const wxRealPoint& s, int i)
+{
+    return wxRealPoint(s.x * i, s.y * i);
+}
+
+inline wxRealPoint operator*(int i, const wxRealPoint& s)
+{
+    return wxRealPoint(s.x * i, s.y * i);
+}
+
+inline wxRealPoint operator/(const wxRealPoint& s, unsigned int i)
+{
+    return wxRealPoint(s.x / i, s.y / i);
+}
+
+inline wxRealPoint operator*(const wxRealPoint& s, unsigned int i)
+{
+    return wxRealPoint(s.x * i, s.y * i);
+}
+
+inline wxRealPoint operator*(unsigned int i, const wxRealPoint& s)
+{
+    return wxRealPoint(s.x * i, s.y * i);
+}
+
+inline wxRealPoint operator/(const wxRealPoint& s, long i)
+{
+    return wxRealPoint(s.x / i, s.y / i);
+}
+
+inline wxRealPoint operator*(const wxRealPoint& s, long i)
+{
+    return wxRealPoint(s.x * i, s.y * i);
+}
+
+inline wxRealPoint operator*(long i, const wxRealPoint& s)
+{
+    return wxRealPoint(s.x * i, s.y * i);
+}
+
+inline wxRealPoint operator/(const wxRealPoint& s, unsigned long i)
+{
+    return wxRealPoint(s.x / i, s.y / i);
+}
+
+inline wxRealPoint operator*(const wxRealPoint& s, unsigned long i)
+{
+    return wxRealPoint(s.x * i, s.y * i);
+}
+
+inline wxRealPoint operator*(unsigned long i, const wxRealPoint& s)
+{
+    return wxRealPoint(s.x * i, s.y * i);
+}
+
+inline wxRealPoint operator*(const wxRealPoint& s, double i)
+{
+    return wxRealPoint(int(s.x * i), int(s.y * i));
+}
+
+inline wxRealPoint operator*(double i, const wxRealPoint& s)
+{
+    return wxRealPoint(int(s.x * i), int(s.y * i));
+}
+
+
+// ----------------------------------------------------------------------------
+// wxPoint: 2D point with integer coordinates
+// ----------------------------------------------------------------------------
+
+class WXDLLIMPEXP_CORE wxPoint
 {
 public:
     int x, y;
 
     wxPoint() : x(0), y(0) { }
     wxPoint(int xx, int yy) : x(xx), y(yy) { }
+    wxPoint(const wxRealPoint& pt) : x(int(pt.x)), y(int(pt.y)) { }
 
     // no copy ctor or assignment operator - the defaults are ok
 
-    // comparison
-    bool operator==(const wxPoint& p) const { return x == p.x && y == p.y; }
-    bool operator!=(const wxPoint& p) const { return !(*this == p); }
-
-    // arithmetic operations (component wise)
-    wxPoint operator+(const wxPoint& p) const { return wxPoint(x + p.x, y + p.y); }
-    wxPoint operator-(const wxPoint& p) const { return wxPoint(x - p.x, y - p.y); }
-
+    //assignment operators
     wxPoint& operator+=(const wxPoint& p) { x += p.x; y += p.y; return *this; }
     wxPoint& operator-=(const wxPoint& p) { x -= p.x; y -= p.y; return *this; }
 
     wxPoint& operator+=(const wxSize& s) { x += s.GetWidth(); y += s.GetHeight(); return *this; }
     wxPoint& operator-=(const wxSize& s) { x -= s.GetWidth(); y -= s.GetHeight(); return *this; }
-
-    wxPoint operator+(const wxSize& s) const { return wxPoint(x + s.GetWidth(), y + s.GetHeight()); }
-    wxPoint operator-(const wxSize& s) const { return wxPoint(x - s.GetWidth(), y - s.GetHeight()); }
-
-    wxPoint operator-() const { return wxPoint(-x, -y); }
 };
 
+
+// comparison
+inline bool operator==(const wxPoint& p1, const wxPoint& p2)
+{
+    return p1.x == p2.x && p1.y == p2.y;
+}
+
+inline bool operator!=(const wxPoint& p1, const wxPoint& p2)
+{
+    return !(p1 == p2);
+}
+
+
+// arithmetic operations (component wise)
+inline wxPoint operator+(const wxPoint& p1, const wxPoint& p2)
+{
+    return wxPoint(p1.x + p2.x, p1.y + p2.y);
+}
+
+inline wxPoint operator-(const wxPoint& p1, const wxPoint& p2)
+{
+    return wxPoint(p1.x - p2.x, p1.y - p2.y);
+}
+
+inline wxPoint operator+(const wxPoint& p, const wxSize& s)
+{
+    return wxPoint(p.x + s.x, p.y + s.y);
+}
+
+inline wxPoint operator-(const wxPoint& p, const wxSize& s)
+{
+    return wxPoint(p.x - s.x, p.y - s.y);
+}
+
+inline wxPoint operator+(const wxSize& s, const wxPoint& p)
+{
+    return wxPoint(p.x + s.x, p.y + s.y);
+}
+
+inline wxPoint operator-(const wxSize& s, const wxPoint& p)
+{
+    return wxPoint(s.x - p.x, s.y - p.y);
+}
+
+inline wxPoint operator-(const wxPoint& p)
+{
+    return wxPoint(-p.x, -p.y);
+}
+
+inline wxPoint operator/(const wxPoint& s, int i)
+{
+    return wxPoint(s.x / i, s.y / i);
+}
+
+inline wxPoint operator*(const wxPoint& s, int i)
+{
+    return wxPoint(s.x * i, s.y * i);
+}
+
+inline wxPoint operator*(int i, const wxPoint& s)
+{
+    return wxPoint(s.x * i, s.y * i);
+}
+
+inline wxPoint operator/(const wxPoint& s, unsigned int i)
+{
+    return wxPoint(s.x / i, s.y / i);
+}
+
+inline wxPoint operator*(const wxPoint& s, unsigned int i)
+{
+    return wxPoint(s.x * i, s.y * i);
+}
+
+inline wxPoint operator*(unsigned int i, const wxPoint& s)
+{
+    return wxPoint(s.x * i, s.y * i);
+}
+
+inline wxPoint operator/(const wxPoint& s, long i)
+{
+    return wxPoint(s.x / i, s.y / i);
+}
+
+inline wxPoint operator*(const wxPoint& s, long i)
+{
+    return wxPoint(s.x * i, s.y * i);
+}
+
+inline wxPoint operator*(long i, const wxPoint& s)
+{
+    return wxPoint(s.x * i, s.y * i);
+}
+
+inline wxPoint operator/(const wxPoint& s, unsigned long i)
+{
+    return wxPoint(s.x / i, s.y / i);
+}
+
+inline wxPoint operator*(const wxPoint& s, unsigned long i)
+{
+    return wxPoint(s.x * i, s.y * i);
+}
+
+inline wxPoint operator*(unsigned long i, const wxPoint& s)
+{
+    return wxPoint(s.x * i, s.y * i);
+}
+
+inline wxPoint operator*(const wxPoint& s, double i)
+{
+    return wxPoint(int(s.x * i), int(s.y * i));
+}
+
+inline wxPoint operator*(double i, const wxPoint& s)
+{
+    return wxPoint(int(s.x * i), int(s.y * i));
+}
+
+WX_DECLARE_LIST_WITH_DECL(wxPoint, wxPointList, class WXDLLIMPEXP_CORE);
+
 // ---------------------------------------------------------------------------
 // wxRect
 // ---------------------------------------------------------------------------
 
-class WXDLLEXPORT wxRect
+class WXDLLIMPEXP_CORE wxRect
 {
 public:
     wxRect()
@@ -438,14 +760,10 @@ public:
         return r;
     }
 
-    // compare rectangles
-    bool operator==(const wxRect& rect) const;
-    bool operator!=(const wxRect& rect) const { return !(*this == rect); }
-
     // return true if the point is (not strcitly) inside the rect
     bool Contains(int x, int y) const;
     bool Contains(const wxPoint& pt) const { return Contains(pt.x, pt.y); }
-    // return true if the rectangle is (not strcitly) inside the rect
+    // return true if the rectangle 'rect' is (not strictly) inside this rect
     bool Contains(const wxRect& rect) const;
 
 #if WXWIN_COMPATIBILITY_2_6
@@ -458,15 +776,11 @@ public:
     // return true if the rectangles have a non empty intersection
     bool Intersects(const wxRect& rect) const;
 
+    // like Union() but don't ignore empty rectangles
+    wxRect& operator+=(const wxRect& rect);
 
-    // these are like Union() but don't ignore empty rectangles
-    wxRect operator+(const wxRect& rect) const;
-    wxRect& operator+=(const wxRect& rect)
-    {
-        *this = *this + rect;
-        return *this;
-    }
-
+    // intersections of two rectrangles not testing for empty rectangles
+    wxRect& operator*=(const wxRect& rect);
 
     // centre this rectangle in the given (usually, but not necessarily,
     // larger) one
@@ -486,6 +800,28 @@ public:
     int x, y, width, height;
 };
 
+
+// compare rectangles
+inline bool operator==(const wxRect& r1, const wxRect& r2)
+{
+    return (r1.x == r2.x) && (r1.y == r2.y) &&
+           (r1.width == r2.width) && (r1.height == r2.height);
+}
+
+inline bool operator!=(const wxRect& r1, const wxRect& r2)
+{
+    return !(r1 == r2);
+}
+
+// like Union() but don't treat empty rectangles specially
+WXDLLIMPEXP_CORE wxRect operator+(const wxRect& r1, const wxRect& r2);
+
+// intersections of two rectangles
+WXDLLIMPEXP_CORE wxRect operator*(const wxRect& r1, const wxRect& r2);
+
+
+
+
 #if WXWIN_COMPATIBILITY_2_6
 inline bool wxRect::Inside(int cx, int cy) const { return Contains(cx, cy); }
 inline bool wxRect::Inside(const wxPoint& pt) const { return Contains(pt); }
@@ -493,6 +829,11 @@ inline bool wxRect::Inside(const wxRect& rect) const { return Contains(rect); }
 #endif // WXWIN_COMPATIBILITY_2_6
 
 
+// define functions which couldn't be defined above because of declarations
+// order
+inline void wxSize::IncBy(const wxPoint& pt) { IncBy(pt.x, pt.y); }
+inline void wxSize::DecBy(const wxPoint& pt) { DecBy(pt.x, pt.y); }
+
 // ---------------------------------------------------------------------------
 // Management of pens, brushes and fonts
 // ---------------------------------------------------------------------------
@@ -508,42 +849,9 @@ protected:
     wxList list;
 };
 
-class WXDLLIMPEXP_CORE wxPenList: public wxGDIObjListBase
-{
-public:
-    wxPen *FindOrCreatePen(const wxColour& colour, int width, int style);
-#if WXWIN_COMPATIBILITY_2_6
-    wxDEPRECATED( void AddPen(wxPen*) );
-    wxDEPRECATED( void RemovePen(wxPen*) );
-#endif
-};
-
-class WXDLLIMPEXP_CORE wxBrushList: public wxGDIObjListBase
-{
-public:
-    wxBrush *FindOrCreateBrush(const wxColour& colour, int style = wxSOLID);
-#if WXWIN_COMPATIBILITY_2_6
-    wxDEPRECATED( void AddBrush(wxBrush*) );
-    wxDEPRECATED( void RemoveBrush(wxBrush*) );
-#endif
-};
-
-class WXDLLIMPEXP_CORE wxFontList: public wxGDIObjListBase
-{
-public:
-    wxFont *FindOrCreateFont(int pointSize, int family, int style, int weight,
-                             bool underline = false,
-                             const wxString& face = wxEmptyString,
-                             wxFontEncoding encoding = wxFONTENCODING_DEFAULT);
-#if WXWIN_COMPATIBILITY_2_6
-    wxDEPRECATED( void AddFont(wxFont*) );
-    wxDEPRECATED( void RemoveFont(wxFont*) );
-#endif
-};
-
 WX_DECLARE_STRING_HASH_MAP(wxColour*, wxStringToColourHashMap);
 
-class WXDLLEXPORT wxColourDatabase
+class WXDLLIMPEXP_CORE wxColourDatabase
 {
 public:
     wxColourDatabase();
@@ -576,7 +884,7 @@ private:
     wxStringToColourHashMap *m_map;
 };
 
-class WXDLLEXPORT wxResourceCache: public wxList
+class WXDLLIMPEXP_CORE wxResourceCache: public wxList
 {
 public:
     wxResourceCache() { }
@@ -590,10 +898,6 @@ public:
 // global variables
 // ---------------------------------------------------------------------------
 
-// Lists of GDI objects
-extern WXDLLEXPORT_DATA(wxPenList*)   wxThePenList;
-extern WXDLLEXPORT_DATA(wxBrushList*)   wxTheBrushList;
-extern WXDLLEXPORT_DATA(wxFontList*)    wxTheFontList;
 
 /* Stock objects
 
@@ -613,6 +917,7 @@ public:
         BRUSH_BLUE,
         BRUSH_CYAN,
         BRUSH_GREEN,
+        BRUSH_YELLOW,
         BRUSH_GREY,
         BRUSH_LIGHTGREY,
         BRUSH_MEDIUMGREY,
@@ -623,6 +928,7 @@ public:
         COLOUR_BLUE,
         COLOUR_CYAN,
         COLOUR_GREEN,
+        COLOUR_YELLOW,
         COLOUR_LIGHTGREY,
         COLOUR_RED,
         COLOUR_WHITE,
@@ -635,8 +941,10 @@ public:
         FONT_SWISS,
         PEN_BLACK,
         PEN_BLACKDASHED,
+        PEN_BLUE,
         PEN_CYAN,
         PEN_GREEN,
+        PEN_YELLOW,
         PEN_GREY,
         PEN_LIGHTGREY,
         PEN_MEDIUMGREY,
@@ -664,7 +972,7 @@ protected:
 
     static wxObject* ms_stockObject[ITEMCOUNT];
 
-    DECLARE_NO_COPY_CLASS(wxStockGDI)
+    wxDECLARE_NO_COPY_CLASS(wxStockGDI);
 };
 
 #define wxITALIC_FONT  wxStockGDI::instance().GetFont(wxStockGDI::FONT_ITALIC)
@@ -674,8 +982,10 @@ protected:
 
 #define wxBLACK_DASHED_PEN  wxStockGDI::GetPen(wxStockGDI::PEN_BLACKDASHED)
 #define wxBLACK_PEN         wxStockGDI::GetPen(wxStockGDI::PEN_BLACK)
+#define wxBLUE_PEN          wxStockGDI::GetPen(wxStockGDI::PEN_BLUE)
 #define wxCYAN_PEN          wxStockGDI::GetPen(wxStockGDI::PEN_CYAN)
 #define wxGREEN_PEN         wxStockGDI::GetPen(wxStockGDI::PEN_GREEN)
+#define wxYELLOW_PEN        wxStockGDI::GetPen(wxStockGDI::PEN_YELLOW)
 #define wxGREY_PEN          wxStockGDI::GetPen(wxStockGDI::PEN_GREY)
 #define wxLIGHT_GREY_PEN    wxStockGDI::GetPen(wxStockGDI::PEN_LIGHTGREY)
 #define wxMEDIUM_GREY_PEN   wxStockGDI::GetPen(wxStockGDI::PEN_MEDIUMGREY)
@@ -687,6 +997,7 @@ protected:
 #define wxBLUE_BRUSH         wxStockGDI::GetBrush(wxStockGDI::BRUSH_BLUE)
 #define wxCYAN_BRUSH         wxStockGDI::GetBrush(wxStockGDI::BRUSH_CYAN)
 #define wxGREEN_BRUSH        wxStockGDI::GetBrush(wxStockGDI::BRUSH_GREEN)
+#define wxYELLOW_BRUSH       wxStockGDI::GetBrush(wxStockGDI::BRUSH_YELLOW)
 #define wxGREY_BRUSH         wxStockGDI::GetBrush(wxStockGDI::BRUSH_GREY)
 #define wxLIGHT_GREY_BRUSH   wxStockGDI::GetBrush(wxStockGDI::BRUSH_LIGHTGREY)
 #define wxMEDIUM_GREY_BRUSH  wxStockGDI::GetBrush(wxStockGDI::BRUSH_MEDIUMGREY)
@@ -698,6 +1009,7 @@ protected:
 #define wxBLUE        wxStockGDI::GetColour(wxStockGDI::COLOUR_BLUE)
 #define wxCYAN        wxStockGDI::GetColour(wxStockGDI::COLOUR_CYAN)
 #define wxGREEN       wxStockGDI::GetColour(wxStockGDI::COLOUR_GREEN)
+#define wxYELLOW      wxStockGDI::GetColour(wxStockGDI::COLOUR_YELLOW)
 #define wxLIGHT_GREY  wxStockGDI::GetColour(wxStockGDI::COLOUR_LIGHTGREY)
 #define wxRED         wxStockGDI::GetColour(wxStockGDI::COLOUR_RED)
 #define wxWHITE       wxStockGDI::GetColour(wxStockGDI::COLOUR_WHITE)
@@ -707,49 +1019,51 @@ protected:
 #define wxSTANDARD_CURSOR   wxStockGDI::GetCursor(wxStockGDI::CURSOR_STANDARD)
 
 // 'Null' objects
-extern WXDLLEXPORT_DATA(wxBitmap)     wxNullBitmap;
-extern WXDLLEXPORT_DATA(wxIcon)       wxNullIcon;
-extern WXDLLEXPORT_DATA(wxCursor)     wxNullCursor;
-extern WXDLLEXPORT_DATA(wxPen)        wxNullPen;
-extern WXDLLEXPORT_DATA(wxBrush)      wxNullBrush;
-extern WXDLLEXPORT_DATA(wxPalette)     wxNullPalette;
-extern WXDLLEXPORT_DATA(wxFont)       wxNullFont;
-extern WXDLLEXPORT_DATA(wxColour)     wxNullColour;
+extern WXDLLIMPEXP_DATA_CORE(wxBitmap)     wxNullBitmap;
+extern WXDLLIMPEXP_DATA_CORE(wxIcon)       wxNullIcon;
+extern WXDLLIMPEXP_DATA_CORE(wxCursor)     wxNullCursor;
+extern WXDLLIMPEXP_DATA_CORE(wxPen)        wxNullPen;
+extern WXDLLIMPEXP_DATA_CORE(wxBrush)      wxNullBrush;
+extern WXDLLIMPEXP_DATA_CORE(wxPalette)    wxNullPalette;
+extern WXDLLIMPEXP_DATA_CORE(wxFont)       wxNullFont;
+extern WXDLLIMPEXP_DATA_CORE(wxColour)     wxNullColour;
+extern WXDLLIMPEXP_DATA_CORE(wxIconBundle) wxNullIconBundle;
 
-extern WXDLLEXPORT_DATA(wxColourDatabase*)  wxTheColourDatabase;
+extern WXDLLIMPEXP_DATA_CORE(wxColourDatabase*)  wxTheColourDatabase;
 
-extern WXDLLEXPORT_DATA(const wxChar) wxPanelNameStr[];
+extern WXDLLIMPEXP_DATA_CORE(const char) wxPanelNameStr[];
 
-extern WXDLLEXPORT_DATA(const wxSize) wxDefaultSize;
-extern WXDLLEXPORT_DATA(const wxPoint) wxDefaultPosition;
+extern WXDLLIMPEXP_DATA_CORE(const wxSize) wxDefaultSize;
+extern WXDLLIMPEXP_DATA_CORE(const wxPoint) wxDefaultPosition;
 
 // ---------------------------------------------------------------------------
 // global functions
 // ---------------------------------------------------------------------------
 
 // resource management
-extern void WXDLLEXPORT wxInitializeStockLists();
-extern void WXDLLEXPORT wxDeleteStockLists();
+extern void WXDLLIMPEXP_CORE wxInitializeStockLists();
+extern void WXDLLIMPEXP_CORE wxDeleteStockLists();
 
 // is the display colour (or monochrome)?
-extern bool WXDLLEXPORT wxColourDisplay();
+extern bool WXDLLIMPEXP_CORE wxColourDisplay();
 
 // Returns depth of screen
-extern int WXDLLEXPORT wxDisplayDepth();
+extern int WXDLLIMPEXP_CORE wxDisplayDepth();
 #define wxGetDisplayDepth wxDisplayDepth
 
 // get the display size
-extern void WXDLLEXPORT wxDisplaySize(int *width, int *height);
-extern wxSize WXDLLEXPORT wxGetDisplaySize();
-extern void WXDLLEXPORT wxDisplaySizeMM(int *width, int *height);
-extern wxSize WXDLLEXPORT wxGetDisplaySizeMM();
+extern void WXDLLIMPEXP_CORE wxDisplaySize(int *width, int *height);
+extern wxSize WXDLLIMPEXP_CORE wxGetDisplaySize();
+extern void WXDLLIMPEXP_CORE wxDisplaySizeMM(int *width, int *height);
+extern wxSize WXDLLIMPEXP_CORE wxGetDisplaySizeMM();
+extern wxSize WXDLLIMPEXP_CORE wxGetDisplayPPI();
 
 // Get position and size of the display workarea
-extern void WXDLLEXPORT wxClientDisplayRect(int *x, int *y, int *width, int *height);
-extern wxRect WXDLLEXPORT wxGetClientDisplayRect();
+extern void WXDLLIMPEXP_CORE wxClientDisplayRect(int *x, int *y, int *width, int *height);
+extern wxRect WXDLLIMPEXP_CORE wxGetClientDisplayRect();
 
 // set global cursor
-extern void WXDLLEXPORT wxSetCursor(const wxCursor& cursor);
+extern void WXDLLIMPEXP_CORE wxSetCursor(const wxCursor& cursor);
 
 #endif
     // _WX_GDICMNH__
