@@ -3,19 +3,21 @@
 // Purpose:     Information window (when app is busy)
 // Author:      Vaclav Slavik
 // Copyright:   (c) 1999 Vaclav Slavik
-// RCS-ID:      $Id: busyinfo.h 48946 2007-09-26 06:40:58Z JS $
+// RCS-ID:      $Id: busyinfo.h 62789 2009-12-05 19:57:58Z PC $
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
-#ifndef __BUSYINFO_H__
-#define __BUSYINFO_H__
+#ifndef _WX_BUSYINFO_H_
+#define _WX_BUSYINFO_H_
 
 #include "wx/defs.h"
 
 #if wxUSE_BUSYINFO
 
-class WXDLLIMPEXP_CORE wxFrame;
-class WXDLLIMPEXP_CORE wxWindow;
+#include "wx/object.h"
+
+class WXDLLIMPEXP_FWD_CORE wxFrame;
+class WXDLLIMPEXP_FWD_CORE wxWindow;
 
 //--------------------------------------------------------------------------------
 // wxBusyInfo
@@ -23,7 +25,7 @@ class WXDLLIMPEXP_CORE wxWindow;
 //                  Can be used in exactly same way as wxBusyCursor
 //--------------------------------------------------------------------------------
 
-class WXDLLEXPORT wxBusyInfo : public wxObject
+class WXDLLIMPEXP_CORE wxBusyInfo : public wxObject
 {
 public:
     wxBusyInfo(const wxString& message, wxWindow *parent = NULL);
@@ -33,10 +35,8 @@ public:
 private:
     wxFrame *m_InfoFrame;
 
-    DECLARE_NO_COPY_CLASS(wxBusyInfo)
+    wxDECLARE_NO_COPY_CLASS(wxBusyInfo);
 };
 
-
 #endif // wxUSE_BUSYINFO
-
-#endif // __BUSYINFO_H__
+#endif // _WX_BUSYINFO_H_
