@@ -2,50 +2,33 @@
 // Name:        dcscreen.h
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: dcscreen.h 35055 2005-08-02 22:58:06Z MW $
+// Id:          $Id: dcscreen.h 50547 2007-12-06 16:22:00Z PC $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
-#ifndef __GTKDCSCREENH__
-#define __GTKDCSCREENH__
+#ifndef _WX_GTKDCSCREEN_H_
+#define _WX_GTKDCSCREEN_H_
 
-#include "wx/dcclient.h"
+#include "wx/dcscreen.h"
+#include "wx/gtk/dcclient.h"
 
 //-----------------------------------------------------------------------------
-// classes
+// wxScreenDCImpl
 //-----------------------------------------------------------------------------
 
-class WXDLLIMPEXP_CORE wxScreenDC;
-
-//-----------------------------------------------------------------------------
-// wxScreenDC
-//-----------------------------------------------------------------------------
-
-class WXDLLIMPEXP_CORE wxScreenDC : public wxPaintDC
+class WXDLLIMPEXP_CORE wxScreenDCImpl : public wxWindowDCImpl
 {
 public:
-    wxScreenDC();
-    virtual ~wxScreenDC();
+    wxScreenDCImpl( wxScreenDC *owner );
+    ~wxScreenDCImpl();
 
-    static bool StartDrawingOnTop( wxWindow *window );
-    static bool StartDrawingOnTop( wxRect *rect = (wxRect *) NULL );
-    static bool EndDrawingOnTop();
-
-    // implementation
-
-    static GdkWindow  *sm_overlayWindow;
-    static int         sm_overlayWindowX;
-    static int         sm_overlayWindowY;
-
-protected:
     virtual void DoGetSize(int *width, int *height) const;
 
-private:
-    DECLARE_DYNAMIC_CLASS(wxScreenDC)
+protected:
+    void Init();
+
+    DECLARE_ABSTRACT_CLASS(wxScreenDCImpl)
 };
 
-#endif
-
-    // __GTKDCSCREENH__
-
+#endif // _WX_GTKDCSCREEN_H_
