@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     16/11/98
-// RCS-ID:      $Id: listimpl.cpp 38893 2006-04-24 17:59:10Z VZ $
+// RCS-ID:      $Id: listimpl.cpp 49277 2007-10-20 22:02:08Z VZ $
 // Copyright:   (c) 1998 Vadim Zeitlin
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -20,8 +20,8 @@
     name::BaseListType name::EmptyList;
 
 #else // !wxUSE_STL
-
-    #define _DEFINE_LIST(T, name)         \
+    #undef WX_DEFINE_LIST_2
+    #define WX_DEFINE_LIST_2(T, name)     \
         void wx##name##Node::DeleteData() \
         {                                 \
             delete (T *)GetData();        \
@@ -30,10 +30,7 @@
     // redefine the macro so that now it will generate the class implementation
     // old value would provoke a compile-time error if this file is not included
     #undef  WX_DEFINE_LIST
-    #define WX_DEFINE_LIST(name) _DEFINE_LIST(_WX_LIST_ITEM_TYPE_##name, name)
-
-    // don't pollute preprocessor's name space
-    //#undef  _DEFINE_LIST
+    #define WX_DEFINE_LIST(name) WX_DEFINE_LIST_2(_WX_LIST_ITEM_TYPE_##name, name)
 
 #endif // wxUSE_STL/!wxUSE_STL
 
