@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by: Agron Selimaj
 // Created:     01/02/97
-// RCS-ID:      $Id: listctrl.h 54419 2008-06-29 01:35:24Z VZ $
+// RCS-ID:      $Id: listctrl.h 64532 2010-06-09 13:55:48Z FM $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -13,8 +13,15 @@
 #define _WX_LISTCTRL_H_
 
 #include "wx/textctrl.h"
+#include "wx/dynarray.h"
+#include "wx/vector.h"
 
 class WXDLLIMPEXP_FWD_CORE wxImageList;
+class wxMSWListItemData;
+
+// define this symbol to indicate the availability of SetColumnsOrder() and
+// related functions
+#define wxHAS_LISTCTRL_COLUMN_ORDER
 
 /*
     The wxListCtrl can show lists of items in four different modes:
@@ -70,7 +77,7 @@ class WXDLLIMPEXP_FWD_CORE wxImageList;
 
  */
 
-class WXDLLEXPORT wxListCtrl: public wxControl
+class WXDLLIMPEXP_CORE wxListCtrl: public wxControl
 {
 public:
     /*
@@ -114,13 +121,25 @@ public:
     bool GetColumn(int col, wxListItem& item) const;
 
     // Sets information about this column
-    bool SetColumn(int col, const wxListItem& item) ;
+    bool SetColumn(int col, const wxListItem& item);
 
     // Gets the column width
     int GetColumnWidth(int col) const;
 
     // Sets the column width
-    bool SetColumnWidth(int col, int width) ;
+    bool SetColumnWidth(int col, int width);
+
+
+    // Gets the column order from its index or index from its order
+    int GetColumnOrder(int col) const;
+    int GetColumnIndexFromOrder(int order) const;
+
+    // Gets the column order for all columns
+    wxArrayInt GetColumnsOrder() const;
+
+    // Sets the column order for all columns
+    bool SetColumnsOrder(const wxArrayInt& orders);
+
 
     // Gets the number of items that can fit vertically in the
     // visible area of the list control (list or report view)
@@ -135,50 +154,48 @@ public:
     wxTextCtrl* GetEditControl() const;
 
     // Gets information about the item
-    bool GetItem(wxListItem& info) const ;
+    bool GetItem(wxListItem& info) const;
 
     // Sets information about the item
-    bool SetItem(wxListItem& info) ;
+    bool SetItem(wxListItem& info);
 
     // Sets a string field at a particular column
     long SetItem(long index, int col, const wxString& label, int imageId = -1);
 
     // Gets the item state
-    int  GetItemState(long item, long stateMask) const ;
+    int  GetItemState(long item, long stateMask) const;
 
     // Sets the item state
-    bool SetItemState(long item, long state, long stateMask) ;
+    bool SetItemState(long item, long state, long stateMask);
 
     // Sets the item image
-    bool SetItemImage(long item, int image, int selImage = -1) ;
+    bool SetItemImage(long item, int image, int selImage = -1);
     bool SetItemColumnImage(long item, long column, int image);
 
     // Gets the item text
-    wxString GetItemText(long item) const ;
+    wxString GetItemText(long item, int col = 0) const;
 
     // Sets the item text
-    void SetItemText(long item, const wxString& str) ;
+    void SetItemText(long item, const wxString& str);
 
     // Gets the item data
-    wxUIntPtr GetItemData(long item) const ;
+    wxUIntPtr GetItemData(long item) const;
 
     // Sets the item data
-#if wxABI_VERSION >= 20804
     bool SetItemPtrData(long item, wxUIntPtr data);
-#endif // wxABI 2.8.4+
-    bool SetItemData(long item, long data);
+    bool SetItemData(long item, long data) { return SetItemPtrData(item, data); }
 
     // Gets the item rectangle
-    bool GetItemRect(long item, wxRect& rect, int code = wxLIST_RECT_BOUNDS) const ;
+    bool GetItemRect(long item, wxRect& rect, int code = wxLIST_RECT_BOUNDS) const;
 
     // Gets the subitem rectangle in report mode
-    bool GetSubItemRect(long item, long subItem, wxRect& rect, int code = wxLIST_RECT_BOUNDS) const ;
+    bool GetSubItemRect(long item, long subItem, wxRect& rect, int code = wxLIST_RECT_BOUNDS) const;
 
     // Gets the item position
-    bool GetItemPosition(long item, wxPoint& pos) const ;
+    bool GetItemPosition(long item, wxPoint& pos) const;
 
     // Sets the item position
-    bool SetItemPosition(long item, const wxPoint& pos) ;
+    bool SetItemPosition(long item, const wxPoint& pos);
 
     // Gets the number of items in the list control
     int GetItemCount() const;
@@ -212,22 +229,22 @@ public:
 
     // Gets the index of the topmost visible item when in
     // list or report view
-    long GetTopItem() const ;
+    long GetTopItem() const;
 
     // Add or remove a single window style
-    void SetSingleStyle(long style, bool add = true) ;
+    void SetSingleStyle(long style, bool add = true);
 
     // Set the whole window style
-    void SetWindowStyleFlag(long style) ;
+    void SetWindowStyleFlag(long style);
 
     // Searches for an item, starting from 'item'.
     // item can be -1 to find the first item that matches the
     // specified flags.
     // Returns the item or -1 if unsuccessful.
-    long GetNextItem(long item, int geometry = wxLIST_NEXT_ALL, int state = wxLIST_STATE_DONTCARE) const ;
+    long GetNextItem(long item, int geometry = wxLIST_NEXT_ALL, int state = wxLIST_STATE_DONTCARE) const;
 
     // Gets one of the three image lists
-    wxImageList *GetImageList(int which) const ;
+    wxImageList *GetImageList(int which) const;
 
     // Sets the image list
     // N.B. There's a quirk in the Win95 list view implementation.
@@ -236,8 +253,8 @@ public:
     // haven't specified wxLIST_MASK_IMAGE when inserting.
     // So you have to set a NULL small-icon image list to be sure that
     // the wxLC_LIST mode works without icons. Of course, you may want icons...
-    void SetImageList(wxImageList *imageList, int which) ;
-    void AssignImageList(wxImageList *imageList, int which) ;
+    void SetImageList(wxImageList *imageList, int which);
+    void AssignImageList(wxImageList *imageList, int which);
 
     // are we in report mode?
     bool InReportView() const { return HasFlag(wxLC_REPORT); }
@@ -259,7 +276,7 @@ public:
     bool DeleteItem(long item);
 
     // Deletes all items
-    bool DeleteAllItems() ;
+    bool DeleteAllItems();
 
     // Deletes a column
     bool DeleteColumn(int col);
@@ -271,13 +288,13 @@ public:
     void ClearAll();
 
     // Edit the label
-    wxTextCtrl* EditLabel(long item, wxClassInfo* textControlClass = CLASSINFO(wxTextCtrl));
+    wxTextCtrl* EditLabel(long item, wxClassInfo* textControlClass = wxCLASSINFO(wxTextCtrl));
 
     // End label editing, optionally cancelling the edit
     bool EndEditLabel(bool cancel);
 
     // Ensures this item is visible
-    bool EnsureVisible(long item) ;
+    bool EnsureVisible(long item);
 
     // Find an item whose label matches this string, starting from the item after 'start'
     // or the beginning if 'start' is -1.
@@ -338,7 +355,7 @@ public:
     // or zero if the two items are equivalent.
 
     // data is arbitrary data to be passed to the sort function.
-    bool SortItems(wxListCtrlCompare fn, long data);
+    bool SortItems(wxListCtrlCompare fn, wxIntPtr data);
 
     // IMPLEMENTATION
     virtual bool MSWCommand(WXUINT param, WXWORD id);
@@ -385,9 +402,12 @@ protected:
     // free memory taken by all internal data
     void FreeAllInternalData();
 
+    // get the internal data object for this item (may return NULL)
+    wxMSWListItemData *MSWGetItemData(long item) const;
+
     // get the item attribute, either by quering it for virtual control, or by
     // returning the one previously set using setter methods for a normal one
-    wxListItemAttr *DoGetItemAttr(long item) const;
+    wxListItemAttr *DoGetItemColumnAttr(long item, long column) const;
 
 
     wxTextCtrl*       m_textCtrl;        // The control used for editing a label
@@ -402,10 +422,9 @@ protected:
                                     // keep track of inserted/deleted columns
     long              m_count;      // Keep track of item count to save calls to
                                     // ListView_GetItemCount
-    bool              m_ignoreChangeMessages;
 
-    // true if we have any internal data (user data & attributes)
-    bool m_AnyInternalData;
+    // all wxMSWListItemData objects we use
+    wxVector<wxMSWListItemData *> m_internalData;
 
     // true if we have any items with custom attributes
     bool m_hasAnyAttr;
@@ -427,6 +446,12 @@ protected:
     // return the attribute for the item (may return NULL if none)
     virtual wxListItemAttr *OnGetItemAttr(long item) const;
 
+    // return the attribute for the given item and column (may return NULL if none)
+    virtual wxListItemAttr *OnGetItemColumnAttr(long item, long WXUNUSED(column)) const
+    {
+        return OnGetItemAttr(item);
+    }
+
 private:
     // process NM_CUSTOMDRAW notification message
     WXLPARAM OnCustomDraw(WXLPARAM lParam);
@@ -441,9 +466,10 @@ private:
     // destroy m_textCtrl if it's currently valid and reset it to NULL
     void DeleteEditControl();
 
+
     DECLARE_DYNAMIC_CLASS(wxListCtrl)
     DECLARE_EVENT_TABLE()
-    DECLARE_NO_COPY_CLASS(wxListCtrl)
+    wxDECLARE_NO_COPY_CLASS(wxListCtrl);
 };
 
 #endif // _WX_LISTCTRL_H_
