@@ -4,11 +4,14 @@
 // Author:      Wlodzimierz ABX Skiba
 // Modified by:
 // Created:     01.05.2004
-// RCS-ID:      $Id: resources.h 27829 2004-06-16 06:17:27Z ABX $
+// RCS-ID:      $Id: resources.h 64940 2010-07-13 13:29:13Z VZ $
 // Copyright:   (c) Wlodzimierz Skiba
-// License:     wxWindows licence
+// Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
 
+// Windows CE dummy menu for SHCreateMenuBar()
+#define wxIDM_SHMENU             40000
+
 #define IDM_LEFT                 40001
 #define IDM_RIGHT                40002
 #define IDM_ITEM                 40003
@@ -22,4 +25,3 @@
 #define IDR_MENUBAR_LEFT_MENU    40031
 #define IDR_MENUBAR_RIGHT_MENU   40032
 #define IDR_MENUBAR_BOTH_MENUS   40033
-
