@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: prntbase.h 42713 2006-10-30 11:56:12Z ABX $
+// RCS-ID:      $Id: prntbase.h 64451 2010-05-31 14:59:03Z VZ $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -22,22 +22,24 @@
 #include "wx/scrolwin.h"
 #include "wx/dialog.h"
 #include "wx/frame.h"
+#include "wx/dc.h"
 
-class WXDLLEXPORT wxDC;
-class WXDLLEXPORT wxButton;
-class WXDLLEXPORT wxChoice;
-class WXDLLEXPORT wxPrintout;
-class WXDLLEXPORT wxPrinterBase;
-class WXDLLEXPORT wxPrintDialogBase;
-class WXDLLEXPORT wxPrintDialog;
-class WXDLLEXPORT wxPageSetupDialogBase;
-class WXDLLEXPORT wxPageSetupDialog;
-class WXDLLEXPORT wxPrintPreviewBase;
-class WXDLLEXPORT wxPreviewCanvas;
-class WXDLLEXPORT wxPreviewControlBar;
-class WXDLLEXPORT wxPreviewFrame;
-class WXDLLEXPORT wxPrintFactory;
-class WXDLLEXPORT wxPrintNativeDataBase;
+class WXDLLIMPEXP_FWD_CORE wxDC;
+class WXDLLIMPEXP_FWD_CORE wxButton;
+class WXDLLIMPEXP_FWD_CORE wxChoice;
+class WXDLLIMPEXP_FWD_CORE wxPrintout;
+class WXDLLIMPEXP_FWD_CORE wxPrinterBase;
+class WXDLLIMPEXP_FWD_CORE wxPrintDialogBase;
+class WXDLLIMPEXP_FWD_CORE wxPrintDialog;
+class WXDLLIMPEXP_FWD_CORE wxPageSetupDialogBase;
+class WXDLLIMPEXP_FWD_CORE wxPageSetupDialog;
+class WXDLLIMPEXP_FWD_CORE wxPrintPreviewBase;
+class WXDLLIMPEXP_FWD_CORE wxPreviewCanvas;
+class WXDLLIMPEXP_FWD_CORE wxPreviewControlBar;
+class WXDLLIMPEXP_FWD_CORE wxPreviewFrame;
+class WXDLLIMPEXP_FWD_CORE wxPrintFactory;
+class WXDLLIMPEXP_FWD_CORE wxPrintNativeDataBase;
+class WXDLLIMPEXP_FWD_CORE wxPrintPreview;
 
 //----------------------------------------------------------------------------
 // error consts
@@ -54,7 +56,7 @@ enum wxPrinterError
 // wxPrintFactory
 //----------------------------------------------------------------------------
 
-class WXDLLEXPORT wxPrintFactory
+class WXDLLIMPEXP_CORE wxPrintFactory
 {
 public:
     wxPrintFactory() {}
@@ -77,7 +79,7 @@ public:
     virtual wxPageSetupDialogBase *CreatePageSetupDialog( wxWindow *parent,
                                                           wxPageSetupDialogData * data = NULL ) = 0;
 
-    virtual wxDC* CreatePrinterDC( const wxPrintData& data ) = 0;
+    virtual wxDCImpl* CreatePrinterDCImpl( wxPrinterDC *owner, const wxPrintData& data ) = 0;
 
     // What to do and what to show in the wxPrintDialog
     // a) Use the generic print setup dialog or a native one?
@@ -101,7 +103,7 @@ private:
     static wxPrintFactory *m_factory;
 };
 
-class WXDLLEXPORT wxNativePrintFactory: public wxPrintFactory
+class WXDLLIMPEXP_CORE wxNativePrintFactory: public wxPrintFactory
 {
 public:
     virtual wxPrinterBase *CreatePrinter( wxPrintDialogData *data );
@@ -121,7 +123,7 @@ public:
     virtual wxPageSetupDialogBase *CreatePageSetupDialog( wxWindow *parent,
                                                           wxPageSetupDialogData * data = NULL );
 
-    virtual wxDC* CreatePrinterDC( const wxPrintData& data );
+    virtual wxDCImpl* CreatePrinterDCImpl( wxPrinterDC *owner, const wxPrintData& data );
 
     virtual bool HasPrintSetupDialog();
     virtual wxDialog *CreatePrintSetupDialog( wxWindow *parent, wxPrintData *data );
@@ -138,7 +140,7 @@ public:
 // wxPrintNativeDataBase
 //----------------------------------------------------------------------------
 
-class WXDLLEXPORT wxPrintNativeDataBase: public wxObject
+class WXDLLIMPEXP_CORE wxPrintNativeDataBase: public wxObject
 {
 public:
     wxPrintNativeDataBase();
@@ -154,7 +156,7 @@ public:
 
 private:
     DECLARE_CLASS(wxPrintNativeDataBase)
-    DECLARE_NO_COPY_CLASS(wxPrintNativeDataBase)
+    wxDECLARE_NO_COPY_CLASS(wxPrintNativeDataBase);
 };
 
 //----------------------------------------------------------------------------
@@ -165,10 +167,10 @@ private:
  * Represents the printer: manages printing a wxPrintout object
  */
 
-class WXDLLEXPORT wxPrinterBase: public wxObject
+class WXDLLIMPEXP_CORE wxPrinterBase: public wxObject
 {
 public:
-    wxPrinterBase(wxPrintDialogData *data = (wxPrintDialogData *) NULL);
+    wxPrinterBase(wxPrintDialogData *data = NULL);
     virtual ~wxPrinterBase();
 
     virtual wxWindow *CreateAbortWindow(wxWindow *parent, wxPrintout *printout);
@@ -198,17 +200,17 @@ public:
 
 private:
     DECLARE_CLASS(wxPrinterBase)
-    DECLARE_NO_COPY_CLASS(wxPrinterBase)
+    wxDECLARE_NO_COPY_CLASS(wxPrinterBase);
 };
 
 //----------------------------------------------------------------------------
 // wxPrinter
 //----------------------------------------------------------------------------
 
-class WXDLLEXPORT wxPrinter: public wxPrinterBase
+class WXDLLIMPEXP_CORE wxPrinter: public wxPrinterBase
 {
 public:
-    wxPrinter(wxPrintDialogData *data = (wxPrintDialogData *) NULL);
+    wxPrinter(wxPrintDialogData *data = NULL);
     virtual ~wxPrinter();
 
     virtual wxWindow *CreateAbortWindow(wxWindow *parent, wxPrintout *printout);
@@ -225,7 +227,7 @@ protected:
 
 private:
     DECLARE_CLASS(wxPrinter)
-    DECLARE_NO_COPY_CLASS(wxPrinter)
+    wxDECLARE_NO_COPY_CLASS(wxPrinter);
 };
 
 //----------------------------------------------------------------------------
@@ -239,10 +241,10 @@ private:
  * object for previewing.
  */
 
-class WXDLLEXPORT wxPrintout: public wxObject
+class WXDLLIMPEXP_CORE wxPrintout: public wxObject
 {
 public:
-    wxPrintout(const wxString& title = wxT("Printout"));
+    wxPrintout(const wxString& title = _("Printout"));
     virtual ~wxPrintout();
 
     virtual bool OnBeginDocument(int startPage, int endPage);
@@ -283,20 +285,26 @@ public:
     void GetPageSizeMM(int *w, int  *h) const { *w = m_pageWidthMM; *h = m_pageHeightMM; }
 
     void SetPPIScreen(int x, int y) { m_PPIScreenX = x; m_PPIScreenY = y; }
+    void SetPPIScreen(const wxSize& ppi) { SetPPIScreen(ppi.x, ppi.y); }
     void GetPPIScreen(int *x, int *y) const { *x = m_PPIScreenX; *y = m_PPIScreenY; }
     void SetPPIPrinter(int x, int y) { m_PPIPrinterX = x; m_PPIPrinterY = y; }
+    void SetPPIPrinter(const wxSize& ppi) { SetPPIPrinter(ppi.x, ppi.y); }
     void GetPPIPrinter(int *x, int *y) const { *x = m_PPIPrinterX; *y = m_PPIPrinterY; }
 
     void SetPaperRectPixels(const wxRect& paperRectPixels) { m_paperRectPixels = paperRectPixels; }
     wxRect GetPaperRectPixels() const { return m_paperRectPixels; }
 
-    virtual bool IsPreview() const { return m_isPreview; }
+    // This must be called by wxPrintPreview to associate itself with the
+    // printout it uses.
+    virtual void SetPreview(wxPrintPreview *preview) { m_preview = preview; }
 
-    virtual void SetIsPreview(bool p) { m_isPreview = p; }
+    wxPrintPreview *GetPreview() const { return m_preview; }
+    virtual bool IsPreview() const { return GetPreview() != NULL; }
 
 private:
     wxString         m_printoutTitle;
     wxDC*            m_printoutDC;
+    wxPrintPreview  *m_preview;
 
     int              m_pageWidthPixels;
     int              m_pageHeightPixels;
@@ -311,11 +319,9 @@ private:
 
     wxRect           m_paperRectPixels;
 
-    bool             m_isPreview;
-
 private:
     DECLARE_ABSTRACT_CLASS(wxPrintout)
-    DECLARE_NO_COPY_CLASS(wxPrintout)
+    wxDECLARE_NO_COPY_CLASS(wxPrintout);
 };
 
 //----------------------------------------------------------------------------
@@ -326,7 +332,7 @@ private:
  * Canvas upon which a preview is drawn.
  */
 
-class WXDLLEXPORT wxPreviewCanvas: public wxScrolledWindow
+class WXDLLIMPEXP_CORE wxPreviewCanvas: public wxScrolledWindow
 {
 public:
     wxPreviewCanvas(wxPrintPreviewBase *preview,
@@ -337,6 +343,8 @@ public:
                     const wxString& name = wxT("canvas"));
     virtual ~wxPreviewCanvas();
 
+    void SetPreview(wxPrintPreviewBase *preview) { m_printPreview = preview; }
+
     void OnPaint(wxPaintEvent& event);
     void OnChar(wxKeyEvent &event);
     // Responds to colour changes
@@ -346,12 +354,13 @@ private:
 #if wxUSE_MOUSEWHEEL
     void OnMouseWheel(wxMouseEvent& event);
 #endif // wxUSE_MOUSEWHEEL
+    void OnIdle(wxIdleEvent& event);
 
     wxPrintPreviewBase* m_printPreview;
 
     DECLARE_CLASS(wxPreviewCanvas)
     DECLARE_EVENT_TABLE()
-    DECLARE_NO_COPY_CLASS(wxPreviewCanvas)
+    wxDECLARE_NO_COPY_CLASS(wxPreviewCanvas);
 };
 
 //----------------------------------------------------------------------------
@@ -362,16 +371,16 @@ private:
  * Default frame for showing preview.
  */
 
-class WXDLLEXPORT wxPreviewFrame: public wxFrame
+class WXDLLIMPEXP_CORE wxPreviewFrame: public wxFrame
 {
 public:
     wxPreviewFrame(wxPrintPreviewBase *preview,
                    wxWindow *parent,
-                   const wxString& title = wxT("Print Preview"),
+                   const wxString& title = _("Print Preview"),
                    const wxPoint& pos = wxDefaultPosition,
                    const wxSize& size = wxDefaultSize,
-                   long style = wxDEFAULT_FRAME_STYLE,
-                   const wxString& name = wxT("frame"));
+                   long style = wxDEFAULT_FRAME_STYLE | wxFRAME_FLOAT_ON_PARENT,
+                   const wxString& name = wxFrameNameStr);
     virtual ~wxPreviewFrame();
 
     void OnCloseWindow(wxCloseEvent& event);
@@ -388,9 +397,11 @@ protected:
     wxWindowDisabler*     m_windowDisabler;
 
 private:
-    DECLARE_CLASS(wxPreviewFrame)
+    void OnChar(wxKeyEvent& event);
+
     DECLARE_EVENT_TABLE()
-    DECLARE_NO_COPY_CLASS(wxPreviewFrame)
+    DECLARE_CLASS(wxPreviewFrame)
+    wxDECLARE_NO_COPY_CLASS(wxPreviewFrame);
 };
 
 //----------------------------------------------------------------------------
@@ -424,7 +435,7 @@ private:
 #define wxID_PREVIEW_LAST       7
 #define wxID_PREVIEW_GOTO       8
 
-class WXDLLEXPORT wxPreviewControlBar: public wxPanel
+class WXDLLIMPEXP_CORE wxPreviewControlBar: public wxPanel
 {
     DECLARE_CLASS(wxPreviewControlBar)
 
@@ -474,7 +485,7 @@ protected:
 
 private:
     DECLARE_EVENT_TABLE()
-    DECLARE_NO_COPY_CLASS(wxPreviewControlBar)
+    wxDECLARE_NO_COPY_CLASS(wxPreviewControlBar);
 };
 
 //----------------------------------------------------------------------------
@@ -485,12 +496,12 @@ private:
  * Programmer creates an object of this class to preview a wxPrintout.
  */
 
-class WXDLLEXPORT wxPrintPreviewBase: public wxObject
+class WXDLLIMPEXP_CORE wxPrintPreviewBase: public wxObject
 {
 public:
     wxPrintPreviewBase(wxPrintout *printout,
-                       wxPrintout *printoutForPrinting = (wxPrintout *) NULL,
-                       wxPrintDialogData *data = (wxPrintDialogData *) NULL);
+                       wxPrintout *printoutForPrinting = NULL,
+                       wxPrintDialogData *data = NULL);
     wxPrintPreviewBase(wxPrintout *printout,
                        wxPrintout *printoutForPrinting,
                        wxPrintData *data);
@@ -516,6 +527,10 @@ public:
     // The preview canvas should call this from OnPaint
     virtual bool PaintPage(wxPreviewCanvas *canvas, wxDC& dc);
 
+    // Updates rendered page by calling RenderPage() if needed, returns true
+    // if there was some change. Preview canvas should call it at idle time
+    virtual bool UpdatePageRendering();
+
     // This draws a blank page onto the preview canvas
     virtual bool DrawBlankPage(wxPreviewCanvas *canvas, wxDC& dc);
 
@@ -551,11 +566,20 @@ public:
     // the currently selected printer.
     virtual void DetermineScaling() = 0;
 
+protected:
+    // helpers for RenderPage():
+    virtual bool RenderPageIntoDC(wxDC& dc, int pageNum);
+    // renders preview into m_previewBitmap
+    virtual bool RenderPageIntoBitmap(wxBitmap& bmp, int pageNum);
+
+    void InvalidatePreviewBitmap();
+
 protected:
     wxPrintDialogData m_printDialogData;
     wxPreviewCanvas*  m_previewCanvas;
     wxFrame*          m_previewFrame;
     wxBitmap*         m_previewBitmap;
+    bool              m_previewFailed;
     wxPrintout*       m_previewPrintout;
     wxPrintout*       m_printPrintout;
     int               m_currentPage;
@@ -575,7 +599,7 @@ protected:
 private:
     void Init(wxPrintout *printout, wxPrintout *printoutForPrinting);
 
-    DECLARE_NO_COPY_CLASS(wxPrintPreviewBase)
+    wxDECLARE_NO_COPY_CLASS(wxPrintPreviewBase);
     DECLARE_CLASS(wxPrintPreviewBase)
 };
 
@@ -583,12 +607,12 @@ private:
 // wxPrintPreview
 //----------------------------------------------------------------------------
 
-class WXDLLEXPORT wxPrintPreview: public wxPrintPreviewBase
+class WXDLLIMPEXP_CORE wxPrintPreview: public wxPrintPreviewBase
 {
 public:
     wxPrintPreview(wxPrintout *printout,
-                   wxPrintout *printoutForPrinting = (wxPrintout *) NULL,
-                   wxPrintDialogData *data = (wxPrintDialogData *) NULL);
+                   wxPrintout *printoutForPrinting = NULL,
+                   wxPrintDialogData *data = NULL);
     wxPrintPreview(wxPrintout *printout,
                    wxPrintout *printoutForPrinting,
                    wxPrintData *data);
@@ -605,6 +629,7 @@ public:
     virtual wxFrame *GetFrame() const;
     virtual wxPreviewCanvas *GetCanvas() const;
     virtual bool PaintPage(wxPreviewCanvas *canvas, wxDC& dc);
+    virtual bool UpdatePageRendering();
     virtual bool DrawBlankPage(wxPreviewCanvas *canvas, wxDC& dc);
     virtual void AdjustScrollbars(wxPreviewCanvas *canvas);
     virtual bool RenderPage(int pageNum);
@@ -628,14 +653,14 @@ private:
 
 private:
     DECLARE_CLASS(wxPrintPreview)
-    DECLARE_NO_COPY_CLASS(wxPrintPreview)
+    wxDECLARE_NO_COPY_CLASS(wxPrintPreview);
 };
 
 //----------------------------------------------------------------------------
 // wxPrintAbortDialog
 //----------------------------------------------------------------------------
 
-class WXDLLEXPORT wxPrintAbortDialog: public wxDialog
+class WXDLLIMPEXP_CORE wxPrintAbortDialog: public wxDialog
 {
 public:
     wxPrintAbortDialog(wxWindow *parent,
@@ -652,7 +677,7 @@ public:
 
 private:
     DECLARE_EVENT_TABLE()
-    DECLARE_NO_COPY_CLASS(wxPrintAbortDialog)
+    wxDECLARE_NO_COPY_CLASS(wxPrintAbortDialog);
 };
 
 #endif // wxUSE_PRINTING_ARCHITECTURE
