@@ -3,7 +3,7 @@
 // Purpose:     XML resource handler for wxGauge
 // Author:      Bob Mitchell
 // Created:     2000/03/21
-// RCS-ID:      $Id: xh_gauge.h 41590 2006-10-03 14:53:40Z VZ $
+// RCS-ID:      $Id: xh_gauge.h 56023 2008-10-01 19:54:57Z VS $
 // Copyright:   (c) 2000 Bob Mitchell and Verant Interactive
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -17,16 +17,12 @@
 
 class WXDLLIMPEXP_XRC wxGaugeXmlHandler : public wxXmlResourceHandler
 {
-    DECLARE_DYNAMIC_CLASS(wxGaugeXmlHandler)
-    enum
-    {
-        wxGAUGE_DEFAULT_RANGE = 100
-    };
-
 public:
     wxGaugeXmlHandler();
     virtual wxObject *DoCreateResource();
     virtual bool CanHandle(wxXmlNode *node);
+
+    DECLARE_DYNAMIC_CLASS(wxGaugeXmlHandler)
 };
 
 #endif // wxUSE_XRC && wxUSE_GAUGE
