@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: gdicmn.cpp 42420 2006-10-26 03:26:57Z RD $
+// RCS-ID:      $Id: gdicmn.cpp 64656 2010-06-20 18:18:23Z VZ $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -25,15 +25,17 @@
     #include "wx/brush.h"
     #include "wx/palette.h"
     #include "wx/icon.h"
+    #include "wx/iconbndl.h"
     #include "wx/cursor.h"
     #include "wx/settings.h"
     #include "wx/bitmap.h"
     #include "wx/colour.h"
     #include "wx/font.h"
+    #include "wx/math.h"
 #endif
 
 
-IMPLEMENT_DYNAMIC_CLASS(wxGDIObject, wxObject)
+IMPLEMENT_ABSTRACT_CLASS(wxGDIObject, wxObject)
 
 
 WXDLLIMPEXP_DATA_CORE(wxBrushList*) wxTheBrushList;
@@ -52,10 +54,15 @@ WXDLLIMPEXP_DATA_CORE(wxPen)     wxNullPen;
 #if wxUSE_PALETTE
 WXDLLIMPEXP_DATA_CORE(wxPalette) wxNullPalette;
 #endif
+WXDLLIMPEXP_DATA_CORE(wxIconBundle) wxNullIconBundle;
 
 const wxSize wxDefaultSize(wxDefaultCoord, wxDefaultCoord);
 const wxPoint wxDefaultPosition(wxDefaultCoord, wxDefaultCoord);
 
+#include "wx/listimpl.cpp"
+WX_DEFINE_LIST(wxPointList)
+
+
 #if wxUSE_EXTENDED_RTTI
 
 // wxPoint
@@ -108,23 +115,6 @@ wxRect::wxRect(const wxPoint& point1, const wxPoint& point2)
     height++;
 }
 
-bool wxRect::operator==(const wxRect& rect) const
-{
-    return ((x == rect.x) &&
-            (y == rect.y) &&
-            (width == rect.width) &&
-            (height == rect.height));
-}
-
-wxRect wxRect::operator+(const wxRect& rect) const
-{
-    int x1 = wxMin(this->x, rect.x);
-    int y1 = wxMin(this->y, rect.y);
-    int y2 = wxMax(y+height, rect.height+rect.y);
-    int x2 = wxMax(x+width, rect.width+rect.x);
-    return wxRect(x1, y1, x2-x1, y2-y1);
-}
-
 wxRect& wxRect::Union(const wxRect& rect)
 {
     // ignore empty rectangles: union with an empty rectangle shouldn't extend
@@ -230,6 +220,43 @@ bool wxRect::Intersects(const wxRect& rect) const
     return r.width != 0;
 }
 
+wxRect& wxRect::operator+=(const wxRect& rect)
+{
+    *this = *this + rect;
+    return *this;
+}
+
+
+wxRect& wxRect::operator*=(const wxRect& rect)
+{
+    *this = *this * rect;
+    return *this;
+}
+
+
+wxRect operator+(const wxRect& r1, const wxRect& r2)
+{
+    int x1 = wxMin(r1.x, r2.x);
+    int y1 = wxMin(r1.y, r2.y);
+    int y2 = wxMax(r1.y+r1.height, r2.height+r2.y);
+    int x2 = wxMax(r1.x+r1.width, r2.width+r2.x);
+    return wxRect(x1, y1, x2-x1, y2-y1);
+}
+
+wxRect operator*(const wxRect& r1, const wxRect& r2)
+{
+    int x1 = wxMax(r1.x, r2.x);
+    int y1 = wxMax(r1.y, r2.y);
+    int y2 = wxMin(r1.y+r1.height, r2.height+r2.y);
+    int x2 = wxMin(r1.x+r1.width, r2.width+r2.x);
+    return wxRect(x1, y1, x2-x1, y2-y1);
+}
+
+wxRealPoint::wxRealPoint(const wxPoint& pt)
+ : x(pt.x), y(pt.y) 
+{
+}
+
 // ============================================================================
 // wxColourDatabase
 // ============================================================================
@@ -383,7 +410,7 @@ void wxColourDatabase::AddColour(const wxString& name, const wxColour& colour)
 
     // ... and we also allow both grey/gray
     wxString colNameAlt = colName;
-    if ( !colNameAlt.Replace(_T("GRAY"), _T("GREY")) )
+    if ( !colNameAlt.Replace(wxT("GRAY"), wxT("GREY")) )
     {
         // but in this case it is not necessary so avoid extra search below
         colNameAlt.clear();
@@ -411,7 +438,7 @@ wxColour wxColourDatabase::Find(const wxString& colour) const
     wxString colName = colour;
     colName.MakeUpper();
     wxString colNameAlt = colName;
-    if ( !colNameAlt.Replace(_T("GRAY"), _T("GREY")) )
+    if ( !colNameAlt.Replace(wxT("GRAY"), wxT("GREY")) )
         colNameAlt.clear();
 
     wxStringToColourHashMap::iterator it = m_map->find(colName);
@@ -499,47 +526,49 @@ void wxStockGDI::DeleteAll()
 {
     for (unsigned i = 0; i < ITEMCOUNT; i++)
     {
-        delete ms_stockObject[i];
-        ms_stockObject[i] = NULL;
+        wxDELETE(ms_stockObject[i]);
     }
 }
 
 const wxBrush* wxStockGDI::GetBrush(Item item)
 {
-    wxBrush* brush = wx_static_cast(wxBrush*, ms_stockObject[item]);
+    wxBrush* brush = static_cast<wxBrush*>(ms_stockObject[item]);
     if (brush == NULL)
     {
         switch (item)
         {
         case BRUSH_BLACK:
-            brush = new wxBrush(*GetColour(COLOUR_BLACK), wxSOLID);
+            brush = new wxBrush(*GetColour(COLOUR_BLACK), wxBRUSHSTYLE_SOLID);
             break;
         case BRUSH_BLUE:
-            brush = new wxBrush(*GetColour(COLOUR_BLUE), wxSOLID);
+            brush = new wxBrush(*GetColour(COLOUR_BLUE), wxBRUSHSTYLE_SOLID);
             break;
         case BRUSH_CYAN:
-            brush = new wxBrush(*GetColour(COLOUR_CYAN), wxSOLID);
+            brush = new wxBrush(*GetColour(COLOUR_CYAN), wxBRUSHSTYLE_SOLID);
             break;
         case BRUSH_GREEN:
-            brush = new wxBrush(*GetColour(COLOUR_GREEN), wxSOLID);
+            brush = new wxBrush(*GetColour(COLOUR_GREEN), wxBRUSHSTYLE_SOLID);
+            break;
+        case BRUSH_YELLOW:
+            brush = new wxBrush(*GetColour(COLOUR_YELLOW), wxBRUSHSTYLE_SOLID);
             break;
         case BRUSH_GREY:
-            brush = new wxBrush(wxColour(wxT("GREY")), wxSOLID);
+            brush = new wxBrush(wxColour(wxT("GREY")), wxBRUSHSTYLE_SOLID);
             break;
         case BRUSH_LIGHTGREY:
-            brush = new wxBrush(*GetColour(COLOUR_LIGHTGREY), wxSOLID);
+            brush = new wxBrush(*GetColour(COLOUR_LIGHTGREY), wxBRUSHSTYLE_SOLID);
             break;
         case BRUSH_MEDIUMGREY:
-            brush = new wxBrush(wxColour(wxT("MEDIUM GREY")), wxSOLID);
+            brush = new wxBrush(wxColour(wxT("MEDIUM GREY")), wxBRUSHSTYLE_SOLID);
             break;
         case BRUSH_RED:
-            brush = new wxBrush(*GetColour(COLOUR_RED), wxSOLID);
+            brush = new wxBrush(*GetColour(COLOUR_RED), wxBRUSHSTYLE_SOLID);
             break;
         case BRUSH_TRANSPARENT:
-            brush = new wxBrush(*GetColour(COLOUR_BLACK), wxTRANSPARENT);
+            brush = new wxBrush(*GetColour(COLOUR_BLACK), wxBRUSHSTYLE_TRANSPARENT);
             break;
         case BRUSH_WHITE:
-            brush = new wxBrush(*GetColour(COLOUR_WHITE), wxSOLID);
+            brush = new wxBrush(*GetColour(COLOUR_WHITE), wxBRUSHSTYLE_SOLID);
             break;
         default:
             wxFAIL;
@@ -551,7 +580,7 @@ const wxBrush* wxStockGDI::GetBrush(Item item)
 
 const wxColour* wxStockGDI::GetColour(Item item)
 {
-    wxColour* colour = wx_static_cast(wxColour*, ms_stockObject[item]);
+    wxColour* colour = static_cast<wxColour*>(ms_stockObject[item]);
     if (colour == NULL)
     {
         switch (item)
@@ -568,6 +597,9 @@ const wxColour* wxStockGDI::GetColour(Item item)
         case COLOUR_GREEN:
             colour = new wxColour(0, 255, 0);
             break;
+        case COLOUR_YELLOW:
+            colour = new wxColour(255, 255, 0);
+            break;
         case COLOUR_LIGHTGREY:
             colour = new wxColour(wxT("LIGHT GREY"));
             break;
@@ -587,7 +619,7 @@ const wxColour* wxStockGDI::GetColour(Item item)
 
 const wxCursor* wxStockGDI::GetCursor(Item item)
 {
-    wxCursor* cursor = wx_static_cast(wxCursor*, ms_stockObject[item]);
+    wxCursor* cursor = static_cast<wxCursor*>(ms_stockObject[item]);
     if (cursor == NULL)
     {
         switch (item)
@@ -611,7 +643,7 @@ const wxCursor* wxStockGDI::GetCursor(Item item)
 
 const wxFont* wxStockGDI::GetFont(Item item)
 {
-    wxFont* font = wx_static_cast(wxFont*, ms_stockObject[item]);
+    wxFont* font = static_cast<wxFont*>(ms_stockObject[item]);
     if (font == NULL)
     {
         switch (item)
@@ -638,40 +670,46 @@ const wxFont* wxStockGDI::GetFont(Item item)
 
 const wxPen* wxStockGDI::GetPen(Item item)
 {
-    wxPen* pen = wx_static_cast(wxPen*, ms_stockObject[item]);
+    wxPen* pen = static_cast<wxPen*>(ms_stockObject[item]);
     if (pen == NULL)
     {
         switch (item)
         {
         case PEN_BLACK:
-            pen = new wxPen(*GetColour(COLOUR_BLACK), 1, wxSOLID);
+            pen = new wxPen(*GetColour(COLOUR_BLACK), 1, wxPENSTYLE_SOLID);
             break;
         case PEN_BLACKDASHED:
-            pen = new wxPen(*GetColour(COLOUR_BLACK), 1, wxSHORT_DASH);
+            pen = new wxPen(*GetColour(COLOUR_BLACK), 1, wxPENSTYLE_SHORT_DASH);
+            break;
+        case PEN_BLUE:
+            pen = new wxPen(*GetColour(COLOUR_BLUE), 1, wxPENSTYLE_SOLID);
             break;
         case PEN_CYAN:
-            pen = new wxPen(*GetColour(COLOUR_CYAN), 1, wxSOLID);
+            pen = new wxPen(*GetColour(COLOUR_CYAN), 1, wxPENSTYLE_SOLID);
             break;
         case PEN_GREEN:
-            pen = new wxPen(*GetColour(COLOUR_GREEN), 1, wxSOLID);
+            pen = new wxPen(*GetColour(COLOUR_GREEN), 1, wxPENSTYLE_SOLID);
+            break;
+        case PEN_YELLOW:
+            pen = new wxPen(*GetColour(COLOUR_YELLOW), 1, wxPENSTYLE_SOLID);
             break;
         case PEN_GREY:
-            pen = new wxPen(wxColour(wxT("GREY")), 1, wxSOLID);
+            pen = new wxPen(wxColour(wxT("GREY")), 1, wxPENSTYLE_SOLID);
             break;
         case PEN_LIGHTGREY:
-            pen = new wxPen(*GetColour(COLOUR_LIGHTGREY), 1, wxSOLID);
+            pen = new wxPen(*GetColour(COLOUR_LIGHTGREY), 1, wxPENSTYLE_SOLID);
             break;
         case PEN_MEDIUMGREY:
-            pen = new wxPen(wxColour(wxT("MEDIUM GREY")), 1, wxSOLID);
+            pen = new wxPen(wxColour(wxT("MEDIUM GREY")), 1, wxPENSTYLE_SOLID);
             break;
         case PEN_RED:
-            pen = new wxPen(*GetColour(COLOUR_RED), 1, wxSOLID);
+            pen = new wxPen(*GetColour(COLOUR_RED), 1, wxPENSTYLE_SOLID);
             break;
         case PEN_TRANSPARENT:
-            pen = new wxPen(*GetColour(COLOUR_BLACK), 1, wxTRANSPARENT);
+            pen = new wxPen(*GetColour(COLOUR_BLACK), 1, wxPENSTYLE_TRANSPARENT);
             break;
         case PEN_WHITE:
-            pen = new wxPen(*GetColour(COLOUR_WHITE), 1, wxSOLID);
+            pen = new wxPen(*GetColour(COLOUR_WHITE), 1, wxPENSTYLE_SOLID);
             break;
         default:
             wxFAIL;
@@ -695,6 +733,8 @@ void wxDeleteStockLists()
     wxDELETE(wxTheBrushList);
     wxDELETE(wxThePenList);
     wxDELETE(wxTheFontList);
+
+    // wxTheColourDatabase is cleaned up by wxAppBase::CleanUp()
 }
 
 // ============================================================================
@@ -709,22 +749,21 @@ wxGDIObjListBase::~wxGDIObjListBase()
 {
     for (wxList::compatibility_iterator node = list.GetFirst(); node; node = node->GetNext())
     {
-        delete wx_static_cast(wxObject*, node->GetData());
+        delete static_cast<wxObject*>(node->GetData());
     }
 }
 
-wxPen *wxPenList::FindOrCreatePen (const wxColour& colour, int width, int style)
+wxPen *wxPenList::FindOrCreatePen (const wxColour& colour, int width, wxPenStyle style)
 {
-    for (wxList::compatibility_iterator node = list.GetFirst(); node; node = node->GetNext())
+    for ( wxList::compatibility_iterator node = list.GetFirst();
+          node;
+          node = node->GetNext() )
     {
-        wxPen *each_pen = (wxPen *) node->GetData ();
-        if (
-                each_pen->GetWidth () == width &&
-                each_pen->GetStyle () == style &&
-                each_pen->GetColour ().Red () == colour.Red () &&
-                each_pen->GetColour ().Green () == colour.Green () &&
-                each_pen->GetColour ().Blue () == colour.Blue ())
-            return each_pen;
+        wxPen * const pen = (wxPen *) node->GetData();
+        if ( pen->GetWidth () == width &&
+                pen->GetStyle () == style &&
+                    pen->GetColour() == colour )
+            return pen;
     }
 
     wxPen* pen = NULL;
@@ -738,17 +777,15 @@ wxPen *wxPenList::FindOrCreatePen (const wxColour& colour, int width, int style)
     return pen;
 }
 
-wxBrush *wxBrushList::FindOrCreateBrush (const wxColour& colour, int style)
+wxBrush *wxBrushList::FindOrCreateBrush (const wxColour& colour, wxBrushStyle style)
 {
-    for (wxList::compatibility_iterator node = list.GetFirst(); node; node = node->GetNext())
+    for ( wxList::compatibility_iterator node = list.GetFirst();
+          node;
+          node = node->GetNext() )
     {
-        wxBrush *each_brush = (wxBrush *) node->GetData ();
-        if (
-                each_brush->GetStyle () == style &&
-                each_brush->GetColour ().Red () == colour.Red () &&
-                each_brush->GetColour ().Green () == colour.Green () &&
-                each_brush->GetColour ().Blue () == colour.Blue ())
-            return each_brush;
+        wxBrush * const brush = (wxBrush *) node->GetData ();
+        if ( brush->GetStyle() == style && brush->GetColour() == colour )
+            return brush;
     }
 
     wxBrush* brush = NULL;
@@ -763,9 +800,9 @@ wxBrush *wxBrushList::FindOrCreateBrush (const wxColour& colour, int style)
 }
 
 wxFont *wxFontList::FindOrCreateFont(int pointSize,
-                                     int family,
-                                     int style,
-                                     int weight,
+                                     wxFontFamily family,
+                                     wxFontStyle style,
+                                     wxFontWeight weight,
                                      bool underline,
                                      const wxString& facename,
                                      wxFontEncoding encoding)
@@ -781,14 +818,14 @@ wxFont *wxFontList::FindOrCreateFont(int pointSize,
              font->GetWeight () == weight &&
              font->GetUnderlined () == underline )
         {
-            int fontFamily = font->GetFamily();
+            wxFontFamily fontFamily = (wxFontFamily)font->GetFamily();
 
 #if defined(__WXGTK__)
             // under GTK the default family is wxSWISS, so looking for a font
             // with wxDEFAULT family should return a wxSWISS one instead of
             // creating a new one
             bool same = (fontFamily == family) ||
-                        (fontFamily == wxSWISS && family == wxDEFAULT);
+                        (fontFamily == wxFONTFAMILY_SWISS && family == wxFONTFAMILY_DEFAULT);
 #else // !GTK
             // VZ: but why elsewhere do we require an exact match? mystery...
             bool same = fontFamily == family;
@@ -862,6 +899,15 @@ wxSize wxGetDisplaySizeMM()
     return wxSize(x, y);
 }
 
+wxSize wxGetDisplayPPI()
+{
+    const wxSize pixels = wxGetDisplaySize();
+    const wxSize mm = wxGetDisplaySizeMM();
+
+    return wxSize((int)((pixels.x * inches2mm) / mm.x),
+                  (int)((pixels.y * inches2mm) / mm.y));
+}
+
 wxResourceCache::~wxResourceCache ()
 {
     wxList::compatibility_iterator node = GetFirst ();
