@@ -4,7 +4,7 @@
 // Author:      Arthur Seaton, Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: memory.cpp 41054 2006-09-07 19:01:45Z ABX $
+// RCS-ID:      $Id: memory.cpp 61508 2009-07-23 20:30:22Z VZ $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -16,7 +16,7 @@
     #pragma hdrstop
 #endif
 
-#if (defined(__WXDEBUG__) && wxUSE_MEMORY_TRACING) || wxUSE_DEBUG_CONTEXT
+#if wxUSE_MEMORY_TRACING || wxUSE_DEBUG_CONTEXT
 
 #include "wx/memory.h"
 
@@ -39,14 +39,14 @@
 #include "wx/ioswrap.h"
 
 #if !defined(__WATCOMC__) && !(defined(__VMS__) && ( __VMS_VER < 70000000 ) )\
-     && !defined( __MWERKS__ ) && !defined(__SALFORDC__)
+     && !defined( __MWERKS__ )
 #include <memory.h>
 #endif
 
 #include <stdarg.h>
 #include <string.h>
 
-#if wxUSE_THREADS && defined(__WXDEBUG__)
+#if wxUSE_THREADS
 #define USE_THREADSAFE_MEMORY_ALLOCATION 1
 #else
 #define USE_THREADSAFE_MEMORY_ALLOCATION 0
@@ -370,7 +370,7 @@ void wxMemStruct::Dump ()
     msg2.Printf(wxT(" at 0x%lX, size %d"), (long)GetActualData(), (int)RequestSize());
     msg += msg2;
 
-    wxDebugContext::OutputDumpLine(msg);
+    wxDebugContext::OutputDumpLine(msg.c_str());
   }
   else
   {
@@ -381,7 +381,7 @@ void wxMemStruct::Dump ()
     wxString msg2;
     msg2.Printf(wxT("non-object data at 0x%lX, size %d"), (long)GetActualData(), (int)RequestSize() );
     msg += msg2;
-    wxDebugContext::OutputDumpLine(msg);
+    wxDebugContext::OutputDumpLine(msg.c_str());
   }
 }
 
@@ -452,6 +452,9 @@ static wxMarkerType markerCalc[2];
 int wxDebugContext::m_balign = (int)((char *)&markerCalc[1] - (char*)&markerCalc[0]);
 int wxDebugContext::m_balignmask = (int)((char *)&markerCalc[1] - (char*)&markerCalc[0]) - 1;
 
+// Pointer to global function to call at shutdown
+wxShutdownNotifyFunction wxDebugContext::sm_shutdownFn;
+
 wxDebugContext::wxDebugContext(void)
 {
 }
@@ -572,26 +575,21 @@ void wxDebugContext::TraverseList (PmSFV func, wxMemStruct *from)
   */
 bool wxDebugContext::PrintList (void)
 {
-#ifdef __WXDEBUG__
-  TraverseList ((PmSFV)&wxMemStruct::PrintNode, (checkPoint ? checkPoint->m_next : (wxMemStruct*)NULL));
+  TraverseList ((PmSFV)&wxMemStruct::PrintNode, (checkPoint ? checkPoint->m_next : NULL));
 
   return true;
-#else
-  return false;
-#endif
 }
 
 bool wxDebugContext::Dump(void)
 {
-#ifdef __WXDEBUG__
   {
-    wxChar* appName = (wxChar*) wxT("application");
+    const wxChar* appName = wxT("application");
     wxString appNameStr;
     if (wxTheApp)
     {
         appNameStr = wxTheApp->GetAppName();
-        appName = WXSTRINGCAST appNameStr;
-        OutputDumpLine(wxT("----- Memory dump of %s at %s -----"), appName, WXSTRINGCAST wxNow() );
+        appName = appNameStr.c_str();
+        OutputDumpLine(wxT("----- Memory dump of %s at %s -----"), appName, static_cast<const wxChar *>(wxNow().c_str()));
     }
     else
     {
@@ -599,18 +597,14 @@ bool wxDebugContext::Dump(void)
     }
   }
 
-  TraverseList ((PmSFV)&wxMemStruct::Dump, (checkPoint ? checkPoint->m_next : (wxMemStruct*)NULL));
+  TraverseList ((PmSFV)&wxMemStruct::Dump, (checkPoint ? checkPoint->m_next : NULL));
 
   OutputDumpLine(wxEmptyString);
   OutputDumpLine(wxEmptyString);
 
   return true;
-#else
-  return false;
-#endif
 }
 
-#ifdef __WXDEBUG__
 struct wxDebugStatsStruct
 {
   long instanceCount;
@@ -635,19 +629,17 @@ static wxDebugStatsStruct *InsertStatsStruct(wxDebugStatsStruct *head, wxDebugSt
   st->next = head;
   return st;
 }
-#endif
 
 bool wxDebugContext::PrintStatistics(bool detailed)
 {
-#ifdef __WXDEBUG__
   {
-    wxChar* appName = (wxChar*) wxT("application");
+    const wxChar* appName = wxT("application");
     wxString appNameStr;
     if (wxTheApp)
     {
         appNameStr = wxTheApp->GetAppName();
-        appName = WXSTRINGCAST appNameStr;
-        OutputDumpLine(wxT("----- Memory statistics of %s at %s -----"), appName, WXSTRINGCAST wxNow() );
+        appName = appNameStr.c_str();
+        OutputDumpLine(wxT("----- Memory statistics of %s at %s -----"), appName, static_cast<const wxChar *>(wxNow().c_str()));
     }
     else
     {
@@ -664,7 +656,7 @@ bool wxDebugContext::PrintStatistics(bool detailed)
 
   wxDebugStatsStruct *list = NULL;
 
-  wxMemStruct *from = (checkPoint ? checkPoint->m_next : (wxMemStruct*)NULL );
+  wxMemStruct *from = (checkPoint ? checkPoint->m_next : NULL );
   if (!from)
     from = wxDebugContext::GetHead ();
 
@@ -726,34 +718,29 @@ bool wxDebugContext::PrintStatistics(bool detailed)
   OutputDumpLine(wxEmptyString);
 
   return true;
-#else
-  (void)detailed;
-  return false;
-#endif
 }
 
 bool wxDebugContext::PrintClasses(void)
 {
   {
-    wxChar* appName = (wxChar*) wxT("application");
+    const wxChar* appName = wxT("application");
     wxString appNameStr;
     if (wxTheApp)
     {
         appNameStr = wxTheApp->GetAppName();
-        appName = WXSTRINGCAST appNameStr;
+        appName = appNameStr.c_str();
         wxLogMessage(wxT("----- Classes in %s -----"), appName);
     }
   }
 
   int n = 0;
-  wxHashTable::compatibility_iterator node;
-  wxClassInfo *info;
+  const wxClassInfo *info;
 
-  wxClassInfo::sm_classTable->BeginFind();
-  node = wxClassInfo::sm_classTable->Next();
-  while (node)
+  for (wxClassInfo::const_iterator node = wxClassInfo::begin_classinfo(),
+                                    end = wxClassInfo::end_classinfo();
+       node != end; ++node)
   {
-    info = (wxClassInfo *)node->GetData();
+    info = *node;
     if (info->GetClassName())
     {
         wxString msg(info->GetClassName());
@@ -776,7 +763,6 @@ bool wxDebugContext::PrintClasses(void)
 
         wxLogMessage(msg);
     }
-    node = wxClassInfo::sm_classTable->Next();
     n ++;
   }
   wxLogMessage(wxEmptyString);
@@ -799,7 +785,7 @@ int wxDebugContext::Check(bool checkAll)
 {
   int nFailures = 0;
 
-  wxMemStruct *from = (checkPoint ? checkPoint->m_next : (wxMemStruct*)NULL );
+  wxMemStruct *from = (checkPoint ? checkPoint->m_next : NULL );
   if (!from || checkAll)
     from = wxDebugContext::GetHead ();
 
@@ -845,21 +831,26 @@ void wxDebugContext::OutputDumpLine(const wxChar *szFormat, ...)
     int count;
     va_list argptr;
     va_start(argptr, szFormat);
-    buf[sizeof(buf)/sizeof(wxChar)-1] = _T('\0');
+    buf[WXSIZEOF(buf)-1] = wxT('\0');
 
     // keep 3 bytes for a \r\n\0
-    count = wxVsnprintf(buf, sizeof(buf)/sizeof(wxChar)-3, szFormat, argptr);
+    count = wxVsnprintf(buf, WXSIZEOF(buf)-3, szFormat, argptr);
 
     if ( count < 0 )
-        count = sizeof(buf)/sizeof(wxChar)-3;
-    buf[count]=_T('\r');
-    buf[count+1]=_T('\n');
-    buf[count+2]=_T('\0');
+        count = WXSIZEOF(buf)-3;
+    buf[count]=wxT('\r');
+    buf[count+1]=wxT('\n');
+    buf[count+2]=wxT('\0');
 
     wxMessageOutputDebug dbgout;
     dbgout.Printf(buf);
 }
 
+void wxDebugContext::SetShutdownNotifyFunction(wxShutdownNotifyFunction shutdownFn)
+{
+    sm_shutdownFn = shutdownFn;
+}
+
 
 #if USE_THREADSAFE_MEMORY_ALLOCATION
 static bool memSectionOk = false;
@@ -896,7 +887,6 @@ static MemoryCriticalSection memLocker;
 #endif // USE_THREADSAFE_MEMORY_ALLOCATION
 
 
-#ifdef __WXDEBUG__
 #if !(defined(__WXMSW__) && (defined(WXUSINGDLL) || defined(WXMAKINGDLL_BASE)))
 #if wxUSE_GLOBAL_MEMORY_OPERATORS
 void * operator new (size_t size, wxChar * fileName, int lineNum)
@@ -1043,8 +1033,6 @@ void wxDebugFree(void * buf, bool WXUNUSED(isVect) )
     free((char *)st);
 }
 
-#endif // __WXDEBUG__
-
 // Trace: send output to the current debugging stream
 void wxTrace(const wxChar * ...)
 {
@@ -1131,6 +1119,22 @@ void wxTraceLevel(int, const wxChar * ...)
 // All global variables are initialized to 0 at the very beginning, and this is just fine.
 int wxDebugContextDumpDelayCounter::sm_count;
 
+wxDebugContextDumpDelayCounter::wxDebugContextDumpDelayCounter()
+{
+    sm_count++;
+}
+
+wxDebugContextDumpDelayCounter::~wxDebugContextDumpDelayCounter()
+{
+    if ( !--sm_count )
+    {
+        // Notify app if we've been asked to do that
+        if( wxDebugContext::sm_shutdownFn )
+            wxDebugContext::sm_shutdownFn();
+        DoDump();
+    }
+}
+
 void wxDebugContextDumpDelayCounter::DoDump()
 {
     if (wxDebugContext::CountObjectsLeft(true) > 0)
@@ -1145,4 +1149,4 @@ void wxDebugContextDumpDelayCounter::DoDump()
 // least one cleanup counter object
 static wxDebugContextDumpDelayCounter wxDebugContextDumpDelayCounter_One;
 
-#endif // (defined(__WXDEBUG__) && wxUSE_MEMORY_TRACING) || wxUSE_DEBUG_CONTEXT
+#endif // wxUSE_MEMORY_TRACING || wxUSE_DEBUG_CONTEXT
