@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/04/2003
-// RCS-ID:      $Id: taskbarcmn.cpp 44138 2007-01-07 19:44:14Z VZ $
+// RCS-ID:      $Id: taskbarcmn.cpp 62789 2009-12-05 19:57:58Z PC $
 // Copyright:   (c) Julian Smart, 2003
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////
@@ -16,25 +16,30 @@
     #pragma hdrstop
 #endif
 
-#ifdef wxHAS_TASK_BAR_ICON
+#if wxUSE_TASKBARICON
+
+#include "wx/taskbar.h"
 
 #ifndef WX_PRECOMP
     #include "wx/app.h"
+    #include "wx/list.h"
     #include "wx/menu.h"
 #endif
 
+extern WXDLLIMPEXP_DATA_CORE(wxList) wxPendingDelete;
+
 // DLL options compatibility check:
 WX_CHECK_BUILD_OPTIONS("wxAdvanced")
 
-#include "wx/taskbar.h"
-
-DEFINE_EVENT_TYPE( wxEVT_TASKBAR_MOVE )
-DEFINE_EVENT_TYPE( wxEVT_TASKBAR_LEFT_DOWN )
-DEFINE_EVENT_TYPE( wxEVT_TASKBAR_LEFT_UP )
-DEFINE_EVENT_TYPE( wxEVT_TASKBAR_RIGHT_DOWN )
-DEFINE_EVENT_TYPE( wxEVT_TASKBAR_RIGHT_UP )
-DEFINE_EVENT_TYPE( wxEVT_TASKBAR_LEFT_DCLICK )
-DEFINE_EVENT_TYPE( wxEVT_TASKBAR_RIGHT_DCLICK )
+wxDEFINE_EVENT( wxEVT_TASKBAR_MOVE, wxTaskBarIconEvent );
+wxDEFINE_EVENT( wxEVT_TASKBAR_LEFT_DOWN, wxTaskBarIconEvent );
+wxDEFINE_EVENT( wxEVT_TASKBAR_LEFT_UP, wxTaskBarIconEvent );
+wxDEFINE_EVENT( wxEVT_TASKBAR_RIGHT_DOWN, wxTaskBarIconEvent );
+wxDEFINE_EVENT( wxEVT_TASKBAR_RIGHT_UP, wxTaskBarIconEvent );
+wxDEFINE_EVENT( wxEVT_TASKBAR_LEFT_DCLICK, wxTaskBarIconEvent );
+wxDEFINE_EVENT( wxEVT_TASKBAR_RIGHT_DCLICK, wxTaskBarIconEvent );
+wxDEFINE_EVENT( wxEVT_TASKBAR_BALLOON_TIMEOUT, wxTaskBarIconEvent );
+wxDEFINE_EVENT( wxEVT_TASKBAR_BALLOON_CLICK, wxTaskBarIconEvent );
 
 
 BEGIN_EVENT_TABLE(wxTaskBarIconBase, wxEvtHandler)
@@ -51,4 +56,9 @@ void wxTaskBarIconBase::OnRightButtonDown(wxTaskBarIconEvent& WXUNUSED(event))
     }
 }
 
-#endif // defined(wxHAS_TASK_BAR_ICON)
+void wxTaskBarIconBase::Destroy()
+{
+    wxPendingDelete.Append(this);
+}
+
+#endif // wxUSE_TASKBARICON
