@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: validate.cpp 39656 2006-06-09 21:21:53Z ABX $
+// RCS-ID:      $Id: validate.cpp 58227 2009-01-19 13:55:27Z VZ $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -34,7 +34,7 @@ bool wxValidator::ms_isSilent = false;
 
 wxValidator::wxValidator()
 {
-  m_validatorWindow = (wxWindow *) NULL;
+  m_validatorWindow = NULL;
 }
 
 wxValidator::~wxValidator()
