@@ -1,10 +1,10 @@
 ///////////////////////////////////////////////////////////////////////////////
-// Name:        notebook.cpp
-// Purpose:     implementation of wxNotebook
+// Name:        src/generic/notebook.cpp
+// Purpose:     generic implementation of wxNotebook
 // Author:      Julian Smart
 // Modified by:
 // Created:     17/09/98
-// RCS-ID:      $Id: notebook.cpp,v 1.36.2.1 2006/05/29 20:37:07 MW Exp $
+// RCS-ID:      $Id: notebook.cpp 58480 2009-01-28 13:41:43Z VZ $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -17,29 +17,25 @@
 // headers
 // ----------------------------------------------------------------------------
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "notebook.h"
-#endif
-
-#ifdef __VMS
-#pragma message disable unscomzer
-#endif
-
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
 #ifdef __BORLANDC__
-#pragma hdrstop
+    #pragma hdrstop
 #endif
 
-#ifndef __WXPALMOS__
+#if wxUSE_NOTEBOOK
 
-#include  "wx/string.h"
-#include  "wx/log.h"
-#include  "wx/settings.h"
-#include  "wx/generic/imaglist.h"
 #include  "wx/notebook.h"
-#include  "wx/dcclient.h"
+
+#ifndef WX_PRECOMP
+    #include "wx/string.h"
+    #include "wx/log.h"
+    #include  "wx/dcclient.h"
+    #include  "wx/settings.h"
+#endif
+
+#include  "wx/imaglist.h"
 #include  "wx/generic/tabg.h"
 
 // ----------------------------------------------------------------------------
@@ -53,10 +49,7 @@
 // event table
 // ----------------------------------------------------------------------------
 
-DEFINE_EVENT_TYPE(wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGED)
-DEFINE_EVENT_TYPE(wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGING)
-
-BEGIN_EVENT_TABLE(wxNotebook, wxControl)
+BEGIN_EVENT_TABLE(wxNotebook, wxBookCtrlBase)
     EVT_NOTEBOOK_PAGE_CHANGED(wxID_ANY, wxNotebook::OnSelChange)
     EVT_SIZE(wxNotebook::OnSize)
     EVT_PAINT(wxNotebook::OnPaint)
@@ -65,8 +58,7 @@ BEGIN_EVENT_TABLE(wxNotebook, wxControl)
     EVT_NAVIGATION_KEY(wxNotebook::OnNavigationKey)
 END_EVENT_TABLE()
 
-IMPLEMENT_DYNAMIC_CLASS(wxNotebook, wxControl)
-IMPLEMENT_DYNAMIC_CLASS(wxNotebookEvent, wxCommandEvent)
+IMPLEMENT_DYNAMIC_CLASS(wxNotebook, wxBookCtrlBase)
 
 // ============================================================================
 // implementation
@@ -87,30 +79,30 @@ class WXDLLEXPORT wxNotebookTabView: public wxTabView
 {
 DECLARE_DYNAMIC_CLASS(wxNotebookTabView)
 public:
-  wxNotebookTabView(wxNotebook* notebook, long style = wxTAB_STYLE_DRAW_BOX | wxTAB_STYLE_COLOUR_INTERIOR);
-  ~wxNotebookTabView(void);
+    wxNotebookTabView(wxNotebook* notebook, long style = wxTAB_STYLE_DRAW_BOX | wxTAB_STYLE_COLOUR_INTERIOR);
+    virtual ~wxNotebookTabView(void);
 
-  // Called when a tab is activated
-  virtual void OnTabActivate(int activateId, int deactivateId);
-  // Allows vetoing
-  virtual bool OnTabPreActivate(int activateId, int deactivateId);
+    // Called when a tab is activated
+    virtual void OnTabActivate(int activateId, int deactivateId);
+    // Allows vetoing
+    virtual bool OnTabPreActivate(int activateId, int deactivateId);
 
-  // map integer ids used by wxTabView to wxNotebookPage pointers
-  int GetId(wxNotebookPage *page);
-  wxNotebookPage *GetPage(int id) { return m_idToPage[id]; }
+    // map integer ids used by wxTabView to wxNotebookPage pointers
+    int GetId(wxNotebookPage *page);
+    wxNotebookPage *GetPage(int id) { return m_idToPage[id]; }
 
 protected:
-  wxNotebook* m_notebook;
+    wxNotebook* m_notebook;
 
 private:
-  wxIntToNotebookPageHashMap m_idToPage;
-  wxNotebookPageToIntHashMap m_pageToId;
-  int m_nextid;
+    wxIntToNotebookPageHashMap m_idToPage;
+    wxNotebookPageToIntHashMap m_pageToId;
+    int m_nextid;
 };
 
 static int GetPageId(wxTabView *tabview, wxNotebookPage *page)
 {
-    return wx_static_cast(wxNotebookTabView*, tabview)->GetId(page);
+    return static_cast<wxNotebookTabView*>(tabview)->GetId(page);
 }
 
 // ----------------------------------------------------------------------------
@@ -120,7 +112,7 @@ static int GetPageId(wxTabView *tabview, wxNotebookPage *page)
 // common part of all ctors
 void wxNotebook::Init()
 {
-    m_tabView = (wxNotebookTabView*) NULL;
+    m_tabView = NULL;
     m_nSelection = -1;
 }
 
@@ -154,13 +146,14 @@ bool wxNotebook::Create(wxWindow *parent,
     // base init
     SetName(name);
 
+    if ( (style & wxBK_ALIGN_MASK) == wxBK_DEFAULT )
+        style |= wxBK_TOP;
+
     m_windowId = id == wxID_ANY ? NewControlId() : id;
 
     if (!wxControl::Create(parent, id, pos, size, style|wxNO_BORDER, wxDefaultValidator, name))
         return false;
 
-    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
-
     SetTabView(new wxNotebookTabView(this));
 
     return true;
@@ -193,6 +186,12 @@ int wxNotebook::SetSelection(size_t nPage)
     return 0;
 }
 
+int wxNotebook::ChangeSelection(size_t nPage)
+{
+    // FIXME: currently it does generate events too
+    return SetSelection(nPage);
+}
+
 #if 0
 void wxNotebook::AdvanceSelection(bool bForward)
 {
@@ -231,7 +230,7 @@ wxString wxNotebook::GetPageText(size_t nPage) const
         return wxEmptyString;
 }
 
-int wxNotebook::GetPageImage(size_t nPage) const
+int wxNotebook::GetPageImage(size_t WXUNUSED_UNLESS_DEBUG(nPage)) const
 {
     wxASSERT( IS_VALID_PAGE(nPage) );
 
@@ -239,7 +238,8 @@ int wxNotebook::GetPageImage(size_t nPage) const
     return 0;
 }
 
-bool wxNotebook::SetPageImage(size_t nPage, int nImage)
+bool wxNotebook::SetPageImage(size_t WXUNUSED_UNLESS_DEBUG(nPage),
+                              int WXUNUSED(nImage))
 {
     wxASSERT( IS_VALID_PAGE(nPage) );
 
@@ -248,19 +248,19 @@ bool wxNotebook::SetPageImage(size_t nPage, int nImage)
 }
 
 // set the size (the same for all pages)
-void wxNotebook::SetPageSize(const wxSize& size)
+void wxNotebook::SetPageSize(const wxSize& WXUNUSED(size))
 {
     // TODO
 }
 
 // set the padding between tabs (in pixels)
-void wxNotebook::SetPadding(const wxSize& padding)
+void wxNotebook::SetPadding(const wxSize& WXUNUSED(padding))
 {
     // TODO
 }
 
 // set the size of the tabs for wxNB_FIXEDWIDTH controls
-void wxNotebook::SetTabSize(const wxSize& sz)
+void wxNotebook::SetTabSize(const wxSize& WXUNUSED(sz))
 {
     // TODO
 }
@@ -289,17 +289,17 @@ bool wxNotebook::DeletePage(size_t nPage)
 
     if (m_pages.GetCount() == 0)
     {
-      m_nSelection = -1;
-      m_tabView->SetTabSelection(-1, false);
+        m_nSelection = -1;
+        m_tabView->SetTabSelection(-1, false);
     }
     else if (m_nSelection > -1)
     {
-      m_nSelection = -1;
+        m_nSelection = -1;
 
-      m_tabView->SetTabSelection(GetPageId(m_tabView, GetPage(0)), false);
+        m_tabView->SetTabSelection(GetPageId(m_tabView, GetPage(0)), false);
 
-      if (m_nSelection != 0)
-        ChangePage(-1, 0);
+        if (m_nSelection != 0)
+            ChangePage(-1, 0);
     }
 
     RefreshLayout(false);
@@ -405,7 +405,7 @@ bool wxNotebook::InsertPage(size_t nPage,
                             wxNotebookPage *pPage,
                             const wxString& strText,
                             bool bSelect,
-                            int imageId)
+                            int WXUNUSED(imageId))
 {
     wxASSERT( pPage != NULL );
     wxCHECK( IS_VALID_PAGE(nPage) || nPage == GetPageCount(), false );
@@ -541,7 +541,7 @@ bool wxNotebook::RefreshLayout(bool force)
     return true;
 }
 
-void wxNotebook::OnSelChange(wxNotebookEvent& event)
+void wxNotebook::OnSelChange(wxBookCtrlEvent& event)
 {
     // is it our tab control?
     if ( event.GetEventObject() == this )
@@ -565,15 +565,17 @@ void wxNotebook::OnSetFocus(wxFocusEvent& event)
 
 void wxNotebook::OnNavigationKey(wxNavigationKeyEvent& event)
 {
-    if ( event.IsWindowChange() ) {
+    if ( event.IsWindowChange() )
+    {
         // change pages
         AdvanceSelection(event.GetDirection());
     }
     else {
         // pass to the parent
-        if ( GetParent() ) {
+        if ( GetParent() )
+        {
             event.SetCurrentFocus(this);
-            GetParent()->ProcessEvent(event);
+            GetParent()->ProcessWindowEvent(event);
         }
     }
 }
@@ -692,15 +694,15 @@ wxNotebookTabView::~wxNotebookTabView(void)
 
 int wxNotebookTabView::GetId(wxNotebookPage *page)
 {
-  int& id = m_pageToId[page];
+    int& id = m_pageToId[page];
 
-  if (!id)
-  {
-    id = m_nextid++;
-    m_idToPage[id] = page;
-  }
+    if (!id)
+    {
+        id = m_nextid++;
+        m_idToPage[id] = page;
+    }
 
-  return id;
+    return id;
 }
 
 // Called when a tab is activated
@@ -709,7 +711,7 @@ void wxNotebookTabView::OnTabActivate(int activateId, int deactivateId)
   if (!m_notebook)
     return;
 
-  wxNotebookEvent event(wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGED, m_notebook->GetId());
+  wxBookCtrlEvent event(wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGED, m_notebook->GetId());
 
   // Translate from wxTabView's ids (which aren't position-dependent)
   // to wxNotebook's (which are).
@@ -732,7 +734,7 @@ bool wxNotebookTabView::OnTabPreActivate(int activateId, int deactivateId)
 
   if (m_notebook)
   {
-    wxNotebookEvent event(wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGING, m_notebook->GetId());
+    wxBookCtrlEvent event(wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGING, m_notebook->GetId());
 
     // Translate from wxTabView's ids (which aren't position-dependent)
     // to wxNotebook's (which are).
@@ -753,4 +755,4 @@ bool wxNotebookTabView::OnTabPreActivate(int activateId, int deactivateId)
   return retval;
 }
 
-#endif // __WXPALMOS__
+#endif // wxUSE_NOTEBOOK
