@@ -1,11 +1,11 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        progdlgg.h
+// Name:        src/generic/progdlgg.cpp
 // Purpose:     wxProgressDialog class
-// Author:      Karsten Ballder
+// Author:      Karsten Ballueder
 // Modified by:
 // Created:     09.05.1999
-// RCS-ID:      $Id: progdlgg.cpp,v 1.93.2.1 2006/05/29 17:34:54 MW Exp $
-// Copyright:   (c) Karsten Ballder
+// RCS-ID:      $Id: progdlgg.cpp 64656 2010-06-20 18:18:23Z VZ $
+// Copyright:   (c) Karsten Ballueder
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
@@ -17,10 +17,6 @@
 // headers
 // ----------------------------------------------------------------------------
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-    #pragma implementation "progdlgg.h"
-#endif
-
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
@@ -41,10 +37,12 @@
     #include "wx/intl.h"
     #include "wx/dcclient.h"
     #include "wx/timer.h"
+    #include "wx/settings.h"
+    #include "wx/app.h"
 #endif
 
-#include "wx/generic/progdlgg.h"
-#include "wx/settings.h"
+#include "wx/progdlg.h"
+#include "wx/evtloop.h"
 
 // ---------------------------------------------------------------------------
 // macros
@@ -96,12 +94,12 @@ IMPLEMENT_CLASS(wxProgressDialog, wxDialog)
 // wxProgressDialog creation
 // ----------------------------------------------------------------------------
 
-wxProgressDialog::wxProgressDialog(wxString const &title,
-                                   wxString const &message,
+wxProgressDialog::wxProgressDialog(const wxString& title,
+                                   const wxString& message,
                                    int maximum,
                                    wxWindow *parent,
                                    int style)
-                : wxDialog(parent, wxID_ANY, title),
+                : wxDialog(GetParentForModalDialog(parent, style), wxID_ANY, title),
                   m_skip(false),
                   m_delay(3),
                   m_hasAbortButton(false),
@@ -114,8 +112,6 @@ wxProgressDialog::wxProgressDialog(wxString const &title,
     m_hasAbortButton = (style & wxPD_CAN_ABORT) != 0;
     m_hasSkipButton = (style & wxPD_CAN_SKIP) != 0;
 
-    bool isPda = (wxSystemSettings::GetScreenType() <= wxSYS_SCREEN_PDA);
-
 #if defined(__WXMSW__) && !defined(__WXUNIVERSAL__)
     // we have to remove the "Close" button from the title bar then as it is
     // confusing to have it - it doesn't work anyhow
@@ -143,78 +139,76 @@ wxProgressDialog::wxProgressDialog(wxString const &title,
 
     m_parentTop = wxGetTopLevelParent(parent);
 
-    wxClientDC dc(this);
-    dc.SetFont(wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT));
-    long widthText = 0;
-    dc.GetTextExtent(message, &widthText, NULL, NULL, NULL, NULL);
-
-    wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
+    // top-level sizerTop
+    wxSizer * const sizerTop = new wxBoxSizer(wxVERTICAL);
 
     m_msg = new wxStaticText(this, wxID_ANY, message);
-    sizer->Add(m_msg, 0, wxLEFT | wxTOP, 2*LAYOUT_MARGIN);
-
-    wxSize sizeDlg,
-           sizeLabel = m_msg->GetSize();
-    sizeDlg.y = 2*LAYOUT_MARGIN + sizeLabel.y;
+    sizerTop->Add(m_msg, 0, wxLEFT | wxTOP, 2*LAYOUT_MARGIN);
 
     if ( maximum > 0 )
     {
         int gauge_style = wxGA_HORIZONTAL;
-        if ( ( style & wxPD_SMOOTH ) == wxPD_SMOOTH )
+        if ( style & wxPD_SMOOTH )
             gauge_style |= wxGA_SMOOTH;
-        m_gauge = new wxGauge(this, wxID_ANY, m_maximum,
-                              wxDefaultPosition, wxDefaultSize,
-                              gauge_style );
+        m_gauge = new wxGauge
+                      (
+                        this,
+                        wxID_ANY,
+                        m_maximum,
+                        wxDefaultPosition,
+                        // make the progress bar sufficiently long
+                        wxSize(wxMin(wxGetClientDisplayRect().width/3, 300), -1),
+                        gauge_style
+                      );
 
-        sizer->Add(m_gauge, 0, wxLEFT | wxRIGHT | wxTOP | wxEXPAND, 2*LAYOUT_MARGIN);
+        sizerTop->Add(m_gauge, 0, wxLEFT | wxRIGHT | wxTOP | wxEXPAND, 2*LAYOUT_MARGIN);
         m_gauge->SetValue(0);
-
-        wxSize sizeGauge = m_gauge->GetSize();
-        sizeDlg.y += 2*LAYOUT_MARGIN + sizeGauge.y;
     }
     else
-        m_gauge = (wxGauge *)NULL;
+    {
+        m_gauge = NULL;
+    }
 
     // create the estimated/remaining/total time zones if requested
-    m_elapsed = m_estimated = m_remaining = (wxStaticText*)NULL;
-    m_display_estimated = m_last_timeupdate = m_break = 0;
+    m_elapsed =
+    m_estimated =
+    m_remaining = NULL;
+    m_display_estimated =
+    m_last_timeupdate =
+    m_break = 0;
     m_ctdelay = 0;
 
-    // if we are going to have at least one label, remmeber it in this var
-    wxStaticText *label = NULL;
-
     // also count how many labels we really have
     size_t nTimeLabels = 0;
 
+    wxSizer * const sizerLabels = new wxFlexGridSizer(2);
+
     if ( style & wxPD_ELAPSED_TIME )
     {
         nTimeLabels++;
 
-        label =
-        m_elapsed = CreateLabel(_("Elapsed time : "), sizer);
+        m_elapsed = CreateLabel(_("Elapsed time:"), sizerLabels);
     }
 
     if ( style & wxPD_ESTIMATED_TIME )
     {
         nTimeLabels++;
 
-        label =
-        m_estimated = CreateLabel(_("Estimated time : "), sizer);
+        m_estimated = CreateLabel(_("Estimated time:"), sizerLabels);
     }
 
     if ( style & wxPD_REMAINING_TIME )
     {
         nTimeLabels++;
 
-        label =
-        m_remaining = CreateLabel(_("Remaining time : "), sizer);
+        m_remaining = CreateLabel(_("Remaining time:"), sizerLabels);
     }
+    sizerTop->Add(sizerLabels, 0, wxALIGN_CENTER_HORIZONTAL | wxTOP, LAYOUT_MARGIN);
 
     if ( nTimeLabels > 0 )
     {
         // set it to the current time
         m_timeStart = wxGetCurrentTime();
-        sizeDlg.y += nTimeLabels * (label->GetSize().y + LAYOUT_MARGIN);
     }
 
 #if defined(__SMARTPHONE__)
@@ -223,10 +217,12 @@ wxProgressDialog::wxProgressDialog(wxString const &title,
     if ( m_hasAbortButton )
         SetLeftMenu(wxID_CANCEL);
 #else
-    m_btnAbort = m_btnSkip = (wxButton *)NULL;
-    bool sizeDlgModified = false;
+    m_btnAbort =
+    m_btnSkip = NULL;
+
     wxBoxSizer *buttonSizer = new wxBoxSizer(wxHORIZONTAL);
 
+    // Windows dialogs usually have buttons in the lower right corner
     const int sizerFlags =
 #if defined(__WXMSW__) || defined(__WXPM__)
                            wxALIGN_RIGHT | wxALL
@@ -237,39 +233,22 @@ wxProgressDialog::wxProgressDialog(wxString const &title,
 
     if ( m_hasSkipButton )
     {
-        m_btnSkip = new wxButton(this, wxID_SKIP, _("Skip"));
+        m_btnSkip = new wxButton(this, wxID_SKIP, _("&Skip"));
 
-        // Windows dialogs usually have buttons in the lower right corner
         buttonSizer->Add(m_btnSkip, 0, sizerFlags, LAYOUT_MARGIN);
-        sizeDlg.y += 2*LAYOUT_MARGIN + wxButton::GetDefaultSize().y;
-        sizeDlgModified = true;
     }
 
     if ( m_hasAbortButton )
     {
         m_btnAbort = new wxButton(this, wxID_CANCEL);
 
-        // Windows dialogs usually have buttons in the lower right corner
         buttonSizer->Add(m_btnAbort, 0, sizerFlags, LAYOUT_MARGIN);
-        if(!sizeDlgModified)
-            sizeDlg.y += 2*LAYOUT_MARGIN + wxButton::GetDefaultSize().y;
     }
 
-    sizer->Add(buttonSizer, 0, sizerFlags, LAYOUT_MARGIN );
+    sizerTop->Add(buttonSizer, 0, sizerFlags, LAYOUT_MARGIN );
 #endif // __SMARTPHONE__/!__SMARTPHONE__
 
-    SetSizerAndFit(sizer);
-
-    if (!isPda)
-    {
-        sizeDlg.y += 2*LAYOUT_MARGIN;
-
-        // try to make the dialog not square but rectangular of reasonable width
-        sizeDlg.x = (wxCoord)wxMax(widthText, 4*sizeDlg.y/3);
-        sizeDlg.x *= 3;
-        sizeDlg.x /= 2;
-        SetClientSize(sizeDlg);
-    }
+    SetSizerAndFit(sizerTop);
 
     Centre(wxCENTER_FRAME | wxBOTH);
 
@@ -298,33 +277,28 @@ wxProgressDialog::wxProgressDialog(wxString const &title,
     Update();
 }
 
-wxStaticText *wxProgressDialog::CreateLabel(const wxString& text,
-                                            wxSizer *sizer)
+wxStaticText *
+wxProgressDialog::CreateLabel(const wxString& text, wxSizer *sizer)
 {
-    wxBoxSizer *locsizer = new wxBoxSizer(wxLARGESMALL(wxHORIZONTAL,wxVERTICAL));
-
-    wxStaticText *dummy = new wxStaticText(this, wxID_ANY, text);
-    wxStaticText *label = new wxStaticText(this, wxID_ANY, _("unknown"));
+    wxStaticText *label = new wxStaticText(this, wxID_ANY, text);
+    wxStaticText *value = new wxStaticText(this, wxID_ANY, _("unknown"));
 
     // select placement most native or nice on target GUI
 #if defined(__SMARTPHONE__)
-    // label and time to the left in two rows
-    locsizer->Add(dummy, 1, wxALIGN_LEFT);
-    locsizer->Add(label, 1, wxALIGN_LEFT);
-    sizer->Add(locsizer, 0, wxALIGN_LEFT | wxTOP | wxLEFT, LAYOUT_MARGIN);
-#elif defined(__WXMSW__) || defined(__WXPM__) || defined(__WXMAC__)
-    // label and time centered in one row
-    locsizer->Add(dummy, 1, wxLARGESMALL(wxALIGN_RIGHT,wxALIGN_LEFT));
-    locsizer->Add(label, 1, wxALIGN_LEFT | wxLEFT, LAYOUT_MARGIN);
-    sizer->Add(locsizer, 0, wxALIGN_CENTER_HORIZONTAL | wxTOP, LAYOUT_MARGIN);
+    // value and time to the left in two rows
+    sizer->Add(label, 1, wxALIGN_LEFT);
+    sizer->Add(value, 1, wxALIGN_LEFT);
+#elif defined(__WXMSW__) || defined(__WXPM__) || defined(__WXMAC__) || defined(__WXGTK20__)
+    // value and time centered in one row
+    sizer->Add(label, 1, wxLARGESMALL(wxALIGN_RIGHT,wxALIGN_LEFT) | wxTOP | wxRIGHT, LAYOUT_MARGIN);
+    sizer->Add(value, 1, wxALIGN_LEFT | wxTOP, LAYOUT_MARGIN);
 #else
-    // label and time to the right in one row
-    sizer->Add(locsizer, 0, wxALIGN_RIGHT | wxRIGHT | wxTOP, LAYOUT_MARGIN);
-    locsizer->Add(dummy);
-    locsizer->Add(label, 0, wxLEFT, LAYOUT_MARGIN);
+    // value and time to the right in one row
+    sizer->Add(label);
+    sizer->Add(value, 0, wxLEFT, LAYOUT_MARGIN);
 #endif
 
-    return label;
+    return value;
 }
 
 // ----------------------------------------------------------------------------
@@ -334,6 +308,9 @@ wxStaticText *wxProgressDialog::CreateLabel(const wxString& text,
 bool
 wxProgressDialog::Update(int value, const wxString& newmsg, bool *skip)
 {
+    if ( !DoBeforeUpdate(skip) )
+        return false;
+
     wxASSERT_MSG( value == -1 || m_gauge, wxT("cannot update non existent dialog") );
 
 #ifdef __WXMSW__
@@ -342,19 +319,10 @@ wxProgressDialog::Update(int value, const wxString& newmsg, bool *skip)
 
     wxASSERT_MSG( value <= m_maximum, wxT("invalid progress value") );
 
-    // fill up the gauge if value == maximum because this means that the dialog
-    // is going to close and the gauge shouldn't be partly empty in this case
-    if ( m_gauge && value <= m_maximum )
-    {
-        m_gauge->SetValue(value == m_maximum ? value : value + 1);
-    }
+    if ( m_gauge )
+        m_gauge->SetValue(value);
 
-    if ( !newmsg.empty() && newmsg != m_msg->GetLabel() )
-    {
-        m_msg->SetLabel(newmsg);
-
-        wxYieldIfNeeded() ;
-    }
+    UpdateMessage(newmsg);
 
     if ( (m_elapsed || m_remaining || m_estimated) && (value != 0) )
     {
@@ -419,7 +387,7 @@ wxProgressDialog::Update(int value, const wxString& newmsg, bool *skip)
         // so that we return true below and that out [Cancel] handler knew what
         // to do
         m_state = Finished;
-        if( !(GetWindowStyle() & wxPD_AUTO_HIDE) )
+        if( !HasFlag(wxPD_AUTO_HIDE) )
         {
             EnableClose();
             DisableSkip();
@@ -433,8 +401,17 @@ wxProgressDialog::Update(int value, const wxString& newmsg, bool *skip)
                 m_msg->SetLabel(_("Done."));
             }
 
-            wxYieldIfNeeded() ;
+            wxCHECK_MSG(wxEventLoopBase::GetActive(), false,
+                        "wxProgressDialog::Update needs a running event loop");
 
+            // allow the window to repaint:
+            // NOTE: since we yield only for UI events with this call, there
+            //       should be no side-effects
+            wxEventLoopBase::GetActive()->YieldFor(wxEVT_CATEGORY_UI);
+
+            // NOTE: this call results in a new event loop being created
+            //       and to a call to ProcessPendingEvents() (which may generate
+            //       unwanted re-entrancies).
             (void)ShowModal();
         }
         else // auto hide
@@ -447,18 +424,9 @@ wxProgressDialog::Update(int value, const wxString& newmsg, bool *skip)
             Hide();
         }
     }
-    else
+    else // not at maximum yet
     {
-        // we have to yield because not only we want to update the display but
-        // also to process the clicks on the cancel and skip buttons
-        wxYieldIfNeeded() ;
-
-        if ( (m_skip) && (skip != NULL) && (*skip == false) )
-        {
-            *skip = true;
-            m_skip = false;
-            EnableSkip();
-        }
+        DoAfterUpdate();
     }
 
     // update the display in case yielding above didn't do it
@@ -467,6 +435,66 @@ wxProgressDialog::Update(int value, const wxString& newmsg, bool *skip)
     return m_state != Canceled;
 }
 
+bool wxProgressDialog::Pulse(const wxString& newmsg, bool *skip)
+{
+    if ( !DoBeforeUpdate(skip) )
+        return false;
+
+    wxASSERT_MSG( m_gauge, wxT("cannot update non existent dialog") );
+
+    // show a bit of progress
+    m_gauge->Pulse();
+
+    UpdateMessage(newmsg);
+
+    if (m_elapsed || m_remaining || m_estimated)
+    {
+        unsigned long elapsed = wxGetCurrentTime() - m_timeStart;
+
+        SetTimeLabel(elapsed, m_elapsed);
+        SetTimeLabel((unsigned long)-1, m_estimated);
+        SetTimeLabel((unsigned long)-1, m_remaining);
+    }
+
+    DoAfterUpdate();
+
+    return m_state != Canceled;
+}
+
+bool wxProgressDialog::DoBeforeUpdate(bool *skip)
+{
+    wxCHECK_MSG(wxEventLoopBase::GetActive(), false,
+                "wxProgressDialog::DoBeforeUpdate needs a running event loop");
+
+    // we have to yield because not only we want to update the display but
+    // also to process the clicks on the cancel and skip buttons
+    // NOTE: using YieldFor() this call shouldn't give re-entrancy problems
+    //       for event handlers not interested to UI/user-input events.
+    wxEventLoopBase::GetActive()->YieldFor(wxEVT_CATEGORY_UI|wxEVT_CATEGORY_USER_INPUT);
+
+    Update();
+
+    if ( m_skip && skip && !*skip )
+    {
+        *skip = true;
+        m_skip = false;
+        EnableSkip();
+    }
+
+    return m_state != Canceled;
+}
+
+void wxProgressDialog::DoAfterUpdate()
+{
+    wxCHECK_RET(wxEventLoopBase::GetActive(),
+                "wxProgressDialog::DoAfterUpdate needs a running event loop");
+
+    // allow the window to repaint:
+    // NOTE: since we yield only for UI events with this call, there
+    //       should be no side-effects
+    wxEventLoopBase::GetActive()->YieldFor(wxEVT_CATEGORY_UI);
+}
+
 void wxProgressDialog::Resume()
 {
     m_state = Continue;
@@ -489,6 +517,53 @@ bool wxProgressDialog::Show( bool show )
     return wxDialog::Show(show);
 }
 
+int wxProgressDialog::GetValue() const
+{
+    if (m_gauge)
+        return m_gauge->GetValue();
+    return wxNOT_FOUND;
+}
+
+int wxProgressDialog::GetRange() const
+{
+    if (m_gauge)
+        return m_gauge->GetRange();
+    return wxNOT_FOUND;
+}
+
+wxString wxProgressDialog::GetMessage() const
+{
+    return m_msg->GetLabel();
+}
+
+void wxProgressDialog::SetRange(int maximum)
+{
+    wxASSERT_MSG(m_gauge, "The dialog should have been constructed with a range > 0");
+    wxASSERT_MSG(maximum > 0, "Invalid range");
+
+    m_gauge->SetRange(maximum);
+    m_maximum = maximum;
+
+#if defined(__WXMSW__) || defined(__WXPM__)
+    // we can't have values > 65,536 in the progress control under Windows, so
+    // scale everything down
+    m_factor = m_maximum / 65536 + 1;
+    m_maximum /= m_factor;
+#endif // __WXMSW__
+}
+
+
+bool wxProgressDialog::WasCancelled() const
+{
+    return m_hasAbortButton && m_state == Canceled;
+}
+
+bool wxProgressDialog::WasSkipped() const
+{
+    return m_hasSkipButton && m_skip;
+}
+
+
 // ----------------------------------------------------------------------------
 // event handlers
 // ----------------------------------------------------------------------------
@@ -558,10 +633,9 @@ wxProgressDialog::~wxProgressDialog()
 
 void wxProgressDialog::ReenableOtherWindows()
 {
-    if ( GetWindowStyle() & wxPD_APP_MODAL )
+    if ( HasFlag(wxPD_APP_MODAL) )
     {
-        delete m_winDisabler;
-        m_winDisabler = (wxWindowDisabler *)NULL;
+        wxDELETE(m_winDisabler);
     }
     else
     {
@@ -579,10 +653,18 @@ static void SetTimeLabel(unsigned long val, wxStaticText *label)
     if ( label )
     {
         wxString s;
-        unsigned long hours = val / 3600;
-        unsigned long minutes = (val % 3600) / 60;
-        unsigned long seconds = val % 60;
-        s.Printf(wxT("%lu:%02lu:%02lu"), hours, minutes, seconds);
+
+        if (val != (unsigned long)-1)
+        {
+            unsigned long hours = val / 3600;
+            unsigned long minutes = (val % 3600) / 60;
+            unsigned long seconds = val % 60;
+            s.Printf(wxT("%lu:%02lu:%02lu"), hours, minutes, seconds);
+        }
+        else
+        {
+            s = _("Unknown");
+        }
 
         if ( s != label->GetLabel() )
             label->SetLabel(s);
@@ -637,4 +719,20 @@ void wxProgressDialog::EnableClose()
     }
 }
 
+void wxProgressDialog::UpdateMessage(const wxString &newmsg)
+{
+    wxCHECK_RET(wxEventLoopBase::GetActive(),
+                "wxProgressDialog::UpdateMessage needs a running event loop");
+
+    if ( !newmsg.empty() && newmsg != m_msg->GetLabel() )
+    {
+        m_msg->SetLabel(newmsg);
+
+        // allow the window to repaint:
+        // NOTE: since we yield only for UI events with this call, there
+        //       should be no side-effects
+        wxEventLoopBase::GetActive()->YieldFor(wxEVT_CATEGORY_UI);
+    }
+}
+
 #endif // wxUSE_PROGRESSDLG
