@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: splitter.cpp 58400 2009-01-25 14:45:52Z VZ $
+// RCS-ID:      $Id: splitter.cpp 64055 2010-04-20 05:01:39Z SC $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -39,10 +39,10 @@
 
 #include <stdlib.h>
 
-DEFINE_EVENT_TYPE(wxEVT_COMMAND_SPLITTER_SASH_POS_CHANGED)
-DEFINE_EVENT_TYPE(wxEVT_COMMAND_SPLITTER_SASH_POS_CHANGING)
-DEFINE_EVENT_TYPE(wxEVT_COMMAND_SPLITTER_DOUBLECLICKED)
-DEFINE_EVENT_TYPE(wxEVT_COMMAND_SPLITTER_UNSPLIT)
+wxDEFINE_EVENT( wxEVT_COMMAND_SPLITTER_SASH_POS_CHANGED, wxSplitterEvent );
+wxDEFINE_EVENT( wxEVT_COMMAND_SPLITTER_SASH_POS_CHANGING, wxSplitterEvent );
+wxDEFINE_EVENT( wxEVT_COMMAND_SPLITTER_DOUBLECLICKED, wxSplitterEvent );
+wxDEFINE_EVENT( wxEVT_COMMAND_SPLITTER_UNSPLIT, wxSplitterEvent );
 
 IMPLEMENT_DYNAMIC_CLASS(wxSplitterWindow, wxWindow)
 
@@ -61,6 +61,7 @@ BEGIN_EVENT_TABLE(wxSplitterWindow, wxWindow)
     EVT_PAINT(wxSplitterWindow::OnPaint)
     EVT_SIZE(wxSplitterWindow::OnSize)
     EVT_MOUSE_EVENTS(wxSplitterWindow::OnMouseEvent)
+    EVT_MOUSE_CAPTURE_LOST(wxSplitterWindow::OnMouseCaptureLost)
 
 #if defined( __WXMSW__ ) || defined( __WXMAC__)
     EVT_SET_CURSOR(wxSplitterWindow::OnSetCursor)
@@ -71,6 +72,19 @@ END_EVENT_TABLE()
 
 WX_DELEGATE_TO_CONTROL_CONTAINER(wxSplitterWindow, wxWindow)
 
+static bool IsLive(wxSplitterWindow* wnd)
+{
+    // with wxSP_LIVE_UPDATE style the splitter windows are always resized
+    // following the mouse movement while it drags the sash, without it we only
+    // draw the sash at the new position but only resize the windows when the
+    // dragging is finished
+#if defined( __WXMAC__ ) && defined(TARGET_API_MAC_OSX) && TARGET_API_MAC_OSX == 1
+    return true; // Mac can't paint outside paint event - always need live mode
+#else
+    return wnd->HasFlag(wxSP_LIVE_UPDATE);
+#endif
+}
+
 bool wxSplitterWindow::Create(wxWindow *parent, wxWindowID id,
                                    const wxPoint& pos,
                                    const wxSize& size,
@@ -84,10 +98,6 @@ bool wxSplitterWindow::Create(wxWindow *parent, wxWindowID id,
     style &= ~wxBORDER_MASK;
     style |= wxBORDER_NONE;
 
-#if defined(__WXMAC__) && wxMAC_USE_CORE_GRAPHICS
-    // CoreGraphics can't paint sash feedback
-    style |= wxSP_LIVE_UPDATE;
-#endif
 
     if ( !wxWindow::Create(parent, id, pos, size, style, name) )
         return false;
@@ -112,17 +122,16 @@ bool wxSplitterWindow::Create(wxWindow *parent, wxWindowID id,
 
 void wxSplitterWindow::Init()
 {
-    m_container.SetContainerWindow(this);
+    WX_INIT_CONTROL_CONTAINER();
 
     m_splitMode = wxSPLIT_VERTICAL;
     m_permitUnsplitAlways = true;
-    m_windowOne = (wxWindow *) NULL;
-    m_windowTwo = (wxWindow *) NULL;
+    m_windowOne = NULL;
+    m_windowTwo = NULL;
     m_dragMode = wxSPLIT_DRAG_NONE;
     m_oldX = 0;
     m_oldY = 0;
-    m_firstX = 0;
-    m_firstY = 0;
+    m_sashStart = 0;
     m_sashPosition = m_requestedSashPosition = 0;
     m_sashGravity = 0.0;
     m_sashSize = -1; // -1 means use the native sash size
@@ -131,7 +140,7 @@ void wxSplitterWindow::Init()
     m_minimumPaneSize = 0;
     m_sashCursorWE = wxCursor(wxCURSOR_SIZEWE);
     m_sashCursorNS = wxCursor(wxCURSOR_SIZENS);
-    m_sashTrackerPen = new wxPen(*wxBLACK, 2, wxSOLID);
+    m_sashTrackerPen = new wxPen(*wxBLACK, 2, wxPENSTYLE_SOLID);
 
     m_needUpdating = false;
     m_isHot = false;
@@ -185,6 +194,12 @@ void wxSplitterWindow::SetResizeCursor()
 void wxSplitterWindow::OnPaint(wxPaintEvent& WXUNUSED(event))
 {
     wxPaintDC dc(this);
+#ifdef __WXOSX__
+    // as subpanels might have a transparent background we must erase the background
+    // at least on OSX, otherwise traces of the sash will remain 
+    // test with: splitter sample->replace right window
+    dc.Clear();
+#endif
 
     DrawSash(dc);
 }
@@ -217,15 +232,8 @@ void wxSplitterWindow::OnMouseEvent(wxMouseEvent& event)
         return;
     }
 
-    // with wxSP_LIVE_UPDATE style the splitter windows are always resized
-    // following the mouse movement while it drags the sash, without it we only
-    // draw the sash at the new position but only resize the windows when the
-    // dragging is finished
-#if defined( __WXMAC__ ) && defined(TARGET_API_MAC_OSX) && TARGET_API_MAC_OSX == 1
-    bool isLive = true ; // FIXME: why?
-#else
-    bool isLive = HasFlag(wxSP_LIVE_UPDATE);
-#endif
+    bool isLive = IsLive(this);
+
     if (event.LeftDown())
     {
         if ( SashHitTest(x, y) )
@@ -243,13 +251,13 @@ void wxSplitterWindow::OnMouseEvent(wxMouseEvent& event)
                 // shadow sash
                 m_sashPositionCurrent = m_sashPosition;
 
-                DrawSashTracker(x, y);
+                m_oldX = (m_splitMode == wxSPLIT_VERTICAL ? m_sashPositionCurrent : x);
+                m_oldY = (m_splitMode != wxSPLIT_VERTICAL ? m_sashPositionCurrent : y);
+                DrawSashTracker(m_oldX, m_oldY);
             }
 
-            m_oldX = x;
-            m_oldY = y;
-
-            SetResizeCursor();
+            m_ptStart = wxPoint(x,y);
+            m_sashStart = m_sashPosition;
             return;
         }
     }
@@ -277,10 +285,9 @@ void wxSplitterWindow::OnMouseEvent(wxMouseEvent& event)
         // the position of the click doesn't exactly correspond to
         // m_sashPosition, rather it changes it by the distance by which the
         // mouse has moved
-        int diff = m_splitMode == wxSPLIT_VERTICAL ? x - m_oldX : y - m_oldY;
+        int diff = m_splitMode == wxSPLIT_VERTICAL ? x - m_ptStart.x : y - m_ptStart.y;
 
-        int posSashOld = isLive ? m_sashPosition : m_sashPositionCurrent;
-        int posSashNew = OnSashPositionChanging(posSashOld + diff);
+        int posSashNew = OnSashPositionChanging(m_sashStart + diff);
         if ( posSashNew == -1 )
         {
             // change not allowed
@@ -295,7 +302,7 @@ void wxSplitterWindow::OnMouseEvent(wxMouseEvent& event)
                 // We remove the first window from the view
                 wxWindow *removedWindow = m_windowOne;
                 m_windowOne = m_windowTwo;
-                m_windowTwo = (wxWindow *) NULL;
+                m_windowTwo = NULL;
                 OnUnsplit(removedWindow);
                 wxSplitterEvent eventUnsplit(wxEVT_COMMAND_SPLITTER_UNSPLIT, this);
                 eventUnsplit.m_data.win = removedWindow;
@@ -306,7 +313,7 @@ void wxSplitterWindow::OnMouseEvent(wxMouseEvent& event)
             {
                 // We remove the second window from the view
                 wxWindow *removedWindow = m_windowTwo;
-                m_windowTwo = (wxWindow *) NULL;
+                m_windowTwo = NULL;
                 OnUnsplit(removedWindow);
                 wxSplitterEvent eventUnsplit(wxEVT_COMMAND_SPLITTER_UNSPLIT, this);
                 eventUnsplit.m_data.win = removedWindow;
@@ -334,62 +341,53 @@ void wxSplitterWindow::OnMouseEvent(wxMouseEvent& event)
     }
     else if (event.Dragging() && (m_dragMode == wxSPLIT_DRAG_DRAGGING))
     {
-        int diff = m_splitMode == wxSPLIT_VERTICAL ? x - m_oldX : y - m_oldY;
-        if ( !diff )
-        {
-            // nothing to do, mouse didn't really move far enough
-            return;
-        }
+        int diff = m_splitMode == wxSPLIT_VERTICAL ? x - m_ptStart.x : y - m_ptStart.y;
 
-        int posSashOld = isLive ? m_sashPosition : m_sashPositionCurrent;
-        int posSashNew = OnSashPositionChanging(posSashOld + diff);
+        int posSashNew = OnSashPositionChanging(m_sashStart + diff);
         if ( posSashNew == -1 )
         {
             // change not allowed
             return;
         }
 
-        if ( posSashNew == m_sashPosition )
-            return;
-
-        // Erase old tracker
         if ( !isLive )
         {
-            DrawSashTracker(m_oldX, m_oldY);
-        }
+            if ( posSashNew == m_sashPositionCurrent )
+                return;
 
-        if (m_splitMode == wxSPLIT_VERTICAL)
-            x = posSashNew;
-        else
-            y = posSashNew;
-
-        // Remember old positions
-        m_oldX = x;
-        m_oldY = y;
-
-#ifdef __WXMSW__
-        // As we captured the mouse, we may get the mouse events from outside
-        // our window - for example, negative values in x, y. This has a weird
-        // consequence under MSW where we use unsigned values sometimes and
-        // signed ones other times: the coordinates turn as big positive
-        // numbers and so the sash is drawn on the *right* side of the window
-        // instead of the left (or bottom instead of top). Correct this.
-        if ( (short)m_oldX < 0 )
-            m_oldX = 0;
-        if ( (short)m_oldY < 0 )
-            m_oldY = 0;
-#endif // __WXMSW__
-
-        // Draw new one
-        if ( !isLive )
-        {
             m_sashPositionCurrent = posSashNew;
 
+            // Erase old tracker
+            DrawSashTracker(m_oldX, m_oldY);
+
+            m_oldX = (m_splitMode == wxSPLIT_VERTICAL ? m_sashPositionCurrent : x);
+            m_oldY = (m_splitMode != wxSPLIT_VERTICAL ? m_sashPositionCurrent : y);
+
+#ifdef __WXMSW__
+            // As we captured the mouse, we may get the mouse events from outside
+            // our window - for example, negative values in x, y. This has a weird
+            // consequence under MSW where we use unsigned values sometimes and
+            // signed ones other times: the coordinates turn as big positive
+            // numbers and so the sash is drawn on the *right* side of the window
+            // instead of the left (or bottom instead of top). Correct this.
+            if ( (short)m_oldX < 0 )
+                m_oldX = 0;
+            if ( (short)m_oldY < 0 )
+                m_oldY = 0;
+#endif // __WXMSW__
+
+            // Draw new one
             DrawSashTracker(m_oldX, m_oldY);
         }
         else
         {
+            if ( posSashNew == m_sashPosition )
+                return;
+
             DoSetSashPosition(posSashNew);
+
+            // in live mode, the new position is the actual sash position, clear requested position!
+            m_requestedSashPosition = INT_MAX;
             m_needUpdating = true;
         }
     }
@@ -403,6 +401,22 @@ void wxSplitterWindow::OnMouseEvent(wxMouseEvent& event)
     }
 }
 
+void wxSplitterWindow::OnMouseCaptureLost(wxMouseCaptureLostEvent& WXUNUSED(event))
+{
+    if (m_dragMode != wxSPLIT_DRAG_DRAGGING)
+        return;
+
+    m_dragMode = wxSPLIT_DRAG_NONE;
+
+    SetCursor(* wxSTANDARD_CURSOR);
+
+    // Erase old tracker
+    if ( !IsLive(this) )
+    {
+        DrawSashTracker(m_oldX, m_oldY);
+    }
+}
+
 void wxSplitterWindow::OnSize(wxSizeEvent& event)
 {
     // only process this message if we're not iconized - otherwise iconizing
@@ -463,7 +477,7 @@ void wxSplitterWindow::OnSize(wxSizeEvent& event)
 void wxSplitterWindow::SetSashGravity(double gravity)
 {
     wxCHECK_RET( gravity >= 0. && gravity <= 1.,
-                    _T("invalid gravity value") );
+                    wxT("invalid gravity value") );
 
     m_sashGravity = gravity;
 }
@@ -533,33 +547,15 @@ void wxSplitterWindow::DrawSashTracker(int x, int y)
 
     if ( m_splitMode == wxSPLIT_VERTICAL )
     {
-        x1 = x; y1 = 2;
-        x2 = x; y2 = h-2;
-
-        if ( x1 > w )
-        {
-            x1 = w; x2 = w;
-        }
-        else if ( x1 < 0 )
-        {
-            x1 = 0; x2 = 0;
-        }
+        x1 = x2 = wxClip(x, 0, w) + m_sashTrackerPen->GetWidth()/2;
+        y1 = 2;
+        y2 = h-2;
     }
     else
     {
-        x1 = 2; y1 = y;
-        x2 = w-2; y2 = y;
-
-        if ( y1 > h )
-        {
-            y1 = h;
-            y2 = h;
-        }
-        else if ( y1 < 0 )
-        {
-            y1 = 0;
-            y2 = 0;
-        }
+        y1 = y2 = wxClip(y, 0, h) + m_sashTrackerPen->GetWidth()/2;
+        x1 = 2;
+        x2 = w-2;
     }
 
     ClientToScreen(&x1, &y1);
@@ -612,7 +608,7 @@ int wxSplitterWindow::AdjustSashPosition(int sashPos) const
             minSize = m_minimumPaneSize;
 
         int maxSize = GetWindowSize() - minSize - GetBorderSize() - GetSashSize();
-        if ( maxSize > 0 && sashPos > maxSize )
+        if ( maxSize > 0 && sashPos > maxSize && maxSize >= m_minimumPaneSize)
             sashPos = maxSize;
     }
 
@@ -729,14 +725,14 @@ void wxSplitterWindow::SizeWindows()
 // Set pane for unsplit window
 void wxSplitterWindow::Initialize(wxWindow *window)
 {
-    wxASSERT_MSG( (!window || (window && window->GetParent() == this)),
-                  _T("windows in the splitter should have it as parent!") );
+    wxASSERT_MSG( (!window || window->GetParent() == this),
+                  wxT("windows in the splitter should have it as parent!") );
 
     if (window && !window->IsShown())
         window->Show();
 
     m_windowOne = window;
-    m_windowTwo = (wxWindow *) NULL;
+    m_windowTwo = NULL;
     DoSetSashPosition(0);
 }
 
@@ -751,10 +747,10 @@ bool wxSplitterWindow::DoSplit(wxSplitMode mode,
         return false;
 
     wxCHECK_MSG( window1 && window2, false,
-                 _T("can not split with NULL window(s)") );
+                 wxT("can not split with NULL window(s)") );
 
     wxCHECK_MSG( window1->GetParent() == this && window2->GetParent() == this, false,
-                  _T("windows in the splitter should have it as parent!") );
+                  wxT("windows in the splitter should have it as parent!") );
 
     if (! window1->IsShown())
         window1->Show();
@@ -765,15 +761,8 @@ bool wxSplitterWindow::DoSplit(wxSplitMode mode,
     m_windowOne = window1;
     m_windowTwo = window2;
 
-    // remember the sash position we want to set for later if we can't set it
-    // right now (e.g. because the window is too small)
-    m_requestedSashPosition = sashPosition;
-    m_checkRequestedSashPosition = false;
-
-    DoSetSashPosition(ConvertSashPosition(sashPosition));
-
-    SizeWindows();
 
+    SetSashPosition(sashPosition, true);
     return true;
 }
 
@@ -806,13 +795,13 @@ bool wxSplitterWindow::Unsplit(wxWindow *toRemove)
     if ( toRemove == NULL || toRemove == m_windowTwo)
     {
         win = m_windowTwo ;
-        m_windowTwo = (wxWindow *) NULL;
+        m_windowTwo = NULL;
     }
     else if ( toRemove == m_windowOne )
     {
         win = m_windowOne ;
         m_windowOne = m_windowTwo;
-        m_windowTwo = (wxWindow *) NULL;
+        m_windowTwo = NULL;
     }
     else
     {
@@ -922,9 +911,12 @@ wxSize wxSplitterWindow::DoGetBestSize() const
         pSash = &sizeBest.y;
     }
 
-    // account for the border and the sash
+    // account for the sash if the window is actually split
+    if ( m_windowOne && m_windowTwo )
+        *pSash += GetSashSize();
+
+    // account for the border too
     int border = 2*GetBorderSize();
-    *pSash += GetSashSize();
     sizeBest.x += border;
     sizeBest.y += border;
 
@@ -978,12 +970,12 @@ int wxSplitterWindow::OnSashPositionChanging(int newSashPosition)
     {
         // If resultant pane would be too small, enlarge it
         newSashPosition = AdjustSashPosition(newSashPosition);
-    }
 
-    // If the result is out of bounds it means minimum size is too big,
-    // so split window in half as best compromise.
-    if ( newSashPosition < 0 || newSashPosition > window_size )
-        newSashPosition = window_size / 2;
+        // If the result is out of bounds it means minimum size is too big,
+        // so split window in half as best compromise.
+        if ( newSashPosition < 0 || newSashPosition > window_size )
+            newSashPosition = window_size / 2;
+    }
 
     // now let the event handler have it
     //
