@@ -3,7 +3,7 @@
 // Purpose:
 // Author:      Robert Roebling
 // Modified By: Ryan Norton (GtkTreeView implementation)
-// Id:          $Id: listbox.cpp 52940 2008-03-31 16:39:53Z VZ $
+// Id:          $Id: listbox.cpp 64436 2010-05-30 17:45:40Z RR $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -32,7 +32,6 @@
     #include "wx/tooltip.h"
 #endif
 
-#include <gdk/gdk.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
@@ -43,6 +42,8 @@
 extern bool           g_blockEventsOnDrag;
 extern bool           g_blockEventsOnScroll;
 
+
+
 //-----------------------------------------------------------------------------
 // Macro to tell which row the strings are in (1 if native checklist, 0 if not)
 //-----------------------------------------------------------------------------
@@ -61,13 +62,11 @@ extern bool           g_blockEventsOnScroll;
 
 extern "C" {
 static void
-gtk_listbox_row_activated_callback(GtkTreeView        *treeview,
+gtk_listbox_row_activated_callback(GtkTreeView        * WXUNUSED(treeview),
                                    GtkTreePath        *path,
-                                   GtkTreeViewColumn  *col,
+                                   GtkTreeViewColumn  * WXUNUSED(col),
                                    wxListBox          *listbox)
 {
-    if (g_isIdle) wxapp_install_idle_handler();
-
     if (g_blockEventsOnDrag) return;
     if (g_blockEventsOnScroll) return;
 
@@ -80,7 +79,7 @@ gtk_listbox_row_activated_callback(GtkTreeView        *treeview,
 
     if (listbox->IsSelected(sel))
     {
-        GtkTreeEntry* entry = listbox->GtkGetEntry(sel);
+        GtkTreeEntry* entry = listbox->GTKGetEntry(sel);
 
         if (entry)
         {
@@ -105,35 +104,7 @@ gtk_listbox_row_activated_callback(GtkTreeView        *treeview,
         event.SetInt(-1);
     }
 
-    listbox->GetEventHandler()->ProcessEvent( event );
-}
-}
-
-//-----------------------------------------------------------------------------
-// "key_press_event"
-//-----------------------------------------------------------------------------
-
-extern "C" {
-static gint
-gtk_listbox_key_press_callback( GtkWidget *widget,
-                                GdkEventKey *gdk_event,
-                                wxListBox *listbox )
-{
-    if (g_blockEventsOnDrag) return FALSE;
-
-    if ((gdk_event->keyval == GDK_Tab) || (gdk_event->keyval == GDK_ISO_Left_Tab))
-    {
-        wxNavigationKeyEvent new_event;
-        /* GDK reports GDK_ISO_Left_Tab for SHIFT-TAB */
-        new_event.SetDirection( (gdk_event->keyval == GDK_Tab) );
-        /* CTRL-TAB changes the (parent) window, i.e. switch notebook page */
-        new_event.SetWindowChange( (gdk_event->state & GDK_CONTROL_MASK) );
-        new_event.SetCurrentFocus( listbox );
-        if (listbox->GetEventHandler()->ProcessEvent( new_event ))
-            return TRUE;
-    }
-
-    return FALSE;
+    listbox->HandleWindowEvent( event );
 }
 }
 
@@ -143,41 +114,20 @@ gtk_listbox_key_press_callback( GtkWidget *widget,
 
 extern "C" {
 static void
-gtk_listitem_changed_callback( GtkTreeSelection* selection, wxListBox *listbox )
+gtk_listitem_changed_callback(GtkTreeSelection * WXUNUSED(selection),
+                              wxListBox *listbox )
 {
     if (g_blockEventsOnDrag) return;
 
-    if (listbox->m_blockEvent) return;
-
-    wxCommandEvent event(wxEVT_COMMAND_LISTBOX_SELECTED, listbox->GetId() );
-    event.SetEventObject( listbox );
-
-    if (listbox->HasFlag(wxLB_MULTIPLE) || listbox->HasFlag(wxLB_EXTENDED))
+    if (listbox->HasFlag(wxLB_MULTIPLE | wxLB_EXTENDED))
     {
-        wxArrayInt selections;
-        listbox->GetSelections( selections );
-
-        if (selections.GetCount() == 0)
-        {
-            // indicate that this is a deselection
-            event.SetExtraLong( 0 );
-            event.SetInt( -1 );
-
-            listbox->GetEventHandler()->ProcessEvent( event );
-
-            return;
-        }
-        else
-        {
-            // indicate that this is a selection
-            event.SetExtraLong( 1 );
-            event.SetInt( selections[0] );
-
-            listbox->GetEventHandler()->ProcessEvent( event );
-        }
+        listbox->CalcAndSendEvent();
     }
-    else
+    else // single selection
     {
+        wxCommandEvent event(wxEVT_COMMAND_LISTBOX_SELECTED, listbox->GetId() );
+        event.SetEventObject( listbox );
+
         int index = listbox->GetSelection();
         if (index == wxNOT_FOUND)
         {
@@ -185,13 +135,13 @@ gtk_listitem_changed_callback( GtkTreeSelection* selection, wxListBox *listbox )
             event.SetExtraLong( 0 );
             event.SetInt( -1 );
 
-            listbox->GetEventHandler()->ProcessEvent( event );
+            listbox->HandleWindowEvent( event );
 
             return;
         }
         else
         {
-            GtkTreeEntry* entry = listbox->GtkGetEntry( index );
+            GtkTreeEntry* entry = listbox->GTKGetEntry( index );
 
             // indicate that this is a selection
             event.SetExtraLong( 1 );
@@ -206,7 +156,7 @@ gtk_listitem_changed_callback( GtkTreeSelection* selection, wxListBox *listbox )
             else if ( listbox->HasClientUntypedData() )
                 event.SetClientData( gtk_tree_entry_get_userdata(entry) );
 
-            listbox->GetEventHandler()->ProcessEvent( event );
+            listbox->HandleWindowEvent( event );
 
             g_object_unref (entry);
         }
@@ -214,6 +164,78 @@ gtk_listitem_changed_callback( GtkTreeSelection* selection, wxListBox *listbox )
 }
 }
 
+//-----------------------------------------------------------------------------
+// "key_press_event"
+//-----------------------------------------------------------------------------
+
+extern "C" {
+static gboolean
+gtk_listbox_key_press_callback( GtkWidget *WXUNUSED(widget),
+                                GdkEventKey *gdk_event,
+                                wxListBox *listbox )
+{
+    if ((gdk_event->keyval == GDK_Return) ||
+        (gdk_event->keyval == GDK_ISO_Enter) ||
+        (gdk_event->keyval == GDK_KP_Enter))
+    {
+        int index = -1;
+        if (!listbox->HasMultipleSelection())
+            index = listbox->GetSelection();
+        else
+        {
+            wxArrayInt sels;
+            if (listbox->GetSelections( sels ) < 1)
+                return FALSE;
+            index = sels[0];
+        }
+
+        if (index != wxNOT_FOUND)
+        {
+            wxCommandEvent event(wxEVT_COMMAND_LISTBOX_DOUBLECLICKED, listbox->GetId() );
+            event.SetEventObject( listbox );
+
+            GtkTreeEntry* entry = listbox->GTKGetEntry( index );
+
+            // indicate that this is a selection
+            event.SetExtraLong( 1 );
+
+            event.SetInt( index );
+            event.SetString(wxConvUTF8.cMB2WX(gtk_tree_entry_get_label(entry)));
+
+            if ( listbox->HasClientObjectData() )
+                event.SetClientObject(
+                    (wxClientData*) gtk_tree_entry_get_userdata(entry)
+                                 );
+            else if ( listbox->HasClientUntypedData() )
+                event.SetClientData( gtk_tree_entry_get_userdata(entry) );
+
+            /* bool ret = */ listbox->HandleWindowEvent( event );
+
+            g_object_unref (entry);
+
+//          wxMac and wxMSW always invoke default action
+//          if (!ret)
+            {
+                // DClick not handled -> invoke default action
+                wxWindow *tlw = wxGetTopLevelParent( listbox );
+                if (tlw)
+                {
+                    GtkWindow *gtk_window = GTK_WINDOW( tlw->GetHandle() );
+                    if (gtk_window)
+                        gtk_window_activate_default( gtk_window );
+                }
+            }
+
+            // Always intercept, otherwise we'd get another dclick
+            // event from row_activated
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+}
+}
+
 //-----------------------------------------------------------------------------
 // GtkTreeEntry destruction (to destroy client data)
 //-----------------------------------------------------------------------------
@@ -236,7 +258,7 @@ static void gtk_tree_entry_destroy_cb(GtkTreeEntry* entry,
 //-----------------------------------------------------------------------------
 
 extern "C" {
-static gint gtk_listbox_sort_callback(GtkTreeModel *model,
+static gint gtk_listbox_sort_callback(GtkTreeModel * WXUNUSED(model),
                                       GtkTreeIter  *a,
                                       GtkTreeIter  *b,
                                       wxListBox    *listbox)
@@ -257,8 +279,8 @@ static gint gtk_listbox_sort_callback(GtkTreeModel *model,
 
     //We compare collate keys here instead of calling g_utf8_collate
     //as it is rather slow (and even the docs reccommend this)
-    int ret = strcasecmp(gtk_tree_entry_get_collate_key(entry),
-                         gtk_tree_entry_get_collate_key(entry2));
+    int ret = strcmp(gtk_tree_entry_get_collate_key(entry),
+                     gtk_tree_entry_get_collate_key(entry2));
 
     g_object_unref (entry);
     g_object_unref (entry2);
@@ -272,8 +294,8 @@ static gint gtk_listbox_sort_callback(GtkTreeModel *model,
 //-----------------------------------------------------------------------------
 
 extern "C" {
-static gboolean gtk_listbox_searchequal_callback(GtkTreeModel* model,
-                                                 gint column,
+static gboolean gtk_listbox_searchequal_callback(GtkTreeModel * WXUNUSED(model),
+                                                 gint WXUNUSED(column),
                                                  const gchar* key,
                                                  GtkTreeIter* iter,
                                                  wxListBox* listbox)
@@ -287,8 +309,8 @@ static gboolean gtk_listbox_searchequal_callback(GtkTreeModel* model,
     wxCHECK_MSG(entry, 0, wxT("Could not get entry"));
     wxGtkString keycollatekey(g_utf8_collate_key(key, -1));
 
-    int ret = strcasecmp(keycollatekey,
-                         gtk_tree_entry_get_collate_key(entry));
+    int ret = strcmp(keycollatekey,
+                     gtk_tree_entry_get_collate_key(entry));
 
     g_object_unref (entry);
 
@@ -300,7 +322,7 @@ static gboolean gtk_listbox_searchequal_callback(GtkTreeModel* model,
 // wxListBox
 //-----------------------------------------------------------------------------
 
-IMPLEMENT_DYNAMIC_CLASS(wxListBox, wxControl)
+IMPLEMENT_DYNAMIC_CLASS(wxListBox, wxControlWithItems)
 
 // ----------------------------------------------------------------------------
 // construction
@@ -308,7 +330,7 @@ IMPLEMENT_DYNAMIC_CLASS(wxListBox, wxControl)
 
 void wxListBox::Init()
 {
-    m_treeview = (GtkTreeView*) NULL;
+    m_treeview = NULL;
 #if wxUSE_CHECKLISTBOX
     m_hasCheckBoxes = false;
 #endif // wxUSE_CHECKLISTBOX
@@ -332,10 +354,6 @@ bool wxListBox::Create( wxWindow *parent, wxWindowID id,
                         long style, const wxValidator& validator,
                         const wxString &name )
 {
-    m_needParent = true;
-    m_acceptsFocus = true;
-    m_blockEvent = false;
-
     if (!PreCreation( parent, pos, size ) ||
         !CreateBase( parent, id, pos, size, style, validator, name ))
     {
@@ -343,7 +361,8 @@ bool wxListBox::Create( wxWindow *parent, wxWindowID id,
         return false;
     }
 
-    m_widget = gtk_scrolled_window_new( (GtkAdjustment*) NULL, (GtkAdjustment*) NULL );
+    m_widget = gtk_scrolled_window_new( NULL, NULL );
+    g_object_ref(m_widget);
     if (style & wxLB_ALWAYS_SB)
     {
       gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(m_widget),
@@ -356,7 +375,7 @@ bool wxListBox::Create( wxWindow *parent, wxWindowID id,
     }
 
 
-    GtkScrolledWindowSetBorder(m_widget, style);
+    GTKScrolledWindowSetBorder(m_widget, style);
 
     m_treeview = GTK_TREE_VIEW( gtk_tree_view_new( ) );
 
@@ -402,32 +421,23 @@ bool wxListBox::Create( wxWindow *parent, wxWindowID id,
 
     gtk_tree_view_set_enable_search(m_treeview, FALSE);
 
-
-    GtkTreeSelection* selection = gtk_tree_view_get_selection( m_treeview );
-
-    g_signal_connect_after (selection, "changed",
-                            G_CALLBACK (gtk_listitem_changed_callback), this);
-
     GtkSelectionMode mode;
-    if (style & wxLB_MULTIPLE)
+    // GTK_SELECTION_EXTENDED is a deprecated synonym for GTK_SELECTION_MULTIPLE
+    if ( style & (wxLB_MULTIPLE | wxLB_EXTENDED) )
     {
         mode = GTK_SELECTION_MULTIPLE;
     }
-    else if (style & wxLB_EXTENDED)
+    else // no multi-selection flags specified
     {
-        mode = GTK_SELECTION_EXTENDED;
-    }
-    else
-    {
-        // if style was 0 set single mode
         m_windowStyle |= wxLB_SINGLE;
         mode = GTK_SELECTION_SINGLE;
     }
 
+    GtkTreeSelection* selection = gtk_tree_view_get_selection( m_treeview );
     gtk_tree_selection_set_mode( selection, mode );
 
     // Handle sortable stuff
-    if(style & wxLB_SORT)
+    if(HasFlag(wxLB_SORT))
     {
         // Setup sorting in ascending (wx) order
         gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(m_liststore),
@@ -449,22 +459,24 @@ bool wxListBox::Create( wxWindow *parent, wxWindowID id,
     gtk_widget_show( GTK_WIDGET(m_treeview) );
     m_focusWidget = GTK_WIDGET(m_treeview);
 
-    wxListBox::DoInsertItems(wxArrayString(n, choices), 0); // insert initial items
+    Append(n, choices); // insert initial items
 
     // generate dclick events
     g_signal_connect_after(m_treeview, "row-activated",
                      G_CALLBACK(gtk_listbox_row_activated_callback), this);
 
-    // for panel navigation
+    // for intercepting dclick generation by <ENTER>
     g_signal_connect (m_treeview, "key_press_event",
                       G_CALLBACK (gtk_listbox_key_press_callback),
                            this);
-
     m_parent->DoAddChild( this );
 
     PostCreation(size);
     SetInitialSize(size); // need this too because this is a wxControlWithItems
 
+    g_signal_connect_after (selection, "changed",
+                            G_CALLBACK (gtk_listitem_changed_callback), this);
+
     return true;
 }
 
@@ -475,186 +487,154 @@ wxListBox::~wxListBox()
     Clear();
 }
 
+void wxListBox::GTKDisableEvents()
+{
+    GtkTreeSelection* selection = gtk_tree_view_get_selection( m_treeview );
+
+    g_signal_handlers_block_by_func(selection,
+                                (gpointer) gtk_listitem_changed_callback, this);
+}
+
+void wxListBox::GTKEnableEvents()
+{
+    GtkTreeSelection* selection = gtk_tree_view_get_selection( m_treeview );
+
+    g_signal_handlers_unblock_by_func(selection,
+                                (gpointer) gtk_listitem_changed_callback, this);
+
+    UpdateOldSelections();
+}
+
+
+void wxListBox::Update()
+{
+    wxWindow::Update();
+
+    if (m_treeview)
+        gdk_window_process_updates(GTK_WIDGET(m_treeview)->window, TRUE);
+}
+
 // ----------------------------------------------------------------------------
 // adding items
 // ----------------------------------------------------------------------------
 
-void wxListBox::GtkInsertItems(const wxArrayString& items,
-                               void** clientData, unsigned int pos)
+int wxListBox::DoInsertItems(const wxArrayStringsAdapter& items,
+                             unsigned int pos,
+                             void **clientData,
+                             wxClientDataType type)
 {
-    wxCHECK_RET( m_treeview != NULL, wxT("invalid listbox") );
+    wxCHECK_MSG( m_treeview != NULL, wxNOT_FOUND, wxT("invalid listbox") );
 
     InvalidateBestSize();
 
-    // Create and set column ids and GValues
-
-    unsigned int nNum = items.GetCount();
-    unsigned int nCurCount = wxListBox::GetCount();
-    wxASSERT_MSG(pos <= nCurCount, wxT("Invalid index passed to wxListBox"));
-
     GtkTreeIter* pIter = NULL; // append by default
     GtkTreeIter iter;
-    if (pos != nCurCount)
+    if ( pos != GetCount() )
     {
-        gboolean res = gtk_tree_model_iter_nth_child(
-                        GTK_TREE_MODEL(m_liststore),
-                        &iter, NULL, //NULL = parent = get first
-                        (int)pos );
-        if(!res)
-        {
-            wxLogSysError(wxT("internal wxListBox error in insertion"));
-            return;
-        }
+        wxCHECK_MSG( GTKGetIteratorFor(pos, &iter), wxNOT_FOUND,
+                     wxT("internal wxListBox error in insertion") );
 
         pIter = &iter;
     }
 
-    for (unsigned int i = 0; i < nNum; ++i)
+    const unsigned int numItems = items.GetCount();
+    for ( unsigned int i = 0; i < numItems; ++i )
     {
-        wxString label = items[i];
-
         GtkTreeEntry* entry = gtk_tree_entry_new();
-        gtk_tree_entry_set_label(entry, wxGTK_CONV(label));
+        gtk_tree_entry_set_label(entry, wxGTK_CONV(items[i]));
         gtk_tree_entry_set_destroy_func(entry,
                 (GtkTreeEntryDestroy)gtk_tree_entry_destroy_cb,
                             this);
 
-        if (clientData)
-            gtk_tree_entry_set_userdata(entry, clientData[i]);
-
         GtkTreeIter itercur;
         gtk_list_store_insert_before(m_liststore, &itercur, pIter);
 
-#if wxUSE_CHECKLISTBOX
-        if (m_hasCheckBoxes)
-        {
-            gtk_list_store_set(m_liststore, &itercur,
-                                 0, FALSE, //FALSE == not toggled
-                                 1, entry, -1);
-        }
-        else
-#endif
-            gtk_list_store_set(m_liststore, &itercur,
-                                 0, entry, -1);
+        GTKSetItem(itercur, entry);
 
-        g_object_unref (entry); //liststore always refs :)
-    }
-}
+        g_object_unref (entry);
 
-void wxListBox::DoInsertItems(const wxArrayString& items, unsigned int pos)
-{
-    wxCHECK_RET( IsValidInsert(pos), wxT("invalid index in wxListBox::InsertItems") );
-
-    GtkInsertItems(items, NULL, pos);
-}
-
-int wxListBox::DoAppend( const wxString& item )
-{
-    wxCHECK_MSG( m_treeview != NULL, -1, wxT("invalid listbox") );
-
-    InvalidateBestSize();
-
-    GtkTreeEntry* entry = gtk_tree_entry_new();
-    gtk_tree_entry_set_label( entry, wxGTK_CONV(item) );
-    gtk_tree_entry_set_destroy_func(entry,
-                (GtkTreeEntryDestroy)gtk_tree_entry_destroy_cb,
-                            this);
-
-    GtkTreeIter itercur;
-    gtk_list_store_insert_before( m_liststore, &itercur, NULL );
-
-#if wxUSE_CHECKLISTBOX
-    if (m_hasCheckBoxes)
-    {
-            gtk_list_store_set(  m_liststore, &itercur,
-                                 0, FALSE, //FALSE == not toggled
-                                 1, entry, -1);
-    }
-    else
-#endif
-        gtk_list_store_set(m_liststore, &itercur,
-                                 0, entry, -1);
-
-    g_object_unref (entry); //liststore always refs :)
-
-    GtkTreePath* path = gtk_tree_model_get_path(
-                                GTK_TREE_MODEL(m_liststore),
-                                &itercur);
-
-    gint* pIntPath = gtk_tree_path_get_indices(path);
-
-    if (pIntPath == NULL)
-    {
-        wxLogSysError(wxT("internal wxListBox error in insertion"));
-        return wxNOT_FOUND;
+        if (clientData)
+            AssignNewItemClientData(GTKGetIndexFor(itercur), clientData, i, type);
     }
 
-    int index = pIntPath[0];
+    UpdateOldSelections();
 
-    gtk_tree_path_free( path );
-
-    return index;
-}
-
-void wxListBox::DoSetItems( const wxArrayString& items,
-                            void **clientData)
-{
-    Clear();
-    GtkInsertItems(items, clientData, 0);
+    return pos + numItems - 1;
 }
 
 // ----------------------------------------------------------------------------
 // deleting items
 // ----------------------------------------------------------------------------
 
-void wxListBox::Clear()
+void wxListBox::DoClear()
 {
     wxCHECK_RET( m_treeview != NULL, wxT("invalid listbox") );
 
+    GTKDisableEvents(); // just in case
+
     InvalidateBestSize();
 
     gtk_list_store_clear( m_liststore ); /* well, THAT was easy :) */
+
+    GTKEnableEvents();
 }
 
-void wxListBox::Delete(unsigned int n)
+void wxListBox::DoDeleteOneItem(unsigned int n)
 {
     wxCHECK_RET( m_treeview != NULL, wxT("invalid listbox") );
 
     InvalidateBestSize();
 
+    GTKDisableEvents(); // just in case
+
     GtkTreeIter iter;
-    gboolean res = gtk_tree_model_iter_nth_child(
-                        GTK_TREE_MODEL(m_liststore),
-                        &iter, NULL, //NULL = parent = get first
-                        n
-                   );
+    wxCHECK_RET( GTKGetIteratorFor(n, &iter), wxT("wrong listbox index") );
 
-    wxCHECK_RET( res, wxT("wrong listbox index") );
-
-    //this returns false if iter is invalid (i.e. deleting item
-    //at end) but since we don't use iter, well... :)
+    // this returns false if iter is invalid (e.g. deleting item at end) but
+    // since we don't use iter, we ignore the return value
     gtk_list_store_remove(m_liststore, &iter);
+
+    GTKEnableEvents();
 }
 
 // ----------------------------------------------------------------------------
-// get GtkTreeEntry from position (note: you need to g_unref it if valid)
+// helper functions for working with iterators
 // ----------------------------------------------------------------------------
 
-struct _GtkTreeEntry* wxListBox::GtkGetEntry(int n) const
+bool wxListBox::GTKGetIteratorFor(unsigned pos, GtkTreeIter *iter) const
+{
+    if ( !gtk_tree_model_iter_nth_child(GTK_TREE_MODEL(m_liststore),
+                                        iter, NULL, pos) )
+    {
+        wxLogDebug(wxT("gtk_tree_model_iter_nth_child(%u) failed"), pos);
+        return false;
+    }
+
+    return true;
+}
+
+int wxListBox::GTKGetIndexFor(GtkTreeIter& iter) const
+{
+    GtkTreePath *path =
+        gtk_tree_model_get_path(GTK_TREE_MODEL(m_liststore), &iter);
+
+    gint* pIntPath = gtk_tree_path_get_indices(path);
+
+    wxCHECK_MSG( pIntPath, wxNOT_FOUND, wxT("failed to get iterator path") );
+
+    int idx = pIntPath[0];
+
+    gtk_tree_path_free( path );
+
+    return idx;
+}
+
+// get GtkTreeEntry from position (note: you need to g_unref it if valid)
+GtkTreeEntry *wxListBox::GTKGetEntry(unsigned n) const
 {
     GtkTreeIter iter;
-    gboolean res = gtk_tree_model_iter_nth_child(
-                        GTK_TREE_MODEL(m_liststore),
-                        &iter, NULL, //NULL = parent = get first
-                        n );
-
-    if (!res)
-    {
-        wxLogDebug(wxT("gtk_tree_model_iter_nth_child failed\n")
-                   wxT("Passed in value was:[%i]  List size:[%u]"),
-                   n, wxListBox::GetCount() );
+    if ( !GTKGetIteratorFor(n, &iter) )
         return NULL;
-    }
 
 
     GtkTreeEntry* entry = NULL;
@@ -664,6 +644,23 @@ struct _GtkTreeEntry* wxListBox::GtkGetEntry(int n) const
     return entry;
 }
 
+void wxListBox::GTKSetItem(GtkTreeIter& iter, const GtkTreeEntry *entry)
+{
+#if wxUSE_CHECKLISTBOX
+    if ( m_hasCheckBoxes )
+    {
+        gtk_list_store_set(m_liststore, &iter,
+                           0, FALSE, // FALSE == not toggled
+                           1, entry,
+                           -1);
+    }
+    else
+#endif // wxUSE_CHECKLISTBOX
+    {
+        gtk_list_store_set(m_liststore, &iter, 0, entry, -1);
+    }
+}
+
 // ----------------------------------------------------------------------------
 // client data
 // ----------------------------------------------------------------------------
@@ -673,7 +670,7 @@ void* wxListBox::DoGetItemClientData(unsigned int n) const
     wxCHECK_MSG( IsValid(n), NULL,
                  wxT("Invalid index passed to GetItemClientData") );
 
-    GtkTreeEntry* entry = GtkGetEntry(n);
+    GtkTreeEntry* entry = GTKGetEntry(n);
     wxCHECK_MSG(entry, NULL, wxT("could not get entry"));
 
     void* userdata = gtk_tree_entry_get_userdata( entry );
@@ -681,70 +678,47 @@ void* wxListBox::DoGetItemClientData(unsigned int n) const
     return userdata;
 }
 
-wxClientData* wxListBox::DoGetItemClientObject(unsigned int n) const
-{
-    return (wxClientData*) wxListBox::DoGetItemClientData(n);
-}
-
 void wxListBox::DoSetItemClientData(unsigned int n, void* clientData)
 {
     wxCHECK_RET( IsValid(n),
                  wxT("Invalid index passed to SetItemClientData") );
 
-    GtkTreeEntry* entry = GtkGetEntry(n);
+    GtkTreeEntry* entry = GTKGetEntry(n);
     wxCHECK_RET(entry, wxT("could not get entry"));
 
     gtk_tree_entry_set_userdata( entry, clientData );
     g_object_unref (entry);
 }
 
-void wxListBox::DoSetItemClientObject(unsigned int n, wxClientData* clientData)
-{
-    // wxItemContainer already deletes data for us
-    wxListBox::DoSetItemClientData(n, (void*) clientData);
-}
-
 // ----------------------------------------------------------------------------
 // string list access
 // ----------------------------------------------------------------------------
 
-void wxListBox::SetString(unsigned int n, const wxString &string)
+void wxListBox::SetString(unsigned int n, const wxString& label)
 {
     wxCHECK_RET( IsValid(n), wxT("invalid index in wxListBox::SetString") );
     wxCHECK_RET( m_treeview != NULL, wxT("invalid listbox") );
 
-    GtkTreeEntry* entry = GtkGetEntry(n);
+    GtkTreeEntry* entry = GTKGetEntry(n);
     wxCHECK_RET( entry, wxT("wrong listbox index") );
 
-    wxString label = string;
+    // update the item itself
+    gtk_tree_entry_set_label(entry, wxGTK_CONV(label));
 
-    // Don't call the selection event handler if we only changed the item label
-    m_blockEvent = true;
+    // and update the model which will refresh the tree too
+    GtkTreeIter iter;
+    wxCHECK_RET( GTKGetIteratorFor(n, &iter), wxT("failed to get iterator") );
 
-    // RN: This may look wierd but the problem is that the TreeView
-    // doesn't resort or update when changed above and there is no real
-    // notification function...
-    void* userdata = gtk_tree_entry_get_userdata(entry);
-    gtk_tree_entry_set_userdata(entry, NULL); //don't delete on destroy
-    g_object_unref (entry);
+    // FIXME: this resets the checked status of a wxCheckListBox item
 
-    bool bWasSelected = wxListBox::IsSelected(n);
-    wxListBox::Delete(n);
-
-    wxArrayString aItems;
-    aItems.Add(label);
-    GtkInsertItems(aItems, &userdata, n);
-    if (bWasSelected)
-        wxListBox::GtkSetSelection(n, true, true);
-
-    m_blockEvent = false;
+    GTKSetItem(iter, entry);
 }
 
 wxString wxListBox::GetString(unsigned int n) const
 {
     wxCHECK_MSG( m_treeview != NULL, wxEmptyString, wxT("invalid listbox") );
 
-    GtkTreeEntry* entry = GtkGetEntry(n);
+    GtkTreeEntry* entry = GTKGetEntry(n);
     wxCHECK_MSG( entry, wxEmptyString, wxT("wrong listbox index") );
 
     wxString label = wxGTK_CONV_BACK( gtk_tree_entry_get_label(entry) );
@@ -795,14 +769,7 @@ int wxListBox::GetSelection() const
     if (!gtk_tree_selection_get_selected(selection, NULL, &iter))
         return wxNOT_FOUND;
 
-    GtkTreePath* path =
-        gtk_tree_model_get_path(GTK_TREE_MODEL(m_liststore), &iter);
-
-    int sel = gtk_tree_path_get_indices(path)[0];
-
-    gtk_tree_path_free(path);
-
-    return sel;
+    return GTKGetIndexFor(iter);
 }
 
 int wxListBox::GetSelections( wxArrayInt& aSelections ) const
@@ -811,7 +778,7 @@ int wxListBox::GetSelections( wxArrayInt& aSelections ) const
 
     aSelections.Empty();
 
-        int i = 0;
+    int i = 0;
     GtkTreeIter iter;
     GtkTreeSelection* selection = gtk_tree_view_get_selection(m_treeview);
 
@@ -836,74 +803,46 @@ bool wxListBox::IsSelected( int n ) const
     GtkTreeSelection* selection = gtk_tree_view_get_selection(m_treeview);
 
     GtkTreeIter iter;
-    gboolean res = gtk_tree_model_iter_nth_child(
-                        GTK_TREE_MODEL(m_liststore),
-                        &iter, NULL, //NULL = parent = get first
-                        n );
-
-    wxCHECK_MSG( res, false, wxT("Invalid index") );
+    wxCHECK_MSG( GTKGetIteratorFor(n, &iter), false, wxT("Invalid index") );
 
     return gtk_tree_selection_iter_is_selected(selection, &iter);
 }
 
 void wxListBox::DoSetSelection( int n, bool select )
 {
+    wxCHECK_RET( m_treeview != NULL, wxT("invalid listbox") );
+
+    GTKDisableEvents();
+
+    GtkTreeSelection* selection = gtk_tree_view_get_selection(m_treeview);
+
     // passing -1 to SetSelection() is documented to deselect all items
     if ( n == wxNOT_FOUND )
     {
-        // ... and not generate any events in the process
-        GtkDeselectAll();
+        gtk_tree_selection_unselect_all(selection);
+        GTKEnableEvents();
         return;
     }
 
     wxCHECK_RET( IsValid(n), wxT("invalid index in wxListBox::SetSelection") );
 
-    // don't generate the selection event
-    GtkSetSelection(n, select, true);
-}
-
-void wxListBox::GtkDeselectAll()
-{
-    wxCHECK_RET( m_treeview != NULL, wxT("invalid listbox") );
-
-    GtkTreeSelection* selection = gtk_tree_view_get_selection(m_treeview);
-
-    m_blockEvent = true;
-
-    gtk_tree_selection_unselect_all(selection);
-
-    m_blockEvent = false;
-}
-
-void wxListBox::GtkSetSelection(int n, const bool select, const bool blockEvent)
-{
-    wxCHECK_RET( m_treeview != NULL, wxT("invalid listbox") );
-
-    GtkTreeSelection* selection = gtk_tree_view_get_selection(m_treeview);
 
     GtkTreeIter iter;
-    gboolean res = gtk_tree_model_iter_nth_child(
-                        GTK_TREE_MODEL(m_liststore),
-                        &iter, NULL, //NULL = parent = get first
-                        n
-                                                );
-    wxCHECK_RET( res, wxT("Invalid index") );
-
-    m_blockEvent = blockEvent;
+    wxCHECK_RET( GTKGetIteratorFor(n, &iter), wxT("Invalid index") );
 
     if (select)
         gtk_tree_selection_select_iter(selection, &iter);
     else
         gtk_tree_selection_unselect_iter(selection, &iter);
-    
+
     GtkTreePath* path = gtk_tree_model_get_path(
-                            GTK_TREE_MODEL(m_liststore), &iter);
+                        GTK_TREE_MODEL(m_liststore), &iter);
 
     gtk_tree_view_scroll_to_cell(m_treeview, path, NULL, FALSE, 0.0f, 0.0f);
 
     gtk_tree_path_free(path);
 
-    m_blockEvent = false;
+    GTKEnableEvents();
 }
 
 void wxListBox::DoScrollToCell(int n, float alignY, float alignX)
@@ -916,12 +855,8 @@ void wxListBox::DoScrollToCell(int n, float alignY, float alignX)
         return;
 
     GtkTreeIter iter;
-    gtk_tree_model_iter_nth_child(
-                                    GTK_TREE_MODEL(m_liststore),
-                                    &iter,
-                                    NULL, //NULL = parent = get first
-                                    n
-                                 );
+    if ( !GTKGetIteratorFor(n, &iter) )
+        return;
 
     GtkTreePath* path = gtk_tree_model_get_path(
                             GTK_TREE_MODEL(m_liststore), &iter);
@@ -985,10 +920,10 @@ int wxListBox::DoListHitTest(const wxPoint& point) const
 // ----------------------------------------------------------------------------
 
 #if wxUSE_TOOLTIPS
-void wxListBox::ApplyToolTip( GtkTooltips *tips, const wxChar *tip )
+void wxListBox::GTKApplyToolTip( GtkTooltips *tips, const gchar *tip )
 {
     // RN: Is this needed anymore?
-    gtk_tooltips_set_tip( tips, GTK_WIDGET( m_treeview ), wxGTK_CONV(tip), (gchar*) NULL );
+    gtk_tooltips_set_tip( tips, GTK_WIDGET( m_treeview ), tip, NULL );
 }
 #endif // wxUSE_TOOLTIPS
 
