@@ -2,7 +2,7 @@
 // Name:        src/gtk/radiobox.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: radiobox.cpp 58188 2009-01-17 20:38:43Z JS $
+// Id:          $Id: radiobox.cpp 62785 2009-12-05 19:25:04Z PC $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -27,8 +27,6 @@
 #include "wx/gtk/private.h"
 #include <gdk/gdkkeysyms.h>
 
-#include "wx/gtk/win_gtk.h"
-
 //-----------------------------------------------------------------------------
 // wxGTKRadioButtonInfo
 //-----------------------------------------------------------------------------
@@ -49,7 +47,7 @@ public:
 //-----------------------------------------------------------------------------
 
 #include "wx/listimpl.cpp"
-WX_DEFINE_LIST(wxRadioBoxButtonsInfoList)
+WX_DEFINE_LIST( wxRadioBoxButtonsInfoList )
 
 extern bool          g_blockEventsOnDrag;
 
@@ -60,8 +58,6 @@ extern bool          g_blockEventsOnDrag;
 extern "C" {
 static void gtk_radiobutton_clicked_callback( GtkToggleButton *button, wxRadioBox *rb )
 {
-    if (g_isIdle) wxapp_install_idle_handler();
-
     if (!rb->m_hasVMT) return;
     if (g_blockEventsOnDrag) return;
 
@@ -71,7 +67,7 @@ static void gtk_radiobutton_clicked_callback( GtkToggleButton *button, wxRadioBo
     event.SetInt( rb->GetSelection() );
     event.SetString( rb->GetStringSelection() );
     event.SetEventObject( rb );
-    rb->GetEventHandler()->ProcessEvent(event);
+    rb->HandleWindowEvent(event);
 }
 }
 
@@ -82,8 +78,6 @@ static void gtk_radiobutton_clicked_callback( GtkToggleButton *button, wxRadioBo
 extern "C" {
 static gint gtk_radiobox_keypress_callback( GtkWidget *widget, GdkEventKey *gdk_event, wxRadioBox *rb )
 {
-    // don't need to install idle handler, its done from "event" signal
-
     if (!rb->m_hasVMT) return FALSE;
     if (g_blockEventsOnDrag) return FALSE;
 
@@ -98,7 +92,7 @@ static gint gtk_radiobox_keypress_callback( GtkWidget *widget, GdkEventKey *gdk_
         // CTRL-TAB changes the (parent) window, i.e. switch notebook page
         new_event.SetWindowChange( (gdk_event->state & GDK_CONTROL_MASK) );
         new_event.SetCurrentFocus( rb );
-        return rb->GetParent()->GetEventHandler()->ProcessEvent( new_event );
+        return rb->GetParent()->HandleWindowEvent(new_event);
     }
 
     if ((gdk_event->keyval != GDK_Up) &&
@@ -144,46 +138,33 @@ static gint gtk_radiobox_keypress_callback( GtkWidget *widget, GdkEventKey *gdk_
 }
 
 extern "C" {
-static gint gtk_radiobutton_focus_in( GtkWidget *widget,
-                                      GdkEvent *WXUNUSED(event),
-                                      wxRadioBox *win )
+static gint gtk_radiobutton_focus_out( GtkWidget * WXUNUSED(widget),
+                                       GdkEventFocus *WXUNUSED(event),
+                                       wxRadioBox *win )
 {
-    if ( win->m_lostFocus )
-    {
-        // no, we didn't really lose it
-        win->m_lostFocus = FALSE;
-    }
-    else if ( !win->m_hasFocus )
-    {
-        win->m_hasFocus = true;
-
-        wxFocusEvent event( wxEVT_SET_FOCUS, win->GetId() );
-        event.SetEventObject( win );
-
-        // never stop the signal emission, it seems to break the kbd handling
-        // inside the radiobox
-        (void)win->GetEventHandler()->ProcessEvent( event );
-    }
+    // NB: This control is composed of several GtkRadioButton widgets and
+    //     when focus changes from one of them to another in the same
+    //     wxRadioBox, we get a focus-out event followed by focus-in for
+    //     another GtkRadioButton owned by the same control. We don't want
+    //     to generate two spurious wxEVT_SET_FOCUS events in this case,
+    //     so we defer sending wx events until idle time.
+    win->GTKHandleFocusOut();
 
+    // never stop the signal emission, it seems to break the kbd handling
+    // inside the radiobox
     return FALSE;
 }
 }
 
 extern "C" {
-static gint gtk_radiobutton_focus_out( GtkWidget *widget,
-                                       GdkEvent *WXUNUSED(event),
-                                       wxRadioBox *win )
+static gint gtk_radiobutton_focus_in( GtkWidget * WXUNUSED(widget),
+                                      GdkEventFocus *WXUNUSED(event),
+                                      wxRadioBox *win )
 {
-  //    wxASSERT_MSG( win->m_hasFocus, _T("got focus out without any focus in?") );
-  // Replace with a warning, else we dump core a lot!
-  //  if (!win->m_hasFocus)
-  //      wxLogWarning(_T("Radiobox got focus out without any focus in.") );
-
-    // we might have lost the focus, but may be not - it may have just gone to
-    // another button in the same radiobox, so we'll check for it in the next
-    // idle iteration (leave m_hasFocus == true for now)
-    win->m_lostFocus = true;
+    win->GTKHandleFocusIn();
 
+    // never stop the signal emission, it seems to break the kbd handling
+    // inside the radiobox
     return FALSE;
 }
 }
@@ -216,15 +197,6 @@ static void gtk_radiobutton_size_allocate( GtkWidget *widget,
 
 IMPLEMENT_DYNAMIC_CLASS(wxRadioBox,wxControl)
 
-void wxRadioBox::Init()
-{
-    m_needParent = true;
-    m_acceptsFocus = true;
-
-    m_hasFocus =
-    m_lostFocus = false;
-}
-
 bool wxRadioBox::Create( wxWindow *parent, wxWindowID id,
                          const wxString& title,
                          const wxPoint &pos, const wxSize &size,
@@ -252,6 +224,7 @@ bool wxRadioBox::Create( wxWindow *parent, wxWindowID id, const wxString& title,
     }
 
     m_widget = GTKCreateFrame(title);
+    g_object_ref(m_widget);
     wxControl::SetLabel(title);
     if ( HasFlag(wxNO_BORDER) )
     {
@@ -268,7 +241,7 @@ bool wxRadioBox::Create( wxWindow *parent, wxWindowID id, const wxString& title,
     unsigned int num_of_cols = GetColumnCount();
     unsigned int num_of_rows = GetRowCount();
 
-    GtkRadioButton *rbtn = (GtkRadioButton*) NULL;
+    GtkRadioButton *rbtn = NULL;
 
     GtkWidget *table = gtk_table_new( num_of_rows, num_of_cols, FALSE );
     gtk_table_set_col_spacings( GTK_TABLE(table), 1 );
@@ -277,14 +250,15 @@ bool wxRadioBox::Create( wxWindow *parent, wxWindowID id, const wxString& title,
     gtk_container_add( GTK_CONTAINER(m_widget), table );
 
     wxString label;
-    GSList *radio_button_group = (GSList *) NULL;
+    GSList *radio_button_group = NULL;
     for (unsigned int i = 0; i < (unsigned int)n; i++)
     {
         if ( i != 0 )
             radio_button_group = gtk_radio_button_get_group( GTK_RADIO_BUTTON(rbtn) );
 
         label.Empty();
-        for ( const wxChar *pc = choices[i]; *pc; pc++ )
+        for ( wxString::const_iterator pc = choices[i].begin();
+              pc != choices[i].end(); ++pc )
         {
             if ( *pc != wxT('&') )
                 label += *pc;
@@ -380,25 +354,6 @@ bool wxRadioBox::Show( bool show )
     return true;
 }
 
-void wxRadioBox::SetFocus()
-{
-    wxCHECK_RET( m_widget != NULL, wxT("invalid radiobox") );
-
-    if (m_buttonsInfo.GetCount() == 0) return;
-
-    wxRadioBoxButtonsInfoList::compatibility_iterator node = m_buttonsInfo.GetFirst();
-    while (node)
-    {
-        GtkToggleButton *button = GTK_TOGGLE_BUTTON( node->GetData()->button );
-        if (button->active)
-        {
-            gtk_widget_grab_focus( GTK_WIDGET(button) );
-            return;
-        }
-        node = node->GetNext();
-    }
-}
-
 void wxRadioBox::SetSelection( int n )
 {
     wxCHECK_RET( m_widget != NULL, wxT("invalid radiobox") );
@@ -473,8 +428,6 @@ void wxRadioBox::SetString(unsigned int item, const wxString& label)
 
 bool wxRadioBox::Enable( bool enable )
 {
-    bool isEnabled = IsEnabled();
-
     if ( !wxControl::Enable( enable ) )
         return false;
 
@@ -489,8 +442,8 @@ bool wxRadioBox::Enable( bool enable )
         node = node->GetNext();
     }
 
-    if (!isEnabled && enable)
-        wxGtkFixSensitivity(this);
+    if (enable)
+        GTKFixSensitivity();
 
     return true;
 }
@@ -568,9 +521,8 @@ void wxRadioBox::GtkDisableEvents()
     wxRadioBoxButtonsInfoList::compatibility_iterator node = m_buttonsInfo.GetFirst();
     while (node)
     {
-        g_signal_handlers_disconnect_by_func (node->GetData()->button,
-                                              (gpointer) gtk_radiobutton_clicked_callback,
-                                              this);
+        g_signal_handlers_block_by_func(node->GetData()->button,
+            (gpointer)gtk_radiobutton_clicked_callback, this);
 
         node = node->GetNext();
     }
@@ -581,8 +533,8 @@ void wxRadioBox::GtkEnableEvents()
     wxRadioBoxButtonsInfoList::compatibility_iterator node = m_buttonsInfo.GetFirst();
     while (node)
     {
-        g_signal_connect (node->GetData()->button, "clicked",
-                          G_CALLBACK (gtk_radiobutton_clicked_callback), this);
+        g_signal_handlers_unblock_by_func(node->GetData()->button,
+            (gpointer)gtk_radiobutton_clicked_callback, this);
 
         node = node->GetNext();
     }
@@ -615,7 +567,7 @@ void wxRadioBox::GTKWidgetDoSetMnemonic(GtkWidget* w)
 }
 
 #if wxUSE_TOOLTIPS
-void wxRadioBox::ApplyToolTip(GtkTooltips * WXUNUSED(tips), const wxChar *tip)
+void wxRadioBox::GTKApplyToolTip(GtkTooltips * WXUNUSED(tips), const gchar *tip)
 {
     // set this tooltip for all radiobuttons which don't have their own tips
     unsigned n = 0;
@@ -625,8 +577,7 @@ void wxRadioBox::ApplyToolTip(GtkTooltips * WXUNUSED(tips), const wxChar *tip)
     {
         if ( !GetItemToolTip(n) )
         {
-            wxToolTip::Apply(GTK_WIDGET(node->GetData()->button),
-                             wxConvCurrent->cWX2MB(tip));
+            wxToolTip::GTKApply(GTK_WIDGET(node->GetData()->button), tip);
         }
     }
 }
@@ -639,7 +590,7 @@ void wxRadioBox::DoSetItemToolTip(unsigned int n, wxToolTip *tooltip)
     if ( tooltip )
         buf = wxGTK_CONV(tooltip->GetTip());
 
-    wxToolTip::Apply(GTK_WIDGET(m_buttonsInfo[n]->button), buf);
+    wxToolTip::GTKApply(GTK_WIDGET(m_buttonsInfo[n]->button), buf);
 }
 
 #endif // wxUSE_TOOLTIPS
@@ -653,7 +604,9 @@ GdkWindow *wxRadioBox::GTKGetWindow(wxArrayGdkWindows& windows) const
     {
         GtkWidget *button = GTK_WIDGET( node->GetData()->button );
 
-        windows.push_back(button->window);
+        // don't put NULL pointers in the 'windows' array!
+        if (button->window)
+            windows.push_back(button->window);
 
         node = node->GetNext();
     }
@@ -661,22 +614,6 @@ GdkWindow *wxRadioBox::GTKGetWindow(wxArrayGdkWindows& windows) const
     return NULL;
 }
 
-void wxRadioBox::OnInternalIdle()
-{
-    wxControl::OnInternalIdle();
-
-    if ( m_lostFocus )
-    {
-        m_hasFocus = false;
-        m_lostFocus = false;
-
-        wxFocusEvent event( wxEVT_KILL_FOCUS, GetId() );
-        event.SetEventObject( this );
-
-        (void)GetEventHandler()->ProcessEvent( event );
-    }
-}
-
 // static
 wxVisualAttributes
 wxRadioBox::GetClassDefaultAttributes(wxWindowVariant WXUNUSED(variant))
