@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     16.11.97
-// RCS-ID:      $Id: checklst.cpp 62511 2009-10-30 14:11:03Z JMS $
+// RCS-ID:      $Id: checklst.cpp 63226 2010-01-23 13:22:00Z VZ $
 // Copyright:   (c) 1998 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -29,7 +29,7 @@
 #include "wx/checklst.h"
 
 #ifndef WX_PRECOMP
-    #include "wx/msw/wrapwin.h"
+    #include "wx/msw/wrapcctl.h"
     #include "wx/object.h"
     #include "wx/colour.h"
     #include "wx/font.h"
@@ -45,7 +45,9 @@
 
 #include <windowsx.h>
 
+#include "wx/renderer.h"
 #include "wx/msw/private.h"
+#include "wx/msw/dc.h"
 
 // ----------------------------------------------------------------------------
 // private functions
@@ -54,6 +56,16 @@
 // get item (converted to right type)
 #define GetItem(n)    ((wxCheckListBoxItem *)(GetItem(n)))
 
+namespace
+{
+    // space around check mark bitmap in pixels
+    static const int CHECKMARK_EXTRA_SPACE = 1;
+
+    // space betwen check bitmap and text label
+    static const int CHECKMARK_LABEL_SPACE = 2;
+
+} // anonymous namespace
+
 // ============================================================================
 // implementation
 // ============================================================================
@@ -123,164 +135,93 @@ IMPLEMENT_DYNAMIC_CLASS(wxCheckListBox, wxListBox)
 
 class wxCheckListBoxItem : public wxOwnerDrawn
 {
-friend class WXDLLIMPEXP_FWD_CORE wxCheckListBox;
 public:
     // ctor
-    wxCheckListBoxItem(wxCheckListBox *pParent, size_t nIndex);
+    wxCheckListBoxItem(wxCheckListBox *parent);
 
     // drawing functions
     virtual bool OnDrawItem(wxDC& dc, const wxRect& rc, wxODAction act, wxODStatus stat);
 
     // simple accessors and operations
-    bool IsChecked() const { return m_bChecked; }
+    wxCheckListBox *GetParent() const
+        { return m_parent; }
 
-    void Check(bool bCheck);
-    void Toggle() { Check(!IsChecked()); }
+    int GetIndex() const
+        { return m_parent->GetItemIndex(const_cast<wxCheckListBoxItem*>(this)); }
 
-    void SendEvent();
+    wxString GetName() const
+        { return m_parent->GetString(GetIndex()); }
+
+
+    bool IsChecked() const
+        { return m_checked; }
+
+    void Check(bool bCheck)
+        { m_checked = bCheck; }
+
+    void Toggle()
+        { Check(!IsChecked()); }
 
 private:
-    bool            m_bChecked;
-    wxCheckListBox *m_pParent;
-    size_t    m_nIndex;
+    wxCheckListBox *m_parent;
+    bool m_checked;
 
-    DECLARE_NO_COPY_CLASS(wxCheckListBoxItem)
+    wxDECLARE_NO_COPY_CLASS(wxCheckListBoxItem);
 };
 
-wxCheckListBoxItem::wxCheckListBoxItem(wxCheckListBox *pParent, size_t nIndex)
-                  : wxOwnerDrawn(wxEmptyString, true)   // checkable
+wxCheckListBoxItem::wxCheckListBoxItem(wxCheckListBox *parent)
 {
-    m_bChecked = false;
-    m_pParent  = pParent;
-    m_nIndex   = nIndex;
+    m_parent = parent;
+    m_checked = false;
 
-    // we don't initialize m_nCheckHeight/Width vars because it's
-    // done in OnMeasure while they are used only in OnDraw and we
-    // know that there will always be OnMeasure before OnDraw
+    wxSize size = wxRendererNative::Get().GetCheckBoxSize(parent);
+    size.x += 2 * CHECKMARK_EXTRA_SPACE + CHECKMARK_LABEL_SPACE;
 
-    // fix appearance for check list boxes: they don't look quite the same as
-    // menu icons
-    SetOwnMarginWidth(::GetSystemMetrics(SM_CXMENUCHECK) -
-                      2*wxSystemSettings::GetMetric(wxSYS_EDGE_X) + 1);
-
-    SetBackgroundColour(pParent->GetBackgroundColour());
+    SetMarginWidth(size.GetWidth());
+    SetBackgroundColour(parent->GetBackgroundColour());
 }
 
 bool wxCheckListBoxItem::OnDrawItem(wxDC& dc, const wxRect& rc,
                                     wxODAction act, wxODStatus stat)
 {
     // first draw the label
-    if ( IsChecked() )
-        stat = (wxOwnerDrawn::wxODStatus)(stat | wxOwnerDrawn::wxODChecked);
-
     if ( !wxOwnerDrawn::OnDrawItem(dc, rc, act, stat) )
         return false;
 
-
     // now draw the check mark part
-    size_t nCheckWidth  = GetDefaultMarginWidth(),
-           nCheckHeight = m_pParent->GetItemHeight();
+    wxMSWDCImpl *impl = (wxMSWDCImpl*) dc.GetImpl();
+    HDC hdc = GetHdcOf(*impl);
 
-    int x = rc.GetX(),
-        y = rc.GetY();
+    wxSize size = wxRendererNative::Get().GetCheckBoxSize(GetParent());
 
-    HDC hdc = (HDC)dc.GetHDC();
+    // first create bitmap in a memory DC
+    MemoryHDC hdcMem(hdc);
+    CompatibleBitmap hBmpCheck(hdc, size.GetWidth(), size.GetHeight());
 
-    // create pens, brushes &c
-    COLORREF colBg = wxColourToRGB(GetBackgroundColour());
-    AutoHPEN hpenBack(colBg),
-             hpenGray(RGB(0xc0, 0xc0, 0xc0));
-
-    SelectInHDC selPen(hdc, (HGDIOBJ)hpenBack);
-    AutoHBRUSH hbrBack(colBg);
-    SelectInHDC selBrush(hdc, hbrBack);
-
-    // erase the background: it could have been filled with the selected colour
-    Rectangle(hdc, x, y, x + nCheckWidth + 1, rc.GetBottom() + 1);
-
-    // shift check mark 1 pixel to the right, looks better like this
-    x++;
-
-    if ( IsChecked() )
+    // then draw a check mark into it
     {
-        // first create a monochrome bitmap in a memory DC
-        MemoryHDC hdcMem(hdc);
-        MonoBitmap hbmpCheck(nCheckWidth, nCheckHeight);
-        SelectInHDC selBmp(hdcMem, hbmpCheck);
+        SelectInHDC selBmp(hdcMem, hBmpCheck);
 
-        // then draw a check mark into it
-        RECT rect = { 0, 0, nCheckWidth, nCheckHeight };
-        ::DrawFrameControl(hdcMem, &rect,
-#ifdef __WXWINCE__
-                           DFC_BUTTON, DFCS_BUTTONCHECK
-#else
-                           DFC_MENU, DFCS_MENUCHECK
-#endif
-                           );
+        int flags = wxCONTROL_FLAT;
+        if ( IsChecked() )
+            flags |= wxCONTROL_CHECKED;
 
-        // finally copy it to screen DC
-        ::BitBlt(hdc, x, y, nCheckWidth, nCheckHeight, hdcMem, 0, 0, SRCCOPY);
-    }
+        wxDCTemp dcMem(hdcMem);
+        wxRendererNative::Get().DrawCheckBox(GetParent(), dcMem, wxRect(size), flags);
+    } // select hBmpCheck out of hdcMem
 
-    // now we draw the smaller rectangle
-    y++;
-    nCheckWidth  -= 2;
-    nCheckHeight -= 2;
+    // finally draw bitmap to screen
 
-    // draw hollow gray rectangle
-    (void)::SelectObject(hdc, (HGDIOBJ)hpenGray);
+    // position of check mark bitmap
+    int x = rc.GetX() + CHECKMARK_EXTRA_SPACE;
+    int y = rc.GetY() + (rc.GetHeight() - size.GetHeight()) / 2;
 
-    SelectInHDC selBrush2(hdc, ::GetStockObject(NULL_BRUSH));
-    Rectangle(hdc, x, y, x + nCheckWidth, y + nCheckHeight);
-
-    if (stat & wxODHasFocus)
-    {
-        RECT rect;
-        wxCopyRectToRECT(rc, rect);
-        DrawFocusRect(hdc, &rect);
-    }
+    UINT uState = stat & wxOwnerDrawn::wxODSelected ? wxDSB_SELECTED : wxDSB_NORMAL;
+    wxDrawStateBitmap(hdc, hBmpCheck, x, y, uState);
 
     return true;
 }
 
-// change the state of the item and redraw it
-void wxCheckListBoxItem::Check(bool check)
-{
-    m_bChecked = check;
-
-    // index may be changed because new items were added/deleted
-    if ( m_pParent->GetItemIndex(this) != (int)m_nIndex )
-    {
-        // update it
-        int index = m_pParent->GetItemIndex(this);
-
-        wxASSERT_MSG( index != wxNOT_FOUND, wxT("what does this item do here?") );
-
-        m_nIndex = (size_t)index;
-    }
-
-    HWND hwndListbox = (HWND)m_pParent->GetHWND();
-
-    RECT rcUpdate;
-
-    if ( ::SendMessage(hwndListbox, LB_GETITEMRECT,
-                       m_nIndex, (LPARAM)&rcUpdate) == LB_ERR )
-    {
-        wxLogDebug(wxT("LB_GETITEMRECT failed"));
-    }
-
-    ::InvalidateRect(hwndListbox, &rcUpdate, FALSE);
-}
-
-// send an "item checked" event
-void wxCheckListBoxItem::SendEvent()
-{
-    wxCommandEvent event(wxEVT_COMMAND_CHECKLISTBOX_TOGGLED, m_pParent->GetId());
-    event.SetInt(m_nIndex);
-    event.SetEventObject(m_pParent);
-    m_pParent->ProcessCommand(event);
-}
-
 // ----------------------------------------------------------------------------
 // implementation of wxCheckListBox class
 // ----------------------------------------------------------------------------
@@ -339,77 +280,64 @@ bool wxCheckListBox::Create(wxWindow *parent, wxWindowID id,
                              style | wxLB_OWNERDRAW, validator, name);
 }
 
-// misc overloaded methods
-// -----------------------
-
-void wxCheckListBox::Delete(unsigned int n)
-{
-    wxCHECK_RET( IsValid(n),
-                 wxT("invalid index in wxListBox::Delete") );
-
-    wxListBox::Delete(n);
-
-    // free memory
-    delete m_aItems[n];
-
-    m_aItems.RemoveAt(n);
-}
-
-bool wxCheckListBox::SetFont( const wxFont &font )
-{
-    unsigned int i;
-    for ( i = 0; i < m_aItems.GetCount(); i++ )
-        m_aItems[i]->SetFont(font);
-
-    wxListBox::SetFont(font);
-
-    return true;
-}
-
 // create/retrieve item
 // --------------------
 
 // create a check list box item
-wxOwnerDrawn *wxCheckListBox::CreateLboxItem(size_t nIndex)
+wxOwnerDrawn *wxCheckListBox::CreateLboxItem(size_t WXUNUSED(n))
 {
-  wxCheckListBoxItem *pItem = new wxCheckListBoxItem(this, nIndex);
-  return pItem;
+    wxCheckListBoxItem *pItem = new wxCheckListBoxItem(this);
+    return pItem;
 }
 
 // return item size
 // ----------------
 bool wxCheckListBox::MSWOnMeasure(WXMEASUREITEMSTRUCT *item)
 {
-  if ( wxListBox::MSWOnMeasure(item) ) {
-    MEASUREITEMSTRUCT *pStruct = (MEASUREITEMSTRUCT *)item;
+    if ( wxListBox::MSWOnMeasure(item) )
+    {
+        MEASUREITEMSTRUCT *pStruct = (MEASUREITEMSTRUCT *)item;
 
-    // save item height
-    m_nItemHeight = pStruct->itemHeight;
+        wxSize size = wxRendererNative::Get().GetCheckBoxSize(this);
+        size.x += 2 * CHECKMARK_EXTRA_SPACE;
+        size.y += 2 * CHECKMARK_EXTRA_SPACE;
 
-    // add place for the check mark
-    pStruct->itemWidth += wxOwnerDrawn::GetDefaultMarginWidth();
+        // add place for the check mark
+        pStruct->itemWidth += size.GetWidth();
 
-    return true;
+        if ( pStruct->itemHeight < static_cast<unsigned int>(size.GetHeight()) )
+            pStruct->itemHeight = size.GetHeight();
+
+        return true;
+    }
+
+    return false;
   }
 
-  return false;
-}
-
 // check items
 // -----------
 
 bool wxCheckListBox::IsChecked(unsigned int uiIndex) const
 {
-    wxCHECK_MSG( IsValid(uiIndex), false, _T("bad wxCheckListBox index") );
+    wxCHECK_MSG( IsValid(uiIndex), false, wxT("bad wxCheckListBox index") );
 
     return GetItem(uiIndex)->IsChecked();
 }
 
 void wxCheckListBox::Check(unsigned int uiIndex, bool bCheck)
 {
-    wxCHECK_RET( IsValid(uiIndex), _T("bad wxCheckListBox index") );
+    wxCHECK_RET( IsValid(uiIndex), wxT("bad wxCheckListBox index") );
 
     GetItem(uiIndex)->Check(bCheck);
+    RefreshItem(uiIndex);
+}
+
+void wxCheckListBox::Toggle(unsigned int uiIndex)
+{
+    wxCHECK_RET( IsValid(uiIndex), wxT("bad wxCheckListBox index") );
+
+    GetItem(uiIndex)->Toggle();
+    RefreshItem(uiIndex);
 }
 
 // process events
@@ -420,33 +348,33 @@ void wxCheckListBox::OnKeyDown(wxKeyEvent& event)
     // what do we do?
     enum
     {
-        None,
-        Toggle,
-        Set,
-        Clear
+        NONE,
+        TOGGLE,
+        SET,
+        CLEAR
     } oper;
 
     switch ( event.GetKeyCode() )
     {
         case WXK_SPACE:
-            oper = Toggle;
+            oper = TOGGLE;
             break;
 
         case WXK_NUMPAD_ADD:
         case '+':
-            oper = Set;
+            oper = SET;
             break;
 
         case WXK_NUMPAD_SUBTRACT:
         case '-':
-            oper = Clear;
+            oper = CLEAR;
             break;
 
         default:
-            oper = None;
+            oper = NONE;
     }
 
-    if ( oper != None )
+    if ( oper != NONE )
     {
         wxArrayInt selections;
         int count = 0;
@@ -466,31 +394,26 @@ void wxCheckListBox::OnKeyDown(wxKeyEvent& event)
 
         for ( int i = 0; i < count; i++ )
         {
-            wxCheckListBoxItem *item = GetItem(selections[i]);
-            if ( !item )
-            {
-                wxFAIL_MSG( _T("no wxCheckListBoxItem?") );
-                continue;
-            }
+            int nItem = selections[i];
 
             switch ( oper )
             {
-                case Toggle:
-                    item->Toggle();
+                case TOGGLE:
+                    Toggle(nItem);
                     break;
 
-                case Set:
-                case Clear:
-                    item->Check( oper == Set );
+                case SET:
+                case CLEAR:
+                    Check(nItem, oper == SET);
                     break;
 
                 default:
-                    wxFAIL_MSG( _T("what should this key do?") );
+                    wxFAIL_MSG( wxT("what should this key do?") );
             }
 
             // we should send an event as this has been done by the user and
             // not by the program
-            item->SendEvent();
+            SendEvent(nItem);
         }
     }
     else // nothing to do
@@ -502,45 +425,70 @@ void wxCheckListBox::OnKeyDown(wxKeyEvent& event)
 void wxCheckListBox::OnLeftClick(wxMouseEvent& event)
 {
     // clicking on the item selects it, clicking on the checkmark toggles
-    if ( event.GetX() <= wxOwnerDrawn::GetDefaultMarginWidth() )
-    {
-        int nItem = HitTest(event.GetX(), event.GetY());
 
-        if ( nItem != wxNOT_FOUND )
+    int nItem = HitTest(event.GetX(), event.GetY());
+
+    if ( nItem != wxNOT_FOUND )
+    {
+        wxRect rect;
+        GetItemRect(nItem, rect);
+
+        // convert item rect to check mark rect
+        wxSize size = wxRendererNative::Get().GetCheckBoxSize(this);
+        rect.x += CHECKMARK_EXTRA_SPACE;
+        rect.y += (rect.GetHeight() - size.GetHeight()) / 2;
+        rect.SetSize(size);
+
+        if ( rect.Contains(event.GetX(), event.GetY()) )
         {
-            wxCheckListBoxItem *item = GetItem(nItem);
-            item->Toggle();
-            item->SendEvent();
+            // people expect to get "kill focus" event for the currently
+            // focused control before getting events from the other controls
+            // and, equally importantly, they may prevent the focus change from
+            // taking place at all (e.g. because the old control contents is
+            // invalid and needs to be corrected) in which case we shouldn't
+            // generate this event at all
+            SetFocus();
+            if ( FindFocus() == this )
+            {
+                Toggle(nItem);
+                SendEvent(nItem);
+
+                // scroll one item down if the item is the last one
+                // and isn't visible at all
+                int h;
+                GetClientSize(NULL, &h);
+                if ( rect.GetBottom() > h )
+                    ScrollLines(1);
+            }
+        }
+        else
+        {
+            // implement default behaviour: clicking on the item selects it
+            event.Skip();
         }
-        //else: it's not an error, just click outside of client zone
     }
     else
     {
-        // implement default behaviour: clicking on the item selects it
+        // implement default behavior on click outside of client zone
         event.Skip();
     }
 }
 
-int wxCheckListBox::DoHitTestItem(wxCoord x, wxCoord y) const
+wxSize wxCheckListBox::DoGetBestClientSize() const
 {
-    int nItem = (int)::SendMessage
-                             (
-                              (HWND)GetHWND(),
-                              LB_ITEMFROMPOINT,
-                              0,
-                              MAKELPARAM(x, y)
-                             );
+    wxSize best = wxListBox::DoGetBestClientSize();
 
-    return nItem >= (int)m_noItems ? wxNOT_FOUND : nItem;
-}
+    // add room for the checkbox
+    wxSize size = wxRendererNative::Get().GetCheckBoxSize(const_cast<wxCheckListBox*>(this));
+    size.x += 2 * CHECKMARK_EXTRA_SPACE;
+    size.y += 2 * CHECKMARK_EXTRA_SPACE;
 
+    best.x += size.GetWidth();
+    if ( best.y < size.GetHeight() )
+        best.y = size.GetHeight();
 
-wxSize wxCheckListBox::DoGetBestSize() const
-{
-    wxSize best = wxListBox::DoGetBestSize();
-    best.x += wxOwnerDrawn::GetDefaultMarginWidth();  // add room for the checkbox
     CacheBestSize(best);
     return best;
 }
 
-#endif
+#endif // wxUSE_CHECKLISTBOX
