@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     2005-01-08
-// RCS-ID:      $Id: stackwalk.cpp 61341 2009-07-07 09:35:56Z VZ $
+// RCS-ID:      $Id: stackwalk.cpp 62925 2009-12-18 14:47:08Z VZ $
 // Copyright:   (c) 2003-2005 Vadim Zeitlin <vadim@wxwindows.org>
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -68,7 +68,7 @@ void wxStackFrame::OnGetName()
                             pSymbol
                         ) )
     {
-        wxDbgHelpDLL::LogError(_T("SymFromAddr"));
+        wxDbgHelpDLL::LogError(wxT("SymFromAddr"));
         return;
     }
 
@@ -96,7 +96,7 @@ void wxStackFrame::OnGetLocation()
     {
         // it is normal that we don't have source info for some symbols,
         // notably all the ones from the system DLLs...
-        //wxDbgHelpDLL::LogError(_T("SymGetLineFromAddr"));
+        //wxDbgHelpDLL::LogError(wxT("SymGetLineFromAddr"));
         return;
     }
 
@@ -161,7 +161,7 @@ void wxStackFrame::OnParam(PSYMBOL_INFO pSymInfo)
 BOOL CALLBACK
 EnumSymbolsProc(PSYMBOL_INFO pSymInfo, ULONG WXUNUSED(SymSize), PVOID data)
 {
-    wxStackFrame *frame = wx_static_cast(wxStackFrame *, data);
+    wxStackFrame *frame = static_cast<wxStackFrame *>(data);
 
     // we're only interested in parameters
     if ( pSymInfo->Flags & IMAGEHLP_SYMBOL_INFO_PARAMETER )
@@ -190,7 +190,7 @@ void wxStackFrame::OnGetParam()
         // address, this is not a real error
         if ( ::GetLastError() != ERROR_INVALID_ADDRESS )
         {
-            wxDbgHelpDLL::LogError(_T("SymSetContext"));
+            wxDbgHelpDLL::LogError(wxT("SymSetContext"));
         }
 
         return;
@@ -205,7 +205,7 @@ void wxStackFrame::OnGetParam()
                             this                // data to pass to it
                         ) )
     {
-        wxDbgHelpDLL::LogError(_T("SymEnumSymbols"));
+        wxDbgHelpDLL::LogError(wxT("SymEnumSymbols"));
     }
 }
 
@@ -214,14 +214,14 @@ void wxStackFrame::OnGetParam()
 // wxStackWalker
 // ----------------------------------------------------------------------------
 
-void wxStackWalker::WalkFrom(const CONTEXT *pCtx, size_t skip)
+void wxStackWalker::WalkFrom(const CONTEXT *pCtx, size_t skip, size_t maxDepth)
 {
     if ( !wxDbgHelpDLL::Init() )
     {
         // don't log a user-visible error message here because the stack trace
         // is only needed for debugging/diagnostics anyhow and we shouldn't
         // confuse the user by complaining that we couldn't generate it
-        wxLogDebug(_T("Failed to get stack backtrace: %s"),
+        wxLogDebug(wxT("Failed to get stack backtrace: %s"),
                    wxDbgHelpDLL::GetErrorMessage().c_str());
         return;
     }
@@ -240,7 +240,7 @@ void wxStackWalker::WalkFrom(const CONTEXT *pCtx, size_t skip)
                             TRUE    // load symbols for all loaded modules
                         ) )
     {
-        wxDbgHelpDLL::LogError(_T("SymInitialize"));
+        wxDbgHelpDLL::LogError(wxT("SymInitialize"));
 
         return;
     }
@@ -266,9 +266,8 @@ void wxStackWalker::WalkFrom(const CONTEXT *pCtx, size_t skip)
     #error "Need to initialize STACKFRAME on non x86"
 #endif // _M_IX86
 
-    // iterate over all stack frames (but stop after 200 to avoid entering
-    // infinite loop if the stack is corrupted)
-    for ( size_t nLevel = 0; nLevel < 200; nLevel++ )
+    // iterate over all stack frames
+    for ( size_t nLevel = 0; nLevel < maxDepth; nLevel++ )
     {
         // get the next stack frame
         if ( !wxDbgHelpDLL::StackWalk
@@ -285,7 +284,7 @@ void wxStackWalker::WalkFrom(const CONTEXT *pCtx, size_t skip)
                             ) )
         {
             if ( ::GetLastError() )
-                wxDbgHelpDLL::LogError(_T("StackWalk"));
+                wxDbgHelpDLL::LogError(wxT("StackWalk"));
 
             break;
         }
@@ -294,49 +293,46 @@ void wxStackWalker::WalkFrom(const CONTEXT *pCtx, size_t skip)
         if ( nLevel >= skip )
         {
             wxStackFrame frame(nLevel - skip,
-                               (void *)sf.AddrPC.Offset,
+                               wxUIntToPtr(sf.AddrPC.Offset),
                                sf.AddrFrame.Offset);
 
             OnStackFrame(frame);
         }
     }
 
-    // this results in crashes inside ntdll.dll when called from
-    // exception handler ...
-#if 0
     if ( !wxDbgHelpDLL::SymCleanup(hProcess) )
     {
-        wxDbgHelpDLL::LogError(_T("SymCleanup"));
+        wxDbgHelpDLL::LogError(wxT("SymCleanup"));
     }
-#endif
 }
 
-void wxStackWalker::WalkFrom(const _EXCEPTION_POINTERS *ep, size_t skip)
+void wxStackWalker::WalkFrom(const _EXCEPTION_POINTERS *ep, size_t skip, size_t maxDepth)
 {
-    WalkFrom(ep->ContextRecord, skip);
+    WalkFrom(ep->ContextRecord, skip, maxDepth);
 }
 
-void wxStackWalker::WalkFromException()
-{
-    // wxGlobalSEInformation is unavailable if wxUSE_ON_FATAL_EXCEPTION==0
 #if wxUSE_ON_FATAL_EXCEPTION
+
+void wxStackWalker::WalkFromException(size_t maxDepth)
+{
     extern EXCEPTION_POINTERS *wxGlobalSEInformation;
 
     wxCHECK_RET( wxGlobalSEInformation,
-                 _T("wxStackWalker::WalkFromException() can only be called from wxApp::OnFatalException()") );
+                 wxT("wxStackWalker::WalkFromException() can only be called from wxApp::OnFatalException()") );
 
     // don't skip any frames, the first one is where we crashed
-    WalkFrom(wxGlobalSEInformation, 0);
-#endif // wxUSE_ON_FATAL_EXCEPTION
+    WalkFrom(wxGlobalSEInformation, 0, maxDepth);
 }
 
+#endif // wxUSE_ON_FATAL_EXCEPTION
+
 void wxStackWalker::Walk(size_t skip, size_t WXUNUSED(maxDepth))
 {
     // to get a CONTEXT for the current location, simply force an exception and
     // get EXCEPTION_POINTERS from it
     //
     // note:
-    //  1. we additionally skip RaiseException() and WalkFromException() frames
+    //  1. we additionally skip RaiseException() and WalkFrom() frames
     //  2. explicit cast to EXCEPTION_POINTERS is needed with VC7.1 even if it
     //     shouldn't have been according to the docs
     __try
@@ -346,7 +342,8 @@ void wxStackWalker::Walk(size_t skip, size_t WXUNUSED(maxDepth))
     __except( WalkFrom((EXCEPTION_POINTERS *)GetExceptionInformation(),
                        skip + 2), EXCEPTION_CONTINUE_EXECUTION )
     {
-        // never executed because of WalkFromException() return value
+        // never executed because the above expression always evaluates to
+        // EXCEPTION_CONTINUE_EXECUTION
     }
 }
 
@@ -390,19 +387,24 @@ void wxStackFrame::OnGetParam()
 // ----------------------------------------------------------------------------
 
 void
-wxStackWalker::WalkFrom(const CONTEXT * WXUNUSED(pCtx), size_t WXUNUSED(skip))
+wxStackWalker::WalkFrom(const CONTEXT * WXUNUSED(pCtx),
+                        size_t WXUNUSED(skip),
+                        size_t WXUNUSED(maxDepth))
 {
 }
 
 void
 wxStackWalker::WalkFrom(const _EXCEPTION_POINTERS * WXUNUSED(ep),
-                        size_t WXUNUSED(skip))
+                        size_t WXUNUSED(skip),
+                        size_t WXUNUSED(maxDepth))
 {
 }
 
-void wxStackWalker::WalkFromException()
+#if wxUSE_ON_FATAL_EXCEPTION
+void wxStackWalker::WalkFromException(size_t WXUNUSED(maxDepth))
 {
 }
+#endif // wxUSE_ON_FATAL_EXCEPTION
 
 void wxStackWalker::Walk(size_t WXUNUSED(skip), size_t WXUNUSED(maxDepth))
 {
