@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     2006-10-01
-// RCS-ID:      $Id: richtextformatdlg.cpp 45498 2007-04-16 13:03:05Z VZ $
+// RCS-ID:      $Id: richtextformatdlg.cpp 62009 2009-09-22 10:03:04Z JS $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -65,9 +65,9 @@
 #endif
 
 #if 0 // def __WXMAC__
-#define wxRICHTEXT_USE_TOOLBOOK true
+#define wxRICHTEXT_USE_TOOLBOOK 1
 #else
-#define wxRICHTEXT_USE_TOOLBOOK false
+#define wxRICHTEXT_USE_TOOLBOOK 0
 #endif
 
 bool wxRichTextFormattingDialog::sm_showToolTips = false;
@@ -130,7 +130,7 @@ bool wxRichTextFormattingDialog::ApplyStyle(wxRichTextCtrl* ctrl, const wxRichTe
 }
 
 /// Set the attributes and optionally update the display
-bool wxRichTextFormattingDialog::SetStyle(const wxTextAttrEx& style, bool update)
+bool wxRichTextFormattingDialog::SetStyle(const wxTextAttr& style, bool update)
 {
     m_attributes = style;
     if (update)
@@ -335,20 +335,20 @@ int wxRichTextFormattingDialogFactory::GetPageIdCount() const
 /// Set the sheet style, called at the start of wxRichTextFormattingDialog::Create
 bool wxRichTextFormattingDialogFactory::SetSheetStyle(wxRichTextFormattingDialog* dialog)
 {
-    bool useToolBook = wxRICHTEXT_USE_TOOLBOOK;
-    if (useToolBook)
-    {
-        int sheetStyle = wxPROPSHEET_SHRINKTOFIT;
+#if wxRICHTEXT_USE_TOOLBOOK
+    int sheetStyle = wxPROPSHEET_SHRINKTOFIT;
 #ifdef __WXMAC__
-        sheetStyle |= wxPROPSHEET_BUTTONTOOLBOOK;
+    sheetStyle |= wxPROPSHEET_BUTTONTOOLBOOK;
 #else
-        sheetStyle |= wxPROPSHEET_TOOLBOOK;
+    sheetStyle |= wxPROPSHEET_TOOLBOOK;
 #endif
 
-        dialog->SetSheetStyle(sheetStyle);
-        dialog->SetSheetInnerBorder(0);
-        dialog->SetSheetOuterBorder(0);
-    }
+    dialog->SetSheetStyle(sheetStyle);
+    dialog->SetSheetInnerBorder(0);
+    dialog->SetSheetOuterBorder(0);
+#else
+    wxUnusedVar(dialog);
+#endif // wxRICHTEXT_USE_TOOLBOOK
 
     return true;
 }
@@ -356,17 +356,16 @@ bool wxRichTextFormattingDialogFactory::SetSheetStyle(wxRichTextFormattingDialog
 /// Create the main dialog buttons
 bool wxRichTextFormattingDialogFactory::CreateButtons(wxRichTextFormattingDialog* dialog)
 {
-    bool useToolBook = wxRICHTEXT_USE_TOOLBOOK;
-
-    // If using a toolbook, also follow Mac style and don't create buttons
     int flags = wxOK|wxCANCEL;
 #ifndef __WXWINCE__
     if (dialog->GetWindowStyleFlag() & wxRICHTEXT_FORMAT_HELP_BUTTON)
         flags |= wxHELP;
 #endif
 
-    if (!useToolBook)
-        dialog->CreateButtons(flags);
+    // If using a toolbook, also follow Mac style and don't create buttons
+#if !wxRICHTEXT_USE_TOOLBOOK
+    dialog->CreateButtons(flags);
+#endif
 
     return true;
 }
@@ -394,6 +393,17 @@ BEGIN_EVENT_TABLE(wxRichTextFontPreviewCtrl, wxWindow)
     EVT_PAINT(wxRichTextFontPreviewCtrl::OnPaint)
 END_EVENT_TABLE()
 
+wxRichTextFontPreviewCtrl::wxRichTextFontPreviewCtrl(wxWindow *parent, wxWindowID id, const wxPoint& pos, const wxSize& sz, long style)
+{
+    if ((style & wxBORDER_MASK) == wxBORDER_DEFAULT)
+        style |= wxBORDER_THEME;
+
+    wxWindow::Create(parent, id, pos, sz, style);
+
+    SetBackgroundColour(*wxWHITE);
+    m_textEffects = 0;
+}
+
 void wxRichTextFontPreviewCtrl::OnPaint(wxPaintEvent& WXUNUSED(event))
 {
     wxPaintDC dc(this);
@@ -401,11 +411,17 @@ void wxRichTextFontPreviewCtrl::OnPaint(wxPaintEvent& WXUNUSED(event))
     wxSize size = GetSize();
     wxFont font = GetFont();
 
+    if ((GetTextEffects() & wxTEXT_ATTR_EFFECT_SUPERSCRIPT) || (GetTextEffects() & wxTEXT_ATTR_EFFECT_SUBSCRIPT))
+    {
+        double size = static_cast<double>(font.GetPointSize()) / wxSCRIPT_MUL_FACTOR;
+        font.SetPointSize( static_cast<int>(size) );
+    }
+
     if ( font.Ok() )
     {
         dc.SetFont(font);
         // Calculate vertical and horizontal centre
-        long w = 0, h = 0;
+        wxCoord w = 0, h = 0;
 
         wxString text(_("ABCDEFGabcdefg12345"));
         if (GetTextEffects() & wxTEXT_ATTR_EFFECT_CAPITALS)
@@ -415,16 +431,21 @@ void wxRichTextFontPreviewCtrl::OnPaint(wxPaintEvent& WXUNUSED(event))
         int cx = wxMax(2, (size.x/2) - (w/2));
         int cy = wxMax(2, (size.y/2) - (h/2));
 
+        if ( GetTextEffects() & wxTEXT_ATTR_EFFECT_SUPERSCRIPT )
+            cy -= h/2;
+        if ( GetTextEffects() & wxTEXT_ATTR_EFFECT_SUBSCRIPT )
+            cy += h/2;
+
         dc.SetTextForeground(GetForegroundColour());
         dc.SetClippingRegion(2, 2, size.x-4, size.y-4);
         dc.DrawText(text, cx, cy);
-        
+
         if (GetTextEffects() & wxTEXT_ATTR_EFFECT_STRIKETHROUGH)
         {
             dc.SetPen(wxPen(GetForegroundColour(), 1));
             dc.DrawLine(cx, (int) (cy + h/2 + 0.5), cx + w, (int) (cy + h/2 + 0.5));
         }
-        
+
         dc.DestroyClippingRegion();
     }
 }
@@ -441,7 +462,7 @@ wxRichTextFormattingDialog* wxRichTextFormattingDialog::GetDialog(wxWindow* win)
 
 
 // Helper for pages to get the attributes
-wxTextAttrEx* wxRichTextFormattingDialog::GetDialogAttributes(wxWindow* win)
+wxTextAttr* wxRichTextFormattingDialog::GetDialogAttributes(wxWindow* win)
 {
     wxRichTextFormattingDialog* dialog = GetDialog(win);
     if (dialog)
@@ -470,11 +491,14 @@ END_EVENT_TABLE()
 
 IMPLEMENT_CLASS(wxRichTextColourSwatchCtrl, wxControl)
 
-wxRichTextColourSwatchCtrl::wxRichTextColourSwatchCtrl(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style):
-    wxControl(parent, id, pos, size, style)
+wxRichTextColourSwatchCtrl::wxRichTextColourSwatchCtrl(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style)
 {
+    if ((style & wxBORDER_MASK) == wxBORDER_DEFAULT)
+        style |= wxBORDER_THEME;
+
+    wxControl::Create(parent, id, pos, size, style);
+
     SetColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
-    SetBackgroundStyle(wxBG_STYLE_COLOUR);
 }
 
 wxRichTextColourSwatchCtrl::~wxRichTextColourSwatchCtrl()
@@ -496,7 +520,7 @@ void wxRichTextColourSwatchCtrl::OnMouseEvent(wxMouseEvent& event)
         wxColourDialog *dialog = new wxColourDialog(parent, &data);
         // Crashes on wxMac (no m_peer)
 #ifndef __WXMAC__
-        dialog->SetTitle(_("Background colour"));
+        dialog->SetTitle(_("Colour"));
 #endif
         if (dialog->ShowModal() == wxID_OK)
         {
@@ -535,6 +559,9 @@ wxRichTextFontListBox::wxRichTextFontListBox(wxWindow* parent, wxWindowID id, co
 bool wxRichTextFontListBox::Create(wxWindow* parent, wxWindowID id, const wxPoint& pos,
         const wxSize& size, long style)
 {
+    if ((style & wxBORDER_MASK) == wxBORDER_DEFAULT)
+        style |= wxBORDER_THEME;
+
     return wxHtmlListBox::Create(parent, id, pos, size, style);
 }
 
