@@ -3,14 +3,11 @@
 // Purpose:     XML resource compiler
 // Author:      Vaclav Slavik, Eduardo Marques <edrdo@netcabo.pt>
 // Created:     2000/03/05
-// RCS-ID:      $Id: wxrc.cpp 63231 2010-01-23 15:25:18Z VZ $
+// RCS-ID:      $Id: wxrc.cpp 64801 2010-07-03 14:24:27Z VS $
 // Copyright:   (c) 2000 Vaclav Slavik
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
-// To fix compilation when using Sun CC we must define wxT() for this file.
-#define wxNEEDS_T
-
 // For compilers that support precompilation, includes "wx/wx.h".
 #include "wx/wxprec.h"
 
@@ -22,6 +19,7 @@
 #ifndef WX_PRECOMP
     #include "wx/app.h"
     #include "wx/log.h"
+    #include "wx/wxcrtvararg.h"
 #endif
 
 #include "wx/cmdline.h"
@@ -32,6 +30,7 @@
 #include "wx/utils.h"
 #include "wx/hashset.h"
 #include "wx/mimetype.h"
+#include "wx/vector.h"
 
 WX_DECLARE_HASH_SET(wxString, wxStringHash, wxStringEqual, StringSet);
 
@@ -67,8 +66,8 @@ private:
         while (node)
         {
             if (node->GetName() == wxT("object")
-                && node->GetPropVal(wxT("class"),&classValue)
-                && node->GetPropVal(wxT("name"),&nameValue))
+                && node->GetAttribute(wxT("class"),&classValue)
+                && node->GetAttribute(wxT("name"),&nameValue))
             {
                 m_wdata.Add(XRCWidgetData(nameValue,classValue));
             }
@@ -113,7 +112,7 @@ public:
         return m_wdata;
     }
 
-    bool IsRealClass(const wxString& name)
+    bool CanBeUsedWithXRCCTRL(const wxString& name)
     {
         if (name == wxT("tool") ||
             name == wxT("data") ||
@@ -137,10 +136,10 @@ public:
         file.Write(wxT("class ") + m_className + wxT(" : public ") + m_parentClassName
                    + wxT(" {\nprotected:\n"));
         size_t i;
-        for(i=0;i<m_wdata.Count();++i)
+        for(i=0;i<m_wdata.GetCount();++i)
         {
             const XRCWidgetData& w = m_wdata.Item(i);
-            if( !IsRealClass(w.GetClass()) ) continue;
+            if( !CanBeUsedWithXRCCTRL(w.GetClass()) ) continue;
             if( w.GetName().Length() == 0 ) continue;
             file.Write(
                 wxT(" ") + w.GetClass() + wxT("* ") + w.GetName()
@@ -152,10 +151,10 @@ public:
                    +  wxT("\"), wxT(\"")
                    +  m_parentClassName
                    +  wxT("\"));\n"));
-        for(i=0;i<m_wdata.Count();++i)
+        for(i=0;i<m_wdata.GetCount();++i)
         {
             const XRCWidgetData& w = m_wdata.Item(i);
-            if( !IsRealClass(w.GetClass()) ) continue;
+            if( !CanBeUsedWithXRCCTRL(w.GetClass()) ) continue;
             if( w.GetName().Length() == 0 ) continue;
             file.Write( wxT("  ")
                         + w.GetName()
@@ -204,6 +203,22 @@ public:
 WX_DECLARE_OBJARRAY(XRCWndClassData,ArrayOfXRCWndClassData);
 WX_DEFINE_OBJARRAY(ArrayOfXRCWndClassData)
 
+struct ExtractedString
+{
+    ExtractedString() : lineNo(-1) {}
+    ExtractedString(const wxString& str_,
+                    const wxString& filename_, int lineNo_)
+        : str(str_), filename(filename_), lineNo(lineNo_)
+    {}
+
+    wxString str;
+
+    wxString filename;
+    int      lineNo;
+};
+
+typedef wxVector<ExtractedString> ExtractedStrings;
+
 
 class XmlResApp : public wxAppConsole
 {
@@ -225,8 +240,8 @@ private:
     void MakePackagePython(const wxArrayString& flist);
 
     void OutputGettext();
-    wxArrayString FindStrings();
-    wxArrayString FindStrings(wxXmlNode *node);
+    ExtractedStrings FindStrings();
+    ExtractedStrings FindStrings(const wxString& filename, wxXmlNode *node);
 
     bool flagVerbose, flagCPP, flagPython, flagGettext;
     wxString parOutput, parFuncname, parOutputPath;
@@ -244,23 +259,22 @@ int XmlResApp::OnRun()
 {
     static const wxCmdLineEntryDesc cmdLineDesc[] =
     {
-        { wxCMD_LINE_SWITCH, wxT("h"), wxT("help"),  wxT("show help message"),
-              wxCMD_LINE_VAL_NONE, wxCMD_LINE_OPTION_HELP },
-        { wxCMD_LINE_SWITCH, wxT("v"), wxT("verbose"), wxT("be verbose"), (wxCmdLineParamType)0, 0 },
-        { wxCMD_LINE_SWITCH, wxT("e"), wxT("extra-cpp-code"),  wxT("output C++ header file with XRC derived classes"), (wxCmdLineParamType)0, 0 },
-        { wxCMD_LINE_SWITCH, wxT("c"), wxT("cpp-code"),  wxT("output C++ source rather than .rsc file"), (wxCmdLineParamType)0, 0 },
-        { wxCMD_LINE_SWITCH, wxT("p"), wxT("python-code"),  wxT("output wxPython source rather than .rsc file"), (wxCmdLineParamType)0, 0 },
-        { wxCMD_LINE_SWITCH, wxT("g"), wxT("gettext"),  wxT("output list of translatable strings (to stdout or file if -o used)"), (wxCmdLineParamType)0, 0 },
-        { wxCMD_LINE_OPTION, wxT("n"), wxT("function"),  wxT("C++/Python function name (with -c or -p) [InitXmlResource]"), (wxCmdLineParamType)0, 0 },
-        { wxCMD_LINE_OPTION, wxT("o"), wxT("output"),  wxT("output file [resource.xrs/cpp]"), (wxCmdLineParamType)0, 0 },
+        { wxCMD_LINE_SWITCH, "h", "help",  "show help message", wxCMD_LINE_VAL_NONE, wxCMD_LINE_OPTION_HELP },
+        { wxCMD_LINE_SWITCH, "v", "verbose", "be verbose" },
+        { wxCMD_LINE_SWITCH, "e", "extra-cpp-code",  "output C++ header file with XRC derived classes" },
+        { wxCMD_LINE_SWITCH, "c", "cpp-code",  "output C++ source rather than .rsc file" },
+        { wxCMD_LINE_SWITCH, "p", "python-code",  "output wxPython source rather than .rsc file" },
+        { wxCMD_LINE_SWITCH, "g", "gettext",  "output list of translatable strings (to stdout or file if -o used)" },
+        { wxCMD_LINE_OPTION, "n", "function",  "C++/Python function name (with -c or -p) [InitXmlResource]" },
+        { wxCMD_LINE_OPTION, "o", "output",  "output file [resource.xrs/cpp]" },
 #if 0 // not yet implemented
-        { wxCMD_LINE_OPTION, wxT("l"), wxT("list-of-handlers"),  wxT("output list of necessary handlers to this file"), (wxCmdLineParamType)0, 0 },
+        { wxCMD_LINE_OPTION, "l", "list-of-handlers",  "output list of necessary handlers to this file" },
 #endif
-        { wxCMD_LINE_PARAM,  NULL, NULL, wxT("input file(s)"),
+        { wxCMD_LINE_PARAM,  NULL, NULL, "input file(s)",
               wxCMD_LINE_VAL_STRING,
               wxCMD_LINE_PARAM_MULTIPLE | wxCMD_LINE_OPTION_MANDATORY },
 
-        { wxCMD_LINE_NONE, NULL, NULL, NULL, (wxCmdLineParamType)0, 0 }
+        wxCMD_LINE_DESC_END
     };
 
     wxCmdLineParser parser(cmdLineDesc, argc, argv);
@@ -287,14 +301,14 @@ int XmlResApp::OnRun()
 
 void XmlResApp::ParseParams(const wxCmdLineParser& cmdline)
 {
-    flagGettext = cmdline.Found(wxT("g"));
-    flagVerbose = cmdline.Found(wxT("v"));
-    flagCPP = cmdline.Found(wxT("c"));
-    flagPython = cmdline.Found(wxT("p"));
-    flagH = flagCPP && cmdline.Found(wxT("e"));
+    flagGettext = cmdline.Found("g");
+    flagVerbose = cmdline.Found("v");
+    flagCPP = cmdline.Found("c");
+    flagPython = cmdline.Found("p");
+    flagH = flagCPP && cmdline.Found("e");
 
 
-    if (!cmdline.Found(wxT("o"), &parOutput))
+    if (!cmdline.Found("o", &parOutput))
     {
         if (flagGettext)
             parOutput = wxEmptyString;
@@ -317,7 +331,7 @@ void XmlResApp::ParseParams(const wxCmdLineParser& cmdline)
     }
     if (!parOutputPath) parOutputPath = wxT(".");
 
-    if (!cmdline.Found(wxT("n"), &parFuncname))
+    if (!cmdline.Found("n", &parFuncname))
         parFuncname = wxT("InitXmlResource");
 
     for (size_t i = 0; i < cmdline.GetParamCount(); i++)
@@ -388,7 +402,7 @@ wxArrayString XmlResApp::PrepareTempFiles()
 {
     wxArrayString flist;
 
-    for (size_t i = 0; i < parFiles.Count(); i++)
+    for (size_t i = 0; i < parFiles.GetCount(); i++)
     {
         if (flagVerbose)
             wxPrintf(wxT("processing ") + parFiles[i] +  wxT("...\n"));
@@ -403,7 +417,7 @@ wxArrayString XmlResApp::PrepareTempFiles()
         }
 
         wxString name, ext, path;
-        wxSplitPath(parFiles[i], &path, &name, &ext);
+        wxFileName::SplitPath(parFiles[i], &path, &name, &ext);
 
         FindFilesInXML(doc.GetRoot(), flist, path);
         if (flagH)
@@ -412,8 +426,8 @@ wxArrayString XmlResApp::PrepareTempFiles()
                 wxString classValue,nameValue;
                 while(node){
                     if(node->GetName() == wxT("object")
-                     && node->GetPropVal(wxT("class"),&classValue)
-                     && node->GetPropVal(wxT("name"),&nameValue)){
+                     && node->GetAttribute(wxT("class"),&classValue)
+                     && node->GetAttribute(wxT("name"),&nameValue)){
 
                       aXRCWndClassData.Add(
                         XRCWndClassData(nameValue,classValue,node)
@@ -447,7 +461,7 @@ static bool NodeContainsFilename(wxXmlNode *node)
    // wxBitmapButton:
    wxXmlNode *parent = node->GetParent();
    if (parent != NULL &&
-       parent->GetPropVal(wxT("class"), wxT("")) == wxT("wxBitmapButton") &&
+       parent->GetAttribute(wxT("class"), wxT("")) == wxT("wxBitmapButton") &&
        (name == wxT("focus") ||
         name == wxT("disabled") ||
         name == wxT("hover") ||
@@ -457,15 +471,17 @@ static bool NodeContainsFilename(wxXmlNode *node)
    // wxBitmap or wxIcon toplevel resources:
    if ( name == wxT("object") )
    {
-       wxString klass = node->GetPropVal(wxT("class"), wxEmptyString);
-       if (klass == wxT("wxBitmap") || klass == wxT("wxIcon"))
+       wxString klass = node->GetAttribute(wxT("class"), wxEmptyString);
+       if (klass == wxT("wxBitmap") ||
+               klass == wxT("wxIcon") ||
+                klass == wxT("data") )
            return true;
    }
 
    // URLs in wxHtmlWindow:
    if ( name == wxT("url") &&
         parent != NULL &&
-        parent->GetPropVal(wxT("class"), wxT("")) == wxT("wxHtmlWindow") )
+        parent->GetAttribute(wxT("class"), wxT("")) == wxT("wxHtmlWindow") )
    {
        // FIXME: this is wrong for e.g. http:// URLs
        return true;
@@ -522,7 +538,7 @@ void XmlResApp::FindFilesInXML(wxXmlNode *node, wxArrayString& flist, const wxSt
 
 void XmlResApp::DeleteTempFiles(const wxArrayString& flist)
 {
-    for (size_t i = 0; i < flist.Count(); i++)
+    for (size_t i = 0; i < flist.GetCount(); i++)
         wxRemoveFile(parOutputPath + wxFILE_SEP_PATH + flist[i]);
 }
 
@@ -532,7 +548,7 @@ void XmlResApp::MakePackageZIP(const wxArrayString& flist)
 {
     wxString files;
 
-    for (size_t i = 0; i < flist.Count(); i++)
+    for (size_t i = 0; i < flist.GetCount(); i++)
         files += flist[i] + wxT(" ");
     files.RemoveLast();
 
@@ -566,10 +582,11 @@ static wxString FileToCppArray(wxString filename, int num)
     wxASSERT_MSG( offset >= 0 , wxT("Invalid file length") );
 
     const size_t lng = wx_truncate_cast(size_t, offset);
-    wxASSERT_MSG( !(lng - offset), wxT("Huge file not supported") );
+    wxASSERT_MSG( static_cast<wxFileOffset>(lng) == offset,
+                  wxT("Huge file not supported") );
 
     snum.Printf(wxT("%i"), num);
-    output.Printf(wxT("static size_t xml_res_size_") + snum + wxT(" = %i;\n"), lng);
+    output.Printf(wxT("static size_t xml_res_size_") + snum + wxT(" = %zu;\n"), lng);
     output += wxT("static unsigned char xml_res_file_") + snum + wxT("[] = {\n");
     // we cannot use string literals because MSVC is dumb wannabe compiler
     // with arbitrary limitation to 2048 strings :(
@@ -601,56 +618,56 @@ static wxString FileToCppArray(wxString filename, int num)
 void XmlResApp::MakePackageCPP(const wxArrayString& flist)
 {
     wxFFile file(parOutput, wxT("wt"));
-    size_t i;
+    unsigned i;
 
     if (flagVerbose)
         wxPrintf(wxT("creating C++ source file ") + parOutput +  wxT("...\n"));
 
-    file.Write(wxT("")
-wxT("//\n")
-wxT("// This file was automatically generated by wxrc, do not edit by hand.\n")
-wxT("//\n\n")
-wxT("#include <wx/wxprec.h>\n")
-wxT("\n")
-wxT("#ifdef __BORLANDC__\n")
-wxT("    #pragma hdrstop\n")
-wxT("#endif\n")
-wxT("\n")
-wxT("")
-wxT("#include <wx/filesys.h>\n")
-wxT("#include <wx/fs_mem.h>\n")
-wxT("#include <wx/xrc/xmlres.h>\n")
-wxT("#include <wx/xrc/xh_all.h>\n")
-wxT("\n")
-wxT("#if wxCHECK_VERSION(2,8,5) && wxABI_VERSION >= 20805\n")
-wxT("    #define XRC_ADD_FILE(name, data, size, mime) \\\n")
-wxT("        wxMemoryFSHandler::AddFileWithMimeType(name, data, size, mime)\n")
-wxT("#else\n")
-wxT("    #define XRC_ADD_FILE(name, data, size, mime) \\\n")
-wxT("        wxMemoryFSHandler::AddFile(name, data, size)\n")
-wxT("#endif\n")
-wxT("\n"));
+    file.Write(""
+"//\n"
+"// This file was automatically generated by wxrc, do not edit by hand.\n"
+"//\n\n"
+"#include <wx/wxprec.h>\n"
+"\n"
+"#ifdef __BORLANDC__\n"
+"    #pragma hdrstop\n"
+"#endif\n"
+"\n"
+""
+"#include <wx/filesys.h>\n"
+"#include <wx/fs_mem.h>\n"
+"#include <wx/xrc/xmlres.h>\n"
+"#include <wx/xrc/xh_all.h>\n"
+"\n"
+"#if wxCHECK_VERSION(2,8,5) && wxABI_VERSION >= 20805\n"
+"    #define XRC_ADD_FILE(name, data, size, mime) \\\n"
+"        wxMemoryFSHandler::AddFileWithMimeType(name, data, size, mime)\n"
+"#else\n"
+"    #define XRC_ADD_FILE(name, data, size, mime) \\\n"
+"        wxMemoryFSHandler::AddFile(name, data, size)\n"
+"#endif\n"
+"\n");
 
-    for (i = 0; i < flist.Count(); i++)
+    for (i = 0; i < flist.GetCount(); i++)
         file.Write(
               FileToCppArray(parOutputPath + wxFILE_SEP_PATH + flist[i], i));
 
-    file.Write(wxT("")
-wxT("void ") + parFuncname + wxT("()\n")
-wxT("{\n")
-wxT("\n")
-wxT("    // Check for memory FS. If not present, load the handler:\n")
-wxT("    {\n")
-wxT("        wxMemoryFSHandler::AddFile(wxT(\"XRC_resource/dummy_file\"), wxT(\"dummy one\"));\n")
-wxT("        wxFileSystem fsys;\n")
-wxT("        wxFSFile *f = fsys.OpenFile(wxT(\"memory:XRC_resource/dummy_file\"));\n")
-wxT("        wxMemoryFSHandler::RemoveFile(wxT(\"XRC_resource/dummy_file\"));\n")
-wxT("        if (f) delete f;\n")
-wxT("        else wxFileSystem::AddHandler(new wxMemoryFSHandler);\n")
-wxT("    }\n")
-wxT("\n"));
+    file.Write(""
+"void " + parFuncname + "()\n"
+"{\n"
+"\n"
+"    // Check for memory FS. If not present, load the handler:\n"
+"    {\n"
+"        wxMemoryFSHandler::AddFile(wxT(\"XRC_resource/dummy_file\"), wxT(\"dummy one\"));\n"
+"        wxFileSystem fsys;\n"
+"        wxFSFile *f = fsys.OpenFile(wxT(\"memory:XRC_resource/dummy_file\"));\n"
+"        wxMemoryFSHandler::RemoveFile(wxT(\"XRC_resource/dummy_file\"));\n"
+"        if (f) delete f;\n"
+"        else wxFileSystem::AddHandler(new wxMemoryFSHandler);\n"
+"    }\n"
+"\n");
 
-    for (i = 0; i < flist.Count(); i++)
+    for (i = 0; i < flist.GetCount(); i++)
     {
         wxString s;
 
@@ -670,19 +687,19 @@ wxT("\n"));
         }
 #endif // wxUSE_MIMETYPE
 
-        s.Printf(wxT("    XRC_ADD_FILE(wxT(\"XRC_resource/") + flist[i] +
-                 wxT("\"), xml_res_file_%i, xml_res_size_%i, wxT(\"%s\"));\n"),
+        s.Printf("    XRC_ADD_FILE(wxT(\"XRC_resource/" + flist[i] +
+                 "\"), xml_res_file_%u, xml_res_size_%u, wxT(\"%s\"));\n",
                  i, i, mime.c_str());
         file.Write(s);
     }
 
-    for (i = 0; i < parFiles.Count(); i++)
+    for (i = 0; i < parFiles.GetCount(); i++)
     {
-        file.Write(wxT("    wxXmlResource::Get()->Load(wxT(\"memory:XRC_resource/") +
-                   GetInternalFileName(parFiles[i], flist) + wxT("\"));\n"));
+        file.Write("    wxXmlResource::Get()->Load(wxT(\"memory:XRC_resource/" +
+                   GetInternalFileName(parFiles[i], flist) + "\"));\n");
     }
 
-    file.Write(wxT("}\n"));
+    file.Write("}\n");
 
 
 }
@@ -694,19 +711,19 @@ void XmlResApp::GenCPPHeader()
 
     wxFFile file(heaFileName, wxT("wt"));
     file.Write(
-wxT("//\n")
-wxT("// This file was automatically generated by wxrc, do not edit by hand.\n")
-wxT("//\n\n")
-wxT("#ifndef __")  + fileSpec + wxT("_h__\n")
-wxT("#define __")  + fileSpec + wxT("_h__\n")
+"//\n"
+"// This file was automatically generated by wxrc, do not edit by hand.\n"
+"//\n\n"
+"#ifndef __"  + fileSpec + "_h__\n"
+"#define __"  + fileSpec + "_h__\n"
 );
-    for(size_t i=0;i<aXRCWndClassData.Count();++i){
+    for(size_t i=0;i<aXRCWndClassData.GetCount();++i){
                 aXRCWndClassData.Item(i).GenerateHeaderCode(file);
     }
     file.Write(
-                wxT("\nvoid \n")
+                "\nvoid \n"
                 + parFuncname
-                + wxT("();\n#endif\n"));
+                + "();\n#endif\n");
 }
 
 static wxString FileToPythonArray(wxString filename, int num)
@@ -719,10 +736,11 @@ static wxString FileToPythonArray(wxString filename, int num)
     wxASSERT_MSG( offset >= 0 , wxT("Invalid file length") );
 
     const size_t lng = wx_truncate_cast(size_t, offset);
-    wxASSERT_MSG( !(offset - lng), wxT("Huge file not supported") );
+    wxASSERT_MSG( static_cast<wxFileOffset>(lng) == offset,
+                  wxT("Huge file not supported") );
 
     snum.Printf(wxT("%i"), num);
-    output = wxT("    xml_res_file_") + snum + wxT(" = '''\\\n");
+    output = "    xml_res_file_" + snum + " = '''\\\n";
 
     unsigned char *buffer = new unsigned char[lng];
     file.Read(buffer, lng);
@@ -761,62 +779,62 @@ static wxString FileToPythonArray(wxString filename, int num)
 void XmlResApp::MakePackagePython(const wxArrayString& flist)
 {
     wxFFile file(parOutput, wxT("wt"));
-    size_t i;
+    unsigned i;
 
     if (flagVerbose)
         wxPrintf(wxT("creating Python source file ") + parOutput +  wxT("...\n"));
 
     file.Write(
-       wxT("#\n")
-       wxT("# This file was automatically generated by wxrc, do not edit by hand.\n")
-       wxT("#\n\n")
-       wxT("import wx\n")
-       wxT("import wx.xrc\n\n")
+       "#\n"
+       "# This file was automatically generated by wxrc, do not edit by hand.\n"
+       "#\n\n"
+       "import wx\n"
+       "import wx.xrc\n\n"
     );
 
 
-    file.Write(wxT("def ") + parFuncname + wxT("():\n"));
+    file.Write("def " + parFuncname + "():\n");
 
-    for (i = 0; i < flist.Count(); i++)
+    for (i = 0; i < flist.GetCount(); i++)
         file.Write(
           FileToPythonArray(parOutputPath + wxFILE_SEP_PATH + flist[i], i));
 
     file.Write(
-        wxT("    # check if the memory filesystem handler has been loaded yet, and load it if not\n")
-        wxT("    wx.MemoryFSHandler.AddFile('XRC_resource/dummy_file', 'dummy value')\n")
-        wxT("    fsys = wx.FileSystem()\n")
-        wxT("    f = fsys.OpenFile('memory:XRC_resource/dummy_file')\n")
-        wxT("    wx.MemoryFSHandler.RemoveFile('XRC_resource/dummy_file')\n")
-        wxT("    if f is not None:\n")
-        wxT("        f.Destroy()\n")
-        wxT("    else:\n")
-        wxT("        wx.FileSystem.AddHandler(wx.MemoryFSHandler())\n")
-        wxT("\n")
-        wxT("    # load all the strings as memory files and load into XmlRes\n")
+        "    # check if the memory filesystem handler has been loaded yet, and load it if not\n"
+        "    wx.MemoryFSHandler.AddFile('XRC_resource/dummy_file', 'dummy value')\n"
+        "    fsys = wx.FileSystem()\n"
+        "    f = fsys.OpenFile('memory:XRC_resource/dummy_file')\n"
+        "    wx.MemoryFSHandler.RemoveFile('XRC_resource/dummy_file')\n"
+        "    if f is not None:\n"
+        "        f.Destroy()\n"
+        "    else:\n"
+        "        wx.FileSystem.AddHandler(wx.MemoryFSHandler())\n"
+        "\n"
+        "    # load all the strings as memory files and load into XmlRes\n"
         );
 
 
-    for (i = 0; i < flist.Count(); i++)
+    for (i = 0; i < flist.GetCount(); i++)
     {
         wxString s;
-        s.Printf(wxT("    wx.MemoryFSHandler.AddFile('XRC_resource/") + flist[i] +
-                 wxT("', xml_res_file_%i)\n"), i);
+        s.Printf("    wx.MemoryFSHandler.AddFile('XRC_resource/" + flist[i] +
+                 "', xml_res_file_%u)\n", i);
         file.Write(s);
     }
-    for (i = 0; i < parFiles.Count(); i++)
+    for (i = 0; i < parFiles.GetCount(); i++)
     {
-        file.Write(wxT("    wx.xrc.XmlResource.Get().Load('memory:XRC_resource/") +
-                   GetInternalFileName(parFiles[i], flist) + wxT("')\n"));
+        file.Write("    wx.xrc.XmlResource.Get().Load('memory:XRC_resource/" +
+                   GetInternalFileName(parFiles[i], flist) + "')\n");
     }
 
-    file.Write(wxT("\n"));
+    file.Write("\n");
 }
 
 
 
 void XmlResApp::OutputGettext()
 {
-    wxArrayString str = FindStrings();
+    ExtractedStrings str = FindStrings();
 
     wxFFile fout;
     if (parOutput.empty())
@@ -824,19 +842,28 @@ void XmlResApp::OutputGettext()
     else
         fout.Open(parOutput, wxT("wt"));
 
-    for (size_t i = 0; i < str.GetCount(); i++)
-        fout.Write(wxT("_(\"") + str[i] + wxT("\");\n"));
+    for (ExtractedStrings::const_iterator i = str.begin(); i != str.end(); ++i)
+    {
+        const wxFileName filename(i->filename);
+
+        wxString s;
+        s.Printf("#line %d \"%s\"\n",
+                 i->lineNo, filename.GetFullPath(wxPATH_UNIX));
+
+        fout.Write(s);
+        fout.Write("_(\"" + i->str + "\");\n");
+    }
 
     if (!parOutput) fout.Detach();
 }
 
 
 
-wxArrayString XmlResApp::FindStrings()
+ExtractedStrings XmlResApp::FindStrings()
 {
-    wxArrayString arr, a2;
+    ExtractedStrings arr, a2;
 
-    for (size_t i = 0; i < parFiles.Count(); i++)
+    for (size_t i = 0; i < parFiles.GetCount(); i++)
     {
         if (flagVerbose)
             wxPrintf(wxT("processing ") + parFiles[i] +  wxT("...\n"));
@@ -848,7 +875,8 @@ wxArrayString XmlResApp::FindStrings()
             retCode = 1;
             continue;
         }
-        a2 = FindStrings(doc.GetRoot());
+        a2 = FindStrings(parFiles[i], doc.GetRoot());
+
         WX_APPEND_ARRAY(arr, a2);
     }
 
@@ -895,9 +923,10 @@ static wxString ConvertText(const wxString& str)
 }
 
 
-wxArrayString XmlResApp::FindStrings(wxXmlNode *node)
+ExtractedStrings
+XmlResApp::FindStrings(const wxString& filename, wxXmlNode *node)
 {
-    wxArrayString arr;
+    ExtractedStrings arr;
 
     wxXmlNode *n = node;
     if (n == NULL) return arr;
@@ -924,16 +953,24 @@ wxArrayString XmlResApp::FindStrings(wxXmlNode *node)
             // ...and known to contain translatable string
         {
             if (!flagGettext ||
-                node->GetPropVal(wxT("translate"), wxT("1")) != wxT("0"))
+                node->GetAttribute(wxT("translate"), wxT("1")) != wxT("0"))
             {
-                arr.Add(ConvertText(n->GetContent()));
+                arr.push_back
+                    (
+                        ExtractedString
+                        (
+                            ConvertText(n->GetContent()),
+                            filename,
+                            n->GetLineNumber()
+                        )
+                    );
             }
         }
 
         // subnodes:
         if (n->GetType() == wxXML_ELEMENT_NODE)
         {
-            wxArrayString a2 = FindStrings(n);
+            ExtractedStrings a2 = FindStrings(filename, n);
             WX_APPEND_ARRAY(arr, a2);
         }
 
