@@ -12,7 +12,10 @@ import os
 import webbrowser
 import wx
 import wx.lib.scrolledpanel as scrolled
-import wx.lib.platebtn as platebtn
+try:
+    import wx.lib.platebtn as platebtn
+except ImportError:
+    import platebtn
 
 #-----------------------------------------------------------------------------#
 
@@ -27,6 +30,7 @@ class TestPanel(scrolled.ScrolledPanel):
 
         # Event Handlers
         self.Bind(wx.EVT_BUTTON, self.OnButton)
+        self.Bind(wx.EVT_TOGGLEBUTTON, self.OnToggleButton)
         self.Bind(wx.EVT_MENU, self.OnMenu)
 
     def __DoLayout(self):
@@ -72,6 +76,8 @@ class TestPanel(scrolled.ScrolledPanel):
         hsizer2.Add((15, 15))
         hsizer3 = wx.BoxSizer(wx.HORIZONTAL)
         hsizer3.Add((15, 15))
+        hsizer4 = wx.BoxSizer(wx.HORIZONTAL)
+        hsizer4.Add((15, 15))
 
         # Button Styles
         default = platebtn.PB_STYLE_DEFAULT
@@ -79,6 +85,7 @@ class TestPanel(scrolled.ScrolledPanel):
         sqgrad  = platebtn.PB_STYLE_SQUARE | platebtn.PB_STYLE_GRADIENT
         gradient = platebtn.PB_STYLE_GRADIENT
         droparrow = platebtn.PB_STYLE_DROPARROW
+        toggle = default | platebtn.PB_STYLE_TOGGLE
 
         # Create a number of different PlateButtons
         # Each button is created in the below loop by using the data set in this
@@ -98,11 +105,15 @@ class TestPanel(scrolled.ScrolledPanel):
                  # Row 3
                  (devil, "Custom Color",       default, None,    None, wx.RED, True),
                  (monkey, "Gradient Highlight", gradient, None,  None, None,   True),
-                 (monkey, "Custom Gradient",   gradient, None,   None, wx.Color(245, 55, 245), True),
+                 (monkey, "Custom Gradient",   gradient, None,   None, wx.Colour(245, 55, 245), True),
                  (None,  "Drop Arrow",                  droparrow, None,    None, None,   True),
                  (devil,  "",                  default, None,    None, None,   True),
                  (bookmark,  "",               default, None,    True, None,   True),
                  (monkey,  "",                 square,  None,    None, None,   True),
+                 # Row 4
+                 (None,  "Toggle PlateButton", toggle, None,    None, None,  True),
+                 (devil, "Toggle w/Bitmap",    toggle, None,    None, None,  True),
+                 (None,  "Toggle w/Menu",      toggle, None,    True, None,  True),
                  ]
 
         # Make and layout three rows of buttons in the panel
@@ -125,8 +136,10 @@ class TestPanel(scrolled.ScrolledPanel):
                 tsizer = hsizer1
             elif btype.index(btn) < 10:
                 tsizer = hsizer2
-            else:
+            elif btype.index(btn) < 17:
                 tsizer = hsizer3
+            else:
+                tsizer = hsizer4
 
             tbtn = platebtn.PlateButton(panel, wx.ID_ANY, btn[1], btn[0], style=bstyle)
 
@@ -170,7 +183,8 @@ class TestPanel(scrolled.ScrolledPanel):
                         (txt_sz, 0, wx.ALIGN_LEFT),
                         ((10, 10)), (hsizer1, 0, wx.EXPAND), ((10, 10)), 
                         (hsizer2, 0, wx.EXPAND), ((10, 10)), 
-                        (hsizer3, 0, wx.EXPAND), ((10, 10))])
+                        (hsizer3, 0, wx.EXPAND), ((10, 10)),
+                        (hsizer4, 0, wx.EXPAND), ((10, 10))])
         panel.SetSizer(vsizer)
         
     def OnDropArrowPressed(self, evt):
@@ -180,6 +194,11 @@ class TestPanel(scrolled.ScrolledPanel):
         self.log.write("BUTTON CLICKED: Id: %d, Label: %s" % \
                        (evt.GetId(), evt.GetEventObject().LabelText))
 
+    def OnToggleButton(self, evt):
+        self.log.write("TOGGLE BUTTON CLICKED: Id: %d, Label: %s, Pressed: %s" % \
+                       (evt.GetId(), evt.GetEventObject().LabelText,
+                        evt.GetEventObject().IsPressed()))
+
     def OnChildFocus(self, evt):
         """Override ScrolledPanel.OnChildFocus to prevent erratic
         scrolling on wxMac.
@@ -197,7 +216,7 @@ class TestPanel(scrolled.ScrolledPanel):
         e_obj = evt.GetEventObject()
         mitem = e_obj.FindItemById(evt.GetId())
         if mitem != wx.NOT_FOUND:
-            label = mitem.GetLabel()
+            label = mitem.GetItemLabel()
             if label.startswith('http://'):
                 webbrowser.open(label, True)
 
