@@ -1,4 +1,4 @@
-# -*- coding: utf-8 -*-
+# -*- coding: iso-8859-1 -*-
 #----------------------------------------------------------------------
 # Name:        make_installer.py
 # Purpose:     A script to create the wxPython windows installer
@@ -6,7 +6,7 @@
 # Author:      Robin Dunn
 #
 # Created:     30-April-2001
-# RCS-ID:      $Id: make_installer.py 67484 2011-04-13 21:05:17Z RD $
+# RCS-ID:      $Id: make_installer.py 65772 2010-10-05 23:37:22Z RD $
 # Copyright:   (c) 2003 by Total Control Software
 # Licence:     wxWindows license
 #----------------------------------------------------------------------
@@ -36,12 +36,12 @@ if os.environ.has_key("INNO5"):
 ISS_Template = r'''
 
 [Setup]
-AppName = wxPython%(SHORTVER)s-%(CHARTYPE)s-%(PYVER)s
-AppVerName = wxPython %(VERSION)s (%(CHARTYPE)s) for Python %(PYTHONVER)s
-OutputBaseFilename = wxPython%(SHORTVER)s-win%(BITS)s-%(CHARTYPE)s-%(VERSION)s-%(PYVER)s
-AppCopyright = Copyright 2008 Total Control Software
+AppName = wxPython%(SHORTVER)s-%(PYVER)s
+AppVerName = wxPython %(VERSION)s for Python %(PYTHONVER)s
+OutputBaseFilename = wxPython%(SHORTVER)s-win%(BITS)s-%(VERSION)s-%(PYVER)s
+AppCopyright = Copyright 2010 Total Control Software
 DefaultDirName = {code:GetInstallDir|c:\DoNotInstallHere}
-DefaultGroupName = wxPython %(VERSION)s (%(CHARTYPE)s) for Python %(PYTHONVER)s
+DefaultGroupName = wxPython %(VERSION)s for Python %(PYTHONVER)s
 PrivilegesRequired = %(PRIV)s
 OutputDir = dist
 DisableStartupPrompt = true
@@ -61,7 +61,7 @@ AppPublisher = Total Control Software
 AppPublisherURL = http://wxPython.org/
 AppSupportURL = http://wxPython.org/maillist.php
 AppUpdatesURL = http://wxPython.org/download.php
-AppVersion = %(VERSION)s-%(CHARTYPE)s
+AppVersion = %(VERSION)s
 
 UninstallFilesDir = {app}\%(PKGDIR)s
 LicenseFile = licence\licence.txt
@@ -103,7 +103,8 @@ Source: "wx\_wizard.pyd";                      DestDir: "{app}\%(PKGDIR)s\wx"; C
 Source: "wx\_xrc.pyd";                         DestDir: "{app}\%(PKGDIR)s\wx"; Components: core; Flags: comparetimestamp
 Source: "wx\_aui.pyd";                         DestDir: "{app}\%(PKGDIR)s\wx"; Components: core; Flags: comparetimestamp
 Source: "wx\_richtext.pyd";                    DestDir: "{app}\%(PKGDIR)s\wx"; Components: core; Flags: comparetimestamp
-
+Source: "wx\_dataview.pyd";                    DestDir: "{app}\%(PKGDIR)s\wx"; Components: core; Flags: comparetimestamp
+Source: "wx\_propgrid.pyd";                    DestDir: "{app}\%(PKGDIR)s\wx"; Components: core; Flags: comparetimestamp
 
 Source: "wx\*.py";                              DestDir: "{app}\%(PKGDIR)s\wx"; Components: core
 Source: "wx\build\*.py";                        DestDir: "{app}\%(PKGDIR)s\wx\build"; Components: core
@@ -123,6 +124,7 @@ Source: "wx\lib\editor\*.txt";                  DestDir: "{app}\%(PKGDIR)s\wx\li
 Source: "wx\lib\mixins\*.py";                   DestDir: "{app}\%(PKGDIR)s\wx\lib\mixins"; Components: core
 Source: "wx\lib\masked\*.py";                   DestDir: "{app}\%(PKGDIR)s\wx\lib\masked"; Components: core
 Source: "wx\lib\ogl\*.py";                      DestDir: "{app}\%(PKGDIR)s\wx\lib\ogl"; Components: core
+Source: "wx\lib\pdfviewer\*.py";                DestDir: "{app}\%(PKGDIR)s\wx\lib\pdfviewer"; Components: core
 Source: "wx\lib\floatcanvas\*.py";              DestDir: "{app}\%(PKGDIR)s\wx\lib\floatcanvas"; Components: core
 Source: "wx\lib\floatcanvas\Utilities\*.py";    DestDir: "{app}\%(PKGDIR)s\wx\lib\floatcanvas\Utilities"; Components: core
 Source: "wx\lib\pubsub\*.py";                   DestDir: "{app}\%(PKGDIR)s\wx\lib\pubsub"; Components: core
@@ -169,17 +171,13 @@ Source: "wx\tools\Editra\src\*.py";                          DestDir: "{app}\%(P
 Source: "wx\tools\Editra\src\autocomp\*.py";                 DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\autocomp"; Components: core
 Source: "wx\tools\Editra\src\ebmlib\*.py";                   DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\ebmlib"; Components: core
 Source: "wx\tools\Editra\src\eclib\*.py";                    DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\eclib"; Components: core
-Source: "wx\tools\Editra\src\ebmlib\*.py";                   DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\ebmlib"; Components: core
 Source: "wx\tools\Editra\src\extern\*.py";                   DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern"; Components: core
 Source: "wx\tools\Editra\src\extern\README";                 DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern"; Components: core
 Source: "wx\tools\Editra\src\extern\aui\*.py";               DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\aui"; Components: core
-Source: "wx\tools\Editra\src\extern\dexml\*.py";               DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\dexml"; Components: core
-Source: "wx\tools\Editra\src\extern\dexml\*.txt";               DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\dexml"; Components: core
 Source: "wx\tools\Editra\src\extern\pygments\*.py";          DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments"; Components: core
 Source: "wx\tools\Editra\src\extern\pygments\filters\*.py";    DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\filters"; Components: core
 Source: "wx\tools\Editra\src\extern\pygments\formatters\*.py"; DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\formatters"; Components: core
 Source: "wx\tools\Editra\src\extern\pygments\lexers\*.py";     DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\lexers"; Components: core
-Source: "wx\tools\Editra\src\extern\pygments\styles\*.py";     DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\styles"; Components: core
 Source: "wx\tools\Editra\src\syntax\*.py";                   DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\syntax"; Components: core
 Source: "wx\tools\Editra\src\syntax\README";                 DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\src\syntax"; Components: core
 Source: "wx\tools\Editra\styles\*.ess";                      DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra\styles"; Components: core
@@ -196,14 +194,6 @@ Source: "wx\tools\Editra\__init__.py";                       DestDir: "{app}\%(P
 Source: "wx\tools\Editra\launcher.py";                       DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra"; Components: core
 Source: "wx\tools\Editra\Editra";                            DestDir: "{app}\%(PKGDIR)s\wx\tools\Editra"; Components: core
 
-
-Source: "wxPython\*.py";                          DestDir: "{app}\%(PKGDIR)s\wxPython"; Components: core
-Source: "wxPython\lib\*.py";                      DestDir: "{app}\%(PKGDIR)s\wxPython\lib"; Components: core
-Source: "wxPython\lib\colourchooser\*.py";        DestDir: "{app}\%(PKGDIR)s\wxPython\lib\colourchooser"; Components: core
-Source: "wxPython\lib\editor\*.py";               DestDir: "{app}\%(PKGDIR)s\wxPython\lib\editor"; Components: core
-Source: "wxPython\lib\mixins\*.py";               DestDir: "{app}\%(PKGDIR)s\wxPython\lib\mixins"; Components: core
-Source: "wxPython\tools\*.py";                    DestDir: "{app}\%(PKGDIR)s\wxPython\tools"; Components: core
-
 Source: "wxversion\wxversion.py";           DestDir: "{app}";  Flags: sharedfile;  Components: core
 Source: "src\wx.pth";                       DestDir: "{app}";  Flags: sharedfile;  Components: pthfile
 
@@ -278,6 +268,8 @@ Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\masked\*.pyc";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\masked\*.pyo";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\ogl\*.pyc";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\ogl\*.pyo";
+Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\pdfviewer\*.pyc";
+Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\pdfviewer\*.pyo";
 
 Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\pubsub\*.pyc";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\lib\pubsub\*.pyo";
@@ -316,10 +308,6 @@ Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\syntax\*.pyc";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\syntax\*.pyo";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\*.pyc";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\*.pyo";
-Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\aui\*.pyc";
-Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\aui\*.pyo";
-Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\dexml\*.pyc";
-Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\dexml\*.pyo";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\*.pyc";      
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\*.pyo";      
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\filters\*.pyc"; 
@@ -328,8 +316,6 @@ Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\formatt
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\formatters\*.pyo";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\lexers\*.pyc";    
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\lexers\*.pyo";    
-Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\styles\*.pyc";    
-Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\extern\pygments\styles\*.pyo";    
 
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\autocomp\*.pyc";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\autocomp\*.pyo";
@@ -340,19 +326,6 @@ Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\eclib\*.pyo";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\*.pyc";
 Type: files; Name: "{app}\%(PKGDIR)s\wx\tools\Editra\src\*.pyo";
 
-Type: files; Name: "{app}\%(PKGDIR)s\wxPython\*.pyc";
-Type: files; Name: "{app}\%(PKGDIR)s\wxPython\*.pyo";
-Type: files; Name: "{app}\%(PKGDIR)s\wxPython\lib\*.pyc";
-Type: files; Name: "{app}\%(PKGDIR)s\wxPython\lib\*.pyo";
-Type: files; Name: "{app}\%(PKGDIR)s\wxPython\lib\colourchooser\*.pyc";
-Type: files; Name: "{app}\%(PKGDIR)s\wxPython\lib\colourchooser\*.pyo";
-Type: files; Name: "{app}\%(PKGDIR)s\wxPython\lib\editor\*.pyc";
-Type: files; Name: "{app}\%(PKGDIR)s\wxPython\lib\editor\*.pyo";
-Type: files; Name: "{app}\%(PKGDIR)s\wxPython\lib\mixins\*.pyc";
-Type: files; Name: "{app}\%(PKGDIR)s\wxPython\lib\mixins\*.pyo";
-Type: files; Name: "{app}\%(PKGDIR)s\wxPython\tools\*.pyc";
-Type: files; Name: "{app}\%(PKGDIR)s\wxPython\tools\*.pyo";
-
 Type: files; Name: "{app}\wxversion.pyc";
 Type: files; Name: "{app}\wxversion.pyo";
 
@@ -376,19 +349,19 @@ begin
     (* -------------------------------------------------------------- *)
     (* Figure out what to use as a default installation dir           *)
 
-    if not RegQueryStringValue(HKEY_CURRENT_USER,
+    if not RegQueryStringValue(HKEY_LOCAL_MACHINE,
                                'Software\Python\PythonCore\%(PYTHONVER)s\InstallPath',
                                '', PythonDir) then begin
 
-        if not RegQueryStringValue(HKEY_LOCAL_MACHINE,
+        if not RegQueryStringValue(HKEY_CURRENT_USER,
                                    'Software\Python\PythonCore\%(PYTHONVER)s\InstallPath',
                                    '', PythonDir) then begin
 
-            if not RegQueryStringValue(HKEY_CURRENT_USER,
+            if not RegQueryStringValue(HKEY_LOCAL_MACHINE,
                                        'Software\Wow6432Node\Python\PythonCore\%(PYTHONVER)s\InstallPath',
                                        '', PythonDir) then begin
 
-                if not RegQueryStringValue(HKEY_LOCAL_MACHINE,
+                if not RegQueryStringValue(HKEY_CURRENT_USER,
                                            'Software\Wow6432Node\Python\PythonCore\%(PYTHONVER)s\InstallPath',
                                            '', PythonDir) then begin
 
@@ -478,7 +451,7 @@ ISS_DocDemo_Template = r'''
 AppName = wxPython%(SHORTVER)s-docs-demos
 AppVerName = wxPython Docs and Demos %(VERSION)s
 OutputBaseFilename = wxPython%(SHORTVER)s-win32-docs-demos-%(VERSION)s
-AppCopyright = Copyright 2008 Total Control Software
+AppCopyright = Copyright 2009 Total Control Software
 DefaultDirName = {pf}\wxPython%(SHORTVER)s Docs and Demos
 DefaultGroupName = wxPython%(SHORTVER)s Docs Demos and Tools
 PrivilegesRequired = none
@@ -764,7 +737,7 @@ def find_DLLs():
     if os.environ.get('CPU', '') == 'AMD64':
         # Just hard-code it for now until a good solution for finding
         # the right dumpbin can be found...
-        return '28uh', sys.version[:3]
+        return '291u', sys.version[:3]
         
     WXDLLVER = PYTHONVER = None
 
@@ -914,11 +887,6 @@ Building Win32 installer for wxPython:
         IF22 = ""
 
     MSLU=''
-    CHARTYPE='ansi'
-    if "UNICODE=1" in sys.argv:
-        if os.environ.get('CPU', '') != 'AMD64':
-            MSLU=r'Source: "distrib\msw\unicows.dll"; DestDir: "{code:GetPythonDir}"; Components: core; Flags: replacesameversion sharedfile' % vars()
-        CHARTYPE='unicode'
 
     f = open(ISSFILE, "w")
     f.write(ISS_Template % vars())
