@@ -4,7 +4,7 @@
 // Author:      Paul Lanier
 // Modified by:
 // Created:     05/25/06
-// RCS-ID:      $Id: pseudodc.cpp 51090 2008-01-08 04:36:23Z RD $
+// RCS-ID:      $Id: pseudodc.cpp 58701 2009-02-07 07:24:38Z KO $
 // Copyright:   (c) wxWidgets team
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -113,7 +113,7 @@ wxBitmap &GetGreyBitmap(wxBitmap &bmp)
 // pdcDrawPolyPolygonOp constructor
 // ----------------------------------------------------------------------------
 pdcDrawPolyPolygonOp::pdcDrawPolyPolygonOp(int n, int count[], wxPoint points[],
-                 wxCoord xoffset, wxCoord yoffset, int fillStyle) 
+                 wxCoord xoffset, wxCoord yoffset, wxPolygonFillMode fillStyle) 
 {
     m_n=n; m_xoffset=xoffset; m_yoffset=yoffset; m_fillStyle=fillStyle;
     int total_n=0;
@@ -181,7 +181,7 @@ pdcDrawLinesOp::~pdcDrawLinesOp()
 // pdcDrawPolygonOp constructor
 // ----------------------------------------------------------------------------
 pdcDrawPolygonOp::pdcDrawPolygonOp(int n, wxPoint points[],
-             wxCoord xoffset, wxCoord yoffset, int fillStyle)
+             wxCoord xoffset, wxCoord yoffset, wxPolygonFillMode fillStyle)
 {
     m_n=n; m_xoffset=xoffset; m_yoffset=yoffset; m_fillStyle=fillStyle;
     if (n)
@@ -556,14 +556,7 @@ PyObject *wxPseudoDC::FindObjects(wxCoord x, wxCoord y,
                 memdc.SetPen(bgpen);
                 memdc.DrawRectangle(viewrect);
                 memdc.SetLogicalFunction(wxCOPY);
-#ifdef __WXMAC__
-                // wxAND is not supported on wxMac, but it doesn't seem to
-                // hurt anything to use wxCOPY instead...
                 memdc.Blit(x-radius,y-radius,2*radius,2*radius,&maskdc,0,0,wxCOPY);
-#else
-                // AND with circle bitmap
-                memdc.Blit(x-radius,y-radius,2*radius,2*radius,&maskdc,0,0,wxAND);
-#endif
                 // clear and update rgn2
                 memdc.SelectObject(wxNullBitmap);
                 rgn2.Clear();
