@@ -3,7 +3,7 @@
 # Ported And Enhanced From wxWidgets Contribution (Aj Bommarito) By:
 #
 # Andrea Gavana, @ 16 September 2005
-# Latest Revision: 01 Dec 2009, 14.00 GMT
+# Latest Revision: 26 Aug 2010, 10.00 GMT
 #
 #
 # TODO/Caveats List
@@ -67,9 +67,9 @@ This class supports the following window styles:
 ==================== =========== ==================================================
 Window Styles        Hex Value   Description
 ==================== =========== ==================================================
-``TB_SIMPLE``                0x1 A simple ToasterBox, with background image and text customization can be created.
+``TB_SIMPLE``                0x1 A simple `ToasterBox`, with background image and text customization can be created.
 ``TB_ONTIME``                0x1 `ToasterBox` will close after a specified amount of time.
-``TB_COMPLEX``               0x2 ToasterBoxes with different degree of complexity can be created. You can add as  many controls as you want, provided that you call the AddPanel() method and pass  to it a dummy frame and a wx.Panel. See the demo for details.
+``TB_COMPLEX``               0x2 ToasterBoxes with different degree of complexity can be created. You can add as  many controls as you want, provided that you call the L{AddPanel} method and pass  to it a dummy frame and a `wx.Panel`. See the demo for details.
 ``TB_ONCLICK``               0x2 `ToasterBox` can be closed by clicking anywhere on the `ToasterBox` frame.
 ``TB_DEFAULT_STYLE``   0x2008002 Default window style for `ToasterBox`, with no caption nor close box.
 ``TB_CAPTION``        0x22009806 `ToasterBox` will have a caption, with the possibility to set a title for the `ToasterBox` frame, and a close box.
@@ -87,7 +87,7 @@ License And Version
 
 ToasterBox is distributed under the wxPython license.
 
-Latest revision: Andrea Gavana @ 01 Dec 2009, 14.00 GMT
+Latest revision: Andrea Gavana @ 26 Aug 2010, 10.00 GMT
 
 Version 0.3
 
@@ -96,7 +96,6 @@ Version 0.3
 import textwrap
 import wx
 
-from wx.lib.statbmp import GenStaticBitmap as StaticBitmap
 
 # Define Window List, We Use It Globally
 winlist = []
@@ -121,6 +120,9 @@ TB_ONCLICK = 2
 TB_SCR_TYPE_UD = 1
 # scroll from down to up
 TB_SCR_TYPE_DU = 2
+# fade in/out 
+TB_SCR_TYPE_FADE = 4
+
 
 # ------------------------------------------------------------------------------ #
 # Class ToasterBox
@@ -146,7 +148,7 @@ class ToasterBox(wx.Timer):
          ====================== ======= ================================
          `ToasterBox` Style      Value  Description
          ====================== ======= ================================
-         ``TB_SIMPLE``              0x1 A simple `ToasterBox`, with background image and text customization can be created
+         ``TB_SIMPLE``              0x1 A simple L{ToasterBox}, with background image and text customization can be created
          ``TB_COMPLEX``             0x2 `ToasterBoxes` with different degree of complexity can be created. You can add as many controls as you want, provided that you call the L{AddPanel} method and pass to it a dummy frame and a `wx.Panel`.
          ====================== ======= ================================
 
@@ -156,8 +158,8 @@ class ToasterBox(wx.Timer):
          ====================== ========== ================================
          Window Style           Hex Value  Description
          ====================== ========== ================================
-         ``TB_DEFAULT_STYLE``   0x2008002  Default window style for `ToasterBox`, with no caption nor close box.
-         ``TB_CAPTION``         0x22009806 `ToasterBox` will have a caption, with the possibility to set a title for the `ToasterBox` frame, and a close box.
+         ``TB_DEFAULT_STYLE``   0x2008002  Default window style for L{ToasterBox}, with no caption nor close box.
+         ``TB_CAPTION``         0x22009806 L{ToasterBox} will have a caption, with the possibility to set a title for the L{ToasterBox} frame, and a close box.
          ====================== ========== ================================
        
         :param `closingstyle`: the closing style for L{ToasterBox}. Can be one of the
@@ -166,8 +168,8 @@ class ToasterBox(wx.Timer):
          ==================== =========== ==================================================
          Closing Styles       Hex Value   Description
          ==================== =========== ==================================================
-         ``TB_ONTIME``                0x1 `ToasterBox` will close after a specified amount of time.
-         ``TB_ONCLICK``               0x2 `ToasterBox` can be closed by clicking anywhere on the `ToasterBox` frame.
+         ``TB_ONTIME``                0x1 L{ToasterBox} will close after a specified amount of time.
+         ``TB_ONCLICK``               0x2 L{ToasterBox} can be closed by clicking anywhere on the L{ToasterBox} frame.
          ==================== =========== ==================================================
 
         :param `scrollType`: the scrolling direction for L{ToasterBox}. Can be one of the
@@ -176,8 +178,9 @@ class ToasterBox(wx.Timer):
          ==================== =========== ==================================================
          Scroll Styles        Hex Value   Description
          ==================== =========== ==================================================
-         ``TB_SCR_TYPE_UD``           0x1 `ToasterBox` will scroll from up to down
-         ``TB_SCR_TYPE_DU``           0x2 `ToasterBox` will scroll from down to up
+         ``TB_SCR_TYPE_UD``           0x1 L{ToasterBox} will scroll from up to down
+         ``TB_SCR_TYPE_DU``           0x2 L{ToasterBox} will scroll from down to up
+         ``TB_SCR_TYPE_FADE``         0x4 L{ToasterBox} will fade in/out (without scrolling). 
          ==================== =========== ==================================================
          
         """
@@ -189,6 +192,8 @@ class ToasterBox(wx.Timer):
         self._popupposition = wx.Point(100,100)
         self._popuptop = wx.Point(0,0)
         self._popupsize = wx.Size(150, 170)
+        self._usefocus = True
+        self._originalfocus = wx.Window.FindFocus()
 
         self._backgroundcolour = wx.WHITE
         self._foregroundcolour = wx.BLACK
@@ -199,19 +204,23 @@ class ToasterBox(wx.Timer):
         self._tbstyle = tbstyle
         self._windowstyle = windowstyle
         self._closingstyle = closingstyle
-
+        self._scrollType = scrollType
+        
         self._panel = None
 
         self._bottomright = wx.Point(wx.GetDisplaySize().GetWidth(),
-                                    wx.GetDisplaySize().GetHeight())
+                                     wx.GetDisplaySize().GetHeight())
 
         if parent is not None:
             parent.Bind(wx.EVT_ICONIZE, lambda evt: [w.Hide() for w in winlist])
+            self._moveTimer = wx.Timer(parent, -1)
+            parent.Bind(wx.EVT_TIMER, self.OnMoveTimer, self._moveTimer)
  
         self._tb = ToasterBoxWindow(self._parent, self, self._tbstyle, self._windowstyle,
-                                    self._closingstyle, scrollType=scrollType)
+                                    self._closingstyle, scrollType=self._scrollType)
 
 
+        
     def SetPopupPosition(self, pos):
         """
         Sets the L{ToasterBox} position on screen.
@@ -275,7 +284,11 @@ class ToasterBox(wx.Timer):
         if colour is None:
             colour = wx.WHITE
 
+        if isinstance(colour, basestring):
+            colour = wx.NamedColour(colour)
+            
         self._backgroundcolour = colour
+        self._tb.SetPopupBackgroundColour(self._backgroundcolour)
 
 
     def SetPopupTextColour(self, colour=None):
@@ -291,6 +304,9 @@ class ToasterBox(wx.Timer):
         if colour is None:
             colour = wx.BLACK
 
+        if isinstance(colour, basestring):
+            colour = wx.NamedColour(colour)
+            
         self._foregroundcolour = colour
 
 
@@ -334,14 +350,15 @@ class ToasterBox(wx.Timer):
         """
         Sets the L{ToasterBox} background image.
 
-        :param `bitmap`: a valid `wx.Bitmap` object. If defaulted to ``None``, then
-         no background bitmap is used.
+        :param `bitmap`: a valid `wx.Bitmap` object or filename. If defaulted
+         to ``None``, then no background bitmap is used.
          
         :note: Use this method only for a L{ToasterBox} created with the ``TB_SIMPLE`` style.
         """
 
         if bitmap is not None:
-            bitmap = wx.Bitmap(bitmap, wx.BITMAP_TYPE_BMP)
+            if isinstance(bitmap, basestring):
+                bitmap = wx.Bitmap(bitmap)
 
         self._bitmap = bitmap
 
@@ -392,6 +409,7 @@ class ToasterBox(wx.Timer):
         self._tb.SetPopupPosition((self._popupposition[0], self._popupposition[1]))
         self._tb.SetPopupPauseTime(self._pausetime)
         self._tb.SetPopupScrollSpeed(self._sleeptime)
+        self._tb.SetUseFocus(self._usefocus, self._originalfocus)
 
         if self._tbstyle == TB_SIMPLE:
             self._tb.SetPopupTextColour(self._foregroundcolour)
@@ -457,6 +475,24 @@ class ToasterBox(wx.Timer):
         self._tb.SetTitle(title)
 
 
+    def SetUseFocus(self, focus):
+        """
+        If `focus` is ``True``, Instructs L{ToasterBox} to steal the focus from the
+        parent application, otherwise it returns the focus to the original owner.
+
+        :param `focus`: ``True`` to set the focus on L{ToasterBox}, ``False`` to
+         return it to the original owner.
+        """
+
+        self._usefocus = focus
+
+
+    def GetUseFocus(self):
+        """ Returns whether L{ToasterBox} will steal the focus from the parent application. """
+
+        return self._usefocus
+    
+
     def Notify(self):
         """ It's time to hide a L{ToasterBox}. """
 
@@ -475,23 +511,39 @@ class ToasterBox(wx.Timer):
         if not node:
             return
 
+        self._startPos = node.GetPosition()[1]
+        self._moveTimer.Start(self._sleeptime)
+
+
+    def OnMoveTimer(self, event):
+        """
+        Handles the ``wx.EVT_TIMER`` event for L{ToasterBox}, moving the new window
+        on top of the last one created.
+
+        :param `event`: a `wx.TimerEvent` event to be processed.
+        """
+
+        current = self._startPos
+        if current >= self._popupposition[1]:
+            self._moveTimer.Stop()
+
         # move windows to fill in blank space
-        for i in xrange(node.GetPosition()[1], self._popupposition[1], 4):
-            if i > self._popupposition[1]:
-                i = self._popupposition[1]
+        
+        if current > self._popupposition[1]:
+            current = self._popupposition[1]
 
-            # loop through all the windows
-            for j in xrange(0, len(winlist)):
-                ourNewHeight = i - (j*self._popupsize[1] - 8)
-                tmpTb = winlist[j]
-                # reset where the object THINKS its supposed to be
-                tmpTb.SetPopupPosition((self._popupposition[0], ourNewHeight))
-                # actually move it
-                tmpTb.SetDimensions(self._popupposition[0], ourNewHeight, tmpTb.GetSize().GetWidth(),
-                                    tmpTb.GetSize().GetHeight())
-
-            wx.Usleep(self._sleeptime)
+        # loop through all the windows
+        for j in xrange(0, len(winlist)):
+            ourNewHeight = current - (j*self._popupsize[1] - 8)
+            tmpTb = winlist[j]
+            # reset where the object THINKS its supposed to be
+            tmpTb.SetPopupPosition((self._popupposition[0], ourNewHeight))
+            # actually move it
+            tmpTb.SetDimensions(self._popupposition[0], ourNewHeight, tmpTb.GetSize().GetWidth(),
+                                tmpTb.GetSize().GetHeight())
 
+        self._startPos += 4
+        
 
     def CleanList(self):
         """ Cleans the window list, erasing the stack of L{ToasterBox} objects. """
@@ -503,6 +555,7 @@ class ToasterBox(wx.Timer):
         while node:
             if not node.IsShown():
                 winlist.remove(node)
+                node.Close()
                 try:
                     node = winlist[0]
                 except:
@@ -543,7 +596,7 @@ class ToasterBoxWindow(wx.Frame):
          ====================== ======= ================================
          `ToasterBox` Style      Value  Description
          ====================== ======= ================================
-         ``TB_SIMPLE``              0x1 A simple `ToasterBox`, with background image and text customization can be created
+         ``TB_SIMPLE``              0x1 A simple L{ToasterBox}, with background image and text customization can be created
          ``TB_COMPLEX``             0x2 `ToasterBoxes` with different degree of complexity can be created. You can add as many controls as you want, provided that you call the L{AddPanel} method and pass to it a dummy frame and a `wx.Panel`.
          ====================== ======= ================================
 
@@ -553,8 +606,8 @@ class ToasterBoxWindow(wx.Frame):
          ====================== ========== ================================
          Window Style           Hex Value  Description
          ====================== ========== ================================
-         ``TB_DEFAULT_STYLE``   0x2008002  Default window style for `ToasterBox`, with no caption nor close box.
-         ``TB_CAPTION``         0x22009806 `ToasterBox` will have a caption, with the possibility to set a title for the `ToasterBox` frame, and a close box.
+         ``TB_DEFAULT_STYLE``   0x2008002  Default window style for L{ToasterBox}, with no caption nor close box.
+         ``TB_CAPTION``         0x22009806 L{ToasterBox} will have a caption, with the possibility to set a title for the L{ToasterBox} frame, and a close box.
          ====================== ========== ================================
        
         :param `closingstyle`: the closing style for L{ToasterBoxWindow}. Can be one of the
@@ -563,8 +616,8 @@ class ToasterBoxWindow(wx.Frame):
          ==================== =========== ==================================================
          Closing Styles       Hex Value   Description
          ==================== =========== ==================================================
-         ``TB_ONTIME``                0x1 `ToasterBox` will close after a specified amount of time.
-         ``TB_ONCLICK``               0x2 `ToasterBox` can be closed by clicking anywhere on the `ToasterBox` frame.
+         ``TB_ONTIME``                0x1 L{ToasterBox} will close after a specified amount of time.
+         ``TB_ONCLICK``               0x2 L{ToasterBox} can be closed by clicking anywhere on the L{ToasterBox} frame.
          ==================== =========== ==================================================
 
         :param `scrollType`: the scrolling direction for L{ToasterBoxWindow}. Can be one of the
@@ -573,8 +626,9 @@ class ToasterBoxWindow(wx.Frame):
          ==================== =========== ==================================================
          Scroll Styles        Hex Value   Description
          ==================== =========== ==================================================
-         ``TB_SCR_TYPE_UD``           0x1 `ToasterBox` will scroll from up to down
-         ``TB_SCR_TYPE_DU``           0x2 `ToasterBox` will scroll from down to up
+         ``TB_SCR_TYPE_UD``           0x1 L{ToasterBox} will scroll from up to down
+         ``TB_SCR_TYPE_DU``           0x2 L{ToasterBox} will scroll from down to up
+         ``TB_SCR_TYPE_FADE``         0x4 L{ToasterBox} will fade in/out (without scrolling).
          ==================== =========== ==================================================
 
         """
@@ -596,7 +650,7 @@ class ToasterBoxWindow(wx.Frame):
         self._tbstyle = tbstyle
         self._windowstyle = windowstyle
         self._closingstyle = closingstyle
-        self._scrollType = scrollType
+        self._backgroundcolour = wx.WHITE
 
         if tbstyle == TB_COMPLEX:
             self.sizer = wx.BoxSizer(wx.VERTICAL)
@@ -607,6 +661,13 @@ class ToasterBoxWindow(wx.Frame):
             self.Bind(wx.EVT_CLOSE, self.OnClose)
             self.SetTitle("")
 
+        if scrollType == TB_SCR_TYPE_FADE and not self.CanSetTransparent():
+            import warnings
+            warnings.warn("The style ``TB_SCR_TYPE_FADE`` is not supported on this platform.")
+            scrollType = TB_SCR_TYPE_DU
+            
+        self._scrollType = scrollType
+
         if self._closingstyle & TB_ONCLICK and self._windowstyle != TB_CAPTION:
             self.Bind(wx.EVT_LEFT_DOWN, self.OnMouseDown)
 
@@ -616,6 +677,16 @@ class ToasterBoxWindow(wx.Frame):
         self.SetDimensions(self._bottomright.x, self._bottomright.y,
                            framesize.GetWidth(), framesize.GetHeight())
 
+        self._scrollTimer = wx.Timer(self, -1)
+        self._alphaTimer = wx.Timer(self, -1)
+        
+        self.Bind(wx.EVT_TIMER, self.OnScrollTimer, self._scrollTimer)
+        self.Bind(wx.EVT_TIMER, self.AlphaCycle, self._alphaTimer)
+
+        if not self._tbstyle & TB_COMPLEX:
+            self.Bind(wx.EVT_PAINT, self.OnPaint)
+            self.SetBackgroundStyle(wx.BG_STYLE_CUSTOM)
+        
 
     def OnClose(self, event):
         """
@@ -639,7 +710,7 @@ class ToasterBoxWindow(wx.Frame):
         event.Skip()
 
 
-    def SetPopupBitmap(self, bitmap):
+    def SetPopupBitmap(self, bitmap=None):
         """
         Sets the L{ToasterBox} background image.
 
@@ -649,14 +720,13 @@ class ToasterBoxWindow(wx.Frame):
         :note: Use this method only for a L{ToasterBox} created with the ``TB_SIMPLE`` style.
         """
 
-        bitmap = bitmap.ConvertToImage()
-        xsize, ysize = self.GetSize()
-        bitmap = bitmap.Scale(xsize, ysize)
-        bitmap = bitmap.ConvertToBitmap()
-        self._staticbitmap = StaticBitmap(self, -1, bitmap, pos=(0,0))
-
-        if self._closingstyle & TB_ONCLICK and self._windowstyle != TB_CAPTION:
-            self._staticbitmap.Bind(wx.EVT_LEFT_DOWN, self.OnMouseDown)
+        if bitmap is None:
+            self._staticbitmap = None
+        else:
+            bitmap = bitmap.ConvertToImage()
+            xsize, ysize = self.GetSize()
+            bitmap = bitmap.Scale(xsize, ysize)
+            self._staticbitmap = bitmap.ConvertToBitmap()
 
 
     def SetPopupSize(self, size):
@@ -755,9 +825,9 @@ class ToasterBoxWindow(wx.Frame):
             raise Exception("\nERROR: Panel Can Not Be Added When Using TB_SIMPLE ToasterBox Style")
 
         self.sizer.Add(panel, 1, wx.EXPAND)
-        self.sizer.Layout()
         self.SetSizer(self.sizer)
-
+        self.Layout()
+        
         if self._closingstyle & TB_ONCLICK and self._windowstyle != TB_CAPTION:
             panel.Bind(wx.EVT_LEFT_DOWN, self.OnMouseDown)
 
@@ -809,6 +879,8 @@ class ToasterBoxWindow(wx.Frame):
             # when you try to make the window too small.
             return False
 
+        self._direction = wx.UP
+        self.SetupPositions()
         self.ScrollUp()
         timerid = wx.NewId()
         self.showtime = wx.Timer(self, timerid)
@@ -823,6 +895,10 @@ class ToasterBoxWindow(wx.Frame):
 
         self.showtime.Stop()
         del self.showtime
+
+        self._direction = wx.DOWN
+        self.SetupPositions()
+
         self.ScrollDown()
 
 
@@ -837,6 +913,7 @@ class ToasterBoxWindow(wx.Frame):
         """
 
         self.SetBackgroundColour(colour)
+        self._backgroundcolour = colour
 
 
     def SetPopupTextColour(self, colour):
@@ -852,133 +929,251 @@ class ToasterBoxWindow(wx.Frame):
         self._textcolour = colour
 
 
-    def ScrollUp(self):
+    def SetUseFocus(self, focus, originalfocus):
+        """
+        If `focus` is ``True``, Instructs L{ToasterBoxWindow} to steal the focus from the
+        parent application, otherwise it returns the focus to the original owner.
+
+        :param `focus`: ``True`` to set the focus on L{ToasterBoxWindow}, ``False`` to
+         return it to the original owner;
+        :param `originalfocus`: an instance of `wx.Window`, representing a pointer to
+         the window which originally had the focus
+        """
+
+        self._usefocus = focus
+        self._originalfocus = originalfocus
+
+
+    def OnScrollTimer(self, event):
+        """
+        Handles the ``wx.EVT_TIMER`` event for L{ToasterBoxWindow} scrolling up/down.
+
+        :param `event`: a `wx.TimerEvent` event to be processed.
+        """
+
+        if self._direction == wx.UP:
+            self.TearUp()
+        else:
+            self.TearDown()
+            
+        
+    def TearUp(self):
         """ Scrolls the L{ToasterBox} up, which means gradually showing it. """
 
-        self.Show(True)
+        self._windowsize = self._windowsize + self._step
+        step = self._currentStep
 
-        # walk the Y value up in a raise motion
-        xpos = self.GetPosition().x
-        ypos = self._bottomright[1]
-        windowsize = 0
+        if step < self._dialogtop[1]:
+            step = self._dialogtop[1]
 
-        # checking the type of the scroll (from up to down or from down to up)
+         # checking the type of the scroll (from up to down or from down to up)
         if self._scrollType == TB_SCR_TYPE_UD:
-            start = self._dialogtop[1]
-            stop = ypos
-            step = self._step
+            dimY = self._dialogtop[1]
         elif self._scrollType == TB_SCR_TYPE_DU:
-            start = ypos
-            stop = self._dialogtop[1]
-            step = -self._step
-        else:
-            errMsg = ("scrollType not supported (in ToasterBoxWindow.ScrollUp): %s" %
-                      self._scrollType)
-            raise ValueError(errMsg)
+            dimY = step
 
-        for i in xrange(start, stop, step):
-            if i < self._dialogtop[1]:
-                i = self._dialogtop[1]
+        self.SetDimensions(self._dialogtop[0], dimY, self.GetSize().GetWidth(), self._windowsize)
 
-            windowsize = windowsize + self._step
+        self.Refresh(False)
 
-            # checking the type of the scroll (from up to down or from down to up)
-            if self._scrollType == TB_SCR_TYPE_UD:
-                dimY = self._dialogtop[1]
-            elif self._scrollType == TB_SCR_TYPE_DU:
-                dimY = i
-            else:
-                errMsg = ("scrollType not supported (in ToasterBoxWindow.ScrollUp): %s" %
-                          self._scrollType)
-                raise ValueError(errMsg)
+        self._currentStep += self._scrollStep
 
-            self.SetDimensions(self._dialogtop[0], dimY, self.GetSize().GetWidth(),
-                               windowsize)
+        if self._currentStep not in range(self._start, self._stop, self._scrollStep):
+            self._scrollTimer.Stop()
+            self.Update()
 
             if self._tbstyle == TB_SIMPLE:
                 self.DrawText()
 
-            wx.Usleep(self._sleeptime)
-            self.Update()
-            self.Refresh()
+            if self._usefocus:
+                self.SetFocus()
+            else:
+                self._originalfocus.SetFocus()
 
-        self.Update()
-
-        if self._tbstyle == TB_SIMPLE:
-            self.DrawText()
-
-        self.SetFocus()
-
-
-    def ScrollDown(self):
+        
+    def TearDown(self):
         """ Scrolls the L{ToasterBox} down, which means gradually hiding it. """
 
-        # walk down the Y value
-        windowsize = self.GetSize().GetHeight()
+        self._windowsize = self._windowsize - self._step
+        step = self._currentStep
 
-        # checking the type of the scroll (from up to down or from down to up)
-        if self._scrollType == TB_SCR_TYPE_UD:
-            start = self._bottomright.y
-            stop = self._dialogtop[1]
-            step = -self._step
-        elif self._scrollType == TB_SCR_TYPE_DU:
-            start = self._dialogtop[1]
-            stop = self._bottomright.y
-            step = self._step
-        else:
-            errMsg = ("scrollType not supported (in ToasterBoxWindow.ScrollUp): %s" %
-                      self._scrollType)
-            raise ValueError(errMsg)
-
-        for i in xrange(start, stop, step):
-            if i > self._bottomright.y:
-                i = self._bottomright.y
-
-            windowsize = windowsize - self._step
-
-            if windowsize <= 0:
-               break
-            
+        if step > self._bottomright.y:
+            step = self._bottomright.y
+        
+        if self._windowsize > 0:            
             # checking the type of the scroll (from up to down or from down to up)
             if self._scrollType == TB_SCR_TYPE_UD:
                 dimY = self._dialogtop[1]
             elif self._scrollType == TB_SCR_TYPE_DU:
-                dimY = i
-            else:
-                errMsg = ("scrollType not supported (in ToasterBoxWindow.ScrollUp): %s" %
-                          self._scrollType)
-                raise ValueError(errMsg)
+                dimY = step
 
             self.SetDimensions(self._dialogtop[0], dimY,
-                               self.GetSize().GetWidth(), windowsize)
+                               self.GetSize().GetWidth(), self._windowsize)
 
-            wx.Usleep(self._sleeptime)
+            self.Update()
             self.Refresh()
 
-        self.Hide()
-        if self._parent2:
-            self._parent2.Notify()
+            self._currentStep += self._scrollStep
+            
+        else:            
+            self._scrollTimer.Stop()
+            self.Hide()
+            if self._parent2:
+                self._parent2.Notify()
 
 
-    def DrawText(self):
-        """ Draws the text label for a L{ToasterBox} with ``TB_SIMPLE`` style set. """
-      
-        if self._staticbitmap is not None:
-            dc = wx.ClientDC(self._staticbitmap)
+    def SetupPositions(self):
+        """ Sets up the position, size and scrolling step for L{ToasterBoxWindow}. """
+
+        if self._scrollType == TB_SCR_TYPE_FADE:
+            self.SetPosition(wx.Point(*self._dialogtop))
+            return
+
+        if self._direction == wx.UP:
+            # walk the Y value up in a raise motion
+            self._xpos = self.GetPosition().x
+            self._ypos = self._bottomright[1]
+            self._windowsize = 0
+
+            # checking the type of the scroll (from up to down or from down to up)
+            if self._scrollType == TB_SCR_TYPE_UD:
+                self._start = self._dialogtop[1]
+                self._stop = self._ypos
+                self._scrollStep = self._step
+            elif self._scrollType == TB_SCR_TYPE_DU:
+                self._start = self._ypos
+                self._stop = self._dialogtop[1]
+                self._scrollStep = -self._step
+
         else:
+
+            # walk down the Y value
+            self._windowsize = self.GetSize().GetHeight()
+
+            # checking the type of the scroll (from up to down or from down to up)
+            if self._scrollType == TB_SCR_TYPE_UD:
+                self._start = self._bottomright.y
+                self._stop = self._dialogtop[1]
+                self._scrollStep = -self._step
+            elif self._scrollType == TB_SCR_TYPE_DU:
+                self._start = self._dialogtop[1]
+                self._stop = self._bottomright.y
+                self._scrollStep = self._step
+
+        self._currentStep = self._start
+
+
+    def ScrollUp(self):
+        """ Scrolls the L{ToasterBox} up, which means gradually showing it. """
+
+        if self._scrollType == TB_SCR_TYPE_FADE:
+            self._amount = 0
+            self._delta = 5
+            self.SetSize(self.GetSize())
+            self._alphaTimer.Start(self._sleeptime)
+        else:
+            self.Show(True)
+            self._scrollTimer.Start(self._sleeptime)
+    
+
+    def ScrollDown(self):
+        """ Scrolls the L{ToasterBox} down, which means gradually hiding it. """
+
+        if self._scrollType == TB_SCR_TYPE_FADE:
+            self._amount = 255
+            self._delta = -5
+            self._alphaTimer.Start(self._sleeptime)
+        else:
+            self._scrollTimer.Start(self._sleeptime)
+
+
+    def OnPaint(self, event):
+        """
+        Handles the ``wx.EVT_PAINT`` event for L{ToasterBoxWindow}.
+
+        :param `event`: a `wx.PaintEvent` event to be processed.
+
+        :note: This event is handled and processed only if the style ``TB_SIMPLE`` is
+         given to L{ToasterBox}.
+        """
+        
+        dc = wx.AutoBufferedPaintDC(self)
+        self.DrawText(dc)
+        
+            
+    def DrawText(self, dc=None):
+        """
+        Draws the text label for a L{ToasterBox} with ``TB_SIMPLE`` style set.
+
+        :param `dc`: an instance of `wx.DC`. If defaulted to ``None``, a `wx.ClientDC`
+         will be created on the fly.
+        """
+      
+        if dc is None:
             dc = wx.ClientDC(self)
            
+        dc.SetBackground(wx.Brush(self._backgroundcolour))
+        dc.Clear()
+        
+        if self._staticbitmap:
+            dc.DrawBitmap(self._staticbitmap, 0, 0)
         dc.SetFont(self._textfont)
-
+        dc.SetTextForeground(self._textcolour)
+        
         if not hasattr(self, "text_coords"):
             self._getTextCoords(dc)
-
-        fg = dc.GetTextForeground()
-        dc.SetTextForeground(self._textcolour)
         dc.DrawTextList(*self.text_coords)
-        dc.SetTextForeground(fg)
 
 
+    def AlphaCycle(self, event):
+        """
+        Handles the ``wx.EVT_TIMER`` event for L{ToasterBoxWindow}.
+
+        :param `event`: a `wx.TimerEvent` event to be processed.
+        """
+
+        # Increase (or decrease) the alpha channel
+        self._amount += self._delta
+
+        if self._tbstyle == TB_SIMPLE:
+            self.Refresh(False)
+            
+        if self._amount > 255 or self._amount < 0:
+            # We're done, stop the timer
+            self._alphaTimer.Stop()
+
+            if self._amount < 0:
+                self.Hide()
+                if self._parent2:
+                    self._parent2.Notify()
+
+            elif self._amount > 255:
+                if self._usefocus:
+                    self.SetFocus()
+                else:
+                    self._originalfocus.SetFocus()
+
+            return
+
+        # Make the ToasterBoxWindow more or less transparent
+        self.MakeWindowTransparent(self._amount)
+        if not self.IsShown():
+            self.Show()
+            
+
+    def MakeWindowTransparent(self, amount):
+        """
+        Makes the L{ToasterBoxWindow} window transparent.
+
+        :param `amount`: the alpha channel value.
+        """
+
+        if not self.CanSetTransparent():
+            return
+        
+        self.SetTransparent(amount)
+        
+
     def _getTextCoords(self, dc):
         """
         Draw the user specified text.
