@@ -7,7 +7,7 @@
 #
 # Author:      Mike Fletcher
 #
-# RCS-ID:      $Id: filebrowsebutton.py 54039 2008-06-08 23:00:10Z RD $
+# RCS-ID:      $Id: filebrowsebutton.py 65245 2010-08-11 01:56:28Z RD $
 # Copyright:   (c) 2000 by Total Control Software
 # Licence:     wxWindows license
 #----------------------------------------------------------------------
@@ -159,6 +159,7 @@ class FileBrowseButton(wx.Panel):
             directory = directory [0]
         else:
             directory = self.startDirectory
+            current = ''
         dlg = wx.FileDialog(self, self.dialogTitle, directory, current,
                             self.fileMask, self.fileMode)
 
@@ -181,16 +182,6 @@ class FileBrowseButton(wx.Panel):
         self.callCallback =  save
 
 
-    def Enable (self, value=True):
-        """ Convenient enabling/disabling of entire control """
-        self.label.Enable (value)
-        self.textControl.Enable (value)
-        return self.browseButton.Enable (value)
-
-    def Disable (self,):
-        """ Convenient disabling of entire control """
-        self.Enable(False)
-
     def GetLabel( self ):
         """ Retrieve the label's current text """
         return self.label.GetLabel()
