@@ -201,11 +201,8 @@ class ImageView(wx.Window):
             evt.Skip()
             return
         dc = evt.GetDC()
-        if not dc:
-            dc = wx.ClientDC(self)
-            rect = self.GetUpdateRegion().GetBox()
-            dc.SetClippingRect(rect)
-        self.PaintBackground(dc, self.dark_bg)
+        if dc:
+            self.PaintBackground(dc, self.dark_bg)
 
     def PaintBackground(self, dc, painter, rect=None):
         if painter is None:
@@ -511,7 +508,11 @@ class ImageDialog(wx.Dialog):
         if self.fl_ext == "All":
             all_files = []
 
-            for ftypes in self.fl_types[1:-1]:    # get list of all available image types
+            if self.fl_types[-1] == 'All Files':
+                allTypes = self.fl_types[-1:]
+            else:
+                allTypes = self.fl_types[1:]
+            for ftypes in allTypes:    # get list of all
                 filter = self.fl_ext_types[ftypes]
                 #print "filter = ", filter
                 self.fl_val = FindFiles(self, self.set_dir, filter)
@@ -713,160 +714,39 @@ class FindFiles:
 # This part of the file was generated by C:\Python25\Scripts\img2py
 # then edited slightly.
 
-import cStringIO, zlib
-
+from wx.lib.embeddedimage import PyEmbeddedImage
 
 IMG_CATALOG = {}
 
-class ImageClass: pass
+IMG_CATALOG['White'] = PyEmbeddedImage(
+    "iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAIAAADZF8uwAAAAA3NCSVQICAjb4U/gAAAAIUlE"
+    "QVQYlWNgIAIwMjAw/P//H58KRkYmYkwaVUScIqIAAMjRAxRV8+5MAAAAAElFTkSuQmCC")
 
-def getWhiteData():
-    return zlib.decompress(
-'x\xda\xeb\x0c\xf0s\xe7\xe5\x92\xe2b``\xe0\xf5\xf4p\t\x02\xd2< \xcc\xc1\x04$\
-o\x8a\x9f\xde\x00\xa4\x98\x8b\x9d<C888n?\xf4\x7f\x00\xe4*z\xba8\x86HLMNP`202\
-0\xf8\xf3\xff\xbf\xfc|.77\xb5$\x1f\xa9P\x979J\x8b\x18\x18N\\d\x16\t\xfd\xfc\
-\xce\x07\xa8\x98\xc1\xd3\xd5\xcfe\x9dSB\x13\x00\xcc1\x1b\xb3' )
+IMG_CATALOG['Grey'] = PyEmbeddedImage(
+    "iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAIAAADZF8uwAAAAA3NCSVQICAjb4U/gAAAAIklE"
+    "QVQYlWNgIAIwMjAwnDlzBo8KExMTJmJMGlVEnCKiAAC24wMULFLZGAAAAABJRU5ErkJggg==")
 
-def getWhiteBitmap():
-    return wx.BitmapFromImage(getWhiteImage())
+IMG_CATALOG['Black'] = PyEmbeddedImage(
+    "iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAIAAADZF8uwAAAAA3NCSVQICAjb4U/gAAAADklE"
+    "QVQYlWNgGAVDFQAAAbwAATN8mzYAAAAASUVORK5CYII=")
 
-def getWhiteImage():
-    stream = cStringIO.StringIO(getWhiteData())
-    return wx.ImageFromStream(stream)
+IMG_CATALOG['Checked'] = PyEmbeddedImage(
+    "iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAIAAADZF8uwAAAAA3NCSVQICAjb4U/gAAAAMUlE"
+    "QVQYlWNgIAIwMjAwnDlzBlnI2NgYRQUjIxMxJtFZEQsDhkvPnj07sG4iShFRAAAougYW+urT"
+    "ZwAAAABJRU5ErkJggg==")
 
-IMG_CATALOG['White'] = ImageClass()
-IMG_CATALOG['White'].getData = getWhiteData
-IMG_CATALOG['White'].getImage = getWhiteImage
-IMG_CATALOG['White'].getBitmap = getWhiteBitmap
+IMG_CATALOG['NoFrame'] = PyEmbeddedImage(
+    "iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAIAAADZF8uwAAAAA3NCSVQICAjb4U/gAAAANklE"
+    "QVQYla2PQQoAIBACnej/X7ZbUEQtkudhVKkQJNm+EdAqpggCgB+m44kFml1bY39q0k15Bsuc"
+    "CR/z8ajiAAAAAElFTkSuQmCC")
 
+IMG_CATALOG['BoxFrame'] = PyEmbeddedImage(
+    "iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAIAAADZF8uwAAAAA3NCSVQICAjb4U/gAAAAQ0lE"
+    "QVQYlZ2O0QoAIAgDd9L//7I9CFEhJu1psmNOaghJ7l4RYJ0m1U0R2X4vevcHVOiG0tcHBABh"
+    "8nWpIhpPLtn0rwm4WyD966x3sgAAAABJRU5ErkJggg==")
 
-#----------------------------------------------------------------------
-def getGreyData():
-    return zlib.decompress(
-'x\xda\xeb\x0c\xf0s\xe7\xe5\x92\xe2b``\xe0\xf5\xf4p\t\x02\xd2< \xcc\xc1\x04$\
-o\x8a\x9f\xde\x00\xa4\x98\x8b\x9d<C888n?\xf4\x7f\x00\xe4*y\xba8\x86HLMNP`202\
-0\x98cY\xcc\xd6\xcf%,,\xac\x96\xe4#\x15\xea2Gi\x11\x03\xc3\xb6\xc7\xcc":A7%\
-\x80\xaa\x19<]\xfd\\\xd69%4\x01\x00{m\x18s' )
+IMG_CATALOG['CropFrame'] = PyEmbeddedImage(
+    "iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAIAAADZF8uwAAAAA3NCSVQICAjb4U/gAAAASUlE"
+    "QVQYlb2QMQrAQAgEZ0P+q0/RF5tCuIMUh2myhcgyjCAMIiAiDoS7XxPTCLrXZmaAJKCqgMz8"
+    "YHpD7ThBkvpcz93z6wtGeQD/sQ8bfXs8NAAAAABJRU5ErkJggg==")
 
-def getGreyBitmap():
-    return wx.BitmapFromImage(getGreyImage())
-
-def getGreyImage():
-    stream = cStringIO.StringIO(getGreyData())
-    return wx.ImageFromStream(stream)
-
-IMG_CATALOG['Grey'] = ImageClass()
-IMG_CATALOG['Grey'].getData = getGreyData
-IMG_CATALOG['Grey'].getImage = getGreyImage
-IMG_CATALOG['Grey'].getBitmap = getGreyBitmap
-
-
-#----------------------------------------------------------------------
-def getBlackData():
-    return zlib.decompress(
-'x\xda\xeb\x0c\xf0s\xe7\xe5\x92\xe2b``\xe0\xf5\xf4p\t\x02\xd2< \xcc\xc1\x04$\
-o\x8a\x9f\xde\x00\xa4\x98\x8b\x9d<C888n?\xf4\x7f\x00\xe4\xf2y\xba8\x86HLMN\
-\x90`u\x16e``\xdc\xc3\xc0h\\3\xdb\x0c(\xc3\xe0\xe9\xea\xe7\xb2\xce)\xa1\t\
-\x00\xb6`\x12\x08' )
-
-def getBlackBitmap():
-    return wx.BitmapFromImage(getBlackImage())
-
-def getBlackImage():
-    stream = cStringIO.StringIO(getBlackData())
-    return wx.ImageFromStream(stream)
-
-IMG_CATALOG['Black'] = ImageClass()
-IMG_CATALOG['Black'].getData = getBlackData
-IMG_CATALOG['Black'].getImage = getBlackImage
-IMG_CATALOG['Black'].getBitmap = getBlackBitmap
-
-
-#----------------------------------------------------------------------
-def getCheckedData():
-    return zlib.decompress(
-'x\xda\xeb\x0c\xf0s\xe7\xe5\x92\xe2b``\xe0\xf5\xf4p\t\x02\xd2< \xcc\xc1\x04$\
-o\x8a\x9f\xde\x00\xa4\x98\x8b\x9d<C888n?\xf4\x7f\x00\xe4\x1az\xba8\x86HLMNP`\
-2020\x98cY\xcc\x16y\xe2\xc6\r\tWVeeaC\xb5\x8b\x91\x82\xdc\xccm\xde\xe7\xe7\
-\xd9Zo\xc8S\xf2\x12\x0cd`\xd0\xd8\xc5&\xf6\xeb\xd5\xe5t\xa0f\x06OW?\x97uN\tM\
-\x00qL\x1f\x94' )
-
-def getCheckedBitmap():
-    return wx.BitmapFromImage(getCheckedImage())
-
-def getCheckedImage():
-    stream = cStringIO.StringIO(getCheckedData())
-    return wx.ImageFromStream(stream)
-
-IMG_CATALOG['Checked'] = ImageClass()
-IMG_CATALOG['Checked'].getData = getCheckedData
-IMG_CATALOG['Checked'].getImage = getCheckedImage
-IMG_CATALOG['Checked'].getBitmap = getCheckedBitmap
-
-
-#----------------------------------------------------------------------
-def getNoFrameData():
-    return zlib.decompress(
-"x\xda\xeb\x0c\xf0s\xe7\xe5\x92\xe2b``\xe0\xf5\xf4p\t\x02\xd2< \xcc\xc1\x04$\
-o\x8a\x9f\xde\x00\xa4\x98\x8b\x9d<C888n?\xf4\x7f\x00\xe4\x9ay\xba8\x86HL]\
-\xdb\xef\xc8\xc5\xa0 \xc04\xf7\xc5\xff\xf8m\xd1\x01.\xba\x93\x9e'\x86\xac\
-\x14P\xb9\xb9O\xf0\x82\xd62\x0e\xa6\x06\xf9e\x8f;Yg\xc5F'\xd7g]\xf2\xadd;=\
-\x87S\xfe\xf3\xc7\x15\x8f\x80&0x\xba\xfa\xb9\xacsJh\x02\x00\x07\xac't" )
-
-def getNoFrameBitmap():
-    return wx.BitmapFromImage(getNoFrameImage())
-
-def getNoFrameImage():
-    stream = cStringIO.StringIO(getNoFrameData())
-    return wx.ImageFromStream(stream)
-
-IMG_CATALOG['NoFrame'] = ImageClass()
-IMG_CATALOG['NoFrame'].getData = getNoFrameData
-IMG_CATALOG['NoFrame'].getImage = getNoFrameImage
-IMG_CATALOG['NoFrame'].getBitmap = getNoFrameBitmap
-
-
-#----------------------------------------------------------------------
-def getBoxFrameData():
-    return zlib.decompress(
-"x\xda\xeb\x0c\xf0s\xe7\xe5\x92\xe2b``\xe0\xf5\xf4p\t\x02\xd2< \xcc\xc1\x04$\
-o\x8a\x9f\xde\x00\xa4\x98\x8b\x9d<C888n?\xf4\x7f\x00\xe4:{\xba8\x86HL\x9d\
-\xdbw\x91\x8bA\x81\x83\xb9\xfc\xd2\xff\xff\x9bl9\x02\x15\xd5\xdefnJ\xf6\xcb\
-\xe2\xf0|\x17'\x980W\xed\xaa\xaf\xe0\xcd:\xfd\xaa\xef\xec!/\xda.]ggaH\xfcT\
-\xbaRI\xca_\xef\xe6\x97\xf5\x9c;\xa2\x15\xfe\xbe^S\xbe\th\x1c\x83\xa7\xab\
-\x9f\xcb:\xa7\x84&\x00k\xdd.\x08" )
-
-def getBoxFrameBitmap():
-    return wx.BitmapFromImage(getBoxFrameImage())
-
-def getBoxFrameImage():
-    stream = cStringIO.StringIO(getBoxFrameData())
-    return wx.ImageFromStream(stream)
-
-IMG_CATALOG['BoxFrame'] = ImageClass()
-IMG_CATALOG['BoxFrame'].getData = getBoxFrameData
-IMG_CATALOG['BoxFrame'].getImage = getBoxFrameImage
-IMG_CATALOG['BoxFrame'].getBitmap = getBoxFrameBitmap
-
-
-#----------------------------------------------------------------------
-def getCropFrameData():
-    return zlib.decompress(
-"x\xda\xeb\x0c\xf0s\xe7\xe5\x92\xe2b``\xe0\xf5\xf4p\t\x02\xd2< \xcc\xc1\x04$\
-o\x8a\x9f\xde\x00\xa4\x98\x8b\x9d<C888n?\xf4\x7f\x00\xe4zz\xba8\x86HL\xdd;\
-\xc1\x90\xeb\x80\x03\x07K\xba\xf3\xbf\xd5\xfe\x17\xc5g;\xedh\x16i\xcf\xdc\
-\xd4z\xc2\xa8G\x81GIA\x89\xafew\xbc\xf0e\x8e]\xd7\xd3\xd2\x1aT\x16\xacj8\xf3\
-'\xa1\xca\xf9\xad\x85\xe3\xa4_1\xe7\xef~~\xcd\xedV\xc9\xf0\x7f#\xbftm\xb5\
-\x8d\t\x03\xc8TW?\x97uN\tM\x00\x9c@0\x82" )
-
-def getCropFrameBitmap():
-    return wx.BitmapFromImage(getCropFrameImage())
-
-def getCropFrameImage():
-    stream = cStringIO.StringIO(getCropFrameData())
-    return wx.ImageFromStream(stream)
-
-IMG_CATALOG['CropFrame'] = ImageClass()
-IMG_CATALOG['CropFrame'].getData = getCropFrameData
-IMG_CATALOG['CropFrame'].getImage = getCropFrameImage
-IMG_CATALOG['CropFrame'].getBitmap = getCropFrameBitmap
