@@ -6,7 +6,7 @@
 # Author:      Robin Dunn
 #
 # Created:     26-Jan-2007
-# RCS-ID:      $Id: inspection.py 58930 2009-02-16 01:13:47Z RD $
+# RCS-ID:      $Id: inspection.py 65244 2010-08-11 01:55:49Z RD $
 # Copyright:   (c) 2007 by Total Control Software
 # Licence:     wxWindows license
 #----------------------------------------------------------------------------
@@ -19,7 +19,8 @@
 import wx
 import wx.py
 import wx.stc
-import wx.aui
+#import wx.aui as aui
+import wx.lib.agw.aui as aui
 import wx.lib.utils as utils
 import sys
 import inspect
@@ -127,10 +128,10 @@ class InspectionFrame(wx.Frame):
         panel = wx.Panel(self, size=self.GetClientSize())
 
         # tell FrameManager to manage this frame
-        self.mgr = wx.aui.AuiManager(panel,
-                                     wx.aui.AUI_MGR_DEFAULT
-                                     | wx.aui.AUI_MGR_TRANSPARENT_DRAG
-                                     | wx.aui.AUI_MGR_ALLOW_ACTIVE_PANE)
+        self.mgr = aui.AuiManager(panel,
+                                  aui.AUI_MGR_DEFAULT
+                                  | aui.AUI_MGR_TRANSPARENT_DRAG
+                                  | aui.AUI_MGR_ALLOW_ACTIVE_PANE)
 
         # make the child tools
         self.tree = InspectionTree(panel, size=(100,300))
@@ -158,17 +159,17 @@ class InspectionFrame(wx.Frame):
 
         # put the chlid tools in AUI panes
         self.mgr.AddPane(self.info,
-                         wx.aui.AuiPaneInfo().Name("info").Caption("Object Info").
+                         aui.AuiPaneInfo().Name("info").Caption("Object Info").
                          CenterPane().CaptionVisible(True).
                          CloseButton(False).MaximizeButton(True)
                          )
         self.mgr.AddPane(self.tree,
-                         wx.aui.AuiPaneInfo().Name("tree").Caption("Widget Tree").
+                         aui.AuiPaneInfo().Name("tree").Caption("Widget Tree").
                          CaptionVisible(True).Left().Dockable(True).Floatable(True).
                          BestSize((280,200)).CloseButton(False).MaximizeButton(True)
                          )
         self.mgr.AddPane(self.crust,
-                         wx.aui.AuiPaneInfo().Name("crust").Caption("PyCrust").
+                         aui.AuiPaneInfo().Name("crust").Caption("PyCrust").
                          CaptionVisible(True).Bottom().Dockable(True).Floatable(True).
                          BestSize((400,200)).CloseButton(False).MaximizeButton(True)
                          )
@@ -179,6 +180,9 @@ class InspectionFrame(wx.Frame):
             config = wx.Config('wxpyinspector')
         self.config = config
         self.Bind(wx.EVT_CLOSE, self.OnClose)
+        if self.Parent:
+            tlw = self.Parent.GetTopLevelParent()
+            tlw.Bind(wx.EVT_CLOSE, self.OnClose)
         self.LoadSettings(self.config)
         self.crust.shell.lineNumbers = False
         self.crust.shell.setDisplayLineNumbers(False)
@@ -254,13 +258,17 @@ class InspectionFrame(wx.Frame):
 
     def OnClose(self, evt):
         self.SaveSettings(self.config)
-        self.mgr.UnInit()
-        del self.mgr
         evt.Skip()
+        if hasattr(self, 'mgr'):
+            self.mgr.UnInit()
+            del self.mgr
+            if self.Parent:
+                tlw = self.Parent.GetTopLevelParent()
+                tlw.Unbind(wx.EVT_CLOSE, handler=self.OnClose)
 
 
     def UpdateInfo(self):
-        self.info.Update(self.obj)
+        self.info.UpdateInfo(self.obj)
 
 
     def SetObj(self, obj):
@@ -348,8 +356,9 @@ class InspectionFrame(wx.Frame):
             
     def OnWatchEventsUI(self, evt):
         item = self.tree.GetSelection()
-        obj = self.tree.GetItemPyData(item)
-        evt.Enable(isinstance(obj, wx.Window))
+        if item:
+            obj = self.tree.GetItemPyData(item)
+            evt.Enable(isinstance(obj, wx.Window))
 
 
     def OnToggleFilling(self, evt):
@@ -409,7 +418,7 @@ INCLUDE_INSPECTOR = True
 
 USE_CUSTOMTREECTRL = False
 if USE_CUSTOMTREECTRL:
-    import wx.lib.customtreectrl as CT
+    import wx.lib.agw.customtreectrl as CT
     TreeBaseClass = CT.CustomTreeCtrl
 else:
     TreeBaseClass = wx.TreeCtrl
@@ -431,7 +440,7 @@ class InspectionTree(TreeBaseClass):
             self.SetWindowVariant(wx.WINDOW_VARIANT_SMALL)
 
 
-    def BuildTree(self, startWidget, includeSizers=False):
+    def BuildTree(self, startWidget, includeSizers=False, expandFrame=False):
         if self.GetCount():
             self.DeleteAllItems()
             self.roots = []
@@ -448,12 +457,13 @@ class InspectionTree(TreeBaseClass):
         # Expand the subtree containing the startWidget, and select it.
         if not startWidget or not isinstance(startWidget, wx.Window):
             startWidget = wx.GetApp().GetTopWindow()
-        top = wx.GetTopLevelParent(startWidget)
-        topItem = self.FindWidgetItem(top)
-        if topItem:
-            self.ExpandAllChildren(topItem)
-        self.SelectObj(startWidget)
+        if expandFrame:
+            top = wx.GetTopLevelParent(startWidget)
+            topItem = self.FindWidgetItem(top)
+            if topItem:
+                self.ExpandAllChildren(topItem)
         self.built = True
+        self.SelectObj(startWidget)
 
 
     def _AddWidget(self, parentItem, widget, includeSizers):
@@ -469,7 +479,9 @@ class InspectionTree(TreeBaseClass):
         # Add any children not in the sizer, or all children if we're
         # not showing the sizers
         for child in widget.GetChildren():
-            if not child in widgetsInSizer and not child.IsTopLevel():
+            if (not child in widgetsInSizer and
+                (not child.IsTopLevel() or
+                 isinstance(child, wx.PopupWindow))):
                 self._AddWidget(item, child, includeSizers)
 
         return item
@@ -526,6 +538,7 @@ class InspectionTree(TreeBaseClass):
         """
         return "%s (\"%s\")" % (widget.__class__.__name__, widget.GetName())
 
+    
     def GetTextForSizer(self, sizer):
         """
         Returns the string to be used in the tree for a sizer
@@ -541,8 +554,10 @@ class InspectionTree(TreeBaseClass):
 
 
     def OnSelectionChanged(self, evt):
-        obj = self.GetItemPyData(evt.GetItem())
-        self.toolFrame.SetObj(obj)
+        item = evt.GetItem()
+        if item:
+            obj = self.GetItemPyData(item)
+            self.toolFrame.SetObj(obj)
 
 
 #---------------------------------------------------------------------------
@@ -576,7 +591,7 @@ class InspectionInfoPanel(wx.stc.StyledTextCtrl):
         config.WriteInt('View/Zoom/Info', self.GetZoom())
 
 
-    def Update(self, obj):
+    def UpdateInfo(self, obj):
         st = []
         if not obj:
             st.append("Item is None or has been destroyed.")
@@ -603,6 +618,24 @@ class InspectionInfoPanel(wx.stc.StyledTextCtrl):
 
 
     def FmtWidget(self, obj):
+        def _countChildren(children):
+            count = 0
+            for child in children:
+                if not child.IsTopLevel():
+                    count += 1
+                    count += _countChildren(child.GetChildren())
+            return count
+        def _countAllChildren(children):
+            count = 0
+            for child in children:
+                count += 1
+                count += _countAllChildren(child.GetChildren())
+            return count
+
+        count = len([c for c in obj.GetChildren() if not c.IsTopLevel()])
+        rcount = _countChildren(obj.GetChildren())
+        tlwcount = _countAllChildren(obj.GetChildren())
+
         st = ["Widget:"]
         st.append(self.Fmt('name',        obj.GetName()))
         st.append(self.Fmt('class',       obj.__class__))
@@ -626,7 +659,12 @@ class InspectionInfoPanel(wx.stc.StyledTextCtrl):
         if hasattr(obj, 'GetTitle'):
             st.append(self.Fmt('title',   obj.GetTitle()))
         if hasattr(obj, 'GetValue'):
-            st.append(self.Fmt('value',   obj.GetValue()))
+            try:
+                st.append(self.Fmt('value',   obj.GetValue()))
+            except:
+                pass
+        st.append('    child count = %d (direct)  %d (recursive)  %d (include TLWs)' %
+                  (count, rcount, tlwcount))
         if obj.GetContainingSizer() is not None:
             st.append('')
             sizer = obj.GetContainingSizer()
@@ -752,10 +790,21 @@ class _InspectionHighlighter(object):
     # should non TLWs be flashed too?  Otherwise use a highlight rectangle
     flashAll = False
 
-    color1 = 'red'      # for widgets and sizers
-    color2 = 'red'      # for item boundaries in sizers
-    color3 = '#00008B'  # for items in sizers
+    color1 = 'red'         # for widgets and sizers
+    color2 = 'red'         # for item boundaries in sizers
+    color3 = '#00008B'     # for items in sizers
 
+    highlightTime = 3000   # how long to display the highlights
+    
+                           # how to draw it
+    useOverlay = 'wxMac' in wx.PlatformInfo
+    
+    
+    def __init__(self):
+        if self.useOverlay:
+            self.overlay = wx.Overlay()
+            
+    
     def HighlightCurrentItem(self, tree):
         """
         Draw a highlight rectangle around the item represented by the
@@ -786,9 +835,9 @@ class _InspectionHighlighter(object):
             self.FlickerTLW(win)
             return
         else:
-            pos, useWinDC = self.FindHighlightPos(tlw, win.ClientToScreen((0,0)))
+            pos = self.FindHighlightPos(tlw, win.ClientToScreen((0,0)))
             rect.SetPosition(pos)
-            self.DoHighlight(tlw, rect, self.color1, useWinDC)
+            self.DoHighlight(tlw, rect, self.color1)
 
 
     def HighlightSizerItem(self, item, sizer, penWidth=2):
@@ -796,11 +845,11 @@ class _InspectionHighlighter(object):
         tlw = win.GetTopLevelParent()
         rect = item.GetRect()
         pos = rect.GetPosition()
-        pos, useWinDC = self.FindHighlightPos(tlw, win.ClientToScreen(pos))
+        pos = self.FindHighlightPos(tlw, win.ClientToScreen(pos))
         rect.SetPosition(pos)
         if rect.width < 1: rect.width = 1
         if rect.width < 1: rect.width = 1
-        self.DoHighlight(tlw, rect, self.color1, useWinDC, penWidth)
+        self.DoHighlight(tlw, rect, self.color1, penWidth)
 
 
     def HighlightSizer(self, sizer):
@@ -808,9 +857,9 @@ class _InspectionHighlighter(object):
         win = sizer.GetContainingWindow()
         tlw = win.GetTopLevelParent()
         pos = sizer.GetPosition()
-        pos, useWinDC = self.FindHighlightPos(tlw, win.ClientToScreen(pos))
+        pos = self.FindHighlightPos(tlw, win.ClientToScreen(pos))
         rect = wx.RectPS(pos, sizer.GetSize())
-        dc = self.DoHighlight(tlw, rect, self.color1, useWinDC)
+        dc, dco = self.DoHighlight(tlw, rect, self.color1)
 
         # Now highlight the actual items within the sizer.  This may
         # get overdrawn by the code below for item boundaries, but if
@@ -835,8 +884,13 @@ class _InspectionHighlighter(object):
         # differently.
         dc.SetPen(wx.Pen(self.color2, 1))
 
+        if isinstance(sizer, wx.WrapSizer):
+            for item in sizer.GetChildren():
+                ir = self.AdjustRect(tlw, win, item.Rect)
+                dc.DrawRectangleRect(ir)
+        
         # wx.BoxSizer, wx.StaticBoxSizer
-        if isinstance(sizer, wx.BoxSizer):
+        elif isinstance(sizer, wx.BoxSizer):
             # NOTE: we have to do some reverse-engineering here for
             # borders because the sizer and sizer item don't give us
             # enough information to know for sure where item
@@ -909,33 +963,37 @@ class _InspectionHighlighter(object):
 
         # Anything else is probably a custom sizer, just highlight the items
         else:
-            del dc
+            del dc, odc
             for item in sizer.GetChildren():
                 self.HighlightSizerItem(item, sizer, 1)
 
 
     def FindHighlightPos(self, tlw, pos):
-        if 'wxMac' in wx.PlatformInfo:
-            # We'll be using a WindowDC in this case so adjust the
+        if self.useOverlay:
+            # We'll be using a ClientDC in this case so adjust the
             # position accordingly
             pos = tlw.ScreenToClient(pos)
-            useWinDC = True
-        else:
-            useWinDC = False
-        return pos, useWinDC
+        return pos
 
 
     def AdjustRect(self, tlw, win,  rect):
-        pos, j = self.FindHighlightPos(tlw, win.ClientToScreen(rect.Position))
+        pos = self.FindHighlightPos(tlw, win.ClientToScreen(rect.Position))
         rect.Position = pos
         return wx.RectPS(pos, rect.Size)
 
 
-    def DoHighlight(self, tlw, rect, colour, useWinDC, penWidth=2):
-        if useWinDC:
-            dc = wx.WindowDC(tlw)
+    def DoHighlight(self, tlw, rect, colour, penWidth=2):
+        if not tlw.IsFrozen():
+            tlw.Freeze()
+
+        if self.useOverlay:
+            dc = wx.ClientDC(tlw)
+            dco = wx.DCOverlay(self.overlay, dc)
+            dco.Clear()
         else:
             dc = wx.ScreenDC()
+            dco = None
+            
         dc.SetPen(wx.Pen(colour, penWidth))
         dc.SetBrush(wx.TRANSPARENT_BRUSH)
 
@@ -943,13 +1001,28 @@ class _InspectionHighlighter(object):
         dc.DrawRectangleRect(drawRect)
 
         drawRect.Inflate(2,2)
-        if not useWinDC:
+        if not self.useOverlay:
             pos = tlw.ScreenToClient(drawRect.GetPosition())
             drawRect.SetPosition(pos)
-        wx.CallLater(3000, tlw.RefreshRect, drawRect)
+        wx.CallLater(self.highlightTime, self.DoUnhighlight, tlw, drawRect)
 
-        return dc
+        return dc, dco
 
+    
+    def DoUnhighlight(self, tlw, rect):
+        if not tlw:
+            return
+        if tlw.IsFrozen():
+            tlw.Thaw()
+        if self.useOverlay:
+            dc = wx.ClientDC(tlw)
+            dco = wx.DCOverlay(self.overlay, dc)
+            dco.Clear()
+            del dc, dco
+            self.overlay.Reset()
+        else:
+            tlw.RefreshRect(rect)
+            
 
     def FlickerTLW(self, tlw):
         """
@@ -961,6 +1034,7 @@ class _InspectionHighlighter(object):
         tlw.Hide()
         self.cl = wx.CallLater(300, self._Toggle, tlw)
 
+        
     def _Toggle(self, tlw):
         if tlw.IsShown():
             tlw.Hide()
