@@ -6,7 +6,7 @@
 # Author:       Pierre Hjlm (from C++ original by Julian Smart)
 #
 # Created:      2004-05-08
-# RCS-ID:       $Id: _lines.py 44515 2007-02-16 23:32:37Z RD $
+# RCS-ID:       $Id: _lines.py 57920 2009-01-09 04:18:29Z RD $
 # Copyright:    (c) 2004 Pierre Hjlm - 1998 Julian Smart
 # Licence:      wxWindows license
 #----------------------------------------------------------------------------
@@ -488,6 +488,11 @@ class LineShape(Shape):
                 self._labelObjects[i].RemoveFromCanvas(self._canvas)
         self.ClearArrowsAtPosition(-1)
 
+    # Override Delete to unlink before deleting 
+    def Delete(self): 
+        self.Unlink() 
+        Shape.Delete(self)
+        
     def SetEnds(self, x1, y1, x2, y2):
         """Set the end positions of the line."""
         self._lineControlPoints[0] = wx.RealPoint(x1, y1)
