@@ -6,7 +6,7 @@
 # Author:      Robin Dunn
 #
 # Created:     3-Sept-2008
-# RCS-ID:      $Id: wxcairo.py 63524 2010-02-19 22:14:22Z RD $
+# RCS-ID:      $Id: wxcairo.py 63525 2010-02-19 22:16:18Z RD $
 # Copyright:   (c) 2008 by Total Control Software
 # Licence:     wxWindows license
 #----------------------------------------------------------------------
@@ -138,12 +138,9 @@ def FontFaceFromFont(font):
     """
     
     if 'wxMac' in wx.PlatformInfo:
-        # NOTE: This currently uses the ATSUFontID, but wxMac may
-        # someday transition to the CGFont.  If so, this API call will
-        # need to be changed.
         fontfaceptr = voidp(
-            cairoLib.cairo_quartz_font_face_create_for_atsu_font_id(
-                font.MacGetATSUFontID()) )
+            cairoLib.cairo_quartz_font_face_create_for_cgfont(
+                voidp(font.OSXGetCGFont())) )
         fontface = pycairoAPI.FontFace_FromFontFace(fontfaceptr)
 
 
