@@ -5,8 +5,8 @@ based on wxPython's wxStyledTextCtrl.
 Sponsored by Orbtech - Your source for Python programming expertise."""
 
 __author__ = "Patrick K. O'Brien <pobrien@orbtech.com>"
-__cvsid__ = "$Id: shell.py,v 1.9.2.10 2006/06/10 23:46:43 RD Exp $"
-__revision__ = "$Revision: 1.9.2.10 $"[11:-2]
+__cvsid__ = "$Id: shell.py 63480 2010-02-14 05:34:39Z RD $"
+__revision__ = "$Revision: 63480 $"[11:-2]
 
 import wx
 from wx import stc
@@ -24,8 +24,13 @@ from pseudo import PseudoFileIn
 from pseudo import PseudoFileOut
 from pseudo import PseudoFileErr
 from version import VERSION
+from magic import magic
+from path import ls,cd,pwd,sx
 
 sys.ps3 = '<-- '  # Input prompt.
+USE_MAGIC=True
+# Force updates from long-running commands after this many seconds
+PRINT_UPDATE_MAX_TIME=2
 
 NAVKEYS = (wx.WXK_END, wx.WXK_LEFT, wx.WXK_RIGHT,
            wx.WXK_UP, wx.WXK_DOWN, wx.WXK_PRIOR, wx.WXK_NEXT)
@@ -217,6 +222,16 @@ class ShellFacade:
         return list
 
 
+#DNM
+DISPLAY_TEXT="""
+Author: %r
+Py Version: %s
+Py Shell Revision: %s
+Py Interpreter Revision: %s
+Python Version: %s
+wxPython Version: %s
+wxPython PlatformInfo: %s
+Platform: %s"""
 
 class Shell(editwindow.EditWindow):
     """Shell based on StyledTextCtrl."""
@@ -272,6 +287,9 @@ class Shell(editwindow.EditWindow):
 
         # Keep track of multi-line commands.
         self.more = False
+        
+        # For use with forced updates during long-running scripts
+        self.lastUpdate=None
 
         # Create the command history.  Commands are added into the
         # front of the list (ie. at index 0) as they are entered.
@@ -292,6 +310,22 @@ class Shell(editwindow.EditWindow):
         self.Bind(wx.EVT_CHAR, self.OnChar)
         self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
 
+        # Assign handler for the context menu
+        self.Bind(wx.EVT_CONTEXT_MENU, self.OnContextMenu)
+        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUI)
+        
+        # Assign handlers for edit events
+        self.Bind(wx.EVT_MENU, lambda evt: self.Cut(), id=wx.ID_CUT)
+        self.Bind(wx.EVT_MENU, lambda evt: self.Copy(), id=wx.ID_COPY)
+        self.Bind(wx.EVT_MENU, lambda evt: self.CopyWithPrompts(), id=frame.ID_COPY_PLUS)
+        self.Bind(wx.EVT_MENU, lambda evt: self.Paste(), id=wx.ID_PASTE)
+        self.Bind(wx.EVT_MENU, lambda evt: self.PasteAndRun(), id=frame.ID_PASTE_PLUS)
+        self.Bind(wx.EVT_MENU, lambda evt: self.SelectAll(), id=wx.ID_SELECTALL)
+        self.Bind(wx.EVT_MENU, lambda evt: self.Clear(), id=wx.ID_CLEAR)
+        self.Bind(wx.EVT_MENU, lambda evt: self.Undo(), id=wx.ID_UNDO)
+        self.Bind(wx.EVT_MENU, lambda evt: self.Redo(), id=wx.ID_REDO)
+        
+
         # Assign handler for idle time.
         self.waiting = False
         self.Bind(wx.EVT_IDLE, self.OnIdle)
@@ -343,11 +377,12 @@ class Shell(editwindow.EditWindow):
     def showIntro(self, text=''):
         """Display introductory text in the shell."""
         if text:
-            if not text.endswith(os.linesep):
-                text += os.linesep
             self.write(text)
         try:
-            self.write(self.interp.introText)
+            if self.interp.introText:
+                if text and not text.endswith(os.linesep):
+                    self.write(os.linesep)
+                self.write(self.interp.introText)
         except AttributeError:
             pass
 
@@ -359,6 +394,10 @@ class Shell(editwindow.EditWindow):
         import __builtin__
         __builtin__.close = __builtin__.exit = __builtin__.quit = \
             'Click on the close button to leave the application.'
+        __builtin__.cd = cd
+        __builtin__.ls = ls
+        __builtin__.pwd = pwd
+        __builtin__.sx = sx
 
 
     def quit(self):
@@ -388,15 +427,8 @@ class Shell(editwindow.EditWindow):
 
     def about(self):
         """Display information about Py."""
-        text = """
-Author: %r
-Py Version: %s
-Py Shell Revision: %s
-Py Interpreter Revision: %s
-Python Version: %s
-wxPython Version: %s
-wxPython PlatformInfo: %s
-Platform: %s""" % \
+        #DNM
+        text = DISPLAY_TEXT % \
         (__author__, VERSION, self.revision, self.interp.revision,
          sys.version.split()[0], wx.VERSION_STRING, str(wx.PlatformInfo),
          sys.platform)
@@ -417,7 +449,7 @@ Platform: %s""" % \
         # commands/responses.
         if not self.CanEdit():
             return
-        key = event.KeyCode()
+        key = event.GetKeyCode()
         currpos = self.GetCurrentPos()
         stoppos = self.promptPosEnd
         # Return (Enter) needs to be ignored in this handler.
@@ -452,7 +484,7 @@ Platform: %s""" % \
     def OnKeyDown(self, event):
         """Key down event handler."""
 
-        key = event.KeyCode()
+        key = event.GetKeyCode()
         # If the auto-complete window is up let it do its thing.
         if self.AutoCompActive():
             event.Skip()
@@ -480,13 +512,16 @@ Platform: %s""" % \
                     li += 1
                 endP = self.GetLineEndPosition(li-1)
                 self.ShowLines(li0, li-1)
-                self.SetSelection( startP, endP ) # select reappearing text to allow "hide again"
+                # select reappearing text to allow "hide again"
+                self.SetSelection( startP, endP )
                 return
             startP,endP = self.GetSelection()
             endP-=1
-            startL,endL = self.LineFromPosition(startP), self.LineFromPosition(endP)
+            startL = self.LineFromPosition(startP)
+            endL = self.LineFromPosition(endP)
 
-            if endL == self.LineFromPosition(self.promptPosEnd): # never hide last prompt
+            # never hide last prompt
+            if endL == self.LineFromPosition(self.promptPosEnd):
                 endL -= 1
 
             m = self.MarkerGet(startL)
@@ -497,7 +532,9 @@ Platform: %s""" % \
         if key == wx.WXK_F12: #seb
             if self.noteMode:
                 # self.promptPosStart not used anyway - or ? 
-                self.promptPosEnd = self.PositionFromLine( self.GetLineCount()-1 ) + len(str(sys.ps1))
+                self.promptPosEnd = \
+                   self.PositionFromLine( self.GetLineCount()-1 ) + \
+                   len(str(sys.ps1))
                 self.GotoLine(self.GetLineCount())
                 self.GotoPos(self.promptPosEnd)
                 self.prompt()  #make sure we have a prompt
@@ -517,7 +554,8 @@ Platform: %s""" % \
 
         # Return (Enter) is used to submit a command to the
         # interpreter.
-        if (not controlDown and not shiftDown and not altDown) and key in [wx.WXK_RETURN, wx.WXK_NUMPAD_ENTER]:
+        if (not controlDown and not shiftDown and not altDown) and \
+           key in [wx.WXK_RETURN, wx.WXK_NUMPAD_ENTER]:
             if self.CallTipActive():
                 self.CallTipCancel()
             self.processLine()
@@ -583,7 +621,7 @@ Platform: %s""" % \
             self.CopyWithPromptsPrefixed()
 
         # Home needs to be aware of the prompt.
-        elif key == wx.WXK_HOME:
+        elif controlDown and key == wx.WXK_HOME:
             home = self.promptPosEnd
             if currpos > home:
                 self.SetCurrentPos(home)
@@ -593,6 +631,23 @@ Platform: %s""" % \
             else:
                 event.Skip()
 
+        # Home needs to be aware of the prompt.
+        elif key == wx.WXK_HOME:
+            home = self.promptPosEnd
+            if currpos > home:
+                [line_str,line_len] = self.GetCurLine()
+                pos=self.GetCurrentPos()
+                if line_str[:4] in [sys.ps1,sys.ps2,sys.ps3]:
+                    self.SetCurrentPos(pos+4-line_len)
+                    #self.SetCurrentPos(home)
+                    if not selecting and not shiftDown:
+                        self.SetAnchor(pos+4-line_len)
+                        self.EnsureCaretVisible()
+                else:
+                    event.Skip()
+            else:
+                event.Skip()
+        
         #
         # The following handlers modify text, so we need to see if
         # there is a selection that includes text prior to the prompt.
@@ -615,21 +670,21 @@ Platform: %s""" % \
             self.PasteAndRun()
             
         # Replace with the previous command from the history buffer.
-        elif (controlDown and key == wx.WXK_UP) \
+        elif (controlDown and not shiftDown and key == wx.WXK_UP) \
                  or (altDown and key in (ord('P'), ord('p'))):
             self.OnHistoryReplace(step=+1)
             
         # Replace with the next command from the history buffer.
-        elif (controlDown and key == wx.WXK_DOWN) \
+        elif (controlDown and not shiftDown and key == wx.WXK_DOWN) \
                  or (altDown and key in (ord('N'), ord('n'))):
             self.OnHistoryReplace(step=-1)
             
         # Insert the previous command from the history buffer.
-        elif (shiftDown and key == wx.WXK_UP) and self.CanEdit():
+        elif (controlDown and shiftDown and key == wx.WXK_UP) and self.CanEdit():
             self.OnHistoryInsert(step=+1)
             
         # Insert the next command from the history buffer.
-        elif (shiftDown and key == wx.WXK_DOWN) and self.CanEdit():
+        elif (controlDown and shiftDown and key == wx.WXK_DOWN) and self.CanEdit():
             self.OnHistoryInsert(step=-1)
             
         # Search up the history for the text in front of the cursor.
@@ -654,10 +709,15 @@ Platform: %s""" % \
         
         # Don't allow line deletion.
         elif controlDown and key in (ord('L'), ord('l')):
+            # TODO : Allow line deletion eventually...
+            #event.Skip()
             pass
 
         # Don't allow line transposition.
         elif controlDown and key in (ord('T'), ord('t')):
+            # TODO : Allow line transposition eventually...
+            # TODO : Will have to adjust markers accordingly and test if allowed...
+            #event.Skip()
             pass
 
         # Basic navigation keys should work anywhere.
@@ -904,9 +964,16 @@ Platform: %s""" % \
         """Send command to the interpreter for execution."""
         if not silent:
             self.write(os.linesep)
+        
+        #DNM
+        if USE_MAGIC:
+            command=magic(command)
+         
         busy = wx.BusyCursor()
         self.waiting = True
+        self.lastUpdate=None
         self.more = self.interp.push(command)
+        self.lastUpdate=None
         self.waiting = False
         del busy
         if not self.more:
@@ -924,7 +991,7 @@ Platform: %s""" % \
         and (len(self.history) == 0 or command != self.history[0]):
             self.history.insert(0, command)
             dispatcher.send(signal="Shell.addHistory", command=command)
-
+    
     def write(self, text):
         """Display text in the shell.
 
@@ -932,7 +999,14 @@ Platform: %s""" % \
         text = self.fixLineEndings(text)
         self.AddText(text)
         self.EnsureCaretVisible()
-
+        
+        if self.waiting:
+            if self.lastUpdate==None:
+                self.lastUpdate=time.time()
+            if time.time()-self.lastUpdate > PRINT_UPDATE_MAX_TIME:
+                self.Update()
+                self.lastUpdate=time.time()
+    
     def fixLineEndings(self, text):
         """Return text with line endings replaced by OS-specific endings."""
         lines = text.split('\r\n')
@@ -970,9 +1044,34 @@ Platform: %s""" % \
             self.promptPosEnd = self.GetCurrentPos()
             # Keep the undo feature from undoing previous responses.
             self.EmptyUndoBuffer()
-        # XXX Add some autoindent magic here if more.
+        
         if self.more:
-            self.write(' '*4)  # Temporary hack indentation.
+            line_num=self.GetCurrentLine()
+            currentLine=self.GetLine(line_num)
+            previousLine=self.GetLine(line_num-1)[len(prompt):]
+            pstrip=previousLine.strip()
+            lstrip=previousLine.lstrip()
+            
+            # Get the first alnum word:
+            first_word=[]
+            for i in pstrip:
+                if i.isalnum():
+                    first_word.append(i)
+                else:
+                    break
+            first_word = ''.join(first_word)
+            
+            if pstrip == '':
+                # because it is all whitespace!
+                indent=previousLine.strip('\n').strip('\r')
+            else:
+                indent=previousLine[:(len(previousLine)-len(lstrip))]
+                if pstrip[-1]==':' and \
+                    first_word in ['if','else','elif','for','while',
+                                   'def','class','try','except','finally']:
+                    indent+=' '*4
+            
+            self.write(indent)
         self.EnsureCaretVisible()
         self.ScrollToColumn(0)
 
@@ -1077,15 +1176,14 @@ Platform: %s""" % \
             dispatcher.send(signal='Shell.calltip', sender=self, calltip=tip)
         if not self.autoCallTip and not forceCallTip:
             return
+        startpos = self.GetCurrentPos()
         if argspec and insertcalltip and self.callTipInsert:
-            startpos = self.GetCurrentPos()
             self.write(argspec + ')')
             endpos = self.GetCurrentPos()
-            self.SetSelection(endpos, startpos)
+            self.SetSelection(startpos, endpos)
         if tip:
-            curpos = self.GetCurrentPos()
-            tippos = curpos - (len(name) + 1)
-            fallback = curpos - self.GetColumn(curpos)
+            tippos = startpos - (len(name) + 1)
+            fallback = startpos - self.GetColumn(startpos)
             # In case there isn't enough room, only go back to the
             # fallback.
             tippos = max(tippos, fallback)
@@ -1134,7 +1232,8 @@ Platform: %s""" % \
                 if ctindex != -1 and not self.CallTipActive():
                     #insert calltip, if current pos is '(', otherwise show it only
                     self.autoCallTipShow(ctips[:ctindex + 1], 
-                        self.GetCharAt(currpos - 1) == ord('(') and self.GetCurrentPos() == self.GetTextLength(),
+                        self.GetCharAt(currpos - 1) == ord('(') and \
+                           self.GetCurrentPos() == self.GetTextLength(),
                         True)
                 
 
@@ -1335,40 +1434,85 @@ Platform: %s""" % \
 
 
     def LoadSettings(self, config):
-        self.autoComplete              = config.ReadBool('Options/AutoComplete', True)
-        self.autoCompleteIncludeMagic  = config.ReadBool('Options/AutoCompleteIncludeMagic', True)
-        self.autoCompleteIncludeSingle = config.ReadBool('Options/AutoCompleteIncludeSingle', True)
-        self.autoCompleteIncludeDouble = config.ReadBool('Options/AutoCompleteIncludeDouble', True)
+        self.autoComplete              = \
+            config.ReadBool('Options/AutoComplete', True)
+        self.autoCompleteIncludeMagic  = \
+            config.ReadBool('Options/AutoCompleteIncludeMagic', True)
+        self.autoCompleteIncludeSingle = \
+            config.ReadBool('Options/AutoCompleteIncludeSingle', True)
+        self.autoCompleteIncludeDouble = \
+            config.ReadBool('Options/AutoCompleteIncludeDouble', True)
 
         self.autoCallTip = config.ReadBool('Options/AutoCallTip', True)
         self.callTipInsert = config.ReadBool('Options/CallTipInsert', True)
         self.SetWrapMode(config.ReadBool('View/WrapMode', True))
 
-        useAA = config.ReadBool('Options/UseAntiAliasing', self.GetUseAntiAliasing())
-        self.SetUseAntiAliasing(useAA)
         self.lineNumbers = config.ReadBool('View/ShowLineNumbers', True)
         self.setDisplayLineNumbers (self.lineNumbers)
         zoom = config.ReadInt('View/Zoom/Shell', -99)
         if zoom != -99:
             self.SetZoom(zoom)
 
-
     
     def SaveSettings(self, config):
         config.WriteBool('Options/AutoComplete', self.autoComplete)
-        config.WriteBool('Options/AutoCompleteIncludeMagic', self.autoCompleteIncludeMagic)
-        config.WriteBool('Options/AutoCompleteIncludeSingle', self.autoCompleteIncludeSingle)
-        config.WriteBool('Options/AutoCompleteIncludeDouble', self.autoCompleteIncludeDouble)
+        config.WriteBool('Options/AutoCompleteIncludeMagic',
+                         self.autoCompleteIncludeMagic)
+        config.WriteBool('Options/AutoCompleteIncludeSingle',
+                         self.autoCompleteIncludeSingle)
+        config.WriteBool('Options/AutoCompleteIncludeDouble',
+                         self.autoCompleteIncludeDouble)
         config.WriteBool('Options/AutoCallTip', self.autoCallTip)
         config.WriteBool('Options/CallTipInsert', self.callTipInsert)
-        config.WriteBool('Options/UseAntiAliasing', self.GetUseAntiAliasing())
         config.WriteBool('View/WrapMode', self.GetWrapMode())
         config.WriteBool('View/ShowLineNumbers', self.lineNumbers)
         config.WriteInt('View/Zoom/Shell', self.GetZoom())
 
+    def GetContextMenu(self):
+        """
+            Create and return a context menu for the shell.
+            This is used instead of the scintilla default menu
+            in order to correctly respect our immutable buffer.
+        """
+        menu = wx.Menu()
+        menu.Append(wx.ID_UNDO, "Undo")
+        menu.Append(wx.ID_REDO, "Redo")
+        
+        menu.AppendSeparator()
+        
+        menu.Append(wx.ID_CUT, "Cut")
+        menu.Append(wx.ID_COPY, "Copy")
+        menu.Append(frame.ID_COPY_PLUS, "Copy Plus")
+        menu.Append(wx.ID_PASTE, "Paste")
+        menu.Append(frame.ID_PASTE_PLUS, "Paste Plus")
+        menu.Append(wx.ID_CLEAR, "Clear")
+        
+        menu.AppendSeparator()
+        
+        menu.Append(wx.ID_SELECTALL, "Select All")
+        return menu
+        
+    def OnContextMenu(self, evt):
+        menu = self.GetContextMenu()
+        self.PopupMenu(menu)
+        
+    def OnUpdateUI(self, evt):
+        id = evt.Id
+        if id in (wx.ID_CUT, wx.ID_CLEAR):
+            evt.Enable(self.CanCut())
+        elif id in (wx.ID_COPY, frame.ID_COPY_PLUS):
+            evt.Enable(self.CanCopy())
+        elif id in (wx.ID_PASTE, frame.ID_PASTE_PLUS):
+            evt.Enable(self.CanPaste())
+        elif id == wx.ID_UNDO:
+            evt.Enable(self.CanUndo())
+        elif id == wx.ID_REDO:
+            evt.Enable(self.CanRedo())
+        
 
 
-## NOTE: The DnD of file names is disabled until I can figure out how
+
+## NOTE: The DnD of file names is disabled until we can figure out how
 ## best to still allow DnD of text.
 
 
@@ -1428,4 +1572,3 @@ Platform: %s""" % \
 ##             self.shell.SetSelection( pos, pos )
 
 ##         return result
-    
