@@ -8,10 +8,11 @@
 ###############################################################################
 # Plugin Metadata
 """Run the script in the current buffer"""
+__version__ = "1.7"
 
 __author__ = "Cody Precord"
-__svnid__ = "$Id: __init__.py 67241 2011-03-18 22:37:41Z CJP $"
-__revision__ = "$Revision: 67241 $"
+__svnid__ = "$Id: __init__.py 64526 2010-06-08 20:21:47Z CJP $"
+__revision__ = "$Revision: 64526 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -26,7 +27,6 @@ import ed_glob
 import iface
 import plugin
 import ed_msg
-import profiler
 import util
 import syntax.synglob as synglob
 from ed_menu import EdMenuBar
@@ -44,7 +44,9 @@ class Launch(plugin.Plugin):
     INSTALLED = False
     SHELF = None
 
-    __name__ = u'Launch'
+    @property
+    def __name__(self):
+        return u'Launch'
 
     def AllowMultiple(self):
         """Launch allows multiple instances"""
@@ -66,29 +68,27 @@ class Launch(plugin.Plugin):
 
     def GetId(self):
         """The unique identifier of this plugin"""
-        return Launch.ID_LAUNCH
+        return self.ID_LAUNCH
 
     def GetMenuEntry(self, menu):
         """This plugins menu entry"""
-        item = wx.MenuItem(menu, Launch.ID_LAUNCH, Launch.__name__, 
+        item = wx.MenuItem(menu, self.ID_LAUNCH, self.__name__, 
                            _("Run script from current buffer"))
         item.SetBitmap(self.GetBitmap())
         return item
 
     def GetMinVersion(self):
-        return "0.6.20"
+        return "0.4.89"
 
     def GetName(self):
         """The name of this plugin"""
-        return Launch.__name__
+        return self.__name__
 
     def InstallComponents(self, mainw):
         """Install extra menu components
         param mainw: MainWindow Instance
 
         """
-        # Delete obsolete configuration from older versions
-        profiler.Profile_Del('Launch.Config') # New config is Launch.Config2
         tmenu = mainw.GetMenuBar().GetMenuByName("tools")
         tmenu.Insert(0, ed_glob.ID_RUN_LAUNCH, _("Run") + \
                      EdMenuBar.keybinder.GetBinding(ed_glob.ID_RUN_LAUNCH),
