@@ -9,8 +9,9 @@
 
 """Launch User Interface"""
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: launch.py 67403 2011-04-06 13:54:19Z CJP $"
-__revision__ = "$Revision: 67403 $"
+__svnid__ = "$Id: launch.py 64556 2010-06-10 14:57:09Z CJP $"
+__revision__ = "$Revision: 64556 $"
+__version__ = "0.1"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -31,14 +32,16 @@ import ed_txt
 import ed_msg
 import ebmlib
 import eclib
-import ed_basewin
 
 #-----------------------------------------------------------------------------#
 # Globals
+ID_SETTINGS = wx.NewId()
 ID_EXECUTABLE = wx.NewId()
 ID_ARGS = wx.NewId()
+ID_RUN = wx.NewId()
 
 # Profile Settings Key
+LAUNCH_KEY = 'Launch.Config'
 #LAUNCH_PREFS = 'Launch.Prefs' # defined in cfgdlg
 
 # Custom Messages
@@ -53,6 +56,10 @@ REQUEST_RELAUNCH = 'Launch.CanRelaunch'
 _ = wx.GetTranslation
 #-----------------------------------------------------------------------------#
 
+def OnStoreConfig(msg):
+    Profile_Set(LAUNCH_KEY, handlers.GetState())
+ed_msg.Subscribe(OnStoreConfig, cfgdlg.EDMSG_LAUNCH_CFG_EXIT)
+
 def OnRegisterHandler(msg):
     """Register a custom handler
     @param msg: dict(xml=xml_str, loaded=bool)
@@ -67,52 +74,58 @@ ed_msg.Subscribe(OnRegisterHandler, MSG_LAUNCH_REGISTER)
 
 #-----------------------------------------------------------------------------#
 
-class LaunchWindow(ed_basewin.EdBaseCtrlBox):
+class LaunchWindow(eclib.ControlBox):
     """Control window for showing and running scripts"""
     def __init__(self, parent):
-        super(LaunchWindow, self).__init__(parent)
+        eclib.ControlBox.__init__(self, parent)
 
         # Attributes
         self._log = wx.GetApp().GetLog()
-        self._mw = ed_basewin.FindMainWindow(self)
+        self._mw = self.__FindMainWindow()
         self._buffer = OutputDisplay(self)
         self._fnames = list()
-        self._run = None        # Created in __DoLayout
-        self._pbtn = None       # Created in __DoLayout
-        self._clear = None      # Created in __DoLayout
-        self._lockFile = None   # Created in __DoLayout
-        self._chFiles = None    # Created in __DoLayout
+        self._lockFile = None # Created in __DoLayout
+        self._chFiles = None # Created in __DoLayout
         self._worker = None
         self._busy = False
         self._isready = False
-        self._state = dict(file='', lang=0, cfile='', clang=0, last='', 
-                           lastlang=0, prelang=0, largs='', lcmd='')
+        self._config = dict(file='', lang=0,
+                            cfile='', clang=0,
+                            last='', lastlang=0,
+                            prelang=0, largs='',
+                            lcmd='')
+        self._prefs = Profile_Get(cfgdlg.LAUNCH_PREFS, default=None)
 
         # Setup
         self.__DoLayout()
         if not handlers.InitCustomHandlers(ed_glob.CONFIG['CACHE_DIR']):
             util.Log(u"[launch][warn] failed to load launch extensions")
 
-        # Ensure preferences have been initialized
-        if self.Preferences is None:
-            self.Preferences = dict(autoclear=False,
-                                    errorbeep=False,
-                                    wrapoutput=False,
-                                    defaultf=self._buffer.GetDefaultForeground().Get(),
-                                    defaultb=self._buffer.GetDefaultBackground().Get(),
-                                    errorf=self._buffer.GetErrorForeground().Get(),
-                                    errorb=self._buffer.GetErrorBackground().Get(),
-                                    infof=self._buffer.GetInfoForeground().Get(),
-                                    infob=self._buffer.GetInfoBackground().Get(),
-                                    warnf=self._buffer.GetWarningForeground().Get(),
-                                    warnb=self._buffer.GetWarningBackground().Get())
+        hstate = Profile_Get(LAUNCH_KEY)
+        if hstate is not None:
+            handlers.SetState(hstate)
+        if self._prefs is None:
+            Profile_Set(cfgdlg.LAUNCH_PREFS,
+                        dict(autoclear=False,
+                             errorbeep=False,
+                             wrapoutput=False,
+                             defaultf=self._buffer.GetDefaultForeground().Get(),
+                             defaultb=self._buffer.GetDefaultBackground().Get(),
+                             errorf=self._buffer.GetErrorForeground().Get(),
+                             errorb=self._buffer.GetErrorBackground().Get(),
+                             infof=self._buffer.GetInfoForeground().Get(),
+                             infob=self._buffer.GetInfoBackground().Get(),
+                             warnf=self._buffer.GetWarningForeground().Get(),
+                             warnb=self._buffer.GetWarningBackground().Get()))
+            self._prefs = Profile_Get(cfgdlg.LAUNCH_PREFS)
 
+        self._buffer.SetPrefs(self._prefs)
         self.UpdateBufferColors()
-        cbuffer = GetTextBuffer(self.MainWindow)
+        cbuffer = self._mw.GetNotebook().GetCurrentCtrl()
         self.SetupControlBar(cbuffer)
-        self.State['lang'] = GetLangIdFromMW(self.MainWindow)
-        self.UpdateCurrentFiles(self.State['lang'])
-        self.SetFile(cbuffer.GetFileName())
+        self._config['lang'] = GetLangIdFromMW(self._mw)
+        self.UpdateCurrentFiles(self._config['lang'])
+        self.SetFile(GetTextBuffer(self._mw).GetFileName())
 
         # Setup filetype settings
         self.RefreshControlBar()
@@ -121,43 +134,41 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
         self.Bind(wx.EVT_BUTTON, self.OnButton)
         self.Bind(wx.EVT_CHOICE, self.OnChoice)
         self.Bind(wx.EVT_CHECKBOX, self.OnCheck)
-        self.Bind(wx.EVT_WINDOW_DESTROY, self.OnDestroy, self)
         ed_msg.Subscribe(self.OnPageChanged, ed_msg.EDMSG_UI_NB_CHANGED)
         ed_msg.Subscribe(self.OnFileOpened, ed_msg.EDMSG_FILE_OPENED)
         ed_msg.Subscribe(self.OnThemeChanged, ed_msg.EDMSG_THEME_CHANGED)
         ed_msg.Subscribe(self.OnLexerChange, ed_msg.EDMSG_UI_STC_LEXER)
-        ed_msg.Subscribe(self.OnConfigChange,
-                         ed_msg.EDMSG_PROFILE_CHANGE + (handlers.CONFIG_KEY,))
+        ed_msg.Subscribe(self.OnConfigExit, cfgdlg.EDMSG_LAUNCH_CFG_EXIT)
         ed_msg.Subscribe(self.OnRunMsg, MSG_RUN_LAUNCH)
         ed_msg.Subscribe(self.OnRunLastMsg, MSG_RUN_LAST)
         ed_msg.RegisterCallback(self._CanLaunch, REQUEST_ACTIVE)
         ed_msg.RegisterCallback(self._CanReLaunch, REQUEST_RELAUNCH)
 
-    #---- Properties ----#
-    Locked = property(lambda self: self._lockFile.IsChecked())
-    MainWindow = property(lambda self: self._mw)
-    Preferences = property(lambda self: Profile_Get(cfgdlg.LAUNCH_PREFS, default=None),
-                           lambda self, prefs: Profile_Set(cfgdlg.LAUNCH_PREFS, prefs))
-    State = property(lambda self: self._state)
-
-    def OnDestroy(self, evt):
+    def __del__(self):
         ed_msg.Unsubscribe(self.OnPageChanged)
         ed_msg.Unsubscribe(self.OnFileOpened)
         ed_msg.Unsubscribe(self.OnThemeChanged)
         ed_msg.Unsubscribe(self.OnLexerChange)
-        ed_msg.Unsubscribe(self.OnConfigChange)
+        ed_msg.Unsubscribe(self.OnConfigExit)
         ed_msg.Unsubscribe(self.OnRunMsg)
         ed_msg.Unsubscribe(self.OnRunLastMsg)
         ed_msg.UnRegisterCallback(self._CanLaunch)
         ed_msg.UnRegisterCallback(self._CanReLaunch)
+        super(LaunchWindow, self).__del__()
 
     def __DoLayout(self):
         """Layout the window"""
-        ctrlbar = self.CreateControlBar(wx.TOP)
+        #-- Setup ControlBar --#
+        ctrlbar = eclib.ControlBar(self, style=eclib.CTRLBAR_STYLE_GRADIENT)
+        if wx.Platform == '__WXGTK__':
+            ctrlbar.SetWindowStyle(eclib.CTRLBAR_STYLE_DEFAULT)
 
         # Preferences
-        self._pbtn = self.AddPlateButton(u"", ed_glob.ID_PREF)
-        self._pbtn.SetToolTipString(_("Settings"))
+        prefbmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_PREF), wx.ART_MENU)
+        pref = eclib.PlateButton(ctrlbar, ID_SETTINGS, '', prefbmp,
+                                 style=eclib.PB_STYLE_NOBG)
+        pref.SetToolTipString(_("Settings"))
+        ctrlbar.AddControl(pref, wx.ALIGN_LEFT)
 
         # Exe
         ctrlbar.AddControl(wx.StaticText(ctrlbar, label=_("exec") + ":"),
@@ -185,13 +196,45 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
         # Spacer
         ctrlbar.AddStretchSpacer()
         
-        # Buttons
-        self._run = self.AddPlateButton(_("Run"), ed_glob.ID_BIN_FILE, 
-                                        wx.ALIGN_LEFT)
-        self._clear = self.AddPlateButton(_("Clear"), ed_glob.ID_DELETE,
-                                          wx.ALIGN_RIGHT)
+        # Run Button
+        rbmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_BIN_FILE), wx.ART_MENU)
+        if rbmp.IsNull() or not rbmp.IsOk():
+            rbmp = None
+        run = eclib.PlateButton(ctrlbar, ID_RUN, _("Run"), rbmp,
+                                style=eclib.PB_STYLE_NOBG)
+        ctrlbar.AddControl(run, wx.ALIGN_RIGHT)
+
+        # Clear Button
+        cbmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_DELETE), wx.ART_MENU)
+        if cbmp.IsNull() or not cbmp.IsOk():
+            cbmp = None
+        clear = eclib.PlateButton(ctrlbar, wx.ID_CLEAR, _("Clear"),
+                                  cbmp, style=eclib.PB_STYLE_NOBG)
+        ctrlbar.AddControl(clear, wx.ALIGN_RIGHT)
+        ctrlbar.SetVMargin(1, 1)
+        self.SetControlBar(ctrlbar)
+
         self.SetWindow(self._buffer)
 
+    def __FindMainWindow(self):
+        """Find the mainwindow of this control
+        @return: MainWindow or None
+
+        """
+        def IsMainWin(win):
+            """Check if the given window is a main window"""
+            return getattr(tlw, '__name__', '') == 'MainWindow'
+
+        tlw = self.GetTopLevelParent()
+        if IsMainWin(tlw):
+            return tlw
+        elif hasattr(tlw, 'GetParent'):
+            tlw = tlw.GetParent()
+            if IsMainWin(tlw):
+                return tlw
+
+        return None
+
     def _CanLaunch(self):
         """Method to use with RegisterCallback for getting status"""
         val = self.CanLaunch()
@@ -202,7 +245,7 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
     def _CanReLaunch(self):
         """Method to use with RegisterCallback for getting status"""
         val = self.CanLaunch()
-        if not val or not len(self.State['last']):
+        if not val or not len(self._config['last']):
             val = ed_msg.NullValue()
         return val
 
@@ -211,38 +254,46 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
         @return: bool
 
         """
-        return self.TopLevelParent.IsActive() and self._isready
+        parent = self.GetParent()
+        return parent.GetParent().IsActive() and self._isready
 
     def GetFile(self):
         """Get the file that is currently set to be run
         @return: file path
 
         """
-        return self.State['file']
+        return self._config['file']
 
     def GetLastRun(self):
         """Get the last file that was run
         @return: (fname, lang_id)
 
         """
-        return (self.State['last'], self.State['lastlang'])
+        return (self._config['last'], self._config['lastlang'])
+
+    def GetMainWindow(self):
+        """Get the mainwindow that created this instance
+        @return: reference to MainWindow
+
+        """
+        return self._mw
 
     def OnButton(self, evt):
         """Handle events from the buttons on the control bar"""
-        e_obj = evt.GetEventObject()
-        if e_obj == self._pbtn:
+        e_id = evt.GetId()
+        if e_id == ID_SETTINGS:
             app = wx.GetApp()
             win = app.GetWindowInstance(cfgdlg.ConfigDialog)
             if win is None:
-                config = cfgdlg.ConfigDialog(self.MainWindow)
+                config = cfgdlg.ConfigDialog(self._mw)
                 config.CentreOnParent()
                 config.Show()
             else:
                 win.Raise()
-        elif e_obj is self._run:
+        elif e_id == ID_RUN:
             # May be run or abort depending on current state
             self.StartStopProcess()
-        elif e_obj == self._clear:
+        elif e_id == wx.ID_CLEAR:
             self._buffer.Clear()
         else:
             evt.Skip()
@@ -260,7 +311,7 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
             self._chFiles.SetToolTipString(fname)
         elif e_id == ID_EXECUTABLE:
             e_obj = evt.GetEventObject()
-            handler = handlers.GetHandlerById(self.State['lang'])
+            handler = handlers.GetHandlerById(self._config['lang'])
             cmd = e_obj.GetStringSelection()
             e_obj.SetToolTipString(handler.GetCommand(cmd))
         else:
@@ -270,27 +321,28 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
         """CheckBox for Lock File was clicked"""
         e_obj = evt.GetEventObject()
         if e_obj is self._lockFile:
-            if self.Locked:
+            if self._lockFile.IsChecked():
                 self._chFiles.Disable()
             else:
                 self._chFiles.Enable()
-                cbuff = GetTextBuffer(self.MainWindow)
+                cbuff = GetTextBuffer(self._mw)
                 if isinstance(cbuff, ed_basestc.EditraBaseStc):
                     self.UpdateCurrentFiles(cbuff.GetLangId())
                     self.SetupControlBar(cbuff)
         else:
             evt.Skip()
 
-    def OnConfigChange(self, msg):
-        """Update current state when the configuration has been changed
+    def OnConfigExit(self, msg):
+        """Update current state when the config dialog has been closed
         @param msg: Message Object
 
         """
         util.Log("[Launch][info] Saving config to profile")
         self.RefreshControlBar()
+        Profile_Set(LAUNCH_KEY, handlers.GetState()) # TODO is this necessary see other handler
         # Update wordwrapping
         mode = wx.stc.STC_WRAP_NONE
-        if self.Preferences.get('wrapoutput', False):
+        if self._prefs.get('wrapoutput', False):
             mode = wx.stc.STC_WRAP_WORD # should we do wrap char?
         wrapmode = self._buffer.GetWrapMode()
         if wrapmode != mode:
@@ -303,12 +355,12 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
         @param msg: Message Object
 
         """
-        if self.Locked:
+        if self._lockFile.IsChecked():
             return # Mode is locked ignore update
 
         # Update the file choice control
-        self.State['lang'] = GetLangIdFromMW(self.MainWindow)
-        self.UpdateCurrentFiles(self.State['lang'])
+        self._config['lang'] = GetLangIdFromMW(self._mw)
+        self.UpdateCurrentFiles(self._config['lang'])
 
         fname = msg.GetData()
         self.SetFile(fname)
@@ -325,9 +377,9 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
 
         """
         self._log("[launch][info] Lexer changed handler - context %d" %
-                  self.MainWindow.GetId())
+                  self._mw.GetId())
 
-        if self.Locked:
+        if self._lockFile.IsChecked():
             return # Mode is locked ignore update
 
         mdata = msg.GetData()
@@ -339,6 +391,7 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
         # Update regardless of whether lexer has changed or not as the buffer
         # may have the lexer set before the file was saved to disk.
         if fname:
+            #if ftype != self._config['lang']:
             self.UpdateCurrentFiles(ftype)
             self.SetControlBarState(fname, ftype)
 
@@ -350,7 +403,7 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
 
         """
         # The current mode is locked
-        if self.Locked:
+        if self._lockFile.IsChecked():
             return
 
         mval = msg.GetData()
@@ -369,7 +422,7 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
 
         """
         if self.CanLaunch():
-            shelf = self.MainWindow.GetShelf()
+            shelf = self._mw.GetShelf()
             shelf.RaiseWindow(self)
             self.StartStopProcess()
 
@@ -384,64 +437,61 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
             if not len(fname):
                 return
 
-            shelf = self.MainWindow.GetShelf()
+            shelf = self._mw.GetShelf()
             self.UpdateCurrentFiles(ftype)
             self.SetFile(fname)
             self.RefreshControlBar()
             shelf.RaiseWindow(self)
 
-            if self.Preferences.get('autoclear'):
+            if self._prefs.get('autoclear'):
                 self._buffer.Clear()
 
             self.SetProcessRunning(True)
 
-            self.Run(fname, self.State['lcmd'], self.State['largs'], ftype)
+            self.Run(fname, self._config['lcmd'], self._config['largs'], ftype)
 
     def OnThemeChanged(self, msg):
         """Update icons when the theme has been changed
         @param msg: Message Object
 
         """
-        ctrls = ((self._pbtn, ed_glob.ID_PREF),
-                 (self._clear, ed_glob.ID_DELETE))
+        ctrls = ((ID_SETTINGS, ed_glob.ID_PREF),
+                 (wx.ID_CLEAR, ed_glob.ID_DELETE))
         if self._busy:
-            ctrls += ((self._run, ed_glob.ID_STOP),)
+            ctrls += ((ID_RUN, ed_glob.ID_STOP),)
         else:
-            ctrls += ((self._run, ed_glob.ID_BIN_FILE),)
+            ctrls += ((ID_RUN, ed_glob.ID_BIN_FILE),)
 
-        for btn, art in ctrls:
+        for ctrl, art in ctrls:
+            btn = self.FindWindowById(ctrl)
             bmp = wx.ArtProvider.GetBitmap(str(art), wx.ART_MENU)
             btn.SetBitmap(bmp)
             btn.Refresh()
-        self.GetControlBar().Refresh()
 
     def RefreshControlBar(self):
         """Refresh the state of the control bar based on the current config"""
-        handler = handlers.GetHandlerById(self.State['lang'])
+        handler = handlers.GetHandlerById(self._config['lang'])
         cmds = handler.GetAliases()
 
         # Get the controls
         exe_ch = self.FindWindowById(ID_EXECUTABLE)
         args_txt = self.FindWindowById(ID_ARGS)
+        run_btn = self.FindWindowById(ID_RUN)
 
         csel = exe_ch.GetStringSelection()
         exe_ch.SetItems(cmds)
-        ncmds = len(cmds)
-        if ncmds > 0:
+        if len(cmds):
             exe_ch.SetToolTipString(handler.GetCommand(cmds[0]))
 
-        util.Log("[Launch][info] Found commands %s" % repr(cmds))
-        if handler.GetName() != handlers.DEFAULT_HANDLER and \
-           ncmds > 0 and len(self.GetFile()):
-            for ctrl in (exe_ch, args_txt, self._run,
+        util.Log("[Launch][info] Found commands %s" % str(cmds))
+        if handler.GetName() != handlers.DEFAULT_HANDLER and len(self.GetFile()):
+            for ctrl in (exe_ch, args_txt, run_btn,
                          self._chFiles, self._lockFile):
                 ctrl.Enable()
 
             self._isready = True
-            if self.Locked:
-                self._chFiles.Enable(False)
 
-            if self.State['lang'] == self.State['prelang'] and len(csel):
+            if self._config['lang'] == self._config['prelang'] and len(csel):
                 exe_ch.SetStringSelection(csel)
             else:
                 csel = handler.GetDefault()
@@ -451,9 +501,10 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
             self.GetControlBar().Layout()
         else:
             self._isready = False
-            for ctrl in (exe_ch, args_txt, self._run,
+            for ctrl in (exe_ch, args_txt, run_btn,
                          self._chFiles, self._lockFile):
                 ctrl.Disable()
+            self._chFiles.Clear()
 
     def Run(self, fname, cmd, args, ftype):
         """Run the given file
@@ -464,7 +515,7 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
 
         """
         # Find and save the file if it is modified
-        nb = self.MainWindow.GetNotebook()
+        nb = self._mw.GetNotebook()
         for ctrl in nb.GetTextControls():
             tname = ctrl.GetFileName()
             if fname == tname:
@@ -487,27 +538,19 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
 
     def StartStopProcess(self):
         """Run or abort the context of the current process if possible"""
-        if self.Preferences.get('autoclear', False):
+        if self._prefs.get('autoclear'):
             self._buffer.Clear()
 
-        # Check Auto-save preferences
-        if not self._busy:
-            if self.Preferences.get('autosaveall', False):
-                self.MainWindow.SaveAllBuffers()
-            elif self.Preferences.get('autosave', False):
-                self.MainWindow.SaveCurrentBuffer()
-
-        # Start or stop the process
         self.SetProcessRunning(not self._busy)
         if self._busy:
             util.Log("[Launch][info] Starting process")
-            handler = handlers.GetHandlerById(self.State['lang'])
+            handler = handlers.GetHandlerById(self._config['lang'])
             cmd = self.FindWindowById(ID_EXECUTABLE).GetStringSelection()
-            self.State['lcmd'] = cmd
+            self._config['lcmd'] = cmd
             cmd = handler.GetCommand(cmd)
             args = self.FindWindowById(ID_ARGS).GetValue().split()
-            self.State['largs'] = args
-            self.Run(self.State['file'], cmd, args, self.State['lang'])
+            self._config['largs'] = args
+            self.Run(self._config['file'], cmd, args, self._config['lang'])
         else:
             util.Log("[Launch][info] Aborting process")
             self._worker.Abort()
@@ -519,7 +562,7 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
 
         """
         # Set currently selected file
-        self.State['file'] = fname
+        self._config['file'] = fname
         self._chFiles.SetStringSelection(os.path.split(fname)[1])
         self.GetControlBar().Layout()
 
@@ -528,8 +571,8 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
         @param langid: syntax.synglob lang id
 
         """
-        self.State['prelang'] = self.State['lang']
-        self.State['lang'] = langid
+        self._config['prelang'] = self._config['lang']
+        self._config['lang'] = langid
 
     def SetProcessRunning(self, running=True):
         """Set the state of the window into either process running mode
@@ -537,32 +580,33 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
         @keyword running: Is a process running or not
 
         """
+        rbtn = self.FindWindowById(ID_RUN)
         self._busy = running
         if running:
-            self.State['last'] = self.State['file']
-            self.State['lastlang'] = self.State['lang']
-            self.State['cfile'] = self.State['file']
-            self.State['clang'] = self.State['lang']
+            self._config['last'] = self._config['file']
+            self._config['lastlang'] = self._config['lang']
+            self._config['cfile'] = self._config['file']
+            self._config['clang'] = self._config['lang']
             abort = wx.ArtProvider.GetBitmap(str(ed_glob.ID_STOP), wx.ART_MENU)
             if abort.IsNull() or not abort.IsOk():
                 abort = wx.ArtProvider.GetBitmap(wx.ART_ERROR,
                                                  wx.ART_MENU, (16, 16))
-            self._run.SetBitmap(abort)
-            self._run.SetLabel(_("Abort"))
+            rbtn.SetBitmap(abort)
+            rbtn.SetLabel(_("Abort"))
         else:
             rbmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_BIN_FILE), wx.ART_MENU)
             if rbmp.IsNull() or not rbmp.IsOk():
                 rbmp = None
-            self._run.SetBitmap(rbmp)
-            self._run.SetLabel(_("Run"))
+            rbtn.SetBitmap(rbmp)
+            rbtn.SetLabel(_("Run"))
             # If the buffer was changed while this was running we should
             # update to the new buffer now that it has stopped.
-            self.SetFile(self.State['cfile'])
-            self.SetLangId(self.State['clang'])
+            self.SetFile(self._config['cfile'])
+            self.SetLangId(self._config['clang'])
             self.RefreshControlBar()
 
         self.GetControlBar().Layout()
-        self._run.Refresh()
+        rbtn.Refresh()
 
     def SetupControlBar(self, ctrl):
         """Set the state of the controlbar based data found in the buffer
@@ -577,22 +621,21 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
     def SetControlBarState(self, fname, lang_id):
         # Don't update the bars status if the buffer is busy
         if self._buffer.IsRunning():
-            self.State['cfile'] = fname
-            self.State['clang'] = lang_id
+            self._config['cfile'] = fname
+            self._config['clang'] = lang_id
         else:
-            if not self.Locked:
-                self.SetFile(fname)
-                self.SetLangId(lang_id)
+            self.SetFile(fname)
+            self.SetLangId(lang_id)
 
-                # Refresh the control bars view
-                self.RefreshControlBar()
+            # Refresh the control bars view
+            self.RefreshControlBar()
 
     def UpdateBufferColors(self):
         """Update the buffers colors"""
         colors = dict()
         for color in ('defaultf', 'defaultb', 'errorf', 'errorb',
                       'infof', 'infob', 'warnf', 'warnb'):
-            val = self.Preferences.get(color, None)
+            val = self._prefs.get(color, None)
             if val is not None:
                 colors[color] = wx.Colour(*val)
             else:
@@ -612,7 +655,7 @@ class LaunchWindow(ed_basewin.EdBaseCtrlBox):
 
         """
         self._fnames = list()
-        for txt_ctrl in self.MainWindow.GetNotebook().GetTextControls():
+        for txt_ctrl in self._mw.GetNotebook().GetTextControls():
             if lang_id == txt_ctrl.GetLangId():
                 self._fnames.append(txt_ctrl.GetFileName())
 
@@ -635,8 +678,9 @@ class OutputDisplay(eclib.OutputBuffer, eclib.ProcessBufferMixin):
         eclib.ProcessBufferMixin.__init__(self)
 
         # Attributes
-        self._mw = parent.MainWindow
+        self._mw = parent.GetMainWindow()
         self._cfile = ''
+        self._prefs = dict()
 
         # Setup
         font = Profile_Get('FONT1', 'font', wx.Font(11, wx.FONTFAMILY_MODERN,
@@ -644,9 +688,6 @@ class OutputDisplay(eclib.OutputBuffer, eclib.ProcessBufferMixin):
                                                     wx.FONTWEIGHT_NORMAL))
         self.SetFont(font)
 
-    Preferences = property(lambda self: Profile_Get(cfgdlg.LAUNCH_PREFS, default=None),
-                           lambda self, prefs: Profile_Set(cfgdlg.LAUNCH_PREFS, prefs))
-
     def ApplyStyles(self, start, txt):
         """Apply any desired output formatting to the text in
         the buffer.
@@ -658,8 +699,7 @@ class OutputDisplay(eclib.OutputBuffer, eclib.ProcessBufferMixin):
         style = handler.StyleText(self, start, txt)
 
         # Ring the bell if there was an error and option is enabled
-        if style == handlers.STYLE_ERROR and \
-           self.Preferences.get('errorbeep', False):
+        if style == handlers.STYLE_ERROR and self._prefs.get('errorbeep', False):
             wx.Bell()
 
     def DoFilterInput(self, txt):
@@ -704,21 +744,7 @@ class OutputDisplay(eclib.OutputBuffer, eclib.ProcessBufferMixin):
 
     def DoProcessExit(self, code=0):
         """Do all that is needed to be done after a process has exited"""
-        # Peek in the queue to see the last line before the exit line
-        queue = self.GetUpdateQueue()
-        prepend_nl = True
-        if len(queue):
-            line = queue[-1]
-        else:
-            line = self.GetLine(self.GetLineCount() - 1)
-        if line.endswith('\n') or line.endswith('\r'):
-            prepend_nl = False
-        final_line = u">>> %s: %d%s" % (_("Exit Code"), code, os.linesep)
-        # Add an extra line feed if necessary to make sure the final line
-        # is output on a new line.
-        if prepend_nl:
-            final_line = os.linesep + final_line
-        self.AppendUpdate(final_line)
+        self.AppendUpdate(">>> %s: %d%s" % (_("Exit Code"), code, os.linesep))
         self.Stop()
         self.GetParent().SetProcessRunning(False)
 
@@ -736,6 +762,13 @@ class OutputDisplay(eclib.OutputBuffer, eclib.ProcessBufferMixin):
         handler = handlers.GetHandlerById(lang_id)
         return handler
 
+    def SetPrefs(self, prefs):
+        """Set the launch prefs
+        @param prefs: dict
+
+        """
+        self._prefs = prefs
+
 #-----------------------------------------------------------------------------#
 def GetLangIdFromMW(mainw):
     """Get the language id of the file in the current buffer
