@@ -18,8 +18,8 @@ element is defined in the file.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: cbrowser.py 66072 2010-11-08 21:04:45Z CJP $"
-__revision__ = "$Revision: 66072 $"
+__svnid__ = "$Id: cbrowser.py 65716 2010-10-01 20:12:14Z CJP $"
+__revision__ = "$Revision: 65716 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -338,7 +338,7 @@ class CodeBrowserTree(wx.TreeCtrl):
 
     def DeleteChildren(self, item):
         """Delete the children of a given node"""
-        super(CodeBrowserTree, self).DeleteChildren(item)
+        wx.TreeCtrl.DeleteChildren(self, item)
         for key in self.nodes.keys():
             self.nodes[key] = None
 
