@@ -16,8 +16,8 @@ running Editra.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: Editra.py 64249 2010-05-08 15:40:56Z CJP $"
-__revision__ = "$Revision: 64249 $"
+__svnid__ = "$Id: Editra.py 65750 2010-10-03 20:38:53Z CJP $"
+__revision__ = "$Revision: 65750 $"
 
 #--------------------------------------------------------------------------#
 # Dependencies
@@ -323,12 +323,16 @@ class Editra(wx.App, events.AppEventHandlerMixin):
 
         """
         win = self.GetTopWindow()
-        if getattr(win, '__name__', None) != u"MainWindow":
+        if not isinstance(win, ed_main.MainWindow):
             win = self.GetActiveWindow()
             if win is None:
-                return win
+                return None # UI dead?
 
-        return win.GetNotebook().GetCurrentCtrl()
+        if isinstance(win, ed_main.MainWindow):
+            nbook = win.GetNotebook()
+            if nbook:
+                return nbook.GetCurrentCtrl()
+        return None
 
     def GetMainWindows(self):
         """Returns a list of all open main windows
@@ -406,20 +410,28 @@ class Editra(wx.App, events.AppEventHandlerMixin):
 
         """
         window = self.GetTopWindow()
-        if isinstance(window, ed_main.MainWindow):
-            try:
-                encoding = sys.getfilesystemencoding()
-                window.DoOpen(ed_glob.ID_COMMAND_LINE_OPEN,
-                              ed_txt.DecodeString(filename, encoding),
-                              line)
+        if not isinstance(window, ed_main.MainWindow):
+            window = None
+
+        try:
+            encoding = sys.getfilesystemencoding()
+            fname = ed_txt.DecodeString(filename, encoding)
+
+            if profiler.Profile_Get('OPEN_NW', default=False):
+                self.OpenNewWindow(fname, window)
+            elif window:
+                window.DoOpen(ed_glob.ID_COMMAND_LINE_OPEN, fname, line)
 
                 # Make sure the window is brought to the front
                 if window.IsIconized():
                     window.Iconize(False)
                 window.Raise()
-            except Exception, msg:
-                self._log("[app][err] Failed to open drop file: %s" % str(msg))
-                pass
+            else:
+                # Some unlikely error condition
+                self._log("[app][err] OpenFile unknown error: %s" % filename)
+                
+        except Exception, msg:
+            self._log("[app][err] Failed to open file: %s" % str(msg))
 
     def MacNewFile(self):
         """Stub for future use"""
@@ -903,7 +915,7 @@ def PrintHelp(err=None):
         sys.stderr.write(err + os.linesep)
 
     print(("Editra - %s - Developers Text Editor\n"
-       "Cody Precord (2005-2009)\n\n"
+       "Cody Precord (2005-2010)\n\n"
        "usage: Editra [arguments] [files... ]\n\n"
        "Short Arguments:\n"
        "  -c    Set custom configuration directory at runtime\n"
@@ -920,7 +932,7 @@ def PrintHelp(err=None):
        "  --help            Show this help message\n"
        "  --auth            Print the ipc server info\n"
        "  --version         Print version number and exit\n"
-       "  --profileOut arg  Run Editra in the profier (arg is output file)\n"
+       "  --profileOut arg  Run Editra in the profiler (arg is output file)\n"
       ) % ed_glob.VERSION)
 
     if err is None:
@@ -1102,6 +1114,10 @@ def _Main(opts, args):
     # 3. Start Applications Main Loop
     dev_tool.DEBUGP("[main][info] Starting MainLoop...")
     wx.CallAfter(frame.Raise)
+
+    # Install handlers to exit app if os is shutting down/restarting
+    ebmlib.InstallTermHandler(editra_app.Exit, [], dict(force=True))
+
     editra_app.MainLoop()
     dev_tool.DEBUGP("[main][info] MainLoop finished exiting application")
     os._exit(0)
