@@ -14,11 +14,12 @@ Base class for autocompletion providers to implement the completion interface.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: completer.py 60376 2009-04-26 04:35:42Z CJP $"
-__revision__ = "$Revision: 60376 $"
+__svnid__ = "$Id: completer.py 63531 2010-02-21 15:48:53Z CJP $"
+__revision__ = "$Revision: 63531 $"
 
-__all__ = [ 'IMG_FUNCTION', 'IMG_METHOD', 'IMG_CLASS', 'IMG_ATTRIBUTE',
-            'IMG_VARIABLE', 'IMG_ELEMENT', 'IMG_PROPERTY' ]
+__all__ = [ 'TYPE_FUNCTION', 'TYPE_METHOD', 'TYPE_CLASS', 'TYPE_ATTRIBUTE',
+            'TYPE_VARIABLE', 'TYPE_ELEMENT', 'TYPE_PROPERTY', 'TYPE_UNKNOWN',
+            'BaseCompleter', 'Symbol', 'CreateSymbols' ]
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -27,13 +28,80 @@ import wx
 #--------------------------------------------------------------------------#
 
 # Image Type Ids
-IMG_FUNCTION, \
-IMG_METHOD, \
-IMG_CLASS, \
-IMG_ATTRIBUTE, \
-IMG_PROPERTY, \
-IMG_VARIABLE, \
-IMG_ELEMENT  = range(1, 8)
+TYPE_FUNCTION, \
+TYPE_METHOD, \
+TYPE_CLASS, \
+TYPE_ATTRIBUTE, \
+TYPE_PROPERTY, \
+TYPE_VARIABLE, \
+TYPE_ELEMENT, \
+TYPE_UNKNOWN = range(1, 9)
+
+#--------------------------------------------------------------------------#
+class Symbol(object):
+    """ Defines a symbol as parsed by the autocompleter.
+    Symbols with the same name and different type are EQUAL
+    Symbol hash is based on symbol NAME
+    
+    """
+    def __init__(self, name, type):
+        """ Constructor
+        @param Name: Symbol name
+        @param Type: Symbol type, one of the TYPE_FUNCTION ... TYPE_UNKNOWN range
+        
+        """
+        object.__init__(self)
+
+        # Attributes
+        self.__name = unicode(name)
+        self.__type = type
+    
+    def __eq__(self, other):
+        return (self.__name == other.__name)
+
+    def __lt__(self, other):
+        return (self.__name < other.__name)
+
+    def __le__(self, other):
+        return (self.__name <= other.__name)
+
+    def __ne__(self, other):
+        return (self.__name != other.__name)
+
+    def __gt__(self, other):
+        return (self.__name > other.__name)
+
+    def __ge__(self, other):
+        return (self.__name >= other.__name)
+
+    # TODO: this task should probably be delegated to the ui
+    def __str__(self):
+        if self.__type != TYPE_UNKNOWN:
+            return u'?'.join([self.__name, unicode(self.__type)])
+        else:
+            return self.Name
+
+    def __hash__(self):
+        return self.__name.__hash__()
+    
+    @property
+    def Name(self):
+        return self.__name
+
+    @property
+    def Type(self):
+        return self.__type
+
+#--------------------------------------------------------------------------#
+
+def CreateSymbols(arglst, symtype=TYPE_UNKNOWN):
+    """Convert a list of strings to a list of Symbol objects
+    @param arglst: list of strings
+    @keyword symtype: TYPE_FOO 
+    @return: list of Symbols
+
+    """
+    return [ Symbol(obj, symtype) for obj in arglst ]
 
 #--------------------------------------------------------------------------#
 
@@ -51,6 +119,7 @@ class BaseCompleter(object):
         self._log = wx.GetApp().GetLog()
         self._case_sensitive = False
         self._autocomp_after = False
+        self._choose_single = True
 
         self._autocomp_keys = list()
         self._autocomp_stop = u''
@@ -76,15 +145,16 @@ class BaseCompleter(object):
         """
         return u''
 
-    #--- End override in subclass ----#
-
-    def GetAutoCompAfter(self):
-        """Should text insterted by autocomp be placed after the cursor
-        or before it.
-        @return: bool
+    def OnCompletionInserted(self, pos, text):
+        """Called by the buffer when an autocomp selection has been inserted.
+        The completer can override this method to 
+        @param pos: Position the caret was at before the insertion
+        @param text: text that was inserted at pos
 
         """
-        return self._autocomp_after
+        pass
+
+    #--- End override in subclass ----#
 
     def GetAutoCompKeys(self):
         """Returns the list of key codes for activating the autocompletion.
@@ -182,14 +252,6 @@ class BaseCompleter(object):
         """
         return self._case_sensitive
 
-    def SetAutoCompAfter(self, after=False):
-        """Set if text insterted by autocomp should be placed after the cursor
-        or before it.
-        @keyword after: bool
-
-        """
-        self._autocomp_after = after
-
     def SetCaseSensitive(self, sensitive):
         """Set whether this completer is case sensitive or not
         @param sensitive: bool
@@ -197,6 +259,22 @@ class BaseCompleter(object):
         """
         self._case_sensitive = sensitive
 
+    def GetChooseSingle(self):
+        """Get whether the completer should automatically choose a selection
+        when there is only one symbol in the completion list.
+        @return: bool
+
+        """
+        return self._choose_single
+
+    def SetChooseSingle(self, single):
+        """Set whether the completer should automatically choose a selection
+        when there is only one symbol in the completion list.
+        @param single: bool
+
+        """
+        self._choose_single = single
+
     def ShouldCheck(self, cpos):
         """Should completions be attempted
         @param cpos: current buffer position
