@@ -12,8 +12,8 @@ Editra Buisness Model Library:
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: __init__.py 63565 2010-02-27 04:15:45Z CJP $"
-__revision__ = "$Revision: 63565 $"
+__cvsid__ = "$Id: __init__.py 65750 2010-10-03 20:38:53Z CJP $"
+__revision__ = "$Revision: 65750 $"
 
 #-----------------------------------------------------------------------------#
 
@@ -25,6 +25,7 @@ from fileimpl import *
 from txtutil import *
 
 from backupmgr import *
+from calllock import *
 
 # Storage Classes
 from histcache import *
@@ -33,3 +34,5 @@ from clipboard import *
 # Misc
 from miscutil import *
 from cmenumgr import *
+from osutil import *
+
