@@ -20,8 +20,8 @@ Displays information on:
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: infodlg.py 61808 2009-09-02 15:57:55Z CJP $"
-__revision__ = "$Revision: 61808 $"
+__svnid__ = "$Id: infodlg.py 65584 2010-09-21 20:08:47Z CJP $"
+__revision__ = "$Revision: 65584 $"
 
 __all__ = ["FileInfoDlg", "CalcSize", "GetFileType"]
 
@@ -54,9 +54,9 @@ class FileInfoDlg(wx.MiniFrame):
 
         """
         self._fname = fname.split(os.path.sep)[-1]
-        wx.MiniFrame.__init__(self, parent,
-                              title="%s  %s" % (self._fname, _("Info")),
-                              style=wx.DEFAULT_DIALOG_STYLE)
+        super(FileInfoDlg, self).__init__(parent,
+                                          title="%s  %s" % (self._fname, _("Info")),
+                                          style=wx.DEFAULT_DIALOG_STYLE)
 
         # Attributes
         self._file = fname
