@@ -15,8 +15,8 @@ main Ui component of the editor that contains all the other components.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_main.py 60549 2009-05-08 01:49:32Z CJP $"
-__revision__ = "$Revision: 60549 $"
+__svnid__ = "$Id: ed_main.py 65751 2010-10-03 21:24:06Z CJP $"
+__revision__ = "$Revision: 65751 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -36,6 +36,7 @@ import ed_event
 import ed_msg
 import ed_menu
 import ed_print
+import ed_shelf
 import ed_statbar
 import ed_mdlg
 import prefdlg
@@ -59,6 +60,9 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
     @todo: modularize the event handling more (pubsub?)
 
     """
+    # Clipboard ring is limited to 25, why? Because any more is a waste of
+    # memory and an inefficient waste of your time to cyle through.
+    CLIPBOARD = util.EdClipboard(25)
     PRINTER = None
 
     def __init__(self, parent, id_, wsize, title):
@@ -72,7 +76,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
 
         hint = wx.aui.AUI_MGR_TRANSPARENT_HINT
         if wx.Platform == '__WXGTK__':
-            # Use venetian blinds style as trasparent can cause crashes
+            # Use venetian blinds style as transparent can cause crashes
             # on linux when desktop compositing is used.
             hint = wx.aui.AUI_MGR_VENETIAN_BLINDS_HINT
 
@@ -90,13 +94,14 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
 
         # Attributes
         self._loaded = False
+        self._mlock = ebmlib.CallLock()
         self._last_save = u''
         self.LOG = wx.GetApp().GetLog()
         self._exiting = False
         self._handlers = dict(menu=list(), ui=list())
 
         #---- Setup File History ----#
-        self.filehistory = wx.FileHistory(_PGET('FHIST_LVL', 'int', 5))
+        self.filehistory = wx.FileHistory(_PGET('FHIST_LVL', 'int', 9))
 
         #---- Status bar on bottom of window ----#
         self.SetStatusBar(ed_statbar.EdStatBar(self))
@@ -104,14 +109,14 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
 
         #---- End Statusbar Setup ----#
 
-        #---- Notebook that contains the editting buffers ----#
+        #---- Notebook that contains the editing buffers ----#
         self._mpane = ed_mpane.MainPanel(self)
         self.nb = self._mpane.GetWindow()
         self._mgr.AddPane(self._mpane, wx.aui.AuiPaneInfo(). \
                           Name("EditPane").Center().Layer(1).Dockable(False). \
                           CloseButton(False).MaximizeButton(False). \
                           CaptionVisible(False))
-        self._mpane.InitCommandBar() # <- required due to nb dependancies...
+        self._mpane.InitCommandBar() # <- required due to nb dependencies...
 
         #---- Command Bar ----#
         self._mpane.HideCommandBar()
@@ -137,9 +142,6 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
 
         ## Setup additional menu items
         self.filehistory.UseMenu(menbar.GetMenuByName("filehistory"))
-        menbar.GetMenuByName("settings").AppendMenu(ID_LEXER, _("Lexers"),
-                                                    syntax.GenLexerMenu(),
-                                              _("Manually Set a Lexer/Syntax"))
 
         # On mac, do this to make help menu appear in correct location
         # Note it must be done before setting the menu bar and after the
@@ -174,6 +176,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
 
                                        # Edit Menu
                                        (ID_PASTE_AFTER, self.DispatchToControl),
+                                       (ID_CYCLE_CLIPBOARD, self.DispatchToControl),
                                        (ID_COLUMN_MODE, self.DispatchToControl),
                                        (ID_TOGGLE_FOLD, self.DispatchToControl),
                                        (ID_TOGGLE_ALL_FOLDS, self.DispatchToControl),
@@ -191,6 +194,9 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                                        # Format Menu
                                        (ID_FONT, self.OnFont),
 
+                                       # Settings menu
+                                       (ID_LEXER_CUSTOM, self.OnCustomizeLangMenu),
+
                                        # Tool Menu
                                        (ID_COMMAND, self.OnCommandBar),
                                        (ID_STYLE_EDIT, self.OnStyleEdit),
@@ -201,7 +207,8 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                                        (ID_HOMEPAGE, self.OnHelp),
                                        (ID_DOCUMENTATION, self.OnHelp),
                                        (ID_TRANSLATE, self.OnHelp),
-                                       (ID_CONTACT, self.OnHelp)])
+                                       (ID_CONTACT, self.OnHelp),
+                                       (ID_BUG_TRACKER, self.OnHelp)])
 
         self._handlers['menu'].extend([(l_id, self.DispatchToControl)
                                        for l_id in syntax.SYNTAX_IDS])
@@ -221,6 +228,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                                      (ID_CUT, self.OnUpdateClipboardUI),
                                      (ID_PASTE, self.OnUpdateClipboardUI),
                                      (ID_PASTE_AFTER, self.OnUpdateClipboardUI),
+                                     (ID_CYCLE_CLIPBOARD, self.OnUpdateClipboardUI),
                                      (ID_UNDO, self.OnUpdateClipboardUI),
                                      (ID_REDO, self.OnUpdateClipboardUI),
                                      (ID_COLUMN_MODE, self.OnUpdateClipboardUI),
@@ -281,9 +289,12 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         addons.Init(self)
         self._handlers['menu'].extend(addons.GetEventHandlers())
         self._handlers['ui'].extend(addons.GetEventHandlers(ui_evt=True))
-        self._shelf = iface.Shelf(plgmgr)
-        self._shelf.Init(self)
-        self._handlers['ui'].extend(self._shelf.GetUiHandlers())
+        shelf = ed_shelf.Shelf(plgmgr)
+        shelf.Init(self)
+        self._handlers['ui'].extend(shelf.GetUiHandlers())
+        self._shelf = ed_shelf.Shelf.delegate
+        ed_shelf.Shelf.delegate = None
+
         self.LOG("[ed_main][info] Loading Generator plugins")
         generator.Generator(plgmgr).InstallMenu(menbar.GetMenuByName("tools"))
 
@@ -293,12 +304,18 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         ed_msg.PostMessage(ed_msg.EDMSG_DSP_FONT,
                            _PGET('FONT3', 'font', wx.NORMAL_FONT))
 
+        # Message Handlers
+        ed_msg.Subscribe(self.OnUpdateFileHistory, ed_msg.EDMSG_ADD_FILE_HISTORY)
+
         # HACK: for gtk as most linux window managers manage the windows alpha
         #       and set it when its created.
         wx.CallAfter(self.InitWindowAlpha)
 
     __name__ = u"MainWindow"
 
+    def __del__(self):
+        ed_msg.Unsubscribe(self.OnUpdateFileHistory)
+
     #---- End Private Member Functions/Variables ----#
 
     #---- Begin Public Member Function ----#
@@ -308,29 +325,21 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         @type evt: wx.ActivateEvent
 
         """
-        app = wx.GetApp()
-        active = evt.GetActive()
-
-        # Don't needlessly push and pop events if we only have one window open
-        if self._loaded and len(app.GetMainWindows()) == 1:
-            if active:
-                wx.GetApp().SetTopWindow(self)
-                wx.UpdateUIEvent.SetUpdateInterval(205)
-                wx.UpdateUIEvent.SetMode(wx.UPDATE_UI_PROCESS_SPECIFIED)
-                self.SetExtraStyle(wx.WS_EX_PROCESS_UI_UPDATES)
-            else:
-                self.SetExtraStyle(0)
-                wx.UpdateUIEvent.SetMode(wx.UPDATE_UI_PROCESS_ALL)
+        if self._mlock.IsLocked():
+            # Deactivated for popup, leave handlers hooked up
             evt.Skip()
             return
 
+        app = wx.GetApp()
+        active = evt.GetActive()
+
         # Add or remove handlers from the event stack
-        if active:
-            wx.GetApp().SetTopWindow(self)
+        if active and not self._loaded:
             self._loaded = True
+            app.SetTopWindow(self)
 
             # Slow the update interval to reduce overhead
-            wx.UpdateUIEvent.SetUpdateInterval(205)
+            wx.UpdateUIEvent.SetUpdateInterval(215)
             wx.UpdateUIEvent.SetMode(wx.UPDATE_UI_PROCESS_SPECIFIED)
             self.SetExtraStyle(wx.WS_EX_PROCESS_UI_UPDATES)
 
@@ -340,51 +349,43 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             for handler in self._handlers['ui']:
                 app.AddUIHandlerForID(*handler)
 
-            app.SetTopWindow(self)
-
             # HACK find better way to do this later. It seems that on gtk the
-            #      window doesnt get activated until later than it does on the
+            #      window doesn't get activated until later than it does on the
             #      other platforms. So for panels that depend on updating their
             #      initial state we need to send out a fake update message here.
             if wx.Platform == '__WXGTK__':
                 nb = self.GetNotebook()
                 ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CHANGED,
                                    (nb, nb.GetSelection()))
-        else:
-            self.SetExtraStyle(0)
-
-            # HACK set update ui events back to proccess all here in case
-            # opened dialog needs them. Not sure why this is necessary but it
-            # is the only solution I could find to fix the external find
-            # dialogs so that their buttons become enabled when typing in the
-            # text control.
-            #
-            # If the windows that took the active position is another mainwindow
-            # it will set the events back to UPDATE_UI_PROCESS_SPECIFIED to
-            # prevent all the toolbars/menu items of each window from updating
-            # when they dont need to.
-            wx.UpdateUIEvent.SetMode(wx.UPDATE_UI_PROCESS_ALL)
-            for handler in self._handlers['menu']:
-                app.RemoveHandlerForID(handler[0])
-
-            for handler in self._handlers['ui']:
-                app.RemoveUIHandlerForID(handler[0])
-
+        elif not active:
             self._loaded = False
+            self.DeActivate()
 
         evt.Skip()
 
+    def OnUpdateFileHistory(self, msg):
+        """Update filehistory menu for new files that were opened
+        @param msg: Message object (data == filename)
+
+        """
+        # May get notified after/during delete
+        if self:
+            try:
+                self.filehistory.AddFileToHistory(msg.GetData())
+            except wx.PyAssertionError:
+                # ignore errors that wxMac sometimes raises about unicode data
+                pass
+
     def AddFileToHistory(self, fname):
         """Add a file to the windows file history as well as any
         other open windows history.
         @param fname: name of file to add
-        @todo: change the file history to a centrally manaaged object that
+        @todo: change the file history to a centrally managed object that
                all windows pull from to avoid this quick solution.
 
         """
-        for win in wx.GetApp().GetMainWindows():
-            if hasattr(win, 'filehistory'):
-                win.filehistory.AddFileToHistory(fname)
+        if _PGET('FHIST_LVL', 'int', 9) > 0:
+            ed_msg.PostMessage(ed_msg.EDMSG_ADD_FILE_HISTORY, fname)
 
     def AddMenuHandler(self, menu_id, handler):
         """Add a menu event handler to the handler stack
@@ -412,12 +413,14 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         else:
             self._handlers['ui'].append((menu_id, handler))
 
-    def DoOpen(self, evt, fname=u''):
+    def DoOpen(self, evt, fname=u'', lnum=-1):
         """ Do the work of opening a file and placing it
         in a new notebook page.
         @keyword fname: can be optionally specified to open
                         a file without opening a FileDialog
         @type fname: string
+        @keyword lnum: Explicitly set the line number to open the file to
+        @type lnum: int
 
         """
         try:
@@ -432,12 +435,12 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             elif not hasattr(sys, 'frozen'):
                 fdir = os.curdir
 
-            dlg = wx.FileDialog(self, _("Choose a File"), fdir, "",
+            dlg = wx.FileDialog(self, _("Editra: Open"), fdir, "",
                                 ''.join(syntax.GenFileFilters()),
                                 wx.OPEN | wx.MULTIPLE)
             dlg.SetFilterIndex(_PGET('FFILTER', 'int', 0))
 
-            if dlg.ShowModal() == wx.ID_OK:
+            if ebmlib.LockCall(self._mlock, dlg.ShowModal) == wx.ID_OK:
                 _PSET('FFILTER', dlg.GetFilterIndex())
                 for path in dlg.GetPaths():
                     if _PGET('OPEN_NW', default=False):
@@ -452,8 +455,47 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             self.LOG("[ed_main][info] CMD Open File: %s" % fname)
             self.nb.OpenPage(ebmlib.GetPathName(fname),
                              ebmlib.GetFileName(fname), quiet=True)
+            self.nb.GoCurrentPage()
+
+            # lnum arg is only used with command line open
+            if lnum >= 0:
+                buff = self.nb.GetCurrentCtrl()
+                buff.GotoLine(lnum)
+
             self.Raise()
 
+    def DeActivate(self):
+        """Helper method for the App to tell this window to remove
+        all its event handlers.
+
+        """
+        self.SetExtraStyle(0)
+
+        # HACK set update ui events back to process all here in case
+        # opened dialog needs them. Not sure why this is necessary but it
+        # is the only solution I could find to fix the external find
+        # dialogs so that their buttons become enabled when typing in the
+        # text control.
+        #
+        # If the windows that took the active position is another mainwindow
+        # it will set the events back to UPDATE_UI_PROCESS_SPECIFIED to
+        # prevent all the toolbars/menu items of each window from updating
+        # when they dont need to.
+        wx.UpdateUIEvent.SetMode(wx.UPDATE_UI_PROCESS_ALL)
+        self.FlushEventStack()
+
+    def FlushEventStack(self):
+        """Clear the Menu and UpdateUI event handler stack
+        @note: only unregisters this frames handlers from the app
+
+        """
+        app = wx.GetApp()
+        for handler in self._handlers['menu']:
+            app.RemoveHandlerForID(handler[0])
+
+        for handler in self._handlers['ui']:
+            app.RemoveUIHandlerForID(handler[0])
+
     def GetCommandbar(self):
         """Get this windows command bar
         @return: ed_cmdbar.CommandBarBase
@@ -568,7 +610,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                                     fname, _("File Not Found"),
                                     wx.OK | wx.ICON_WARNING)
             mdlg.CenterOnParent()
-            mdlg.ShowModal()
+            ebmlib.LockCall(self._mlock, mdlg.ShowModal)
             mdlg.Destroy()
             # Remove offending file from history
             self.filehistory.RemoveFileFromHistory(fnum)
@@ -616,6 +658,8 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             evt.Skip()
             return
 
+        # TODO: detect when save fails due to encoding and prompt to
+        #       request an encoding from the user.
         for ctrl in ctrls:
             fname = ebmlib.GetFileName(ctrl[1].GetFileName())
             if fname != '':
@@ -630,10 +674,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                     ed_mdlg.SaveErrorDlg(self, fname, err)
                     ctrl[1].GetDocument().ResetAll()
             else:
-                ret_val = self.OnSaveAs(ID_SAVEAS, ctrl[0], ctrl[1])
-                if ret_val:
-                    self._last_save = ctrl[1].GetFileName()
-                    self.AddFileToHistory(self._last_save)
+                self.OnSaveAs(ID_SAVEAS, ctrl[0], ctrl[1])
 
     def OnSaveAs(self, evt, title=u'', page=None):
         """Save File Using a new/different name
@@ -659,7 +700,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                             u''.join(syntax.GenFileFilters()),
                             wx.SAVE | wx.OVERWRITE_PROMPT)
 
-        if dlg.ShowModal() == wx.ID_OK:
+        if ebmlib.LockCall(self.mlock, dlg.ShowModal) == wx.ID_OK:
             path = dlg.GetPath()
             dlg.Destroy()
 
@@ -677,7 +718,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                 self.nb.SetPageText(self.nb.GetSelection(), fname)
                 self.nb.GetCurrentCtrl().FindLexer()
                 self.nb.UpdatePageImage()
-            return result
+                self.AddFileToHistory(ctrl.GetFileName())
         else:
             dlg.Destroy()
 
@@ -693,7 +734,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                                _("Profile") + " (*.ppb)|*.ppb",
                                 wx.SAVE | wx.OVERWRITE_PROMPT)
 
-            if dlg.ShowModal() == wx.ID_OK:
+            if ebmlib.LockCall(self._mlock, dlg.ShowModal) == wx.ID_OK:
                 profiler.TheProfile.Write(dlg.GetPath())
                 self.PushStatusText(_("Profile Saved as: %s") % \
                                     dlg.GetFilename(), SB_INFO)
@@ -713,8 +754,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                                 CONFIG['PROFILE_DIR'], "default.ppb",
                                 _("Profile") + " (*.ppb)|*.ppb", wx.OPEN)
 
-            result = dlg.ShowModal()
-            if result == wx.ID_OK:
+            if ebmlib.LockCall(self._mlock, dlg.ShowModal) == wx.ID_OK:
                 profiler.TheProfile.Load(dlg.GetPath())
                 self.PushStatusText(_("Loaded Profile: %s") % \
                                     dlg.GetFilename(), SB_INFO)
@@ -738,7 +778,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                                _("Session") + " (*.session)|*.session",
                                 wx.SAVE | wx.OVERWRITE_PROMPT)
 
-            if dlg.ShowModal() == wx.ID_OK:
+            if ebmlib.LockCall(self._mlock, dlg.ShowModal) == wx.ID_OK:
                 fname = dlg.GetPath()
                 if fname is None or not len(fname):
                     return
@@ -764,7 +804,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                                 CONFIG['SESSION_DIR'], u"",
                                 _("Session") + " (*.session)|*.session", wx.OPEN)
 
-            if dlg.ShowModal() == wx.ID_OK:
+            if ebmlib.LockCall(self._mlock, dlg.ShowModal) == wx.ID_OK:
                 fname = dlg.GetPath()
                 nbook = self.GetNotebook()
                 rval = nbook.LoadSessionFile(fname)
@@ -839,10 +879,12 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         """
         # Save session files
         session = _PGET('LAST_SESSION')
-        if not isinstance(session, basestring):
+        if not isinstance(session, basestring) or not len(session):
             session = os.path.join(CONFIG['SESSION_DIR'], u"__default.session")
         _PSET('LAST_SESSION', session)
-        self.nb.SaveSessionFile(session)
+        result = self.nb.SaveSessionFile(session)
+        if result is not None:
+            pass # TODO: report error?
 
         # Cleanup Controls
         self._exiting = True
@@ -914,6 +956,11 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         # Finally close the window
         self.LOG("[ed_main][evt] OnClose: Closing Main Frame")
         wx.GetApp().UnRegisterWindow(repr(self))
+
+        # Ensure that event handlers have been un registered from the app
+        wx.UpdateUIEvent.SetMode(wx.UPDATE_UI_PROCESS_ALL)
+
+        # Cleanup
         self.Destroy()
 
     #---- End File Menu Functions ----#
@@ -967,6 +1014,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         paneInfo = self._mgr.GetPane("EditPane")
         if paneInfo.IsMaximized():
             self._mgr.RestorePane(paneInfo)
+            ed_msg.PostMessage(ed_msg.EDMSG_UI_STC_RESTORE, context=self.GetId())
         else:
             self._mgr.RestoreMaximizedPane()
             self._mgr.MaximizePane(paneInfo)
@@ -989,7 +1037,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             fdata = wx.FontData()
             fdata.SetInitialFont(ctrl.GetDefaultFont())
             dlg = wx.FontDialog(self, fdata)
-            result = dlg.ShowModal()
+            result = ebmlib.LockCall(self._mlock, dlg.ShowModal)
             data = dlg.GetFontData()
             dlg.Destroy()
             if result == wx.ID_OK:
@@ -1013,7 +1061,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             import style_editor
             dlg = style_editor.StyleEditor(self)
             dlg.CenterOnParent()
-            dlg.ShowModal()
+            ebmlib.LockCall(self._mlock, dlg.ShowModal)
             dlg.Destroy()
         else:
             evt.Skip()
@@ -1078,27 +1126,65 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                         ID_SELECTALL, ID_UNDO, ID_REDO, ID_CUT, ID_COPY,
                         ID_PASTE, ID_LINE_BEFORE, ID_LINE_AFTER, ID_DUP_LINE,
                         ID_PASTE_AFTER, ID_COLUMN_MODE, ID_TOGGLE_FOLD,
+                        ID_CYCLE_CLIPBOARD,
                         ID_TOGGLE_ALL_FOLDS, ID_DELETE_LINE ]
 
         # Special handling for common clipboard related actions
         has_focus = self.FindFocus()
-        if has_focus != ctrl and e_id in active_only:
-            if has_focus is not None:
-                if e_id == ID_PASTE and hasattr(has_focus, 'Paste'):
-                    has_focus.Paste()
-                elif e_id == ID_CUT and hasattr(has_focus, 'Cut'):
-                    has_focus.Cut()
-                elif e_id == ID_COPY and hasattr(has_focus, 'Copy'):
-                    has_focus.Copy()
-                elif e_id == ID_REDO and hasattr(has_focus, 'Redo'):
-                    has_focus.Redo()
-                elif e_id == ID_UNDO and hasattr(has_focus, 'Undo'):
-                    has_focus.Undo()
+        is_stc = isinstance(has_focus, wx.stc.StyledTextCtrl)
+        if has_focus is not None:
+            if e_id == ID_PASTE and hasattr(has_focus, 'Paste'):
+                has_focus.Paste()
+                return
+            elif e_id == ID_CYCLE_CLIPBOARD:
+                start, end = has_focus.GetSelection()
+                start, end = min(start, end), max(start, end)
+
+                if is_stc:
+                    txt = has_focus.GetTextRange(start, end)
+                elif hasattr(has_focus, 'GetRange'):
+                    txt = has_focus.GetRange(start, end)
                 else:
-                    evt.Skip()
-            else:
-                evt.Skip()
-            return
+                    self.LOG("[ed_main][warn] no range meth in cycle clipboard")
+                    return
+
+                if not MainWindow.CLIPBOARD.IsAtIndex(txt):
+                    MainWindow.CLIPBOARD.Reset()
+
+                next = MainWindow.CLIPBOARD.GetNext()
+                if is_stc:
+                    has_focus.ReplaceSelection(next)
+                elif hasattr(has_focus, 'Replace'):
+                    has_focus.Replace(start, end, next)
+                else:
+                    return
+
+                has_focus.SetSelection(start, start+len(next))
+                return
+            elif e_id == ID_CUT and hasattr(has_focus, 'Cut'):
+                start, end = has_focus.GetSelection()
+                if is_stc:
+                    txt = has_focus.GetTextRange(start, end)
+                elif hasattr(has_focus, 'GetRange'):
+                    txt = has_focus.GetRange(start, end)
+                MainWindow.CLIPBOARD.Put(txt)
+                has_focus.Cut()
+                return
+            elif e_id == ID_COPY and hasattr(has_focus, 'Copy'):
+                start, end = has_focus.GetSelection()
+                if is_stc:
+                    txt = has_focus.GetTextRange(start, end)
+                elif hasattr(has_focus, 'GetRange'):
+                    txt = has_focus.GetRange(start, end)
+                MainWindow.CLIPBOARD.Put(txt)
+                has_focus.Copy()
+                return
+            elif e_id == ID_REDO and hasattr(has_focus, 'Redo'):
+                has_focus.Redo()
+                return
+            elif e_id == ID_UNDO and hasattr(has_focus, 'Undo'):
+                has_focus.Undo()
+                return
 
         menu_ids = list(syntax.SYNTAX_IDS)
         menu_ids.extend([ID_SHOW_EOL, ID_SHOW_WS, ID_INDENT_GUIDES, ID_SYNTAX,
@@ -1109,7 +1195,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                          ID_TRIM_WS, ID_SHOW_EDGE, ID_MACRO_START,
                          ID_MACRO_STOP, ID_MACRO_PLAY, ID_TO_LOWER,
                          ID_TO_UPPER, ID_USE_SOFTTABS,
-                         ID_GOTO_MBRACE, ID_HLCARET_LINE, ID_REVERT_FILE
+                         ID_GOTO_MBRACE, ID_HLCARET_LINE, ID_REVERT_FILE,
                          ])
         menu_ids.extend(active_only)
 
@@ -1137,7 +1223,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                 dlg.SetBitmap(bmp)
             dlg.CenterOnParent()
 
-            if dlg.ShowModal() == wx.ID_OK:
+            if ebmlib.LockCall(self._mlock, dlg.ShowModal) == wx.ID_OK:
                 nenc = dlg.GetEncoding()
                 doc.SetEncoding(nenc)
                 success = ctrl.ReloadFile()[0]
@@ -1163,7 +1249,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             self._paneNavi = eclib.AuiPaneNavigator(self, self._mgr, bmp,
                                                       _("Aui Pane Navigator"))
             self._paneNavi.SetReturnCode(wx.ID_OK)
-            self._paneNavi.ShowModal()
+            ebmlib.LockCall(self._mlock, self._paneNavi.ShowModal)
 
             sel = self._paneNavi.GetSelection()
             self._paneNavi.Destroy()
@@ -1175,6 +1261,9 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
                     if not paneInfo.IsShown():
                         paneInfo.Show()
                         self._mgr.Update()
+                        # Notify activation if the window supports it
+                        if hasattr(paneInfo.window, "OnShowAUIPane"):
+                            paneInfo.window.OnShowAUIPane()
                     paneInfo.window.SetFocus()
         else:
             evt.Skip()
@@ -1216,7 +1305,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             if hasattr(focus, 'CanRedo'):
                 enable = focus.CanRedo()
             evt.Enable(enable)
-        elif e_id in (ID_PASTE, ID_PASTE_AFTER):
+        elif e_id in (ID_PASTE, ID_PASTE_AFTER, ID_CYCLE_CLIPBOARD):
             if hasattr(focus, 'CanPaste'):
                 enable = focus.CanPaste()
             evt.Enable(enable)
@@ -1354,6 +1443,31 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         else:
             evt.Skip()
 
+    def OnCustomizeLangMenu(self, evt):
+        """Show the lexer menu customization dialog"""
+        if evt.GetId() == ID_LEXER_CUSTOM:
+            dlg = eclib.FilterDialog(self, title=_("Customize Menu"),
+                                     style=wx.DEFAULT_DIALOG_STYLE|wx.RESIZE_BORDER)
+            mconfig = _PGET("LEXERMENU", default=list())
+            flters = dict()
+            for item in syntax.SyntaxNames():
+                if item in mconfig:
+                    flters[item] = True
+                else:
+                    flters[item] = False
+            dlg.SetListValues(flters)
+            dlg.SetInitialSize()
+            dlg.CenterOnParent()
+
+            if ebmlib.LockCall(self._mlock, dlg.ShowModal) == wx.ID_OK:
+                includes = dlg.GetIncludes()
+                includes.sort()
+                _PSET("LEXERMENU", includes)
+                ed_msg.PostMessage(ed_msg.EDMSG_CREATE_LEXER_MENU)
+            dlg.Destroy()
+        else:
+            evt.Skip()
+
     def OnHelp(self, evt):
         """Handles help related menu events
         @param evt: Event fired that called this handler
@@ -1365,12 +1479,14 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         if e_id == ID_HOMEPAGE:
             page = HOME_PAGE
         elif e_id == ID_DOCUMENTATION:
-            page = HOME_PAGE + "/?page=doc"
+            page = HOME_PAGE + "/documentation"
         elif e_id == ID_TRANSLATE:
             page = I18N_PAGE
         elif e_id == ID_CONTACT:
             webbrowser.open("mailto:%s" % CONTACT_MAIL)
             return
+        elif e_id == ID_BUG_TRACKER:
+            page = "http://code.google.com/p/editra/issues/list"
         else:
             evt.Skip()
             return
@@ -1384,31 +1500,15 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
         except:
             self.PushStatusText(_("Error: Unable to open %s") % page, SB_INFO)
 
-    def ModifySave(self):
-        """Called when document has been modified prompting
-        a message dialog asking if the user would like to save
-        the document before closing.
-        @return: Result value of whether the file was saved or not
+    def PushStatusText(self, txt, field):
+        """Override so that our custom status bar's method gets called
+        do to these wxFrame methods not being exposed as virtuals.
 
         """
-        name = self.nb.GetCurrentCtrl().GetFileName()
-        if name == u"":
-            name = self.nb.GetPageText(self.nb.GetSelection())
+        sb = self.GetStatusBar()
+        sb.PushStatusText(txt, field)
 
-        dlg = wx.MessageDialog(self,
-                                _("The file: \"%s\" has been modified since "
-                                  "the last save point.\n\nWould you like to "
-                                  "save the changes?") % name,
-                               _("Save Changes?"),
-                               wx.YES_NO | wx.YES_DEFAULT | wx.CANCEL | \
-                               wx.ICON_INFORMATION)
-        result = dlg.ShowModal()
-        dlg.Destroy()
-
-        if result == wx.ID_YES:
-            self.OnSave(wx.MenuEvent(wx.wxEVT_COMMAND_MENU_SELECTED, ID_SAVE))
-
-        return result
+    SetStatusText = PushStatusText
 
     def SetTitle(self, title=u''):
         """Sets the windows title
@@ -1421,6 +1521,21 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
             name = " - " + name
         wx.Frame.SetTitle(self, title + name)
 
+    @classmethod
+    def UpdateClipboardRing(cls):
+        """Update the clipboard ring to sync it with the
+        system clipboard.
+        @note: for internal use only
+
+        """
+        txt = util.GetClipboardText()
+        if txt is None or cls.CLIPBOARD.IsAtIndex(txt):
+            return
+
+        # Something new has come in from an external program
+        cls.CLIPBOARD.Reset()
+        cls.CLIPBOARD.Put(txt)
+
 #-----------------------------------------------------------------------------#
 # Event handlers that don't need to be part of the class
 
