@@ -15,8 +15,8 @@ text documents and files.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_search.py 63753 2010-03-24 00:23:22Z CJP $"
-__revision__ = "$Revision: 63753 $"
+__svnid__ = "$Id: ed_search.py 65148 2010-07-31 17:18:14Z CJP $"
+__revision__ = "$Revision: 65148 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -45,8 +45,8 @@ class EdSearchEngine(ebmlib.SearchEngine):
     """Text searching engine"""
     def __init__(self, query, regex=True, down=True,
                  matchcase=True, wholeword=False):
-        ebmlib.SearchEngine.__init__(self, query, regex, down,
-                                     matchcase, wholeword)
+        super(EdSearchEngine, self).__init__(query, regex, down,
+                                             matchcase, wholeword)
         # Atttributes
         self._offset = 0
 
@@ -95,7 +95,7 @@ class SearchController(object):
         @param getstc: Callable to get the current buffer with
 
         """
-        object.__init__(self)
+        super(SearchController, self).__init__()
 
         # Attributes
         self._parent   = owner
@@ -468,6 +468,13 @@ class SearchController(object):
                      for fname in self._parent.GetTextControls()]
             ed_msg.PostMessage(ed_msg.EDMSG_START_SEARCH,
                                (engine.SearchInFiles, [files,], dict()))
+        elif smode == eclib.LOCATION_IN_CURRENT_DIR:
+            stc = self._stc()
+            path = ebmlib.GetPathName(stc.GetFileName())
+            engine.SetFileFilters(evt.GetFileFilters())
+            ed_msg.PostMessage(ed_msg.EDMSG_START_SEARCH,
+                               (engine.SearchInDirectory,
+                               [path,], dict(recursive=evt.IsRecursive())))
         elif smode == eclib.LOCATION_IN_FILES:
             path = evt.GetDirectory()
             engine.SetFileFilters(evt.GetFileFilters())
@@ -501,9 +508,9 @@ class SearchController(object):
             self._li_sel = self._finddlg.GetLookinSelection()
             self._filters = self._finddlg.GetFileFilters()
 
-            # Store in profile. Only save most recent 5 in history
-            if len(self._li_choices) > 5:
-                choices = self._li_choices[-5:]
+            # Store in profile. Only save most recent 8 in history
+            if len(self._li_choices) > 8:
+                choices = self._li_choices[-8:]
             else:
                 choices = self._li_choices
 
@@ -605,9 +612,9 @@ class SearchController(object):
                 if matches is not None:
                     self.ReplaceInStc(ctrl, matches, rstring, evt.IsRegEx())
                     results += len(matches)
-        elif smode == eclib.LOCATION_IN_FILES:
+        elif smode in (eclib.LOCATION_IN_CURRENT_DIR, eclib.LOCATION_IN_FILES):
             dlg = wx.MessageDialog(self._parent,
-                                   _("Sorry will be ready for next version"),
+                                   _("Sorry will be ready for future version"),
                                    _("Not implemented"),
 #                                   _("Warning this cannot be undone!"),
 #                                   _("Do Replace All?"),
@@ -777,7 +784,7 @@ class EdSearchCtrl(wx.SearchCtrl):
         @param menulen: max length of history menu
 
         """
-        wx.SearchCtrl.__init__(self, parent, id_, value, pos, size, style)
+        super(EdSearchCtrl, self).__init__(parent, id_, value, pos, size, style)
 
         # Attributes
         self._parent     = parent
@@ -1016,7 +1023,6 @@ class EdSearchCtrl(wx.SearchCtrl):
 
         """
         if evt.GetEventType() != wx.wxEVT_KEY_UP:
-            print "SKIPPY"
             evt.Skip()
             return
 
@@ -1199,7 +1205,7 @@ class SearchResultScreen(eclib.ControlBox):
         @param parent: parent window
 
         """
-        eclib.ControlBox.__init__(self, parent)
+        super(SearchResultScreen, self).__init__(parent)
 
         # Attributes
         self._meth = None
@@ -1343,7 +1349,7 @@ class SearchResultList(eclib.OutputBuffer):
     STY_SEARCH_MATCH = eclib.OPB_STYLE_MAX + 1
     RE_FIND_MATCH = re.compile('(.+) \(([0-9]+)\)\: .+')
     def __init__(self, parent):
-        eclib.OutputBuffer.__init__(self, parent)
+        super(SearchResultList, self).__init__(parent)
 
         # Attributes
         self._files = 0
