@@ -15,21 +15,26 @@ messages from ed_msg to display progress of different actions.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_statbar.py 57041 2008-11-30 07:33:36Z CJP $"
-__revision__ = "$Revision: 57041 $"
+__svnid__ = "$Id: ed_statbar.py 65718 2010-10-01 20:25:49Z CJP $"
+__revision__ = "$Revision: 65718 $"
 
 #--------------------------------------------------------------------------#
 # Imports
 import wx
+import wx.stc
 
 # Editra Libraries
 import ed_glob
 import util
 import ed_msg
-from syntax.syntax import GetFtypeDisplayName
-from eclib.pstatbar import ProgressStatusBar
+from syntax.synglob import GetDescriptionFromId
+from eclib import ProgressStatusBar, EncodingDialog
 from extern.decorlib import anythread
 
+#--------------------------------------------------------------------------#
+ 
+_ = wx.GetTranslation
+
 #--------------------------------------------------------------------------#
 
 class EdStatBar(ProgressStatusBar):
@@ -39,17 +44,31 @@ class EdStatBar(ProgressStatusBar):
     """
     ID_CLEANUP_TIMER = wx.NewId()
     def __init__(self, parent):
-        ProgressStatusBar.__init__(self, parent, style=wx.ST_SIZEGRIP)
+        super(EdStatBar, self).__init__(parent, style=wx.ST_SIZEGRIP)
 
-        # Setup
+        # Attributes
         self._pid = parent.GetId() # Save parents id for filtering msgs
         self._widths = list()
+        self._cleanup_timer = wx.Timer(self, EdStatBar.ID_CLEANUP_TIMER)
+        self._eolmenu = wx.Menu()
+        self._log = wx.GetApp().GetLog()
+
+        # Setup
         self.SetFieldsCount(6) # Info, vi stuff, line/progress
         self.SetStatusWidths([-1, 90, 40, 40, 40, 155])
-        self._cleanup_timer = wx.Timer(self, EdStatBar.ID_CLEANUP_TIMER)
+        self._eolmenu.Append(ed_glob.ID_EOL_MAC, u"CR",
+                             _("Change line endings to %s") % u"CR",
+                             kind=wx.ITEM_CHECK)
+        self._eolmenu.Append(ed_glob.ID_EOL_WIN, u"CRLF",
+                             _("Change line endings to %s") % u"CRLF",
+                             kind=wx.ITEM_CHECK)
+        self._eolmenu.Append(ed_glob.ID_EOL_UNIX, u"LF",
+                             _("Change line endings to %s") % u"LF",
+                             kind=wx.ITEM_CHECK)
 
         # Event Handlers
         self.Bind(wx.EVT_LEFT_DCLICK, self.OnLeftDClick)
+        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
         self.Bind(wx.EVT_TIMER, self.OnExpireMessage,
                   id=EdStatBar.ID_CLEANUP_TIMER)
 
@@ -59,16 +78,15 @@ class EdStatBar(ProgressStatusBar):
         ed_msg.Subscribe(self.OnUpdateText, ed_msg.EDMSG_UI_SB_TXT)
         ed_msg.Subscribe(self.OnUpdateDoc, ed_msg.EDMSG_UI_NB_CHANGED)
         ed_msg.Subscribe(self.OnUpdateDoc, ed_msg.EDMSG_FILE_SAVED)
-        ed_msg.Subscribe(self.OnUpdateDoc, ed_msg.EDMSG_UI_STC_LEXER)
-#        ed_msg.Subscribe(self.OnProgress, ed_msg.EDMSG_FILE_OPENING)
         ed_msg.Subscribe(self.OnUpdateDoc, ed_msg.EDMSG_FILE_OPENED)
+        ed_msg.Subscribe(self.OnUpdateDoc, ed_msg.EDMSG_UI_STC_LEXER)
 
     def __del__(self):
         """Unsubscribe from messages"""
         ed_msg.Unsubscribe(self.OnProgress)
         ed_msg.Unsubscribe(self.OnUpdateText)
         ed_msg.Unsubscribe(self.OnUpdateDoc)
-        ProgressStatusBar.__del__(self)
+        super(EdStatBar, self).__del__()
 
     def __SetStatusText(self, txt, field):
         """Safe method to use for setting status text with CallAfter.
@@ -77,47 +95,55 @@ class EdStatBar(ProgressStatusBar):
 
         """
         try:
-            ProgressStatusBar.SetStatusText(self, txt, field)
+            super(EdStatBar, self).SetStatusText(txt, field)
             self.AdjustFieldWidths()
 
-            if field == ed_glob.SB_INFO:
+            if field == ed_glob.SB_INFO and txt != u'':
                 # Start the expiration countdown
                 if self._cleanup_timer.IsRunning():
                     self._cleanup_timer.Stop()
                 self._cleanup_timer.Start(10000, True)
-        except wx.PyDeadObjectError:
+        except wx.PyDeadObjectError, wx.PyAssertionError:
+            # Getting some odd assertion errors on wxMac so just trap
+            # and ignore them for now
+            # glyphCount == (text.length()+1)" failed at graphics.cpp(2048)
+            # in GetPartialTextExtents()
             pass
+        except TypeError, err:
+            self._log("[edstatbar][err] Bad status message: %s" % str(txt))
+            self._log("[edstatbar][err] %s" % err)
 
     def AdjustFieldWidths(self):
-        """Adust each field width of status bar basing on the field text
+        """Adjust each field width of status bar basing on the field text
         @return: None
 
         """
-        widths = []
+        widths = [-1]
         # Calculate required widths
         # NOTE: Order of fields is important
         for field in [ed_glob.SB_BUFF,
                       ed_glob.SB_LEXER,
-                      ed_glob.SB_READONLY,
                       ed_glob.SB_ENCODING,
+                      ed_glob.SB_EOL,
                       ed_glob.SB_ROWCOL]:
-            width = self.GetTextExtent(self.GetStatusText(field))[0]
+            width = self.GetTextExtent(self.GetStatusText(field))[0] + 20
+            if width == 20:
+                width = 0
             widths.append(width)
 
         # Adjust widths
-        widths = [width + 20 for width in widths]
-        widths.insert(0, -1)
-        for idx, width in enumerate(list(widths)):
-            if width == 20:
-                widths[idx] = 0
-
         if widths[-1] < 155:
             widths[-1] = 155
 
+        # Only update if there are changes
         if widths != self._widths:
             self._widths = widths
             self.SetStatusWidths(self._widths)
 
+    def GetMainWindow(self):
+        """Method required for L{ed_msg.mwcontext}"""
+        return self.GetParent()
+
     def OnExpireMessage(self, evt):
         """Handle Expiring the status message when the oneshot timer
         tells us it has expired.
@@ -143,6 +169,37 @@ class EdStatBar(ProgressStatusBar):
         else:
             evt.Skip()
 
+    def OnLeftUp(self, evt):
+        """Handle left clicks on the status bar
+        @param evt: wx.MouseEvent
+
+        """
+        pt = evt.GetPosition()
+        if self.GetFieldRect(ed_glob.SB_EOL).Contains(pt):
+            rect = self.GetFieldRect(ed_glob.SB_EOL)
+            self.PopupMenu(self._eolmenu, (rect.x, rect.y))
+        elif self.GetFieldRect(ed_glob.SB_ENCODING).Contains(pt):
+            nb = self.GetTopLevelParent().GetNotebook()
+            buff = nb.GetCurrentCtrl()
+            dlg = EncodingDialog(nb,
+                                 msg=_("Change the encoding of the current document."),
+                                 title=_("Change Encoding"),
+                                 default=buff.GetEncoding())
+            bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_DOCPROP),
+                                           wx.ART_OTHER)
+            if bmp.IsOk():
+                dlg.SetBitmap(bmp)
+            dlg.CenterOnParent()
+
+            # TODO: should add EdFile callbacks for modification events instead
+            #       of using explicit statusbar refresh.
+            if dlg.ShowModal() == wx.ID_OK:
+                buff.SetEncoding(dlg.GetEncoding())
+                self.UpdateFields()
+            dlg.Destroy()
+        else:
+            evt.Skip()
+
     def OnProgress(self, msg):
         """Set the progress bar's state
         @param msg: Message Object
@@ -165,16 +222,10 @@ class EdStatBar(ProgressStatusBar):
             if mdata[1]:
                 self.Start(75)
             else:
+                # TODO: findout where stray stop event is coming from...
                 self.Stop()
-        elif mtype == ed_msg.EDMSG_FILE_OPENED:
-            self.StopBusy()
-        elif mtype == ed_msg.EDMSG_FILE_OPENING:
-            # Clear any text from the progress field
-            wx.CallAfter(self.__SetStatusText, u'', ed_glob.SB_ROWCOL)
-            # Data is the file path
-            self.SetRange(util.GetFileSize(mdata))
-            self.Start(75)
 
+    @ed_msg.mwcontext
     def OnUpdateDoc(self, msg):
         """Update document related fields
         @param msg: Message Object
@@ -200,7 +251,7 @@ class EdStatBar(ProgressStatusBar):
             wx.CallAfter(self.__SetStatusText, txt, field)
 
     def OnUpdateText(self, msg):
-        """Update the status bar text based on the recieved message
+        """Update the status bar text based on the received message
         @param msg: Message Object
 
         """
@@ -239,13 +290,16 @@ class EdStatBar(ProgressStatusBar):
             wx.CallAfter(self.__SetStatusText, doc.GetEncoding(),
                          ed_glob.SB_ENCODING)
             wx.CallAfter(self.__SetStatusText,
-                         GetFtypeDisplayName(cbuff.GetLangId()),
+                         GetDescriptionFromId(cbuff.GetLangId()),
                          ed_glob.SB_LEXER)
 
-    #        pstatbar.ProgressStatusBar.SetStatusText(self,
-    #                                                 ,
-    #                                                 ed_glob.SB_READONLY)
+            eol = { wx.stc.STC_EOL_CR : u"CR",
+                    wx.stc.STC_EOL_LF : u"LF",
+                    wx.stc.STC_EOL_CRLF : u"CRLF" }
+            wx.CallAfter(self.__SetStatusText,
+                         eol[cbuff.GetEOLMode()],
+                         ed_glob.SB_EOL)
+
         except wx.PyDeadObjectError:
             # May be called asyncronasly after the control is already dead
             return
-
